#include "if_gw.h"
#include "ble_client.h"
#include "wit.h"
#include "rtc.h"
#include "ram.h"
#include "WIT2FSM.h"
#include "if_apkg_code.h"
#include <time.h>
#include <sys/time.h>



#define COLLECT_DEALY_S         5
#define CONNECT_INTERVAL_RANGE  30
#define SENSOR_INTERVAL_TIME    20

static bool g_bWaveStatus = false;
static bool g_bFeaStatus  = false;
static sGwStatus gGwStatus = IDLE_STATUS;
extern sSenInfo gSenInfo;
extern bool g_bIsLowPower;

void if_gataway_process(void)
{
   static sGroupTable*    pxGroupTB  = NULL;
   static sSenTable*      pxSenTable = NULL;
   static unsigned long   operation = 0;
   static unsigned long   useEnable = 0;
   static unsigned char   groupId = 0;
   static unsigned long   numberOfDeliveries = 0;
   static unsigned int    u32SendStartTick = 0;
   static unsigned long   ulOldTime = 0; 
   unsigned char          uctrey = 3;
   unsigned long*         pxUlTimeStamp = NULL;  
   unsigned long          ulCurrentTime = 0;  
   static struct timeval  time_old;
   static struct timeval  time_now;
   switch(gGwStatus)
   { 
      case IDLE_STATUS: 
         for(int i = 0; i < WIT_SENSOR_GROUP_MAX_NUM ; i++)
         {
            pxGroupTB = getGroupTable(i);
            useEnable = pxGroupTB->ulGUseEnable;
            operation = pxGroupTB->ulOperation;
            numberOfDeliveries = pxGroupTB->ulConnectNum;
            // 判断组是否使用
            if(useEnable == 0)
               continue;
            // 判断是否为组执行操作   
            if(operation == 0) 
            {
               ble_clear_dev_complete();
               continue;
            }  
            gGwStatus = COLLECT_STATUS;
            pxUlTimeStamp = WIT_GetWITDataTime();
            // 清除电机工作状态
            for(int i = 0; i < WIT_SENSOR_GROUP_MAX_NUM ; i++ )
            {
               WIT_SetMotorWork(i, false);
            } 
            groupId = pxGroupTB->ulGroup; 
            *pxUlTimeStamp = RTC_InternalGetTimeStamp();
            // ulOldTime = *pxUlTimeStamp;
            ulOldTime = pxGroupTB->ulConnectTime;
            WIT2FSM_AddGatewayID(groupId - 1);
            u32SendStartTick = xTaskGetTickCount();
            break;
         }
      break;
      case COLLECT_STATUS:
         static unsigned long  ulComplete = 0;
         sDevInfo*      pxSenInfo = NULL;  
         bool           bStatus = false;
         unsigned long  ulSleepInvTime = 0;
         unsigned long  ulRestTime = 0;
         bool           bRevFlag = true;
         unsigned long  ulPeriod = 0;
         for(int i = 0 ; i < WIT_SENSOR_MAX_CONNECT_NUM * WIT_SENSOR_GROUP_MAX_NUM; i++ )
         {
            pxSenInfo = ble_find_dev_from_addr(i+1);  
            if(pxSenInfo == NULL)
            {
               continue;
            }
            // 连接, 判断是否有异常震动
            if(pxSenInfo->bIsConnect == true && pxSenInfo->group == groupId)
            {
               printf("采集: addr: %d, index %d\r\n", pxSenInfo->addr, i); 
               pxSenTable = getSensorFromAddr(pxSenInfo->addr);
               if(g_bIsLowPower == true)
                  ulPeriod = pxGroupTB->ulPeriod;
               else
                  ulPeriod = pxGroupTB->ulFeaPeriod;
               // 如果采集了特征值, 获取
               if((ulComplete & (1 << (i % WIT_SENSOR_MAX_CONNECT_NUM))) == 0)
               {  
                  gettimeofday(&time_old, NULL);
                  if(pxSenInfo->senCode != 0)
                  {
                     ulComplete |= 1 << (i % WIT_SENSOR_MAX_CONNECT_NUM);
                     continue;
                  }
                  uctrey = 3;
                  while(uctrey--)
                  {
                     if(pxSenInfo->bIsConnect == true)
                     {
                        if(eWCPReadDataAddress(i, SENSOR_CHR_WRITE_HANDLER, SENSOR_MOTOR_RUN_APKG_ADDR, 1, (unsigned char *)&bStatus) == WCP_OK)
                           break;
                     }
                     vTaskDelay(10 / portTICK_PERIOD_MS);
                  } 
                  if(uctrey == 0)
                  {
                     gSenInfo.xDevInfo[i].cComplete = 1;
                     ble_gap_terminate(gSenInfo.xDevInfo[i].handle, BLE_ERR_REM_USER_CONN_TERM); 
                     pxSenInfo->bIsConnect = false;
                  }
                  if(bStatus == true) 
                  {
                     WIT_SetMotorWork(groupId - 1, bStatus);   
                     bRevFlag = WIT_GetSensorDataAndUpload(i, pxGroupTB, &g_bWaveStatus, &g_bFeaStatus);
                     if(bRevFlag == false)
                     {
                        ble_gap_terminate(gSenInfo.xDevInfo[i].handle, BLE_ERR_REM_USER_CONN_TERM); 
                        gSenInfo.xDevInfo[i].cComplete = 1;
                        ESP_LOGE("ble","超时");
                     }
                  }           
                  if(pxSenTable->ulConfigEnable == 1 && pxSenInfo->bIsConnect == true && bRevFlag == true)
                  {
                     if(if_sensor_config(i, pxSenInfo->addr - 1, &pxSenTable->ulConfigEnable)) 
                     {
                        WIT_SetUpdateConfigureAreaFlag(true);
                     }
                  } 
                  if(pxSenTable->ulUpdate == 1 && pxSenInfo->bIsConnect == true && bRevFlag == true)
                  {
                     if(send_update_bin(i))
                     {
                        pxSenTable->ulUpdate = 0;
                        gSenInfo.xDevInfo[i].bIsConnect = false;
                        WIT_SetUpdateConfigureAreaFlag(true);
                     }
                  }
                  ulComplete |= 1 << (i % WIT_SENSOR_MAX_CONNECT_NUM);
                  ulCurrentTime = RTC_InternalGetTimeStamp();
                  gettimeofday(&time_now, NULL);
                  pxSenInfo->gtms = (time_now.tv_sec - time_old.tv_sec) * 1000 + (time_now.tv_usec - time_old.tv_usec) / 1000;
                  // 休眠
                  if(ulPeriod > ulCurrentTime - ulOldTime)
                  {
                     printf("采集时间pcf %ld s\r\n", ulCurrentTime - ulOldTime);
                     ulSleepInvTime = ulPeriod - (ulCurrentTime - ulOldTime);
                     uctrey = 3;
                     while(uctrey--)
                     {
                        printf("休眠时间1 %ld\r\n", ulSleepInvTime + ((pxSenTable->ulVirAddr - 1) * SENSOR_INTERVAL_TIME));
                        if(pxSenInfo->bIsConnect == true)
                        {
                           if(if_set_wakeup_time(i, ulSleepInvTime + ulCurrentTime, (ulSleepInvTime + ((pxSenTable->ulVirAddr - 1) * SENSOR_INTERVAL_TIME)), false) == true)       
                           {
                              ble_gap_terminate(gSenInfo.xDevInfo[i].handle, BLE_ERR_REM_USER_CONN_TERM); 
                              break;
                           }
                        }
                        vTaskDelay(10 / portTICK_PERIOD_MS);
                     }
                     if(uctrey == 0)
                     {
                        ble_gap_terminate(gSenInfo.xDevInfo[i].handle, BLE_ERR_REM_USER_CONN_TERM); 
                        gSenInfo.xDevInfo[i].cComplete = 1;
                     }
                     pxSenInfo->bIsConnect = false;
                  }
                  else
                  { 
                     ulRestTime = (ulCurrentTime - ulOldTime) % ulPeriod; 
                     ulSleepInvTime = ulPeriod - ulRestTime;          
                     uctrey = 3;
                     while(uctrey--)
                     {
                        printf("休眠时间2 %ld\r\n", ulSleepInvTime + ((pxSenTable->ulVirAddr - 1) * SENSOR_INTERVAL_TIME));
                        if(pxSenInfo->bIsConnect == true) 
                        {
                           if(if_set_wakeup_time(i, ulSleepInvTime + ulCurrentTime, (ulSleepInvTime + ((pxSenTable->ulVirAddr - 1) * SENSOR_INTERVAL_TIME)), false) == true)
                           {
                              ble_gap_terminate(gSenInfo.xDevInfo[i].handle, BLE_ERR_REM_USER_CONN_TERM); 
                              break;
                           }
                        }   
                        vTaskDelay(10 / portTICK_PERIOD_MS);
                     }
                     if(uctrey == 0)
                     {
                        ble_gap_terminate(gSenInfo.xDevInfo[i].handle, BLE_ERR_REM_USER_CONN_TERM);   
                        gSenInfo.xDevInfo[i].cComplete = 1;
                     }
                     pxSenInfo->bIsConnect = false;          
                  }
                  printf("addr: %d 操作完成 \r\n", pxSenInfo->addr);
                  break;
               } 
               // 本次上发组已上发，发送睡眠间隔
               else
               {
                  ulCurrentTime = RTC_InternalGetTimeStamp();
                  ulSleepInvTime =  ulPeriod - ((ulCurrentTime - ulOldTime) % ulPeriod);
                  printf("addr: %d 已操作, 睡眠 %ld\r\n", pxSenInfo->addr, ulSleepInvTime + ((pxSenTable->ulVirAddr - 1) * SENSOR_INTERVAL_TIME));
                  uctrey = 3;
                  while(uctrey--)
                  {
                     if(pxSenInfo->bIsConnect == true)
                     {
                        if(if_set_wakeup_time(i, ulSleepInvTime + ulCurrentTime, (ulSleepInvTime + ((pxSenTable->ulVirAddr - 1) * SENSOR_INTERVAL_TIME)), false) == true)
                        {
                           ble_gap_terminate(gSenInfo.xDevInfo[i].handle, BLE_ERR_REM_USER_CONN_TERM); 
                           break;
                        }
                     }
                     vTaskDelay(10 / portTICK_PERIOD_MS);
                  }
                  if(uctrey == 0)
                  {
                     gSenInfo.xDevInfo[i].cComplete = 1;
                     ble_gap_terminate(gSenInfo.xDevInfo[i].handle, BLE_ERR_REM_USER_CONN_TERM); 
                  }
                  pxSenInfo->bIsConnect = false;
               }   
            }
         }

         if(ulComplete == numberOfDeliveries) 
         {
            ulCurrentTime = RTC_InternalGetTimeStamp();
            ulComplete = 0;
            pxGroupTB->ulOperation = 0;
            pxGroupTB->ulConnectTime = ulSleepInvTime + ulCurrentTime;
            WIT_SetUpdateConfigureAreaFlag(true);
            WIT2FSM_SendGatewayFeatureToServer(groupId - 1);
            gGwStatus = COMPLETE_STATUS;  
            break;  
         }  
         if(abs((long long)xTaskGetTickCount() - (long long)u32SendStartTick) > 4 * 30 * 1000.0f)
         { 
            ulCurrentTime = RTC_InternalGetTimeStamp();
            ulComplete = 0;
            pxGroupTB->ulOperation = 0;
            pxGroupTB->ulConnectTime = ulSleepInvTime + ulCurrentTime;
            WIT_SetUpdateConfigureAreaFlag(true);
            WIT2FSM_SendGatewayFeatureToServer(groupId - 1);
            gGwStatus = COMPLETE_STATUS;
            break;   
         }     
      break;
      case COMPLETE_STATUS:
         if(g_bIsLowPower == true)
         {
            for(int i = 0 ; i < WIT_SENSOR_MAX_CONNECT_NUM * WIT_SENSOR_GROUP_MAX_NUM ; i++)
            {
               if(gSenInfo.xDevInfo[i].bIsConnect == true || gSenInfo.xDevInfo[i].handle != 0xFFFF)
               {
                  ble_gap_terminate(gSenInfo.xDevInfo[i].handle, BLE_ERR_REM_USER_CONN_TERM); 
                  gSenInfo.xDevInfo[i].bIsConnect = false;  
               }
            }
            for(int i = 0 ; i < WIT_SENSOR_MAX_CONNECT_NUM * WIT_SENSOR_GROUP_MAX_NUM ; i++)
            {
               if(gSenInfo.xDevInfo[i].senCode == 1)
               {
                  ErrorStatusSend(gSenInfo.xDevInfo[i].addr, gSenInfo.xDevInfo[i].senCode, ulOldTime);
               }
               if(gSenInfo.xDevInfo[i].bDeliveries == true)
               {
                  printf("上发 addr: %d\r\n", gSenInfo.xDevInfo[i].addr);
                  WIT_SendWaveToServer(groupId - 1, gSenInfo.xDevInfo[i].addr, NULL);
                  gSenInfo.xDevInfo[i].bDeliveries = false;
               }
               gSenInfo.xDevInfo[i].cComplete = 0;
            }
         }
         if(g_bWaveStatus == true)
         {
            pxGroupTB->ulWavePushEnable = 0;
            g_bWaveStatus = false;
         }
         if(g_bFeaStatus == true)
         {
            pxGroupTB->ulFeaPushEnable = 0;	
            g_bFeaStatus = false;
         }
         
         pxGroupTB  = NULL;
         pxSenTable = NULL;
         operation = 0;
         useEnable = 0;
         groupId = 0;
         numberOfDeliveries = 0;
         u32SendStartTick = 0;  
         gGwStatus = IDLE_STATUS;  
         printf("COMPLETE_STATUS\r\n");
      break;
   }
}

bool if_check_wakeup_time(unsigned short group, unsigned short addr, unsigned short index)
{
   sGroupTable    *pxGroupTable = NULL;
   sGroupTable    *pxGroup = NULL;
   unsigned long  ulWakeUpInv = 0;
   unsigned long  ulTimestamp = 0;
   unsigned long  ulIsOperation = 0;
   unsigned long  useEnable = 0;
   unsigned long  currentOperation = 0;
   unsigned long  ulFeaPeriod = 0;
   unsigned long  ConnectTime = 0;
   unsigned long  ulWillConnectCnt = 0;
   
   int repeat = 3;
   ulTimestamp = RTC_InternalGetTimeStamp();
   // 判断组是否使用
   pxGroupTable = getGroupFromGroup(group);
   gSenInfo.xDevInfo[index].bIsConnect = true; 
   if(pxGroupTable == NULL)
   {
      while(repeat--)
      {
         if(if_set_wakeup_time(index, ulTimestamp, 300, false))
         {
            gSenInfo.xDevInfo[index].bIsConnect = false;
            printf("无组 addr: %d\r\n", addr);
            return false;
         }
      }
       return false;
   }
   repeat = 3;
   if(gSenInfo.xDevInfo[index].cComplete == 1)
   {
      while(repeat--)
      {
         if(if_set_wakeup_time(index, ulTimestamp, 300, false))
         {
            gSenInfo.xDevInfo[index].bIsConnect = false;
            printf("无组 addr: %d\r\n", addr);
            return false;
         }
      }
      return false;
   }
   currentOperation = pxGroupTable->ulOperation;
   if(g_bIsLowPower == true)
      ulFeaPeriod = pxGroupTable->ulPeriod;
   else
      ulFeaPeriod = pxGroupTable->ulFeaPeriod;
   ConnectTime = pxGroupTable->ulConnectTime;
   printf("pxGroupTable %ld, Operation %ld\r\n", pxGroupTable->ulGroup, pxGroupTable->ulOperation);
   if(currentOperation == 1)
   {
      return true; 
   }
   // 当前时间连接
   if(ulTimestamp < ConnectTime - CONNECT_INTERVAL_RANGE || ulTimestamp > ConnectTime + CONNECT_INTERVAL_RANGE * 3)
   {
      ESP_LOGE("ble","time %ld, connecttime %ld", ulTimestamp, ConnectTime);
      if(ulTimestamp < ConnectTime - CONNECT_INTERVAL_RANGE)
      {
         ulWakeUpInv = (ConnectTime - ulTimestamp) % ulFeaPeriod;
         if(ulWakeUpInv >= ulFeaPeriod)
            ulWakeUpInv = ulFeaPeriod;
      }
      else if(ulTimestamp > ConnectTime + CONNECT_INTERVAL_RANGE * 3)
      {
         ulWakeUpInv  = ulFeaPeriod - ((ulTimestamp - ConnectTime) % ulFeaPeriod); 
         if(ulWakeUpInv >= ulFeaPeriod)
            ulWakeUpInv = ulFeaPeriod;
      }
      pxGroupTable->ulConnectTime = ulWakeUpInv + ulTimestamp; 
      WIT_SetUpdateConfigureAreaFlag(true);
      repeat = 3;
      while(repeat--)
      {
         if(if_set_wakeup_time(index, ConnectTime, (ulWakeUpInv + ((gSenInfo.xDevInfo[index].ulVirAddr - 1) * SENSOR_INTERVAL_TIME)), false))
         {
            gSenInfo.xDevInfo[index].bIsConnect = false;
            printf("addr: %d, sleep %ld\r\n", addr, (ulWakeUpInv + ((gSenInfo.xDevInfo[index].ulVirAddr - 1) * SENSOR_INTERVAL_TIME)));
            ble_gap_terminate(gSenInfo.xDevInfo[index].handle, BLE_ERR_REM_USER_CONN_TERM);
            break;
         }          
      }  
      if(repeat == 0)
         return false;      
   }
   else
   {
      repeat = 3;
      currentOperation = 1;
      for(int i = 0 ; i< WIT_SENSOR_GROUP_MAX_NUM ; i++ )
      {
         pxGroup = getGroupTable(i);
         useEnable = pxGroup->ulGUseEnable;
         if(pxGroup->ulOperation == 1)
         {
            ulIsOperation = pxGroup->ulOperation;
         }
         if(useEnable == 0)
            continue;
         ulWillConnectCnt +=pxGroup->ulConnectCnt;
      }
      ESP_LOGD("wakeup","sensor num %ld", ulWillConnectCnt);
      if(ulWillConnectCnt > 1) 
      {
         if(ulIsOperation == 0)
         {
            ESP_LOGD("wakeup"," 没有对组操作");
            pxGroupTable->ulOperation = currentOperation;
         }
         else
         {
            ESP_LOGD("wakeup","有对一组操作, 延迟其他组操作");
            repeat = 3;
            while(repeat--)
            {
               if(if_set_wakeup_time(index, ConnectTime, ((pxGroupTable->ulGroup * 300) + (gSenInfo.xDevInfo[index].ulVirAddr - 1) * SENSOR_INTERVAL_TIME), false))
               {
                  pxGroupTable->ulOperation = 0;
                  pxGroupTable->ulConnectTime += 180;
                  gSenInfo.xDevInfo[index].bIsConnect = false;
                  WIT_SetUpdateConfigureAreaFlag(true);
                  printf("addr: %d, sleep %ld\r\n", addr, ((pxGroupTable->ulGroup * 300) + (gSenInfo.xDevInfo[index].ulVirAddr - 1) * SENSOR_INTERVAL_TIME));
                  break;
               }         
            } 
            if(repeat == 0) 
               return false;
         }
      }
      else
      {
         pxGroupTable->ulOperation = currentOperation;
      }
   }
   pxGroupTable = NULL;
   pxGroup = NULL;
   return true;
}

void if_set_gw_status(sGwStatus xGwStatus)
{
   sGroupTable  *pxGroupTable = NULL;
   gGwStatus = xGwStatus;  
   if(gGwStatus == IDLE_STATUS)
   {
      for(int i = 0; i < WIT_SENSOR_GROUP_MAX_NUM; i++)
      {
         pxGroupTable = getGroupTable(i);
         if(pxGroupTable->ulGUseEnable == 1)
         {
            pxGroupTable->ulOperation = 0;
         }
      }
   }
}

void if_calibration_time(unsigned long ulLasttime)
{
   sGroupTable    *pxGroupTable = NULL;
   unsigned long ulTimestamp;
   unsigned long ulLnterval;
   ulTimestamp = RTC_InternalGetTimeStamp();
   for(int i = 0; i < WIT_SENSOR_GROUP_MAX_NUM; i++)
   {
      pxGroupTable = getGroupTable(i);
      if(pxGroupTable->ulGUseEnable == 1)
      {
         if(((int)(pxGroupTable->ulConnectTime - ulLasttime)) > 0)
         {
            ulLnterval = pxGroupTable->ulConnectTime - ulLasttime;
            pxGroupTable->ulConnectTime = ulLnterval + ulTimestamp;
         }
         else
         {
            pxGroupTable->ulConnectTime = ulTimestamp + pxGroupTable->ulPeriod;
         }
      }
   }
}

