#ifndef WIT_H
#define WIT_H

#include "wit_config.h"
#include "stdbool.h"
#if CONFIG_W5100S_SPI_ETHERNET > 0
	#include "../Telecom/eth/W5100S/wizchip_conf.h"
#endif
#include "mb.h"

#include "ts_iot01b.h"

#if CONFIG_BLUETOOTH > 0
   #include "if_ble_config.h"
#endif
/* Exported macros -----------------------------------------------------------*/
#define WIT_MB_SN_ADDR 0x0000
#define WIT_MB_SN_COUNT 2
#define WIT_MB_VERSION_ADDR 0x0002
#define WIT_MB_MODULE_NAME_ADDR 0x0003
#define WIT_MB_MODULE_NAME_COUNT 6
#define WIT_MB_RESET_ADDR 0x0009
#define WIT_MB_CCID_ADDR 0x000D
#define WIT_MB_CCID_COUNT 10
#define WIT_MB_MQTT_BROKER_ADDR 0x0017
#define WIT_MB_MQTT_BROKER_COUNT 32
#define WIT_MB_MQTT_PORT_ADDR 0x0037
#define WIT_MB_MQTT_USER_ADDR 0x0038
#define WIT_MB_MQTT_USER_COUNT 16
#define WIT_MB_MQTT_PASSWORD_ADDR 0x0048
#define WIT_MB_ID_ADDRESS 0x005B
#define WIT_MB_BAUD_RATE_ADDRESS 0x005C
#define WIT_MB_BAUD_RATE_COUNT 2
#define WIT_MB_PARITY_ADDRESS 0x005E
#define WIT_MB_MQTT_PASSWORD_COUNT 16
/*--------------------------------------------------**/
#define WIT_MB_ALARM_ADDR 0x0064
#define WIT_MB_SENSOR_CHANNEL_COUNT_ADDR 0x5FFE
#define WIT_MB_SENSOR_EXIST_STATUS_ADDR  0x5FFF
#define WIT_MB_SENSOR_ID_ADDR           0x6004
#define WIT_MB_SENSOR_TEMP_ADDR         0x6005
#define WIT_MB_SENSOR_FEATURE_ADDR      0x6006
#define WIT_MB_SENSOR_FEATURE_COUNT     468

#define WIT_MB_SENSOR_NUMBER_ONE            0x6401 
#define WIT_MB_SENSOR_NUMBER_TWO            0x6501 
#define WIT_MB_SENSOR_NUMBER_THREE          0x6601 
#define WIT_MB_SENSOR_NUMBER_FOUR           0x6701 
#define WIT_MB_SENSOR_NUMBER_FIVE           0x6801 
#define WIT_MB_SENSOR_NUMBER_SIX            0x6901 
#define WIT_MB_SENSOR_NUMBER_TOTAL_SIZE     0x05FF
#define WIT_MB_SENSOR_NUMBER_SIZE           13

#if CONFIG_BLUETOOTH 
#define WIT_SENSOR_MAX_CONNECT_NUM          6
#define WIT_SENSOR_DEVICE_MAX_NAME_LENGTH   48
#endif
#define WIT_MODULE_NAME_MAX_SIZE  24

#define WIT_MQTT_MAX_NAME_LENGTH 64
#define WIT_MQTT_MAX_BROKER_LENGTH (64 + 4)
#define WIT_MQTT_MAX_PORT_LENGTH 8
#define WIT_MQTT_MAX_USER_LENGTH (32 + 4)
#define WIT_MQTT_MAX_PASSWORD_LENGTH (32 + 4)
#define WIT_TEMP_COUNT (4)
#define WIT_TEMP_COUNT_RESERVE (4)

//阈值定义
typedef struct
{
	float fHighLimit; //上限
	float fLowLimit;  //下限
} Threshold;

//工作阈值与时间记录
typedef struct
{
	unsigned char ucSensorAddr;	 //选用传感器编号
	Threshold xOTempWorkThr;	 //油温上限
	Threshold xXVarWorkThr;		 // x方差上限 -> 振动阈值
	unsigned long ulWorkMinutes; //工作时长
} WorkDetective;

typedef enum
{
	GatewayMode_Learning = 0, //学习模式(不预警 启动阈值自设定功能)
	GatewayMode_Normal = 1	  //正常模式
} GatewayMode;

#if CONFIG_BLUETOOTH 
typedef struct {
	char           cName[WIT_SENSOR_DEVICE_MAX_NAME_LENGTH];
	char           cMac[40];
	unsigned long  ulVirAddr;
	unsigned long  ulPeriod;	
	unsigned long  ulAddr;
	unsigned long  ulGroup;
	unsigned long  ulUpdate;
	unsigned long  ulUseEnable;
	unsigned long  ulConfigEnable;
} sSenTable;

typedef struct {
	unsigned long   ulGroup;
	unsigned long   ulGUseEnable;
	unsigned long   ulOperation;
	unsigned long   ulConnectNum;
	unsigned long   ulConnectCnt;
	unsigned long   ulPeriod;
	unsigned long   ulFeaPeriod;
	unsigned long   ulPeriodCnt;
	unsigned long   ulFeaPeriodCnt;
	unsigned long   ulConnectTime;
	unsigned long   ulWavePushEnable;
	unsigned long   ulFeaPushEnable;
	unsigned long   packid;
	unsigned long   ulWorkMinutes;
	char            groupNo[32];
} sGroupTable;

#endif

//配置数据 (配置存储到内部FLASH --- 阈值与存储数据存储到外部FLASH)
typedef struct 
{
    unsigned short  addr;
    unsigned short id;
    unsigned short sn;
    bool   bIsTimeout;
    bool   bIsUsed;
    bool   bIsPlugIn;
    unsigned int rope_value;
    bool   beingWaiting;
    unsigned char status;
}IOTDev;

typedef struct
{
	char cModuleName[WIT_MODULE_NAME_MAX_SIZE];		  //模块名称
	unsigned long ulModuleNameValue;				  //检查模块名是否配置过
	unsigned long ulPeriod;			//发送周期
	unsigned long ulConfigureValue; //检查以上内容是否配置过
	unsigned long ulSerialNumber;	//序列号
	unsigned long ulPackageID;		//包ID
	bool bConfigUpdated;			//配置是否更新
	unsigned long ulInputSpeed;		//输入转速
	bool bAutoThreshold;			//自动设置阈值功能是否使能
#if CONFIG_ESP32_WIFI > 0
    WIFI_OpMode eOpMode;				   // WIFI 模式
	char cAPName[WIFI_MAX_NAME_LENGTH];	   //处于AP时的热点名称
	char cReserved1[WIFI_MAX_NAME_LENGTH]; //兼容用
	char cReserved2[WIFI_MAX_NAME_LENGTH]; //兼容用
	WIT_WIFIInfo xWIFI; // WIFI配置信息
#endif
	WorkDetective xWorkDet;							  //检测电机是否运行
	unsigned long ulWIFIAndThrValue;				  //检查以上信息是否配置过
	char cTargetIP[4];
	unsigned long ulPort;
	char cMQTTBroker[WIT_MQTT_MAX_BROKER_LENGTH];	  // mqtt服务器
	char cMQTTPort[WIT_MQTT_MAX_PORT_LENGTH];		  // mqtt端口
	char cMQTTUser[WIT_MQTT_MAX_USER_LENGTH];		  // mqtt用户名
	char cMQTTPassword[WIT_MQTT_MAX_PASSWORD_LENGTH]; // mqtt密码
	unsigned long ulMQTTInfoValue;					  //检查以上内容是否配置过
	GatewayMode eGatewayMode;						  //网关工作模式
	unsigned short usTotalLearningCount;			  //学习模式下总共需学习数量
	unsigned short usCurrentLearningCount;			  //学习模式下当前学习数量
	unsigned long ulWavePeriod;						  //波形上发周期
	unsigned char ucModbusID;						  // modbus 地址
	eMBParity eModbusParity;						  // modbus 校验位
	unsigned long ulModbusBaudRate;					  // modbus 波特率
	unsigned long ulLearningInfoValue;				  //检查以上信息是否配置过
	unsigned long ulDelayTime;						  //振动后延时时间
	unsigned long ulDelayInfoValue;					  //检查以上信息是否配置过
	unsigned long ulFeaturePeriodCopy;				  //特征周期备份
	unsigned long ulWavePeriodCopy;					  //波形周期备份
	unsigned long ulCopyValueInfo;					  //检查以上信息是否配置过
	unsigned long spdSel;
	unsigned long spdSyncEnable;
#if CONFIG_BLUETOOTH > 0
	sGroupTable xGroupTable[WIT_SENSOR_GROUP_MAX_NUM];
    sSenTable   xSenTable[WIT_SENSOR_MAX_CONNECT_NUM * WIT_SENSOR_GROUP_MAX_NUM];
	unsigned long ulBleBinSize;
	unsigned long ulSenParmInfoValue;
#if CONFIG_W5100S_SPI_ETHERNET > 0	
	wiz_NetInfo g_xConfigIp;
	unsigned long ulEthNetInfoValue;				  //检查以上信息是否配置过	
#endif
    IOTDev my_IOTDev[IOTDEV_MAX_NUM];
    unsigned long my_IOTThreshold;
#endif
} WIT_Configure;

/* Exported functions --------------------------------------------------------*/
void WIT_Init(void);
void WIT_Run(void);

bool WIT_SendWaveToServer(unsigned char groupId, unsigned char ucSensorAddr, void (*pFuncHandler)(unsigned char, bool));

unsigned short WIT_GetGWAndSensorFeatureCount(void);

// MB Read
unsigned short WIT_MB_GetSerialNumber(unsigned short usAddress);
unsigned short WIT_MB_GetVersion(void);
unsigned short WIT_MB_GetModuleName(unsigned short usAddress);
unsigned short WIT_MB_GetMQTTBroker(unsigned short usAddress);
unsigned short WIT_MB_GetMQTTPort(void);
unsigned short WIT_MB_GetMQTTUser(unsigned short usAddress);
unsigned short WIT_MB_GetMQTTPassword(unsigned short usAddress);
unsigned short WIT_MB_GetAlarm(unsigned short usAddress);
unsigned short WIT_MB_GetSensorExistStatus(void);
unsigned short WIT_MB_GetSensorID(void);
unsigned short WIT_MB_GetSensorTemperture(void);
unsigned short WIT_MB_GetSensorFeature(unsigned short usAddress);
unsigned short WIT_MB_GetNumberSensorFeature(unsigned short usAddress);
unsigned short WIT_MB_GetID(void);
unsigned short WIT_MB_GetBaudRate(unsigned short usAddress);
unsigned short WIT_MB_GetParity(void);
bool ErrorStatusSend(unsigned int sensorAddr, unsigned int uiCode, unsigned long ts);

// MB Write
void WIT_MB_SetMQTTBroker(unsigned short usAddress, unsigned short usValue);
void WIT_MB_SetMQTTPort(unsigned short usValue);
void WIT_MB_SetMQTTUser(unsigned short usAddress, unsigned short usValue);
void WIT_MB_SetMQTTPassword(unsigned short usAddress, unsigned short usValue);
void WIT_MB_SetSensorID(unsigned short usValue);
void WIT_MB_SetChannelCount(unsigned short usValue);
void WIT_MB_SetID(unsigned short usValue);
void WIT_MB_SetBaudRate(unsigned short usAddress, unsigned short usValue);
void WIT_MB_SetParity(unsigned short usValue);

void WIT_SetMotorWork(unsigned char ucIndex, bool bStatus);
bool WIT_GetMotorWork(unsigned char ucIndex);

#if CONFIG_BLUETOOTH > 0
	WIT_Configure *getGWConfigure(void);
	unsigned long *WIT_GetWITDataTime(void);
	sSenTable     *getSensorTable(int index);
	sGroupTable   *getGroupTable(int index);
	void setDevWaveCount(unsigned char index, unsigned long count);
	void setSamplesFrequency(unsigned char index, unsigned short freq);
	unsigned long getDevWaveCount(unsigned char index);
	void setUpdateFlag(unsigned long flag);
	void setBleBinSize(unsigned long size);
	unsigned long getBleBinSize(void);
	void WIT_SetUpdateConfigureAreaFlag(bool bflag);
	void WIT_SetAddrConfigure(unsigned char ucAddr, unsigned long ulFlag);
	bool WIT_GetSensorDataAndUpload(unsigned int uiAddr, sGroupTable  *pxGroupTable, bool *bWaveStatus, bool *bFeaStatus);
	void if_clear_multi_gateway_data(void);
	void clear_sensor_updata_flag(void);
	void clear_feature_use(void);
	sSenTable   *getSensorFromAddr(int addr);
	sGroupTable *getGroupFromGroup(int group);
	void WIT2FSM_AddGatewayID(unsigned char groupId);
	unsigned long *get_ulPort(void);
	char          *sent_mqtt_ip(void);
#endif
#if CONFIG_W5100S_SPI_ETHERNET > 0
	unsigned long get_status_dhcp(void);
	wiz_NetInfo  *get_gw_eth_info(void);
#endif
void Message_Deal_Function(void);
unsigned long getIotConnectNum(void);

#endif
