/* Includes ------------------------------------------------------------------*/
#include "quectel.h"
#include "common.h"
#include "celluar_comm.h"
#include "celluar_event.h"

#if QUECTEL > 0
/* Private macros ------------------------------------------------------------*/
#define QUECTEL_NET_RECEIVED_MESSAGE			"+QIURC: \"recv\""
#define QUECTEL_NET_CLOSED_MESSAGE				"+QIURC: \"closed\""
#define QUECTEL_POWER_DOWN_MESSAGE				"POWER DOWN"
#define QUECTEL_PULL_CARD_MESSAGE				"+CPIN: NOT READY"
#define QUECTEL_PDP_DEACT_MESSAGE				"+QIURC: \"pdpdeact\""
#define QUECTEL_HTTP_READ_MESSAGE				"CONNECT"

/* Private variables ---------------------------------------------------------*/ 
/* Extern variables ----------------------------------------------------------*/ 
extern Celluar g_xCelluar;

/* Private functions ---------------------------------------------------------*/ 
#if CELLUAR_TCPIP_ENABLE > 0
static long 	Quectel_SearchForAPN(void);
static bool 	Quectel_ConfigAPN(char *pcAPN, char *pcUser, char *pcPassword);
#endif

/**
  * @brief  Quectel poll
  * @param  None
  * @retval None
  */
void Quectel_Poll(void)
{
	Celluar_Buffer *pxBuffer;
	xSemaphoreTake(g_xCelluar.xReceivedSemHandle, portMAX_DELAY);
	
	pxBuffer = &g_xCelluar.xReceiveQueue.xReceiveBuffer[(((signed long)g_xCelluar.xReceiveQueue.ulRear - 1) + CELLUAR_QUEUE_SIZE) % CELLUAR_QUEUE_SIZE];
	
#if DEBUG > 0
	Debug_Print("\r\n");
	Debug_PrintData(pxBuffer->ucBuffer, pxBuffer->ulBufferLen);
#endif
	if (memstr(pxBuffer->ucBuffer, QUECTEL_NET_RECEIVED_MESSAGE, pxBuffer->ulBufferLen))
	{
		unsigned char *pucBufferPointer = pxBuffer->ucBuffer;
		unsigned long ulRestHandlerLen = pxBuffer->ulBufferLen;
		
		while ((pucBufferPointer = (unsigned char *)memstr(pucBufferPointer, QUECTEL_NET_RECEIVED_MESSAGE, ulRestHandlerLen)) != NULL)
		{
			unsigned char *pucSocketNum;
			unsigned char ucSocketID 	= 0;
			unsigned char *pucMsgStart;
			unsigned char *pucDataLen;
			unsigned char *pucData;
			unsigned long ulDataLen;
			
			//Socket ID | Data  // ulDataLen = 1(socket id) + len(data)
			pucMsgStart 	= (unsigned char *)strstr((char *)pucBufferPointer, "+QIURC: \"recv\",");
			pucSocketNum 	= pucMsgStart + strlen("+QIURC: \"recv\",");
			ucSocketID 		= atol((char *)pucSocketNum);
			pucDataLen 		= pucSocketNum + ((ucSocketID >= 10) ? 3 : 2);
			pucData 		= (unsigned char *)strstr((char *)pucMsgStart, "\n");
			
			if (pucMsgStart != NULL && pucData != NULL)
			{
				unsigned long ulRestDataLength = 0;
				
				*pucData 			= ucSocketID;
				ulDataLen 			= atol((char *)pucDataLen);
				ulRestDataLength 	= pxBuffer->ucBuffer + pxBuffer->ulBufferLen - 1 - pucData;
				if (ulDataLen <= ulRestDataLength)
					Celluar_EventPost(Celluar_EventNetReceived, pucData, ulDataLen + 1);
				else		//error occured
					break;
				
				//set offset
				pucBufferPointer = pucData + 1 + ulDataLen;
				//clear receive data to ' ' (space)
				memset(pucMsgStart, 32, pucBufferPointer - pucMsgStart);
				//cal rest length
				ulRestHandlerLen = pxBuffer->ucBuffer + pxBuffer->ulBufferLen - pucBufferPointer;
			}
			else
			{
				break;
			}
		}
	}
	else if ((memstr(pxBuffer->ucBuffer, QUECTEL_HTTP_READ_MESSAGE, pxBuffer->ulBufferLen)))
	{
		//NULL
	}
	else if(memstr(pxBuffer->ucBuffer, QUECTEL_NET_CLOSED_MESSAGE, pxBuffer->ulBufferLen))
	{
		char *pcSocketIDPointer = NULL;
		unsigned char ucClosedSocketID;
		
		pcSocketIDPointer = memstr((char *)pxBuffer->ucBuffer, "+QIURC: \"closed\",", pxBuffer->ulBufferLen) + strlen("+QIURC: \"closed\",");
		ucClosedSocketID = atol(pcSocketIDPointer);
		Celluar_EventPost(Celluar_EventNetClosed, &ucClosedSocketID, sizeof(ucClosedSocketID));
	}
	else if (memstr(pxBuffer->ucBuffer, QUECTEL_POWER_DOWN_MESSAGE, pxBuffer->ulBufferLen))
	{
		Celluar_EventPost(Celluar_EventPowerDown, NULL, 0);
	}
	else if (memstr(pxBuffer->ucBuffer, QUECTEL_PULL_CARD_MESSAGE, pxBuffer->ulBufferLen))
	{
		Celluar_EventPost(Celluar_EventPullCard, NULL, 0);
	}
	else if (memstr(pxBuffer->ucBuffer, QUECTEL_PDP_DEACT_MESSAGE, pxBuffer->ulBufferLen))
	{
		Celluar_EventPost(Celluar_EventPDPDeact, NULL, 0);
	}
	

	if (g_xCelluar.bWaiting)
		xSemaphoreGive(g_xCelluar.xResponseSemHandle);
	else
		Celluar_QueuePop(&g_xCelluar.xReceiveQueue);
}

/**
  * @brief  Quectel TCP/IP Init
  * @param  None
  * @retval None
  */
void Quectel_BasicSettings(void)
{
#if CELLUAR_GNSS_ENABLE > 0
	//开启GPS
	Quectel_ControlGNSSPower(true);
#endif
#if CELLUAR_TCPIP_ENABLE > 0
	//显示LAC CI 基站信息
	Quectel_SetNetworkRegistrationMode(2);
	//显示国家信息
	Quectel_SetCOPSMode(0, 2);
#endif
}

#if CELLUAR_TCPIP_ENABLE > 0
/**
  * @brief  Quectel TCP/IP Init
  * @param  None
  * @retval None
  */
void Quectel_TCPIPInit(unsigned long *pulRecommendedAPNIndex)
{
	if (g_xCelluar.bIsPowerOn)
	{
		//检查卡
		if (Quectel_GetCardStatus() == Celluar_OK)
		{
			g_xCelluar.bHasSIMCard = true;
		}
		else
		{
			//无卡
			g_xCelluar.bHasSIMCard = false;
		}
		
		//检查到SIM卡后进行网络设置
		if (g_xCelluar.bHasSIMCard)
		{
			unsigned char ucWaitTime = CELLUAR_TRYCOUNT_90;
			bool bRegisted = false;
			bool bNetAttached = false;
			
			//等待网络 90s超时
			while (ucWaitTime && g_xCelluar.bHasSIMCard)
			{
				//等待网络注册
				if (Quectel_GetRegistrationStatus() == Celluar_OK)
				{
					bRegisted = true;
					break;
				}
				ucWaitTime--;
				vTaskDelay(1000);
			}
			
			ucWaitTime = CELLUAR_TRYCOUNT_90;
			while (bRegisted && ucWaitTime && g_xCelluar.bHasSIMCard)
			{
				//等待网络附着
				if (Quectel_GetNetworkAttachStatus() == Celluar_OK)
				{
					bNetAttached = true;
					break;
				}
				ucWaitTime--;
				vTaskDelay(1000);
			}
			
			if (bNetAttached)
			{
				//检查网络状态
				if (Quectel_GetNetStatus(NULL, 0) == Celluar_OK)
				{
					g_xCelluar.bHasNetwork = true;
				}
				
				//网络附着成功，设置推荐APN，如果失败搜寻APN 在g_pcAPNList
				if (g_xCelluar.bHasNetwork == false && pulRecommendedAPNIndex != NULL && *pulRecommendedAPNIndex < CELLUAR_APN_COUNT)
				{
					if (Quectel_ConfigAPN(g_pcAPNList[*pulRecommendedAPNIndex], "", "") == true)
					{
						g_xCelluar.ulCurrentAPNIndex = *pulRecommendedAPNIndex;
						g_xCelluar.bHasNetwork = true;
					}
				}
					
				//推荐APN不成功，搜寻APN列表
				if (g_xCelluar.bHasNetwork == false)
				{
					long lTemp;
					if ((lTemp = Quectel_SearchForAPN()) != -1)
					{
						if (pulRecommendedAPNIndex != NULL)
							*pulRecommendedAPNIndex = lTemp;
						g_xCelluar.ulCurrentAPNIndex = lTemp;
						g_xCelluar.bHasNetwork = true;
					}
				}
			}
		}	
	}
}
#endif

#if CELLUAR_NTP_ENABLE > 0
bool Quectel_SyncNetworkTime(Calendar *pxCalendar)
{
	return false;
}
#endif

#if CELLUAR_TCPIP_ENABLE > 0
/**
  * @brief  Quectel search apn in buffer
  * @param  None
  * @retval success apn index (-1 error)
  */
static long 	Quectel_SearchForAPN()
{
	unsigned long ulIndex;
	for (ulIndex = 0;ulIndex < CELLUAR_APN_COUNT;ulIndex++)
	{
		if (Quectel_DeactivateGPRS() == Celluar_OK)
		{
			if (Quectel_ConfigAPN(g_pcAPNList[ulIndex], "", ""))
			{
				return ulIndex;
			}
		}
		else
		{
			break;
		}
	}
	
	return -1;
}

/**
  * @brief  Quectel config apn
  * @param  None
  * @retval true / false
  */
static bool 	Quectel_ConfigAPN(char *pcAPN, char *pcUser, char *pcPassword)
{	
	if (Quectel_SetAPN(pcAPN, pcUser, pcPassword) == Celluar_OK)
	{
		if (Quectel_BringUpWirelessConnection() == Celluar_OK)
		{
			if (Quectel_GetLocalIPAddress() == Celluar_OK)
			{
				return true;
			}
		}
	}
	
	return false;
}
#endif

#if CELLUAR_HTTP_ENABLE > 0
/**
  * @brief  Quectel http post
  * @param  None
  * @retval true / false
  */
bool Quectel_HTTPPost(char *pcURL, unsigned char *pucPostData, unsigned long ulPostLen, unsigned char *pucResponse, unsigned long ulBufferLen, unsigned long *pulRespLen, unsigned long *pulErr)
{
	return false;
}

/**
  * @brief  Quectel http get
  * @param  None
  * @retval true / false
  */
bool Quectel_HTTPGet(char *pcURL, unsigned char *pucResponse, unsigned long ulBufferLen, unsigned long *pulRespLen, unsigned long *pulErr)
{
	if (pcURL == NULL)
		return false;
	
	if (Quectel_SetHTTPURL(pcURL) == Celluar_OK)
	{
		if (Quectel_HTTPGetMethod(pulRespLen, pulErr) == Celluar_OK)
		{
			if (Quectel_StoreHTTPServerResponseToRAM() == Celluar_OK)
			{
				if (pucResponse != NULL && ulBufferLen <= *pulRespLen + CELLUAR_HTTP_RESPONSE_HEAD_SIZE)
					return false;
				
				if (pucResponse != NULL && ulBufferLen > *pulRespLen + CELLUAR_HTTP_RESPONSE_HEAD_SIZE)
				{
					//copy
					if (Quectel_ReadHTTPServerResponseFromRAM(0, *pulRespLen, pucResponse, ulBufferLen, pulRespLen) != Celluar_OK)
					{
						return false;
					}
					Quectel_TerminateHTTPService();
				}

				return true;
			}
		}
	}
	
	return false;
}
#endif
#endif

