/* Includes ------------------------------------------------------------------*/
#include "quectel_tcpip.h"

#if QUECTEL > 0
/*Only use context id 1*/
#if CELLUAR_TCPIP_ENABLE > 0
/* Private macros ------------------------------------------------------------*/
#define QUECTEL_SOCKET_STATE_INITIAL			0
#define QUECTEL_SOCKET_STATE_OPENING			1
#define QUECTEL_SOCKET_STATE_CONNECTED			2
#define QUECTEL_SOCKET_STATE_LISTENING			3
#define QUECTEL_SOCKET_STATE_CLOSING			4

/* Private type --------------------------------------------------------------*/

/* Extern variables ----------------------------------------------------------*/
extern Celluar_Socket g_xCelluarSocket[];
extern unsigned long g_ulMaxSocketID;

/**
  * @brief  Quectel get net attach status
  * @param  None
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus Quectel_GetNetworkAttachStatus()
{
	Celluar_ErrorStatus eResult;
	char cCommand[12], cResponse[48];
	
	Celluar_Take();
	sprintf(cCommand, "AT+CGATT?\r");
	if ((eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "+CGATT:", cResponse, sizeof(cResponse), NULL, CELLUAR_WAITTIME_500MS)) == Celluar_OK)
	{
		if (!strstr(cResponse, "+CGATT: 1"))
		{
			eResult = Celluar_Error;
		}
	}
	Celluar_Give();
	return eResult;
}

/**
  * @brief  Quectel set net attach status
  * @param  bStatus:true / false
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus Quectel_NetworkAttach(bool bStatus)
{
	Celluar_ErrorStatus eResult;
	char cCommand[16];
	
	Celluar_Take();
	sprintf(cCommand, "AT+CGATT=%d\r", bStatus ? 1 : 0);
	eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "OK", NULL, 0, NULL, CELLUAR_WAITTIME_500MS);
	Celluar_Give();
	return eResult;
}

/**
  * @brief  Quectel set apn
  * @param  
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus Quectel_SetAPN(char *pcAPN, char *pcUser, char *pcPassword)
{
	Celluar_ErrorStatus eResult;
	char cCommand[200];
	
	Celluar_Take();
	sprintf(cCommand, "AT+QICSGP=1,1,\"%s\",\"%s\",\"%s\",1\r", pcAPN ? pcAPN : "", pcUser ? pcUser : "", pcPassword ? pcPassword : "");
	eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "OK", NULL, 0, NULL, CELLUAR_WAITTIME_2S);
	Celluar_Give();
	return eResult;
}

/**
  * @brief  Bring up wireless connection with GPRS or CSD
  * @param  None
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus Quectel_BringUpWirelessConnection()
{
	Celluar_ErrorStatus eResult;
	char cCommand[12];
	
	Celluar_Take();
	sprintf(cCommand, "AT+QIACT=1\r");
	eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "OK", NULL, 0, NULL, CELLUAR_WAITTIME_2M);
	Celluar_Give();
	return eResult;
}

/**
  * @brief  Get Local IP Address
  * @param  None
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus Quectel_GetLocalIPAddress()
{
	Celluar_ErrorStatus eResult;
	char cCommand[12];
	
	Celluar_Take();
	sprintf(cCommand, "AT+QIACT?\r");
	eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), ".", NULL, 0, NULL, CELLUAR_WAITTIME_500MS);
	Celluar_Give();
	return eResult;
}

/**
  * @brief  GPRS Deactivate
  * @param  None		
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus Quectel_DeactivateGPRS()
{
	Celluar_ErrorStatus eResult;
	char cCommand[16];
	
	Celluar_Take();
	sprintf(cCommand, "AT+QIDEACT=1\r");
	eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "OK", NULL, 0, NULL, CELLUAR_WAITTIME_1S);
	Celluar_Give();
	return eResult;
}

/**
  * @brief  Get socket status
  * @param  pcNetStatus: Return status:
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus Quectel_GetNetStatus(char *pcNetStatus, unsigned long ulBufferSize)
{
	Celluar_ErrorStatus eResult;
	char cCommand[16], cResponse[200];
	
	Celluar_Take();
	sprintf(cCommand, "AT+QISTATE=1,0\r");
	eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "+QISTATE:", cResponse, sizeof(cResponse), NULL, CELLUAR_WAITTIME_500MS);
	Celluar_Give();
	return eResult;
}

/**
  * @brief  Connect only use direct push mode context id 1 TCP
  * @param  ulSocketID
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus Quectel_Connect(unsigned long ulSocketID)
{
	Celluar_ErrorStatus eResult;
	char cCommand[150], cResponse[48], cSuccessMsg[24];
	Celluar_Socket *pxSocket;
	
	if (ulSocketID >= g_ulMaxSocketID)
		return Celluar_Error;
	
	pxSocket = &g_xCelluarSocket[ulSocketID];
	if (pxSocket->bUsed == false)
		return Celluar_Error;
	
	Celluar_Take();
	sprintf(cCommand, "AT+QIOPEN=1,%lu,\"%s\",\"%s\",%s,0,1\r", ulSocketID, pxSocket->cMode, pxSocket->cDestinationHost, pxSocket->cDestinationPort);
	sprintf(cSuccessMsg, "+QIOPEN: %lu,0", ulSocketID);
	if ((eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "+QIOPEN:", cResponse, sizeof(cResponse), NULL, CELLUAR_WAITTIME_2M)) == Celluar_OK)
	{
		if (strstr(cResponse, cSuccessMsg))
		{
			pxSocket->bStatus = true;
		}
		else
		{
			eResult = Celluar_Error;
		}
	}
#if CELLUAR_RECONNECT_ENABLE > 0
	pxSocket->bReconnectEnable = true;
#endif
	Celluar_Give();
	return eResult;
}

/**
  * @brief  Disconnect 
  * @param  ulSocketID
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus Quectel_Disconnect(unsigned long ulSocketID)
{
	Celluar_ErrorStatus eResult;
	char cCommand[20];
	Celluar_Socket *pxSocket;
	
	if (ulSocketID >= g_ulMaxSocketID)
		return Celluar_Error;
	
	pxSocket = &g_xCelluarSocket[ulSocketID];
	if (pxSocket->bUsed == false)
		return Celluar_Error;
	Celluar_Take();	
	sprintf(cCommand, "AT+QICLOSE=%ld\r", ulSocketID);
	eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "OK", NULL, 0, NULL, CELLUAR_WAITTIME_10S);
	pxSocket->bStatus = false;
	Celluar_Give();
	return eResult;
}

/**
  * @brief  send data
  * @param  ulSocketID
  * @param  pucData: pointer to data
  * @param  ulDataSize:  length of data
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus Quectel_Send(unsigned long ulSocketID, unsigned char *pucData, unsigned long ulDataSize)
{
	Celluar_ErrorStatus eResult = Celluar_Error;
	char cCommand[20], cResponse[48] = {0};
	Celluar_Socket *pxSocket;
	
	if (ulSocketID >= g_ulMaxSocketID || pucData == NULL)
		return Celluar_Error;
	
	pxSocket = &g_xCelluarSocket[ulSocketID];
	if (pxSocket->bUsed == false || pxSocket->bStatus == false)
		return Celluar_Error;
	
	Celluar_Take();
	while (ulDataSize)
	{
		unsigned long ulSendSize = ulDataSize > CELLUAR_TCP_UDP_MAX_SIZE ? CELLUAR_TCP_UDP_MAX_SIZE : ulDataSize;

		sprintf(cCommand, "AT+QISEND=%ld,%ld\r", ulSocketID, ulSendSize);
		if ((eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), ">", NULL, 0, NULL, CELLUAR_WAITTIME_500MS)) == Celluar_OK)
		{
			if ((eResult = Celluar_SendCommandAndWaitForResponse((char *)pucData, ulSendSize, "SEND", cResponse, sizeof(cResponse), NULL, CELLUAR_WAITTIME_1M)) == Celluar_OK)
			{
				//SEND OK or SEND FAIL
				if (strstr(cResponse, "SEND FAIL") != NULL)
				{
					eResult = Celluar_Error;   
				}
			}
		}
		
		if (eResult != Celluar_OK)
			break;
		
		ulDataSize 	-= ulSendSize;
		pucData		+= ulSendSize;
	}
	Celluar_Give();
	return eResult;
}

/**
  * @brief  get socket status
  * @param  ulSocketID		
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus Quectel_GetSocketStatus(unsigned long ulSocketID)
{
	Celluar_ErrorStatus eResult;
	char cCommand[20], cResponse[200];
	Celluar_Socket *pxSocket;
#if CELLUAR_MULTIIPCONNECTION_ENABLE > 0
	char cNumber[4];
	char cNetStatus[100];
#endif
	
	if (ulSocketID >= g_ulMaxSocketID)
		return Celluar_Error;
	
	pxSocket = &g_xCelluarSocket[ulSocketID];
	if (pxSocket->bUsed == false)
		return Celluar_Error;
	
	Celluar_Take();
	sprintf(cCommand, "AT+QISTATE=1,%ld\r", ulSocketID);
	if ((eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "OK", cResponse, sizeof(cResponse), NULL, CELLUAR_WAITTIME_500MS)) == Celluar_OK)
	{
		char *pcStart = NULL;
		eResult = Celluar_Error;
		
		pcStart = strstr(cResponse, "+QISTATE: ");
		
		if (pcStart != NULL)
		{
			char *pcSplitedData[6] = {NULL, NULL, NULL, NULL, NULL, NULL};
			
			pcStart += strlen("+QISTATE: ");
			Split(pcSplitedData, 6, pcStart, ",");
			
			if (pcSplitedData[5] != NULL)
			{
				unsigned char ucState;
				
				ucState = atol(pcSplitedData[5]);
				
				if (ucState == QUECTEL_SOCKET_STATE_CONNECTED)
				{
					pxSocket->bStatus = true;
					eResult = Celluar_OK;
				}
			}
		}
	}
	Celluar_Give();
	return eResult;
}

/**
  * @brief  set ssl
  * @param  bStatus: on or off			
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus Quectel_SetSSL(bool bStatus)
{
	//reserved
	return Celluar_Error;
}

#endif
#endif
