/* Includes ------------------------------------------------------------------*/
#include "simcom_ntp.h"

#if SIMCOM > 0
#if CELLUAR_NTP_ENABLE > 0
/**
  * @brief  SIMCom set ntp server
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus SIMCom_SetNTPServerAndTimeZone(char *pcNTPServer, unsigned char ucTimeZone)
{
	Celluar_ErrorStatus eResult;
	char cCommand[60];
	
	if (pcNTPServer == NULL)
		return Celluar_Error;
	
	Celluar_Take();
	sprintf(cCommand, "AT+CNTP=\"%s\",%d\r", pcNTPServer, ucTimeZone);
	eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "OK", NULL, NULL, NULL, CELLUAR_WAITTIME_500MS);
	Celluar_Give();
	return eResult;
}

/**
  * @brief  SIMCom sync time
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus SIMCom_SynchronizeNetworkTime()
{
	Celluar_ErrorStatus eResult;
	char cCommand[12], cResponse[60];
	
	Celluar_Take();
	sprintf(cCommand, "AT+CNTP\r");
	if ((eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "+CNTP:", cResponse, sizeof(cResponse), NULL, CELLUAR_WAITTIME_1M)) == Celluar_OK)
	{
		if (!strstr(cResponse, "+CNTP: 1"))
		{
			eResult = Celluar_Error;
		}
	}
	Celluar_Give();
	return eResult;
}

/**
  * @brief  SIMCom get local time
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus SIMCom_GetLocalTime(Calendar *pxCalendar)
{
	Celluar_ErrorStatus eResult;
	char cCommand[12], cResponse[100];
	
	Celluar_Take();
	sprintf(cCommand, "AT+CCLK?\r");
	if ((eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "+CCLK:", cResponse, sizeof(cResponse), NULL, CELLUAR_WAITTIME_500MS)) == Celluar_OK)
	{
		char *pcSplitedData[2] = {NULL, NULL};
		
		Split(pcSplitedData, 2, cResponse, "\"");
		if (pcSplitedData[1] != NULL)
		{
			if (pxCalendar != NULL)
			{
				pxCalendar->usYear = (pcSplitedData[1][0] - '0') * 10 + (pcSplitedData[1][1] - '0') + 2000;
				pxCalendar->ucMonth = (pcSplitedData[1][3] - '0') * 10 + (pcSplitedData[1][4] - '0');
				pxCalendar->ucDay = (pcSplitedData[1][6] - '0') * 10 + (pcSplitedData[1][7] - '0');
				pxCalendar->ucHour = (pcSplitedData[1][9] - '0') * 10 + (pcSplitedData[1][10] - '0');
				pxCalendar->ucMinute = (pcSplitedData[1][12] - '0') * 10 + (pcSplitedData[1][13] - '0');
				pxCalendar->ucSecond = (pcSplitedData[1][15] - '0') * 10 + (pcSplitedData[1][16] - '0');
			}
		}
		else
		{
			eResult = Celluar_Error;
		}
	}
	Celluar_Give();
	return eResult;
}
#endif
#endif
