/* Includes ------------------------------------------------------------------*/
#include <string.h>
#include "acc.h"
#include "mb.h"
#include "mbmaster.h"
#if CONFIG_BLUETOOTH > 0
	#include "if_apkg_code.h"
	#include <math.h> 
	#include "esp_log.h"
	#include "common.h"
	#include <stdio.h>
#endif

#if CONFIG_RS485_RTU > 0
/* Private macros ------------------------------------------------------------*/
#define ACC_VERSION_ADDR 0x0002
#define ACC_MODE_ADDR 0x0064
#define ACC_STATUS_ADDR 0x006D
#define ACC_WAVE_COUNT_ADDR 0x006E
#define ACC_SAMPLING_RATE_ADDR 0x0076
#define ACC_SHOCK_THR_ADDR 0x0079
#define ACC_ACTIVE_FLAG_ADDR 0x007B
#define ACC_TEMP_ADDR 0x03E8
#define ACC_BASIC_FEATURE_ADDR 0x03E9
#define ACC_FREQ_1X_ADDR 0x03FD
#define ACC_BASIC_FEATURE_TOTAL_COUNT 29
#define ACC_WAVE_ADDR 0x07D0
#define ACC_WAVE_INDEX_ADDR 0x37D0
#define ACC_BEARING_STATUS_ADDR 0x37D1
#define ACC_BEARING_INFO_ADDR 0x6000
#define ACC_BEARING_FEATURE_ADDR 0x8000
#define ACC_GEAR_STATUS_ADDR 0x37D2
#define ACC_GEAR_INFO_ADDR 0x7000
#define ACC_GEAR_FEATURE_ADDR 0x9000
#define ACC_SPEED_ADDR 0x006A
#define ACC_SPEED_STATUS_ADDR 0x37D3
#define ACC_FP_MODE_ADDR 0x0073
// Power Frequency
#define ACC_MB_POWER_FREQUENCY_ADDR 0x008E
#define ACC_POWER_FREQUENCY_ADDR 0x03DB
#define ACC_PRESPEED_ADDR 0x03E2
#define ACC_PRESPEED_CNT 2
/* 扩展特征 */
#define ACC_EXT_AXIS_PER_SIZE 128
//状态位
#define ACC_EXT_INPUT_STATUS_ADDR 0xD004
#define ACC_EXT_OUTPUT_STATUS_ADDR 0xD005
#define ACC_EXT_BEARING_STATUS_ADDR 0xD006
#define ACC_EXT_GEAR_STATUS_ADDR 0xD007
//基本扩展配置
#define ACC_EXT_INPUT_FREQ_RANGE_ADDR 0xD000
#define ACC_EXT_OUTPUT_FREQ_RANGE_ADDR 0xD002
//基本扩展特征
#define ACC_EXT_BASE_FEATURE_ADDR 0xD080
//轴承扩展配置
#define ACC_EXT_BEARING_CFG_ADDR 0xD200
//轴承扩展特征
#define ACC_EXT_BEARING_FEATURE_ADDR 0xD800
//齿轮扩展配置
#define ACC_EXT_GEAR_CFG_ADDR 0xEA00
//齿轮扩展特征
#define ACC_EXT_GEAR_FEATURE_ADDR 0xED00
//扩展状态
#define ACC_EXT_BASE_STATUS_ADDR 0xD004
#define ACC_EXT_BASE_STATUS_COUNT 2
#define ACC_EXT_BEARING_STATUS_ADDR 0xD006
#define ACC_EXT_GEAR_STATUS_ADDR 0xD007
//别名
#define ACC_EXT_BEARING_ALIAS_ADDR 0xD202
#define ACC_EXT_GEAR_ALIAS_ADDR 0xEA03
//风机
#define ACC_FAN_STATUS_ADDR 0x37D4
#define ACC_FAN_CONFIGURE_ADDR 0x007C
#define ACC_FAN_FEATURE_ADDR 0x0497
#define ACC_FAN_ALIAS_ADDR 0x0080
//皮带
#define ACC_BELT_STATUS_ADDR 0x37D5
#define ACC_BELT_CONFIGURE_ADDR 0x007E
#define ACC_BELT_FEATURE_ADDR 0x04A9
#define ACC_BELT_ALIAS_ADDR 0x0081
// Screw
#define ACC_SCREW_STATUS_ADDR 0xF200
#define ACC_SCREW_CONFIGURE_ADDR 0xF213
#define ACC_SCREW_FEATURE_ADDR 0xF201
#define ACC_SCREW_ALIAS_ADDR 0xF230
//Male and Female Screw
#define ACC_MALE_SCREW_STATUS_ADDR 0xF400
#define ACC_FEMALE_SCREW_STATUS_ADDR 0xF500
#define ACC_MALE_SCREW_CONFIGURE_ADDR 0xF413
#define ACC_FEMALE_SCREW_CONFIGURE_ADDR 0xF513
#define ACC_MALE_SCREW_FEATURE_ADDR 0xF401
#define ACC_FEMALE_SCREW_FEATURE_ADDR 0xF501
#define ACC_MALE_SCREW_ALIAS_ADDR 0xF430
#define ACC_FEMALE_SCREW_ALIAS_ADDR 0xF530
// Shaft
#define ACC_SHAFT_STATUS_ADDR 0xF300
#define ACC_SHAFT_CONFIGURE_ADDR 0xF313
#define ACC_SHAFT_FEATURE_ADDR 0xF301
#define ACC_SHAFT_ALIAS_ADDR 0xF330
// Sleeve
#define ACC_SLEEVE_STATUS_ADDR 0xF600
#define ACC_SLEEVE_CONFIGURE_ADDR 0xF649
#define ACC_SLEEVE_FEATURE_ADDR 0xF601
#define ACC_SLEEVE_ALIAS_ADDR 0xF650
// sample sum
#define ACC_SAMPSUM_ADDR 0x006E
// HPF Enable
#define ACC_HPFEN_ADDR 0x0074
// calculate feature
#define ACC_CALFEA_ADDR 0x0078

// SN
#define ACC_SN_ADDR 0

#define ACC_MAX_RETRY_COUNT 6
#define ACC_MAX_WAIT_COUNT 6

#define ACC_WAVE_SPAN 4096
#define ACC_WAVE_READ_PER_MAX_COUNT ACC_MAX_BUFFER_COUNT
#define ACC_WAVE_PER_INDEX_DATA_COUNT 2048

// recalculete feature use old wave
#define ACC_FEATURE_RECALCULETE_ADDR 0x0082

// Speed sync fActual modbus holding register address
#define ACC_MOTORAR_RATE_ADDR 0x0083

// Speed sync fActual modbus holding register address
#define ACC_MTACTSPD_SEL_ADDR 0x0085

// anomal vibration threshold
#define ACC_AMLVIB_THR_ADDR 0x0086
// anomal vibration alarm
#define ACC_AMLVIB_ALARM_ADDR 0x0087
//anomal vibration interval
#define ACC_AMLVIB_IVL_ADDR 0x0088
// Set anomal vibration keep cnt
#define ACC_AMLVIB_KEEPCNT_ADDR 0x0089

// ODR
#define ACC_ODR_ADDR 0x008A
// Resolution
#define ACC_RESOLUTION_ADDR 0x008B
// sensor PSRAM high temperature protect feature
#define ACC_HIGHTEMPPROTECT_ADDR 0x008C


/* Private functions ---------------------------------------------------------*/
static bool Acc_GetPartOfWave(unsigned char ucPort, unsigned char ucAccID, Wave *pxWave, unsigned short usAddress, unsigned long ulWaveCount);
static bool Acc_GetPartOfFeature(unsigned char ucPort, unsigned char ucAccID, float *pfFea, unsigned short usAddress, unsigned long ulFeatureCount);

/**
 * @brief  Acc start measurement
 * @retval true / false
 */
bool Acc_StartCollecting(unsigned char ucPort, unsigned char ucAccID)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && (eMBWriteHoldingRegister(ucPort, ucAccID, ACC_STATUS_ADDR, 0x01) != MB_EMASTERNOERR))
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	else
		return false;
}

/**
 * @brief  Acc get wave
 * @retval true / false
 */
bool Acc_GetWaveIntoExtMemory(unsigned char ucPort, unsigned char ucAccID, Sensor_Wave *pxWave,
							  void (*pFuncMemoryWrite)(unsigned long, unsigned char *, unsigned long), unsigned long ulMemoryAddress)
{
	unsigned short usReadData = 0;
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;
	bool bStatus = true;

	// Read wave count
	while (ucRetry > 0 && (eMBReadHoldingRegister(ucPort, ucAccID, ACC_WAVE_COUNT_ADDR, 0x01, &usReadData) != MB_EMASTERNOERR))
	{
		ucRetry--;
	}

	if (ucRetry > 0)
	{
		int i;
		unsigned short usWaveAddress = ACC_WAVE_ADDR;
		unsigned short usWaveIndex = 0;

		// Store wave count
		pxWave->xWave.ulWaveCount[ucAccID] = usReadData;

		while (usReadData)
		{
			unsigned short usPerReadDataCnt = 0;
			unsigned short usPerIndexTotalCnt = 0;
			unsigned short usReadAddress = 0;
			unsigned short usCopyCount = 0;

			ucRetry = ACC_MAX_RETRY_COUNT;
			while (ucRetry > 0 && (eMBWriteHoldingRegister(ucPort, ucAccID, ACC_WAVE_INDEX_ADDR, usWaveIndex) != MB_EMASTERNOERR))
			{
				ucRetry--;
			}

			usWaveAddress = ACC_WAVE_ADDR;
			if (ucRetry > 0)
			{

				for (i = 0; i < ACC_AXIS_COUNT && bStatus; i++)
				{
					usPerIndexTotalCnt = usReadData > ACC_WAVE_PER_INDEX_DATA_COUNT ? ACC_WAVE_PER_INDEX_DATA_COUNT : usReadData;
					usCopyCount = usPerIndexTotalCnt;
					usReadAddress = usWaveAddress;
					int j = 0;

					while (usPerIndexTotalCnt && bStatus)
					{
						usPerReadDataCnt = usPerIndexTotalCnt > ACC_WAVE_READ_PER_MAX_COUNT ? ACC_WAVE_READ_PER_MAX_COUNT : usPerIndexTotalCnt;
						bStatus = Acc_GetPartOfWave(ucPort, ucAccID, (Wave *)pxWave, usReadAddress, usPerReadDataCnt);

						usReadAddress += usPerReadDataCnt;
						usPerIndexTotalCnt -= usPerReadDataCnt;

						// Store into external memory
						if (pFuncMemoryWrite != NULL)
						{
							pFuncMemoryWrite(ulMemoryAddress + i * ACC_MAX_WAVE_SUPPORT_COUNT * 2 + usWaveIndex * ACC_WAVE_PER_INDEX_DATA_COUNT * 2 + j * ACC_WAVE_READ_PER_MAX_COUNT * 2, (unsigned char *)(pxWave->xWave.sWaveArray), usPerReadDataCnt * 2);
						}
						j++;
					}

					usWaveAddress += ACC_WAVE_SPAN;
				}
			}

			usReadData -= usCopyCount;
			usWaveIndex++;

			if (bStatus == false)
				break;
		}
	}

	if (!ucRetry)
		bStatus = false;

	return bStatus;
}

/**
 * @brief  Acc get feature
 * @retval true / false
 */
bool Acc_GetFeature(unsigned char ucPort, unsigned char ucAccID, Sensor_Feature *pxFeature,
					unsigned short *pusBearingStatus, unsigned short *pusGearStatus, unsigned short *pusSpeedStatus, 
					unsigned short *pusFanStatus, unsigned short *pusBeltStatus, unsigned short *pusScrewStatus, unsigned short *pusShaftStatus, 
					unsigned short *pusMaleScrewStatus, unsigned short *pusFemaleScrewStatus, unsigned short *pusSleeveStatus,unsigned short * pusOdr, unsigned short * pusSampSum, unsigned short * pusHpfEn, unsigned short * pusResolution)
{
	unsigned short usReadData = 0;
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;
	bool bStatus = true;

	*pusSpeedStatus = false;
	// is set speed
	while (ucRetry > 0 && (eMBReadHoldingRegister(ucPort, ucAccID, ACC_SPEED_STATUS_ADDR, 0x01, &usReadData) != MB_EMASTERNOERR))
	{
		ucRetry--;
	}

	if (ucRetry > 0)
	{
		int i;
		unsigned short usAddress = ACC_POWER_FREQUENCY_ADDR;
		unsigned short usCount = sizeof(PowerFrequency_Parameter) / sizeof(Parameter);
		*pusSpeedStatus = usReadData;
		for (i = 0; i < ACC_AXIS_COUNT && bStatus; i++){
		  bStatus = Acc_GetPartOfFeature(ucPort, ucAccID, (float *)&(((Feature *)&pxFeature->x + i)->xPowerFrequency), usAddress, usCount * 2);
      usAddress += usCount * 2;
		}
		
	}

	// Get temp
	while (ucRetry > 0 && (eMBReadHoldingRegister(ucPort, ucAccID, ACC_TEMP_ADDR, 0x01, &usReadData) != MB_EMASTERNOERR))
	{
		ucRetry--;
	}

	if (ucRetry > 0)
	{
		int i, j;
		unsigned short usFeatureAddress = ACC_BASIC_FEATURE_ADDR;
		unsigned short usFeatureCount = ACC_BASIC_FEATURE_READ_COUNT;

		// Get feature
		pxFeature->temp.fValue = (short)usReadData / 100.0f;

		// Read basic feature
		for (j = 0; j < ACC_AXIS_COUNT && bStatus; j++)
		{
			bStatus = Acc_GetPartOfFeature(ucPort, ucAccID, (float *)((Feature *)&pxFeature->x + j), usFeatureAddress, usFeatureCount * 2);
			usFeatureAddress += ACC_BASIC_FEATURE_TOTAL_COUNT * 2;
		}

		// Read frequency feature
		usFeatureAddress = ACC_FREQ_1X_ADDR;
		for (j = 0; j < ACC_AXIS_COUNT && bStatus; j++)
		{
			bStatus = Acc_GetPartOfFeature(ucPort, ucAccID, (float *)&((Feature *)&pxFeature->x + j)->xFreq1x, usFeatureAddress, 2);
			bStatus = Acc_GetPartOfFeature(ucPort, ucAccID, (float *)&((Feature *)&pxFeature->x + j)->xFreq2x, usFeatureAddress + 4, 2);
			bStatus = Acc_GetPartOfFeature(ucPort, ucAccID, (float *)&((Feature *)&pxFeature->x + j)->xFreq3x, usFeatureAddress + 8, 2);
			usFeatureAddress += ACC_BASIC_FEATURE_TOTAL_COUNT * 2;
		}

		while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_BEARING_STATUS_ADDR, 0x01, &usReadData) != MB_EMASTERNOERR)
		{
			ucRetry--;
		}
		if (ucRetry)
		{
			*pusBearingStatus = usReadData;
			for (i = 0; i < ACC_MAX_BEARING_COUNT && bStatus; i++)
			{
				if (usReadData & (1 << i))
				{
					usFeatureCount = sizeof(Bearing_Parameter) / sizeof(Parameter);
					usFeatureAddress = ACC_BEARING_FEATURE_ADDR + i * usFeatureCount * 3 * 2;

					// Read bearing feature
					for (j = 0; j < ACC_AXIS_COUNT && bStatus; j++)
					{
						bStatus = Acc_GetPartOfFeature(ucPort, ucAccID, (float *)&(((Feature *)&pxFeature->x + j)->xBearingFea[i]), usFeatureAddress, usFeatureCount * 2);
						usFeatureAddress += usFeatureCount * 2;
					}
				}
			}
		}

		// Read gear status
		while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_GEAR_STATUS_ADDR, 0x01, &usReadData) != MB_EMASTERNOERR)
		{
			ucRetry--;
		}
		if (ucRetry)
		{
			*pusGearStatus = usReadData;
			for (i = 0; i < ACC_MAX_GEAR_COUNT && bStatus; i++)
			{
				if (usReadData & (1 << i))
				{
					usFeatureCount = sizeof(Gear_Parameter) / sizeof(Parameter);
					usFeatureAddress = ACC_GEAR_FEATURE_ADDR + i * usFeatureCount * 3 * 2;

					// Read gear feature
					for (j = 0; j < ACC_AXIS_COUNT && bStatus; j++)
					{
						bStatus = Acc_GetPartOfFeature(ucPort, ucAccID, (float *)&(((Feature *)&pxFeature->x + j)->xGearFea[i]), usFeatureAddress, usFeatureCount * 2);
						usFeatureAddress += usFeatureCount * 2;
					}
				}
			}
		}
		// Fan
		while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_FAN_STATUS_ADDR, 0x01, &usReadData) != MB_EMASTERNOERR)
		{
			ucRetry--;
		}
		if (ucRetry)
		{
			*pusFanStatus = usReadData;
			if (usReadData == true)
			{
				usFeatureCount = sizeof(Fan_Parameter) / sizeof(Parameter);
				usFeatureAddress = ACC_FAN_FEATURE_ADDR;

				for (j = 0; j < ACC_AXIS_COUNT && bStatus; j++)
				{
					bStatus = Acc_GetPartOfFeature(ucPort, ucAccID, (float *)&(((Feature *)&pxFeature->x + j)->xFan), usFeatureAddress, usFeatureCount * 2);
					usFeatureAddress += usFeatureCount * 2;
				}
			}
		}

		// Belt
		while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_BELT_STATUS_ADDR, 0x01, &usReadData) != MB_EMASTERNOERR)
		{
			ucRetry--;
		}
		if (ucRetry)
		{
			*pusBeltStatus = usReadData;
			if (usReadData == true)
			{
				usFeatureCount = sizeof(Belt_Parameter) / sizeof(Parameter);
				usFeatureAddress = ACC_BELT_FEATURE_ADDR;

				for (j = 0; j < ACC_AXIS_COUNT && bStatus; j++)
				{
					bStatus = Acc_GetPartOfFeature(ucPort, ucAccID, (float *)&(((Feature *)&pxFeature->x + j)->xBelt), usFeatureAddress, usFeatureCount * 2);
					usFeatureAddress += usFeatureCount * 2;
				}
			}
		}
		// Screw
		while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_SCREW_STATUS_ADDR, 0x01, &usReadData) != MB_EMASTERNOERR)
		{
			ucRetry--;
		}
		if (ucRetry)
		{
			*pusScrewStatus = usReadData;
			if (usReadData == true)
			{
				usFeatureCount = sizeof(Screw_Parameter) / sizeof(Parameter);
				usFeatureAddress = ACC_SCREW_FEATURE_ADDR;

				for (j = 0; j < ACC_AXIS_COUNT && bStatus; j++)
				{
					bStatus = Acc_GetPartOfFeature(ucPort, ucAccID, (float *)&(((Feature *)&pxFeature->x + j)->xScrew), usFeatureAddress, usFeatureCount * 2);
					usFeatureAddress += usFeatureCount * 2;
				}
			}
		}
		//Male Screw
		while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_MALE_SCREW_STATUS_ADDR, 0x01, &usReadData) != MB_EMASTERNOERR)
		{
			ucRetry--;
		}
		if (ucRetry)
		{
			*pusMaleScrewStatus = usReadData;
			if (usReadData == true)
			{
				usFeatureCount = sizeof(MaleScrew_Parameter) / sizeof(Parameter);
				usFeatureAddress = ACC_MALE_SCREW_FEATURE_ADDR;

				for (j = 0; j < ACC_AXIS_COUNT && bStatus; j++)
				{
					bStatus = Acc_GetPartOfFeature(ucPort, ucAccID, (float *)&(((Feature *)&pxFeature->x + j)->xMaleScrew), usFeatureAddress, usFeatureCount * 2);
					usFeatureAddress += usFeatureCount * 2;
				}
			}
		}
		//Female Screw
		while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_FEMALE_SCREW_STATUS_ADDR, 0x01, &usReadData) != MB_EMASTERNOERR)
		{
			ucRetry--;
		}
		if (ucRetry)
		{
			*pusFemaleScrewStatus = usReadData;
			if (usReadData == true)
			{
				usFeatureCount = sizeof(FemaleScrew_Parameter) / sizeof(Parameter);
				usFeatureAddress = ACC_FEMALE_SCREW_FEATURE_ADDR;

				for (j = 0; j < ACC_AXIS_COUNT && bStatus; j++)
				{
					bStatus = Acc_GetPartOfFeature(ucPort, ucAccID, (float *)&(((Feature *)&pxFeature->x + j)->xFemaleScrew), usFeatureAddress, usFeatureCount * 2);
					usFeatureAddress += usFeatureCount * 2;
				}
			}
		}
		// Shaft
		while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_SHAFT_STATUS_ADDR, 0x01, &usReadData) != MB_EMASTERNOERR)
		{
			ucRetry--;
		}
		if (ucRetry)
		{
			*pusShaftStatus = usReadData;
			if (usReadData == true)
			{
				usFeatureCount = sizeof(Shaft_Parameter) / sizeof(Parameter);
				usFeatureAddress = ACC_SHAFT_FEATURE_ADDR;

				for (j = 0; j < ACC_AXIS_COUNT && bStatus; j++)
				{
					bStatus = Acc_GetPartOfFeature(ucPort, ucAccID, (float *)&(((Feature *)&pxFeature->x + j)->xShaft), usFeatureAddress, usFeatureCount * 2);
					usFeatureAddress += usFeatureCount * 2;
				}
			}
		}
		// Sleeve
		while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_SLEEVE_STATUS_ADDR, 0x01, &usReadData) != MB_EMASTERNOERR)
		{
			ucRetry--;
		}
		if (ucRetry)
		{
			*pusSleeveStatus = usReadData;
			if (usReadData == true)
			{
				usFeatureCount = sizeof(Sleeve_Parameter) / sizeof(Parameter);
				usFeatureAddress = ACC_SLEEVE_FEATURE_ADDR;

				for (j = 0; j < ACC_AXIS_COUNT && bStatus; j++)
				{
					bStatus = Acc_GetPartOfFeature(ucPort, ucAccID, (float *)&(((Feature *)&pxFeature->x + j)->xSleeve), usFeatureAddress, usFeatureCount * 2);
					usFeatureAddress += usFeatureCount * 2;
				}
			}
		}
		// ODR
		while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_ODR_ADDR, 0x01, &usReadData) != MB_EMASTERNOERR)
		{
			ucRetry--;
		}
		if (ucRetry)
		{
			*pusOdr = usReadData;
		}
		// Sample sum
		while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_SAMPSUM_ADDR, 0x01, &usReadData) != MB_EMASTERNOERR)
		{
			ucRetry--;
		}
		if (ucRetry)
		{
			*pusSampSum = usReadData;
		}
		// HPF enable
		while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_HPFEN_ADDR, 0x01, &usReadData) != MB_EMASTERNOERR)
		{
			ucRetry--;
		}
		if (ucRetry)
		{
			*pusHpfEn = usReadData;
		}
		// Resolution
		while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_RESOLUTION_ADDR, 0x01, &usReadData) != MB_EMASTERNOERR)
		{
			ucRetry--;
		}
		if (ucRetry)
		{
			*pusResolution = usReadData;
		}
	}

	if (!ucRetry)
		bStatus = false;

	return bStatus;
}

/**
 * @brief  Acc is finish ?
 * @retval true / false
 */
bool Acc_GetSensorStatus(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusCode)
{
	unsigned short usWaitCount = ACC_MAX_WAIT_COUNT;
	unsigned short usStatus = 0;

	// Get status
	while (usWaitCount > 0)
	{
		if (eMBReadHoldingRegister(ucPort, ucAccID, ACC_STATUS_ADDR, 0x01, &usStatus) == MB_EMASTERNOERR)
		{
			if (pusCode != NULL)
				*pusCode = usStatus;

			return true;
		}

		usWaitCount--;
		Delay_ms(10);
	}

	if (pusCode != NULL)
		*pusCode = usStatus;
	return false;
}

/**
 * @brief  Acc set bearing info
 * @retval true / false
 */
bool Acc_SetBearingInfo(unsigned char ucPort, unsigned char ucAccID, unsigned char ucNumber, Bearing *pxComponent)
{
	unsigned short usComponentAddress;
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;
	unsigned short usDataInfo[12];
	unsigned short usDataSize = 0;

	if (ucNumber >= ACC_MAX_BEARING_COUNT)
		return false;

	usComponentAddress = ACC_BEARING_INFO_ADDR + ucNumber * sizeof(Bearing) / sizeof(float) * 2;
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxComponent->fBallCount * 100) >> 16);
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxComponent->fBallCount * 100));
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxComponent->fRotateSpeed * 100) >> 16);
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxComponent->fRotateSpeed * 100));
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxComponent->fBallDim * 100) >> 16);
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxComponent->fBallDim * 100));
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxComponent->fPCD * 100) >> 16);
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxComponent->fPCD * 100));
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxComponent->fWorkAngle * 100) >> 16);
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxComponent->fWorkAngle * 100));

	while (ucRetry > 0 && eMBWriteMultiHoldingRegister(ucPort, ucAccID, usComponentAddress, usDataSize, usDataInfo) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_SetBearingFrequency(unsigned char ucPort, unsigned char ucAccID, unsigned char ucNumber, Bearing *pxComponent)
{
	unsigned short usComponentAddress;
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;
	unsigned short usDataInfo[12];
	unsigned short usDataSize = 0;

	if (ucNumber >= ACC_MAX_BEARING_COUNT)
		return false;

	usComponentAddress = ACC_BEARING_INFO_ADDR + ucNumber * sizeof(Bearing) / sizeof(float) * 2 + 10;
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxComponent->fBPFO * 100) >> 16);
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxComponent->fBPFO * 100));
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxComponent->fBPFI * 100) >> 16);
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxComponent->fBPFI * 100));
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxComponent->fBSF * 100) >> 16);
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxComponent->fBSF * 100));
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxComponent->fFTF * 100) >> 16);
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxComponent->fFTF * 100));

	while (ucRetry > 0 && eMBWriteMultiHoldingRegister(ucPort, ucAccID, usComponentAddress, usDataSize, usDataInfo) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

/**
 * @brief  Acc set gear info
 * @retval true / false
 */
bool Acc_SetGearInfo(unsigned char ucPort, unsigned char ucAccID, unsigned char ucNumber, Gear *pxComponent)
{
	unsigned short usComponentAddress;
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;
	unsigned short usDataInfo[12];
	unsigned short usDataSize = 0;

	if (ucNumber >= ACC_MAX_GEAR_COUNT)
		return false;

	usComponentAddress = ACC_GEAR_INFO_ADDR + ucNumber * sizeof(Gear) / sizeof(float) * 2;
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxComponent->fTeethCount * 100) >> 16);
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxComponent->fTeethCount * 100));
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxComponent->fRotateSpeed * 100) >> 16);
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxComponent->fRotateSpeed * 100));

	while (ucRetry > 0 && eMBWriteMultiHoldingRegister(ucPort, ucAccID, usComponentAddress, usDataSize, usDataInfo) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_SetGearFrequency(unsigned char ucPort, unsigned char ucAccID, unsigned char ucNumber, Gear *pxComponent)
{
	unsigned short usComponentAddress;
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;
	unsigned short usDataInfo[12];
	unsigned short usDataSize = 0;

	if (ucNumber >= ACC_MAX_GEAR_COUNT)
		return false;

	usComponentAddress = ACC_GEAR_INFO_ADDR + ucNumber * sizeof(Gear) / sizeof(float) * 2 + 4;
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxComponent->f1xGMF * 100) >> 16);
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxComponent->f1xGMF * 100));
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxComponent->f2xGMF * 100) >> 16);
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxComponent->f2xGMF * 100));
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxComponent->f3xGMF * 100) >> 16);
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxComponent->f3xGMF * 100));

	while (ucRetry > 0 && eMBWriteMultiHoldingRegister(ucPort, ucAccID, usComponentAddress, usDataSize, usDataInfo) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

/**
 * @brief  Acc set speed
 * @retval true / false
 */
bool Acc_SetSpeed(unsigned char ucPort, unsigned char ucAccID, unsigned long ulSpeed)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;
	unsigned short usDataInfo[2];

	usDataInfo[0] = (unsigned short)(ulSpeed >> 16);
	usDataInfo[1] = (unsigned short)(ulSpeed);

	while (ucRetry > 0 && eMBWriteMultiHoldingRegister(ucPort, ucAccID, ACC_SPEED_ADDR, 2, usDataInfo) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

/**
 * @brief  Acc set power frequency
 * @retval true / false
 */
bool Acc_SetPowerFrequency(unsigned char ucPort, unsigned char ucAccID, unsigned short usPowerFrequency)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;
	unsigned short usDataInfo;

	usDataInfo = usPowerFrequency;

	while (ucRetry > 0 && eMBWriteMultiHoldingRegister(ucPort, ucAccID, ACC_MB_POWER_FREQUENCY_ADDR, 1, &usDataInfo) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

/**
 * @brief  Acc clear bearing status
 * @retval true / false
 */
bool Acc_ClearBearingStatus(unsigned char ucPort, unsigned char ucAccID)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_BEARING_STATUS_ADDR, 0) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

/**
 * @brief  Acc clear gear status
 * @retval true / false
 */
bool Acc_ClearGearStatus(unsigned char ucPort, unsigned char ucAccID)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_GEAR_STATUS_ADDR, 0) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

/**
 * @brief  Acc clear speed status
 * @retval true / false
 */
bool Acc_ClearSpeedStatus(unsigned char ucPort, unsigned char ucAccID)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_SPEED_STATUS_ADDR, 0) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_SetPinFunctionToMB(unsigned char ucPort, unsigned char ucAccID)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_FP_MODE_ADDR, 0x0001) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_GetVersion(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusVer)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_VERSION_ADDR, 1, pusVer) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_GetActiveFlag(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusActiveFlag)
{
	unsigned char ucRetry = 1;

	while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_ACTIVE_FLAG_ADDR, 1, pusActiveFlag) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_GetSamplingRate(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusSamplingRate)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_SAMPLING_RATE_ADDR, 1, pusSamplingRate) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_GetMode(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusMode)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_MODE_ADDR, 1, pusMode) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_SetMode(unsigned char ucPort, unsigned char ucAccID, unsigned short usMode)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_MODE_ADDR, usMode) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_SetShockThreshold(unsigned char ucPort, unsigned char ucAccID, unsigned short usThr)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_SHOCK_THR_ADDR, usThr) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

/* Get Extend Feature */
bool Acc_GetExtendFeature(unsigned char ucPort, unsigned char ucAccID, Sensor_Feature *pxFeature, unsigned short *pusBearingExtStatus, unsigned short *pusGearExtStatus,
						  unsigned short *pusInputExtStatus, unsigned short *pusOutputExtStatus)
{
	unsigned short usReadData = 0;
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;
	bool bStatus = true;

	*pusInputExtStatus = false;
	// input range freq status
	while (ucRetry > 0 && (eMBReadHoldingRegister(ucPort, ucAccID, ACC_EXT_INPUT_STATUS_ADDR, 0x01, &usReadData) != MB_EMASTERNOERR))
	{
		ucRetry--;
	}

	if (ucRetry > 0)
	{
		*pusInputExtStatus = usReadData;
	}

	// output range freq status
	while (ucRetry > 0 && (eMBReadHoldingRegister(ucPort, ucAccID, ACC_EXT_OUTPUT_STATUS_ADDR, 0x01, &usReadData) != MB_EMASTERNOERR))
	{
		ucRetry--;
	}

	if (ucRetry > 0)
	{
		*pusOutputExtStatus = usReadData;
	}

	if (ucRetry > 0)
	{
		int i, j;
		unsigned short usFeatureAddress;
		unsigned short usFeatureCount;

		// Read extend base feature
		usFeatureAddress = ACC_EXT_BASE_FEATURE_ADDR;
		usFeatureCount = sizeof(Base_ExtendFeature) / sizeof(Parameter);
		for (j = 0; j < ACC_AXIS_COUNT && bStatus; j++)
		{
			bStatus = Acc_GetPartOfFeature(ucPort, ucAccID, (float *)((ExtendFeature *)&pxFeature->xExt + j), usFeatureAddress, usFeatureCount * 2);
			usFeatureAddress += ACC_EXT_AXIS_PER_SIZE;
		}

		// Read extend bearing status
		ucRetry = ACC_MAX_RETRY_COUNT;
		while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_EXT_BEARING_STATUS_ADDR, 0x01, &usReadData) != MB_EMASTERNOERR)
		{
			ucRetry--;
		}
		if (ucRetry)
		{
			*pusBearingExtStatus = usReadData;
			for (i = 0; i < ACC_MAX_BEARING_COUNT && bStatus; i++)
			{
				if (usReadData & (1 << i))
				{
					usFeatureCount = sizeof(Bearing_ExtendParameter) / sizeof(Parameter);
					usFeatureAddress = ACC_EXT_BEARING_FEATURE_ADDR + i * ACC_EXT_AXIS_PER_SIZE * 3;

					// Read bearing feature
					for (j = 0; j < ACC_AXIS_COUNT && bStatus; j++)
					{
						bStatus = Acc_GetPartOfFeature(ucPort, ucAccID, (float *)&(((ExtendFeature *)&pxFeature->xExt + j)->xBearingExtFea[i]), usFeatureAddress, usFeatureCount * 2);
						usFeatureAddress += ACC_EXT_AXIS_PER_SIZE;
					}
				}
			}
		}

		// Read gear status
		while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_EXT_GEAR_STATUS_ADDR, 0x01, &usReadData) != MB_EMASTERNOERR)
		{
			ucRetry--;
		}
		if (ucRetry)
		{
			*pusGearExtStatus = usReadData;
			for (i = 0; i < ACC_MAX_GEAR_COUNT && bStatus; i++)
			{
				if (usReadData & (1 << i))
				{
					usFeatureCount = sizeof(Gear_ExtendParameter) / sizeof(Parameter);
					usFeatureAddress = ACC_EXT_GEAR_FEATURE_ADDR + i * ACC_EXT_AXIS_PER_SIZE * 3;

					// Read gear feature
					for (j = 0; j < ACC_AXIS_COUNT && bStatus; j++)
					{
						bStatus = Acc_GetPartOfFeature(ucPort, ucAccID, (float *)&(((ExtendFeature *)&pxFeature->xExt + j)->xGearExtFea[i]), usFeatureAddress, usFeatureCount * 2);
						usFeatureAddress += ACC_EXT_AXIS_PER_SIZE;
					}
				}
			}
		}
	}

	if (!ucRetry)
		bStatus = false;

	return bStatus;
}

bool Acc_SetBaseExtendConfig(unsigned char ucPort, unsigned char ucAccID, Base_ExtendConfig *pxExtCfg)
{
	unsigned short usExtCfgAddress;
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;
	unsigned short usDataInfo[12];
	unsigned short usDataSize = 0;

	if (pxExtCfg->usInputStartFreq != 0 || pxExtCfg->usInputStopFreq != 0)
	{
		usExtCfgAddress = ACC_EXT_INPUT_FREQ_RANGE_ADDR;
		usDataInfo[usDataSize++] = pxExtCfg->usInputStartFreq;
		usDataInfo[usDataSize++] = pxExtCfg->usInputStopFreq;

		while (ucRetry > 0 && eMBWriteMultiHoldingRegister(ucPort, ucAccID, usExtCfgAddress, usDataSize, usDataInfo) != MB_EMASTERNOERR)
		{
			ucRetry--;
		}
	}

	if (pxExtCfg->usOutputStartFreq != 0 || pxExtCfg->usOutputStopFreq != 0)
	{
		usDataSize = 0;
		ucRetry = ACC_MAX_RETRY_COUNT;
		usExtCfgAddress = ACC_EXT_OUTPUT_FREQ_RANGE_ADDR;
		usDataInfo[usDataSize++] = pxExtCfg->usOutputStartFreq;
		usDataInfo[usDataSize++] = pxExtCfg->usOutputStopFreq;

		while (ucRetry > 0 && eMBWriteMultiHoldingRegister(ucPort, ucAccID, usExtCfgAddress, usDataSize, usDataInfo) != MB_EMASTERNOERR)
		{
			ucRetry--;
		}
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_SetBearingExtendConfig(unsigned char ucPort, unsigned char ucAccID, unsigned char ucNumber, Bearing_ExtendConfig *pxExtCfg)
{
	unsigned short usExtCfgAddress;
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;
	unsigned short usDataInfo[12];
	unsigned short usDataSize = 0;

	if (ucNumber >= ACC_MAX_BEARING_COUNT)
		return false;

	usExtCfgAddress = ACC_EXT_BEARING_CFG_ADDR + ucNumber * ACC_EXT_AXIS_PER_SIZE;
	usDataInfo[usDataSize++] = pxExtCfg->usStartFreq;
	usDataInfo[usDataSize++] = pxExtCfg->usStopFreq;

	while (ucRetry > 0 && eMBWriteMultiHoldingRegister(ucPort, ucAccID, usExtCfgAddress, usDataSize, usDataInfo) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_SetGearExtendConfig(unsigned char ucPort, unsigned char ucAccID, unsigned char ucNumber, Gear_ExtendConfig *pxExtCfg)
{
	unsigned short usExtCfgAddress;
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;
	unsigned short usDataInfo[12];
	unsigned short usDataSize = 0;

	if (ucNumber >= ACC_MAX_GEAR_COUNT)
		return false;

	usExtCfgAddress = ACC_EXT_GEAR_CFG_ADDR + ucNumber * ACC_EXT_AXIS_PER_SIZE;
	usDataInfo[usDataSize++] = pxExtCfg->usStartFreq;
	usDataInfo[usDataSize++] = pxExtCfg->usStopFreq;
	usDataInfo[usDataSize++] = pxExtCfg->usSideRange;

	while (ucRetry > 0 && eMBWriteMultiHoldingRegister(ucPort, ucAccID, usExtCfgAddress, usDataSize, usDataInfo) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_ClearBaseExtendStauts(unsigned char ucPort, unsigned char ucAccID)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;
	unsigned short usData[ACC_EXT_BASE_STATUS_COUNT] = {0x0000, 0x0000};

	while (ucRetry > 0 && eMBWriteMultiHoldingRegister(ucPort, ucAccID, ACC_EXT_BASE_STATUS_ADDR, ACC_EXT_BASE_STATUS_COUNT, usData) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_ClearBearingExtendStauts(unsigned char ucPort, unsigned char ucAccID)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_EXT_BEARING_STATUS_ADDR, 0) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_ClearGearExtendStauts(unsigned char ucPort, unsigned char ucAccID)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_EXT_GEAR_STATUS_ADDR, 0) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_GetBearingAlias(unsigned char ucPort, unsigned char ucAccID, unsigned char ucNumber, unsigned short *pusAlias)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;
	unsigned short usAddress;

	if (ucNumber >= ACC_MAX_BEARING_COUNT)
		return false;

	usAddress = ACC_EXT_BEARING_ALIAS_ADDR + ucNumber * ACC_EXT_AXIS_PER_SIZE;
	while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, usAddress, 1, pusAlias) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_GetGearAlias(unsigned char ucPort, unsigned char ucAccID, unsigned char ucNumber, unsigned short *pusAlias)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;
	unsigned short usAddress;

	if (ucNumber >= ACC_MAX_GEAR_COUNT)
		return false;

	usAddress = ACC_EXT_GEAR_ALIAS_ADDR + ucNumber * ACC_EXT_AXIS_PER_SIZE;
	while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, usAddress, 1, pusAlias) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_SetBearingAlias(unsigned char ucPort, unsigned char ucAccID, unsigned char ucNumber, unsigned short usAlias)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;
	unsigned short usAddress;

	if (ucNumber >= ACC_MAX_BEARING_COUNT)
		return false;

	usAddress = ACC_EXT_BEARING_ALIAS_ADDR + ucNumber * ACC_EXT_AXIS_PER_SIZE;
	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, usAddress, usAlias) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_SetGearAlias(unsigned char ucPort, unsigned char ucAccID, unsigned char ucNumber, unsigned short usAlias)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;
	unsigned short usAddress;

	if (ucNumber >= ACC_MAX_GEAR_COUNT)
		return false;

	usAddress = ACC_EXT_GEAR_ALIAS_ADDR + ucNumber * ACC_EXT_AXIS_PER_SIZE;
	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, usAddress, usAlias) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_SetFanConfigure(unsigned char ucPort, unsigned char ucAccID, Fan *pxFan)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;
	unsigned short usDataInfo[12];
	unsigned short usDataSize = 0;

	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxFan->fPassFrequency * 100) >> 16);
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxFan->fPassFrequency * 100));

	while (ucRetry > 0 && eMBWriteMultiHoldingRegister(ucPort, ucAccID, ACC_FAN_CONFIGURE_ADDR, usDataSize, usDataInfo) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_SetBeltConfigure(unsigned char ucPort, unsigned char ucAccID, Belt *pxBelt)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;
	unsigned short usDataInfo[12];
	unsigned short usDataSize = 0;

	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxBelt->fPassFrequency * 100) >> 16);
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxBelt->fPassFrequency * 100));

	while (ucRetry > 0 && eMBWriteMultiHoldingRegister(ucPort, ucAccID, ACC_BELT_CONFIGURE_ADDR, usDataSize, usDataInfo) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_ClearFanStatus(unsigned char ucPort, unsigned char ucAccID)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_FAN_STATUS_ADDR, 0) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_ClearBeltStatus(unsigned char ucPort, unsigned char ucAccID)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_BELT_STATUS_ADDR, 0) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_SetFanAlias(unsigned char ucPort, unsigned char ucAccID, unsigned short usAlias)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_FAN_ALIAS_ADDR, usAlias) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_SetBeltAlias(unsigned char ucPort, unsigned char ucAccID, unsigned short usAlias)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_BELT_ALIAS_ADDR, usAlias) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_GetFanAlias(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusAlias)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_FAN_ALIAS_ADDR, 1, pusAlias) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_GetBeltAlias(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusAlias)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_BELT_ALIAS_ADDR, 1, pusAlias) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}
/*---------------------- Screw ---------------------*/
bool Acc_SetScrewConfigure(unsigned char ucPort, unsigned char ucAccID, Screw *pxScrew)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;
	unsigned short usDataInfo[12];
	unsigned short usDataSize = 0;

	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxScrew->fPassFrequency * 100) >> 16);
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxScrew->fPassFrequency * 100));

	while (ucRetry > 0 && eMBWriteMultiHoldingRegister(ucPort, ucAccID, ACC_SCREW_CONFIGURE_ADDR, usDataSize, usDataInfo) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_ClearScrewStatus(unsigned char ucPort, unsigned char ucAccID)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_SCREW_STATUS_ADDR, 0) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_SetScrewAlias(unsigned char ucPort, unsigned char ucAccID, unsigned short usAlias)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_SCREW_ALIAS_ADDR, usAlias) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_GetScrewAlias(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusAlias)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_SCREW_ALIAS_ADDR, 1, pusAlias) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}
/*-------------- Male and Female Screw -------------*/
bool Acc_SetMaleScrewConfigure(unsigned char ucPort, unsigned char ucAccID, Screw *pxScrew)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;
	unsigned short usDataInfo[12];
	unsigned short usDataSize = 0;

	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxScrew->fPassFrequency * 100) >> 16);
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxScrew->fPassFrequency * 100));

	while (ucRetry > 0 && eMBWriteMultiHoldingRegister(ucPort, ucAccID, ACC_MALE_SCREW_CONFIGURE_ADDR, usDataSize, usDataInfo) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_ClearMaleScrewStatus(unsigned char ucPort, unsigned char ucAccID)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_MALE_SCREW_STATUS_ADDR, 0) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_SetMaleScrewAlias(unsigned char ucPort, unsigned char ucAccID, unsigned short usAlias)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_MALE_SCREW_ALIAS_ADDR, usAlias) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_GetMaleScrewAlias(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusAlias)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_MALE_SCREW_ALIAS_ADDR, 1, pusAlias) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_SetFemaleScrewConfigure(unsigned char ucPort, unsigned char ucAccID, Screw *pxScrew)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;
	unsigned short usDataInfo[12];
	unsigned short usDataSize = 0;

	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxScrew->fPassFrequency * 100) >> 16);
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxScrew->fPassFrequency * 100));

	while (ucRetry > 0 && eMBWriteMultiHoldingRegister(ucPort, ucAccID,  ACC_FEMALE_SCREW_CONFIGURE_ADDR, usDataSize, usDataInfo) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_ClearFemaleScrewStatus(unsigned char ucPort, unsigned char ucAccID)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_FEMALE_SCREW_STATUS_ADDR, 0) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_SetFemaleScrewAlias(unsigned char ucPort, unsigned char ucAccID, unsigned short usAlias)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_FEMALE_SCREW_ALIAS_ADDR, usAlias) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_GetFemaleScrewAlias(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusAlias)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_FEMALE_SCREW_ALIAS_ADDR, 1, pusAlias) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}
/*---------------------- Shaft ---------------------*/
bool Acc_SetShaftConfigure(unsigned char ucPort, unsigned char ucAccID, Shaft *pxShaft)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;
	unsigned short usDataInfo[12];
	unsigned short usDataSize = 0;

	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxShaft->fPassFrequency * 100) >> 16);
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxShaft->fPassFrequency * 100));

	while (ucRetry > 0 && eMBWriteMultiHoldingRegister(ucPort, ucAccID, ACC_SHAFT_CONFIGURE_ADDR, usDataSize, usDataInfo) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_ClearShaftStatus(unsigned char ucPort, unsigned char ucAccID)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_SHAFT_STATUS_ADDR, 0) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_SetShaftAlias(unsigned char ucPort, unsigned char ucAccID, unsigned short usAlias)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_SHAFT_ALIAS_ADDR, usAlias) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_GetShaftAlias(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusAlias)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_SHAFT_ALIAS_ADDR, 1, pusAlias) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}
/*---------------------- Sleeve ---------------------*/
bool Acc_SetSleeveConfigure(unsigned char ucPort, unsigned char ucAccID, Sleeve *pxSleeve)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;
	unsigned short usDataInfo[12];
	unsigned short usDataSize = 0;

	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxSleeve->fRotOfAxesFrequency * 100) >> 16);
	usDataInfo[usDataSize++] = (unsigned short)((unsigned long)(pxSleeve->fRotOfAxesFrequency * 100));

	while (ucRetry > 0 && eMBWriteMultiHoldingRegister(ucPort, ucAccID, ACC_SLEEVE_CONFIGURE_ADDR, usDataSize, usDataInfo) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_ClearSleeveStatus(unsigned char ucPort, unsigned char ucAccID)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_SLEEVE_STATUS_ADDR, 0) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_SetSleeveAlias(unsigned char ucPort, unsigned char ucAccID, unsigned short usAlias)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_SLEEVE_ALIAS_ADDR, usAlias) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_GetSleeveAlias(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusAlias)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_SLEEVE_ALIAS_ADDR, 1, pusAlias) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

// Get sensor ODR
bool Acc_GetOdr(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusOdr)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_ODR_ADDR, 1, pusOdr) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}
// Get sensor sample sum
bool Acc_GetSampSum(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusSampSum)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_SAMPSUM_ADDR, 1, pusSampSum) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}
// Get sensor HPF enable
bool Acc_GetHpfEn(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusHpfEn)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_HPFEN_ADDR, 1, pusHpfEn) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}
// Get sensor resolution
bool Acc_GetResolution(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusResolution)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_RESOLUTION_ADDR, 1, pusResolution) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}
// Set sensor ODR
bool Acc_SetOdr(unsigned char ucPort, unsigned char ucAccID, unsigned short usOdr)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_ODR_ADDR, usOdr) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}
// Set sensor sample sum
bool Acc_SetSampSum(unsigned char ucPort, unsigned char ucAccID, unsigned short usSampSum)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_SAMPSUM_ADDR, usSampSum) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}
// Set sensor HPF enable
bool Acc_SetHpfEn(unsigned char ucPort, unsigned char ucAccID, unsigned short usHpfEn)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_HPFEN_ADDR, usHpfEn) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

// Get anomal vibration alarm
bool Acc_GetAmlVibAlarm(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusAmlVibAlarm)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_AMLVIB_ALARM_ADDR, 1, pusAmlVibAlarm) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

// Set anomal vibration threshold
bool Acc_SetAmlVibTh(unsigned char ucPort, unsigned char ucAccID, unsigned short usAmlVibTh)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_AMLVIB_THR_ADDR, usAmlVibTh) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

// Set anomal vibration interval
bool Acc_SetAmlVibInterval(unsigned char ucPort, unsigned char ucAccID, unsigned short usAmlVibInterval)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_AMLVIB_IVL_ADDR, usAmlVibInterval) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

// Set anomal vibration keep cnt
bool Acc_SetAmlVibKeepCnt(unsigned char ucPort, unsigned char ucAccID, unsigned short usAmlVibKeepCnt)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_AMLVIB_KEEPCNT_ADDR, usAmlVibKeepCnt) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

// set calculate feature
bool Acc_SetCalFea(unsigned char ucPort, unsigned char ucAccID, unsigned short uscalFea)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_CALFEA_ADDR, uscalFea) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

// sensor PSRAM high temperature protect feature
bool Acc_GetHighTempProtect(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusHighTempProtect)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_HIGHTEMPPROTECT_ADDR, 1, pusHighTempProtect) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

// sensor PSRAM high temperature protect feature
bool Acc_SetHighTempProtect(unsigned char ucPort, unsigned char ucAccID, unsigned short usHighTempProtect)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_HIGHTEMPPROTECT_ADDR, usHighTempProtect) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_GetSerialNumber(unsigned char ucPort, unsigned char ucAccID, unsigned long *pulSN)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;
	unsigned short usAddress;
	unsigned short usData[2];

	usAddress = ACC_SN_ADDR;
	while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, usAddress, 2, usData) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
	{
		*pulSN = ((unsigned long)usData[0] << 16) | usData[1];
		return true;
	}
	return false;
}
bool Acc_GetPredictedSpeed(unsigned char ucPort, unsigned char ucAccID, Sensor_Feature *pxFeature)
{
	unsigned char ucRetry = ACC_PRESPEED_CNT;
	unsigned short usAddress;
	unsigned short usData[2];
	unsigned long ulData;

	usAddress = ACC_PRESPEED_ADDR;
	while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, usAddress, 2, usData) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
	{
		ulData = (long)((unsigned long)usData[0] << 16 | usData[1]);
		pxFeature->xActualSpd.fValue = ulData / 100.0;
		return true;
	}
	return false;
}
/**
 * @brief  Acc get one of axis original wave
 * @retval true / false
 */
static bool Acc_GetPartOfWave(unsigned char ucPort, unsigned char ucAccID, Wave *pxWave, unsigned short usAddress, unsigned long ulWaveCount)
{
	short *psArr = pxWave->sWaveArray;

	if (pxWave == NULL)
		return true;

	while (ulWaveCount)
	{
		unsigned long ulReadSize = 0;
		unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

		ulReadSize = ulWaveCount > 100 ? 100 : ulWaveCount;
		while (ucRetry > 0 && (eMBReadHoldingRegister(ucPort, ucAccID, usAddress, ulReadSize, (unsigned short *)psArr) != MB_EMASTERNOERR))
		{
			ucRetry--;
			Delay_ms(100);
		}

		if (ucRetry > 0)
		{
			usAddress += ulReadSize;
			ulWaveCount -= ulReadSize;
			psArr += ulReadSize;
		}
		else
		{
			break;
		}
	}

	if (ulWaveCount > 0)
		return false;
	else
		return true;
}

/**
 * @brief  Acc get one of axis feature
 * @retval true / false
 */
static bool Acc_GetPartOfFeature(unsigned char ucPort, unsigned char ucAccID, float *pfFea, unsigned short usAddress, unsigned long ulFeatureCount)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;
	unsigned short usMBBuffer[120];

	while (ucRetry > 0 && (eMBReadHoldingRegister(ucPort, ucAccID, usAddress, ulFeatureCount, usMBBuffer) != MB_EMASTERNOERR))
	{
		ucRetry--;
		Delay_ms(100);
	}

	if (ucRetry > 0)
	{
		int i = 0;

		for (i = 0; i < ulFeatureCount / 2; i++)
		{
			*pfFea = (long)((unsigned long)usMBBuffer[i * 2] << 16 | usMBBuffer[i * 2 + 1]) / 100.0f;
			pfFea += sizeof(Parameter) / sizeof(float);
		}

		return true;
	}
	else
		return false;
}

bool Acc_SetRecalculete(unsigned char ucPort, unsigned char ucAccID, unsigned long is_recalculete)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_FEATURE_RECALCULETE_ADDR, is_recalculete) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_SetMotorARRate(unsigned char ucPort, unsigned char ucAccID, unsigned long MotorARRate)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;
	unsigned short usData[2];

	usData[0] = (unsigned short)(MotorARRate >> 16);
	usData[1] = (unsigned short)(MotorARRate);

	while (ucRetry > 0 && eMBWriteMultiHoldingRegister(ucPort, ucAccID, ACC_MOTORAR_RATE_ADDR, 2, usData) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_GetMotorARRate(unsigned char ucPort, unsigned char ucAccID, unsigned long *MotorARRate)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;
	unsigned short usAddress;
	unsigned short usData[2];

	usAddress = ACC_MOTORAR_RATE_ADDR;
	while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, usAddress, 2, usData) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
	{
		*MotorARRate = ((unsigned long)usData[0] << 16) | usData[1];
		return true;
	}
	return false;
}

bool Acc_SetMotorActualSel(unsigned char ucPort, unsigned char ucAccID, unsigned short sel)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBWriteHoldingRegister(ucPort, ucAccID, ACC_MTACTSPD_SEL_ADDR, sel) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
		return true;
	return false;
}

bool Acc_GetMotorActualSel(unsigned char ucPort, unsigned char ucAccID, unsigned short *sel)
{
	unsigned char ucRetry = ACC_MAX_RETRY_COUNT;

	while (ucRetry > 0 && eMBReadHoldingRegister(ucPort, ucAccID, ACC_MTACTSPD_SEL_ADDR, 1, sel) != MB_EMASTERNOERR)
	{
		ucRetry--;
	}

	if (ucRetry > 0)
	{
		return true;
	}
	return false;
}
#endif

#if CONFIG_BLUETOOTH > 0
void Acc_GetSenBaseFeatureData(unsigned char *ucData, Sensor_Feature *pxSenFea)
{
	Feature *pxFea = NULL;
	Base_ExtendFeature *pxBaseExtFea = NULL;	
	Parameter *pxParm = NULL;
    unsigned int *pxData = NULL;
	unsigned short *pusData = NULL;

	if(ucData == NULL)
		return;

	pusData = (unsigned short *)&ucData[0];
	pxParm = &pxSenFea->temp;
	pxParm->fValue = (float)(*pusData / 100.0);
	pxParm->bIsUsed = true;
    
	for(int i = 0 ; i < SENSOR_ACC_DIMENSIO ; i++ ) {
		pxData = (unsigned int *)&ucData[SENSOR_TEMP_FEATURE_APKG_NUM + i * SENSOR_BASE_EXT_FEATURE_NAX_NUM];
		if(i == 0) {
			pxFea = &pxSenFea->x;
		} else if(i == 1) {
			pxFea = &pxSenFea->y;
		} else {
			pxFea = &pxSenFea->z;
		}
		pxParm = &pxFea->xMean;	
		for(int j = 0; j < SENSOR_BASE_FEATURE_APKG_NUM  / 4 ; j++ ) {
			pxParm->fValue = (float)(*pxData / 100.0);	
			pxParm->bIsUsed = true;
			pxData++;
			pxParm++;
		}
		pxData = (unsigned int *)&ucData[SENSOR_BASE_FEATURE_APKG_NUM + SENSOR_TEMP_FEATURE_APKG_NUM + i * SENSOR_BASE_EXT_FEATURE_NAX_NUM];
		if(i == 0) {
			pxBaseExtFea = &pxSenFea->xExt.xBaseExtFea;
		} else if(i == 1) {
			pxBaseExtFea = &pxSenFea->yExt.xBaseExtFea;
		} else {
			pxBaseExtFea = &pxSenFea->zExt.xBaseExtFea;
		}
		pxParm = &pxBaseExtFea->xHighFreqPeakToPeak;
		for(int j = 0; j < SENSOR_EXT_FEATURE_APKG_NUM  / 4 ; j++ ) {
			pxParm->fValue = (float)(*pxData / 100.0);	
			pxParm->bIsUsed = true;
			pxData++;
			pxParm++;
		}
	}
}

void Acc_ClearSenSpeedFeature(Sensor_Feature *pxSenFea)
{
	Feature *pxFea = NULL;
	Parameter *pxParm = NULL;
	for(int i = 0 ; i < SENSOR_ACC_DIMENSIO ; i++ ) {
		if(i == 0) {
			pxFea = &pxSenFea->x;
		} else if(i == 1) {
			pxFea = &pxSenFea->y;
		} else {
			pxFea = &pxSenFea->z;
		}
		pxParm = &pxFea->xFreq05x;	
		for(int j = 0; j < 20 ; j++ ) {
			pxParm->bIsUsed = false;
			pxParm++;
		}
	}		
}

void Acc_ClearSenInputSpeedFeature(Sensor_Feature *pxSenFea)
{
	Base_ExtendFeature *pxFea = NULL;
	Parameter *pxParm = NULL;
	for(int i = 0 ; i < SENSOR_ACC_DIMENSIO ; i++ ) 
	{
		if(i == 0) {
			pxFea = &pxSenFea->xExt.xBaseExtFea;
		} else if(i == 1) {
			pxFea = &pxSenFea->yExt.xBaseExtFea;
		} else {
			pxFea = &pxSenFea->zExt.xBaseExtFea;
		}
		pxParm = &pxFea->xInputEnvEnergy;
		pxParm->bIsUsed = false;
			
	}		
}

void Acc_ClearSenOutputSpeedFeature(Sensor_Feature *pxSenFea)
{
	Base_ExtendFeature *pxFea = NULL;
	Parameter *pxParm = NULL;
	for(int i = 0 ; i < SENSOR_ACC_DIMENSIO ; i++ ) 
	{
		if(i == 0) {
			pxFea = &pxSenFea->xExt.xBaseExtFea;
		} else if(i == 1) {
			pxFea = &pxSenFea->yExt.xBaseExtFea;
		} else {
			pxFea = &pxSenFea->zExt.xBaseExtFea;
		}
		pxParm = &pxFea->xOutputEnvEnergy;
		pxParm->bIsUsed = false;	
	}		
}

void Acc_GetMotorSpeedFeatureData(unsigned char *ucData, Sensor_Feature *pxSenFea)
{
	Feature *pxFea = NULL;
	Parameter *pxParm = NULL;
    unsigned int *pxData = NULL;
	
    if(ucData == NULL)
		return;

	for(int i = 0 ; i < SENSOR_ACC_DIMENSIO ; i++ ) {
		pxData = (unsigned int *)&ucData[i * SENSOR_BASE_EXT_FEATURE_NAX_NUM];
		if(i == 0) {
			pxFea = &pxSenFea->x;
		} else if(i == 1) {
			pxFea = &pxSenFea->y;
		} else {
			pxFea = &pxSenFea->z;
		}
		pxParm = &pxFea->xFreq05x;		
		for(int j = 0; j < SENSOR_BASE_MOTOR_FEATURE_APKG_NUM  / 4 ; j++ ) {
			pxParm->fValue = (float)(*pxData / 100.0);	
			pxParm->bIsUsed = true;
			pxData++;
			pxParm++;
		}	
	}	
}

void Acc_GetBearingFeatureData(bool bFlag, bool bExtendFlag, unsigned char index, unsigned char *ucData, Sensor_Feature *pxSenFea, char num)
{
	Bearing_Parameter       *pxBearingFea;
	Bearing_ExtendParameter *pxBearingExtFea; 
	Parameter               *pxParm = NULL;
    unsigned int            *pxData = NULL;
    char                    cAxis[4];
    if(ucData == NULL)
		return;
	printf("bearing num = %d\r\n", num);	
	for(int i = 0 ; i < SENSOR_ACC_DIMENSIO ; i++ ) {
        if(i == 0) {
			sprintf(cAxis, "x");
			pxBearingFea = &pxSenFea->x.xBearingFea[index];
			pxBearingExtFea = &pxSenFea->xExt.xBearingExtFea[index];
		} else if(i == 1) {
			sprintf(cAxis, "y");
            pxBearingFea = &pxSenFea->y.xBearingFea[index];
			pxBearingExtFea = &pxSenFea->yExt.xBearingExtFea[index];
		} else {
			sprintf(cAxis, "z");
			pxBearingFea = &pxSenFea->z.xBearingFea[index];
			pxBearingExtFea = &pxSenFea->zExt.xBearingExtFea[index];
		}
        pxParm = &pxBearingFea->xFreqBO;
		if(bFlag == true)
		{
			pxData = (unsigned int *)&ucData[i * SENSOR_BEARING_FEATURE_APKG_NUM];
			sprintf(pxBearingFea->xFreqBO.cName, "%sFreqBO%d", cAxis, num);
			sprintf(pxBearingFea->xFreqBI.cName, "%sFreqBI%d", cAxis, num);
			sprintf(pxBearingFea->xFreqBB.cName, "%sFreqBB%d", cAxis, num);
			sprintf(pxBearingFea->xFreqBF.cName, "%sFreqBF%d", cAxis, num);
			for(int j = 0 ; j < SENSOR_BEARING_BASE_NUM / 4 ; j++ ) 
			{
				pxParm->fValue = (float)(*pxData / 100.0);	
				pxParm->bIsUsed = true;
				pxData++;
				pxParm++;	
			}
		}
		pxParm = &pxBearingExtFea->xEnvEnergy;
		if(bExtendFlag == true)
		{   
			pxData = (unsigned int *)&ucData[i * SENSOR_BEARING_FEATURE_APKG_NUM + 16];
			//轴承扩展特征
		    sprintf(pxBearingExtFea->xEnvEnergy.cName, "%sseB%d", cAxis, num);
			sprintf(pxBearingExtFea->xEnv2Energy.cName, "%sse2B%d", cAxis, num);
			sprintf(pxBearingExtFea->xEnv3Energy.cName, "%sse3B%d", cAxis, num);
			for(int j = 0 ; j < SENSOR_BEARING_ENVENERGY_NUM / 4 ; j++) 
			{
				pxParm->fValue = (float)(*pxData / 100.0);	
				pxParm->bIsUsed = true;
				pxData++;
				pxParm++;	
			}
		}
	}  
}

void Acc_GetGearFeatureData(bool bFlag, bool bExtendFlag, unsigned char index, unsigned char *ucData, Sensor_Feature *pxSenFea ,char num)
{
	Gear_Parameter          *pxGearFea = NULL;
	Gear_ExtendParameter    *pxGearExtFea = NULL;	
	Parameter               *pxParm = NULL;
    unsigned int            *pxData = NULL;		
	char                    cAxis[4]; 

    if(ucData == NULL)
		return;

	for(int i = 0 ; i < SENSOR_ACC_DIMENSIO ; i++ ) {
        if(i == 0) {
			sprintf(cAxis, "x");
			pxGearFea = &pxSenFea->x.xGearFea[index];
			pxGearExtFea = &pxSenFea->xExt.xGearExtFea[index];
		} else if(i == 1) {
			sprintf(cAxis, "y");
            pxGearFea = &pxSenFea->y.xGearFea[index];
			pxGearExtFea = &pxSenFea->yExt.xGearExtFea[index];
		} else {
			sprintf(cAxis, "z");
			pxGearFea = &pxSenFea->z.xGearFea[index];
			pxGearExtFea = &pxSenFea->zExt.xGearExtFea[index];
		}
		pxParm = &pxGearFea->xFreq1xGMF; 
		if(bFlag == true)
		{
			pxData = (unsigned int *)&ucData[i * SENSOR_GEAR_SERIES];
			sprintf(pxGearFea->xFreq1xGMF.cName, "%sFreq1xGMF%d", cAxis, num);
			sprintf(pxGearFea->xFreq2xGMF.cName, "%sFreq2xGMF%d", cAxis, num);
			sprintf(pxGearFea->xFreq3xGMF.cName, "%sFreq3xGMF%d", cAxis, num);
			for(int j = 0 ; j < SENSOR_GEAE_BASE_NUM / 4 ; j++ ) 
			{
				pxParm->fValue = (float)(*pxData / 100.0);	
				pxParm->bIsUsed = true;
				pxData++;
				pxParm++;	
			}
		}
		pxParm = &pxGearExtFea->x1xSideEnergy; 
		if(bExtendFlag == true)
		{
			pxData = (unsigned int *)&ucData[i * SENSOR_GEAR_SERIES + 12];
			//齿轮扩展特征
			sprintf(pxGearExtFea->x1xSideEnergy.cName, "%sSide1xEnergy%d", cAxis, num);
			sprintf(pxGearExtFea->x2xSideEnergy.cName, "%sSide2xEnergy%d", cAxis, num);
			sprintf(pxGearExtFea->x3xSideEnergy.cName, "%sSide3xEnergy%d", cAxis, num);
			sprintf(pxGearExtFea->xEnvEnergy.cName, "%sseG%d", cAxis, num);	
			for(int j = 0 ; j < SENSOR_GEAR_EXT_NUM / 4 ; j++) 
			{	
				pxParm->fValue = (float)(*pxData / 100.0);	
				pxParm->bIsUsed = true;
				pxData++;
				pxParm++;	
			}
		}
	}
}

void  Acc_GetFanFeatureData(unsigned char *ucData, Sensor_Feature *pxSenFea) 
{
	Fan_Parameter *pxFan = NULL;
	Parameter     *pxParm = NULL;
    unsigned int  *pxData = NULL;
	char           num = 0;
	char           cAxis[4];

    if(ucData == NULL)
		return;

	num = ucData[SENSOR_FAN_ALIAS_APKG_ADDR - SENSOR_FAN_FEATURE_AKPG_ADDR];
	printf("fan num %d\r\n", num);
	if(num == 0)
		num = 1;
	for(int i = 0 ; i < SENSOR_ACC_DIMENSIO ; i++ ) {
		pxData = (unsigned int *)&ucData[i * SENSOR_FAN_AXIS_APKG_NUM];	
		if(i == 0) {
			sprintf(cAxis, "x");
			pxFan = &pxSenFea->x.xFan;
		} else if(i == 1) {
			sprintf(cAxis, "y");
			pxFan = &pxSenFea->y.xFan;
		}else {
			sprintf(cAxis, "z");
			pxFan = &pxSenFea->z.xFan;
		}
		pxParm = &pxFan->xSPM1x;
		sprintf(pxFan->xSPM1x.cName, "%sspm1xF%d", cAxis, num);
		sprintf(pxFan->xSPM2x.cName, "%sspm2xF%d", cAxis, num);
		sprintf(pxFan->xSPM3x.cName, "%sspm3xF%d", cAxis, num);
		for(int j = 0; j < SENSOR_FAN_AXIS_APKG_NUM / 4 ; j++ ) {	
			pxParm->fValue = (float)(*pxData / 100.0);	
			pxParm->bIsUsed = true;
			pxData++;
			pxParm++;
		}
	}
}

void Acc_GetBeltFeatureData(unsigned char *ucData, Sensor_Feature *pxSenFea) 
{
	Belt_Parameter  *pxBelt = NULL;	
	Parameter       *pxParm = NULL;
    unsigned int    *pxData = NULL;
	char           num = 0;
	char           cAxis[4];

    if(ucData == NULL)
		return;
	num = ucData[SENSOR_BELT_ALIAS_APKG_ADDR - SENSOR_BELT_FEATURE_APKG_ADDR];
	if(num == 0)
		num = 1;
	for(int i = 0 ; i < SENSOR_ACC_DIMENSIO ; i++ ) {
		pxData = (unsigned int *)&ucData[i * SENSOR_BELT_AXIS_APKG_NUM];	
		if(i == 0) {
			sprintf(cAxis, "x");
			pxBelt = &pxSenFea->x.xBelt;
		} else if(i == 1) {
			sprintf(cAxis, "y");
			pxBelt = &pxSenFea->y.xBelt;
		}else {
			sprintf(cAxis, "z");
			pxBelt = &pxSenFea->z.xBelt;
		}
		sprintf(pxBelt->xBelt1x.cName, "%sbelt1xF%d", cAxis, num);
		sprintf(pxBelt->xSelfVirb.cName, "%sAmbeltVirF%d", cAxis, num);
		pxParm = &pxBelt->xBelt1x;
		pxParm->fValue = (float)(*pxData / 100.0);	
		pxParm->bIsUsed = true;
	}
	for(int i = 0 ; i < SENSOR_ACC_DIMENSIO ; i++ ) {
		pxData = (unsigned int *)&ucData[i * SENSOR_BELT_SELF_VIBRATION_APKG_NUM + 12];	
		if(i == 0) {
			pxBelt = &pxSenFea->x.xBelt;
		} else if(i == 1) {
			pxBelt = &pxSenFea->y.xBelt;
		}else {
			pxBelt = &pxSenFea->z.xBelt;
		}
		pxParm = &pxBelt->xSelfVirb;
		pxParm->fValue = (float)(*pxData / 100.0);	
		pxParm->bIsUsed = true;
	}
}

void Acc_GetScrewFeatureData(unsigned char *ucData, Sensor_Feature *pxSenFea)
{
	Screw_Parameter *pxScrew = NULL;
	Parameter       *pxParm = NULL;
    unsigned int    *pxData = NULL;
    char             num = 0;
	char           cAxis[4];

    if(ucData == NULL)
		return;

	num = ucData[SENSOR_SCREW_ALIAS_APKG_ADDR - SENSOR_SCREW_FEATURE_AKPG_ADDR];
	printf("screw num %d\r\n", num);
	if(num == 0)
		num = 1;
	for(int i = 0 ; i < SENSOR_ACC_DIMENSIO ; i++ ) {
		pxData = (unsigned int *)&ucData[i * SENSOR_SCREW_AXIS_APKG_NUM];	
		if(i == 0) {
			sprintf(cAxis, "x");
			pxScrew = &pxSenFea->x.xScrew;
		} else if(i == 1) {
			sprintf(cAxis, "y");
			pxScrew = &pxSenFea->y.xScrew;
		}else {
			sprintf(cAxis, "z");
			pxScrew = &pxSenFea->z.xScrew;
		}
		pxParm = &pxScrew->xScrew1x;
		sprintf(pxScrew->xScrew1x.cName, "%sscrew1xF%d", cAxis, num);
		sprintf(pxScrew->xScrew2x.cName, "%sscrew2xF%d", cAxis, num);
		sprintf(pxScrew->xScrew3x.cName, "%sscrew3xF%d", cAxis, num);
		for(int j = 0; j < SENSOR_SCREW_AXIS_APKG_NUM / 4 ; j++ ) {
			pxParm->fValue = (float)(*pxData / 100.0);	
			pxParm->bIsUsed = true;
			pxData++;
			pxParm++;
		}
	}
}

void Acc_GetShaftFeatureData(unsigned char *ucData, Sensor_Feature *pxSenFea)
{
	Shaft_Parameter *pxShaft = NULL;
	Parameter       *pxParm = NULL;
    unsigned int    *pxData = NULL;
	char             num = 0;
	char           cAxis[4];

    if(ucData == NULL)
		return;

	num = ucData[SENSOR_SHAFT_ALIAS_APKG_ADDR - SENSOR_SHAFT_FEATURE_AKPG_ADDR];
	printf("shaft num %d\r\n", num);
	if(num == 0)
		num = 1;
	for(int i = 0 ; i < SENSOR_ACC_DIMENSIO ; i++ ) {
		pxData = (unsigned int *)&ucData[i * SENSOR_SCREW_AXIS_APKG_NUM];	
		if(i == 0) {
			sprintf(cAxis, "x");
			pxShaft = &pxSenFea->x.xShaft;
		} else if(i == 1) {
			sprintf(cAxis, "y");
			pxShaft = &pxSenFea->y.xShaft;
		}else {
			sprintf(cAxis, "z");
			pxShaft = &pxSenFea->z.xShaft;
		}
		pxParm = &pxShaft->xShaft1x;
		sprintf(pxShaft->xShaft1x.cName, "%sshaft1xF%d", cAxis, num);
		sprintf(pxShaft->xShaft2x.cName, "%sshaft2xF%d", cAxis, num);
		sprintf(pxShaft->xShaft3x.cName, "%sshaft3xF%d", cAxis, num);
		for(int j = 0; j < SENSOR_SCREW_AXIS_APKG_NUM / 4 ; j++ ) {
			pxParm->fValue = (float)(*pxData / 100.0);	
			pxParm->bIsUsed = true;
			pxData++;
			pxParm++;
		}
	}   
}

void Acc_GetFeMaleFeatureData(unsigned char *ucData, Sensor_Feature *pxSenFea)
{
	FemaleScrew_Parameter *pxFemaleScrew = NULL;
	Parameter             *pxParm = NULL;
	unsigned int          *pxData = NULL;
	char                   num = 0;
	char                   cAxis[4];
	if(ucData == NULL)
		return;

	num = ucData[SENSOR_FEMALE_ALIAS_APKG_ADDR - SENSOR_FEMALE_SCREW_FEATURE_AKPG_ADDR];
	printf("Female num %d\r\n", num);
	if(num == 0)
		num = 1;

    for(int i = 0 ; i < SENSOR_ACC_DIMENSIO ; i++ ) {
		pxData = (unsigned int *)&ucData[i * SENSOR_SCREW_AXIS_APKG_NUM];	
		if(i == 0) {
			sprintf(cAxis, "x");
			pxFemaleScrew = &pxSenFea->x.xFemaleScrew;
		} else if(i == 1) {
			sprintf(cAxis, "y");
			pxFemaleScrew = &pxSenFea->y.xFemaleScrew;
		}else {
			sprintf(cAxis, "z");
			pxFemaleScrew = &pxSenFea->z.xFemaleScrew;
		}
		pxParm = &pxFemaleScrew->xFemaleScrew1x;
		sprintf(pxFemaleScrew->xFemaleScrew1x.cName, "%sfemalescrew1xF%d", cAxis, num);
		sprintf(pxFemaleScrew->xFemaleScrew2x.cName, "%sfemalescrew2xF%d", cAxis, num);
		sprintf(pxFemaleScrew->xFemaleScrew3x.cName, "%sfemalescrew3xF%d", cAxis, num);
		for(int j = 0; j < SENSOR_FMMALE_AXIS_APKG_NUM / 4 ; j++ ) {
			pxParm->fValue = (float)(*pxData / 100.0);	
			pxParm->bIsUsed = true;
			pxData++;
			pxParm++;
		}			
	}
}

void Acc_GetMaleFeatureData(unsigned char *ucData, Sensor_Feature *pxSenFea)
{
	MaleScrew_Parameter   *pxMaleScrew   = NULL;
	Parameter             *pxParm = NULL;
	unsigned int          *pxData = NULL;
	char                  num = 0;
	char                  cAxis[4];
	if(ucData == NULL)
		return;

	num = ucData[SENSOR_MALE_ALIAS_APKG_ADDR - SENSOR_MALE_SCREW_FEATURE_AKPG_ADDR];
	printf("Male num %d\r\n", num);
	if(num == 0)
		num = 1;

	for(int i = 0 ; i < SENSOR_ACC_DIMENSIO ; i++ ) {
		pxData = (unsigned int *)&ucData[i * SENSOR_FMMALE_AXIS_APKG_NUM];	
		if(i == 0) {
			sprintf(cAxis, "x");
			pxMaleScrew = &pxSenFea->x.xMaleScrew;
		} else if(i == 1) {
			sprintf(cAxis, "y");
			pxMaleScrew = &pxSenFea->y.xMaleScrew;
		}else {
			sprintf(cAxis, "z");
			pxMaleScrew = &pxSenFea->z.xMaleScrew;
		}
		pxParm = &pxMaleScrew->xMaleScrew1x;
		sprintf(pxMaleScrew->xMaleScrew1x.cName, "%smalescrew1xF%d", cAxis, num);
		sprintf(pxMaleScrew->xMaleScrew2x.cName, "%smalescrew2xF%d", cAxis, num);
		sprintf(pxMaleScrew->xMaleScrew3x.cName, "%smalescrew3xF%d", cAxis, num);
		for(int j = 0; j < SENSOR_FMMALE_AXIS_APKG_NUM / 4 ; j++ ) {
			pxParm->fValue = (float)(*pxData / 100.0);	
			pxParm->bIsUsed = true;
			pxData++;
			pxParm++;
		}			
	}
}

void Acc_GetPowerFeatureData(unsigned char *ucData, Sensor_Feature *pxSenFea)
{
	PowerFrequency_Parameter *pxPowerFrequency = NULL;
    Parameter                *pxParm = NULL;
	unsigned int             *pxData = NULL;

    if(ucData == NULL)
		return;

	for(int i = 0 ; i < SENSOR_ACC_DIMENSIO ; i++ ) {
		pxData = (unsigned int *)&ucData[i * SENSOR_POWER_AXIS_APKG_NUM];	
		if(i == 0) {
			pxPowerFrequency = &pxSenFea->x.xPowerFrequency;
		} else if(i == 1) {
			pxPowerFrequency = &pxSenFea->y.xPowerFrequency;
		}else {
			pxPowerFrequency = &pxSenFea->z.xPowerFrequency;
		}
		pxParm = &pxPowerFrequency->xNaAmFreq;
		pxParm->fValue = (float)(*pxData / 100.0);	
		pxParm->bIsUsed = true;
	}
	for(int i = 0 ; i < SENSOR_ACC_DIMENSIO ; i++ ) {
		pxData = (unsigned int *)&ucData[i * SENSOR_BELT_SELF_VIBRATION_APKG_NUM + 12];	
		if(i == 0) {
			pxPowerFrequency = &pxSenFea->x.xPowerFrequency;
		} else if(i == 1) {
			pxPowerFrequency = &pxSenFea->y.xPowerFrequency;
		}else {
			pxPowerFrequency = &pxSenFea->z.xPowerFrequency;
		}
		pxParm = &pxPowerFrequency->xEnergy;
		pxParm->fValue = (float)(*pxData / 100.0);	
		pxParm->bIsUsed = true;
	}
}

void Acc_GetDeviceInfoData(unsigned char *ucData, char *psSN, char *psType, char *psCVer, char *psLogo, char *psVer)
{
	unsigned char ucIndex = 0;
	unsigned char ucNum = 0;

	if(ucData == NULL && psSN == NULL && psType == NULL && psCVer == NULL && psLogo == NULL && psVer == NULL)
		return;
	ucIndex = SENSOR_DEVICE_SN_APKG_ADDR - SENSOR_DEVICE_SN_APKG_ADDR;
	ucNum = SENSOR_DEVICE_TYPE_APKG_ADDR - SENSOR_DEVICE_SN_APKG_ADDR;
	if(ucData[ucIndex] == 0)
	{
		sprintf(psSN, "12345678");	
	}
	else
	{
		memcpy(psSN, &ucData[ucIndex], ucNum);  
	}
   
	ucIndex = SENSOR_DEVICE_TYPE_APKG_ADDR - SENSOR_DEVICE_SN_APKG_ADDR;
    ucNum = SENSOR_DEVICE_CHILDTYPE_APKG_ADDR - SENSOR_DEVICE_TYPE_APKG_ADDR;
	if(ucData[ucIndex] == 0)
	{
		sprintf(psType, "BV43");	
	}
	else
	{
		memcpy(psType, &ucData[ucIndex], ucNum); 
	}
	

	ucIndex = SENSOR_DEVICE_CHILDTYPE_APKG_ADDR - SENSOR_DEVICE_SN_APKG_ADDR;
    ucNum =  SENSOR_DEVICE_LOGO_APKG_ADDR - SENSOR_DEVICE_CHILDTYPE_APKG_ADDR;
	if(ucData[ucIndex] == 0)
	{
		sprintf(psCVer, "ANY");	
	}
	else
	{
		memcpy(psCVer, &ucData[ucIndex], ucNum);  
	}
    ucIndex = SENSOR_DEVICE_LOGO_APKG_ADDR - SENSOR_DEVICE_SN_APKG_ADDR;
    ucNum =  SENSOR_DEVICE_VERSION_APKG_ADDR - SENSOR_DEVICE_DATA_APKG_ADDR;
	if(ucData[ucIndex] == 0)
	{
		sprintf(psLogo, "WT");	
	}
	else
	{
		memcpy(psLogo, &ucData[ucIndex], ucNum);  
	}  
	ucIndex = SENSOR_DEVICE_VERSION_APKG_ADDR - SENSOR_DEVICE_SN_APKG_ADDR;
	memcpy(psVer, &ucData[ucIndex], 3);  
}

void Acc_GetSensorData(unsigned char *ucData, unsigned char flag, unsigned short *pusSampleRate, unsigned int *puiResolution)
{
    unsigned char ucOdr = 0;
	unsigned int OdrDivMap[] = {0, 1, 3, 7, 15, 31};
	if(ucData == NULL && pusSampleRate == NULL)
		return;
	if(puiResolution != NULL)	
		*puiResolution = ucData[SENSOR_ODR_APKG_ADDR - SENSOR_FFT_RESOLUTION_APKG_ADDR];
	ucOdr = ucData[0];
	if(flag == 1)
		*pusSampleRate = (26667.0f / (float)(OdrDivMap[ucOdr % 6] + 1));
	if(*pusSampleRate == 0)
		*pusSampleRate = 13333;

}
#endif