/* Includes ------------------------------------------------------------------*/
#include "celluar_gpio.h"
#include "driver/gpio.h"

#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "esp_rom_gpio.h"

/* Private macros ------------------------------------------------------------*/
// modify

/* Private macros ------------------------------------------------------------*/
#define CELLUAR_POWER_KEY_PIN			  1
#define CELLUAR_STATUS_PIN          38
#define POWER_KEY_PIN               46   

#define CELLUAR_POWER_BASE_TIME			500				//500ms		

/**
  * @brief  Celluar GPIO Init (Status PWRKEY Pin)
  * @param  None
  * @retval None
  */
void Celluar_ConfigureGPIO(void)
{
  esp_rom_gpio_pad_select_gpio(CELLUAR_STATUS_PIN);  
  gpio_set_direction(CELLUAR_STATUS_PIN, GPIO_MODE_INPUT);      //      
  // POWER_KEY_PIN
  // esp_rom_gpio_pad_select_gpio(POWER_KEY_PIN);   // 选择个GPIO
	// gpio_set_direction(POWER_KEY_PIN, GPIO_MODE_OUTPUT); // 把这个GPIO作为输出     
  gpio_set_level(POWER_KEY_PIN, 1); 
  
  // // EC20 PWR设置 
  // vTaskDelay(0.5);
	// //2s Pull down
  // gpio_set_level(POWER_KEY_PIN, 0);
	// vTaskDelay(1);
	// //32s Pull up until serial port active
	// gpio_set_level(POWER_KEY_PIN, 1);
	// vTaskDelay(1000);
}

/**
  * @brief  Celluar power on status 
  * @param  None
  * @retval is power on ? (true/false)
  */
bool Celluar_IsPowerOnWithGPIO(void)
{
	return gpio_get_level(CELLUAR_STATUS_PIN) ? true : false;
}

/**
  * @brief  Celluar power off / on
  * @param  None
  * @retval None
  */
void Celluar_TriggerPowerKey()
{
	// modify
	/*SIM868 开机:Low>1s SerialActive>3s；关机:1.5s<Low<33s SerialActive>2s*/
	/*UC20 开机 Low>100ms SerialActive>5s;关机：Low > 0.6s*/
	/*EC25 开机 Low>100ms SerialActive>13s;关机：Low > 0.6s High > 30s*/
	//500ms Pull up
  // EC20 PWR设置 
  gpio_set_level(POWER_KEY_PIN, 1);
  vTaskDelay((CELLUAR_POWER_BASE_TIME) / portTICK_PERIOD_MS);
	//2s Pull down
  gpio_set_level(POWER_KEY_PIN, 0);
	vTaskDelay((CELLUAR_POWER_BASE_TIME * 4) / portTICK_PERIOD_MS);
	//32s Pull up until serial port active
	gpio_set_level(POWER_KEY_PIN, 1);
	// vTaskDelay((CELLUAR_POWER_BASE_TIME * 60) / portTICK_PERIOD_MS);
}
