/* Includes ------------------------------------------------------------------*/
#include "celluar_socket.h"

#if CELLUAR_TCPIP_ENABLE > 0
/* Private macros ------------------------------------------------------------*/

/* Private variables ---------------------------------------------------------*/
Celluar_Socket 	g_xCelluarSocket[CELLUAR_SOCKET_MAX_COUNT];
unsigned long 	g_ulMaxSocketID;

/* Extern variables ----------------------------------------------------------*/

/**
  * @brief  Celluar create socket
  * @param  None
  * @retval -1 (Create error) / socketid
  */
long Celluar_CreateSocket(char *pcMode, char *pcHost, char *pcPort)
{
	unsigned long ulIndex;
	
	if (pcMode == NULL || pcHost == NULL || pcPort == NULL)
		return -1;
	
	for (ulIndex = 0;ulIndex < g_ulMaxSocketID;ulIndex++)
	{
		if (!g_xCelluarSocket[ulIndex].bUsed)
		{
			break;
		}
	}
	
	if (ulIndex == g_ulMaxSocketID)
	{
		return -1;
	}
	else
	{
		strcpy(g_xCelluarSocket[ulIndex].cMode, pcMode);
		strcpy(g_xCelluarSocket[ulIndex].cDestinationHost, pcHost);
		strcpy(g_xCelluarSocket[ulIndex].cDestinationPort, pcPort);
		g_xCelluarSocket[ulIndex].bUsed 			= true;
		g_xCelluarSocket[ulIndex].bStatus 			= false;
	#if CELLUAR_RECONNECT_ENABLE > 0
		g_xCelluarSocket[ulIndex].bReconnectEnable 	= false;
	#endif
		return ulIndex;
	}
}

/**
  * @brief  Celluar change connect info
  * @param  lSocketID
  * @retval None
  */
void Celluar_ChangeConnectInfo(unsigned long ulSocketID, char *pcMode, char *pcHost, char *pcPort)
{
	// if valid
	if (pcMode == NULL || pcHost == NULL || pcPort == NULL)
		return;
	
	if (ulSocketID >= g_ulMaxSocketID)
		return;
	
	if (g_xCelluarSocket[ulSocketID].bUsed == false)
		return;
	
	strcpy(g_xCelluarSocket[ulSocketID].cMode, pcMode);
	strcpy(g_xCelluarSocket[ulSocketID].cDestinationHost, pcHost);
	strcpy(g_xCelluarSocket[ulSocketID].cDestinationPort, pcPort);
}

/**
  * @brief  Celluar release socket
  * @param  lSocketID
  * @retval None
  */
void Celluar_ReleaseSocket(unsigned long ulSocketID)
{
	if (ulSocketID >= g_ulMaxSocketID)
		return;
	
	g_xCelluarSocket[ulSocketID].bUsed 				= false;
	g_xCelluarSocket[ulSocketID].bStatus 			= false;
#if CELLUAR_RECONNECT_ENABLE > 0
	g_xCelluarSocket[ulSocketID].bReconnectEnable 	= false;
#endif
	memset(g_xCelluarSocket[ulSocketID].cMode, 0, sizeof(g_xCelluarSocket[ulSocketID].cMode));
	memset(g_xCelluarSocket[ulSocketID].cDestinationHost, 0, sizeof(g_xCelluarSocket[ulSocketID].cDestinationHost));
	memset(g_xCelluarSocket[ulSocketID].cDestinationPort, 0, sizeof(g_xCelluarSocket[ulSocketID].cDestinationPort));
}

/**
  * @brief  Celluar set socket status
  * @param  lSocketID
  * @retval None
  */
void Celluar_SetSocketStatus(unsigned long ulSocketID, bool bStatus)
{
	if (ulSocketID >= g_ulMaxSocketID || g_xCelluarSocket[ulSocketID].bUsed == false)
		return;
	
	g_xCelluarSocket[ulSocketID].bStatus = bStatus;
}

/**
  * @brief  Celluar get socket status
  * @param  lSocketID
  * @retval None
  */
bool Celluar_GetSocketStatus(unsigned long ulSocketID)
{
	if (ulSocketID >= g_ulMaxSocketID || g_xCelluarSocket[ulSocketID].bUsed == false)
		return false;
	return g_xCelluarSocket[ulSocketID].bStatus;
}

#endif
