/* MQTT (over TCP) Example

   This example code is in the Public Domain (or CC0 licensed, at your option.)

   Unless required by applicable law or agreed to in writing, this
   software is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
   CONDITIONS OF ANY KIND, either express or implied.
*/

#include <stdio.h>
#include <stdint.h>
#include <stddef.h>
#include <string.h>

#include "esp_system.h"
#include "nvs_flash.h"

#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/semphr.h"
#include "freertos/queue.h"
#include "wit.h"

#if CONFIG_W5100S_SPI_ETHERNET > 0
#include "httputil.h"
#endif

#define MAIN_TASK_STK						(configMINIMAL_STACK_SIZE * 10)
#define MAIN_TASK_PRIORITY					(tskIDLE_PRIORITY + 3)

#if CONFIG_W5100S_SPI_ETHERNET > 0
#define HTTP_TASK_STK						(configMINIMAL_STACK_SIZE * 20)
#define HTTP_TASK_PRIORITY					(tskIDLE_PRIORITY + 2)
#endif

#define TWDT_TIMEOUT_S                      3    // TWDT超时时间
#define TASK_RESET_PERIOD_S                 2    // 任务重置周期时间

static void Main_Task(void *pvParameter);
#if CONFIG_W5100S_SPI_ETHERNET > 0
    static void Http_Task(void *pvParameter);
#endif

static int bWITInitFin = 0;

void app_main(void)
{
    int rc = 0;
    ESP_ERROR_CHECK(nvs_flash_init());
    xTaskCreate(Main_Task, "Main Task", MAIN_TASK_STK, NULL, MAIN_TASK_PRIORITY, NULL);
#if CONFIG_W5100S_SPI_ETHERNET > 0
    rc = xTaskCreate(Http_Task, "Http Task", HTTP_TASK_STK, NULL, HTTP_TASK_PRIORITY, NULL);
#endif
    printf("rc  is %d\r\n", rc);
}

static void Main_Task(void *pvParameter)
{
	//Init
    printf("esp_reset_reason %d\r\n", esp_reset_reason());
    printf("+++WIT_Init+++\r\n");  
	WIT_Init();
    printf("---WIT_Init---\r\n");
    bWITInitFin = 1;  
    while(1)
    {
        WIT_Run();
        vTaskDelay(20 / portTICK_PERIOD_MS);
    }
}

#if CONFIG_W5100S_SPI_ETHERNET > 0
static void Http_Task(void *pvParameter)
{
	//Init
    while(1) 
    {
        if(bWITInitFin == 1)
        {
            do_https();   
        }
        vTaskDelay(1 / portTICK_PERIOD_MS);
    }
}
#endif
