/* Includes ------------------------------------------------------------------*/
#include <stdarg.h>
#include "wit.h"
#include "string.h"
#include "flash.h"
#include "pwr_gpio.h"
#include "acc.h"
#include "acc_pin.h"
#include "key.h"
#include "led.h"
#include "mqtts.h"
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/semphr.h"
#include "freertos/queue.h"
#include "freertos/timers.h"
#include "cJSON.h"
#include "mqtt_update.h"
#include "mbmaster.h"
#if CONFIG_RS485_RTU > 0
#include "sensor_update.h"
#endif

#include "udp_update.h"
// Algorithm
#include "algorithm.h"
// DJSON
#include "djson.h"
// Modbus update
#include "modbus_update.h"
#include "GatewayFSM.h"
#include "SensorFSM.h"
#include "debugFSM.h"
#include "WIT2FSM.h"
#include "esp_log.h"
#include "rtc.h"
#include "ram.h"

#if CONFIG_W5100S_SPI_ETHERNET > 0
#include "../infa/APP/httputil.h"
#include "w5100s.h"
#include "dhcp.h"
#include "loopback.h"
#include "wizchip_conf.h"
#include "w5100s_conf.h"
#include "eth.h"
#include "../infa/APP/tcp_server.h"
#include "../eth_parsing/ts_iot01b.h"
#endif
#if CONFIG_EC200S_USB_4G > 0
#include "celluar_comm.h"
#include "celluar.h"
#include "celluar_socket.h"
#endif

#if CONFIG_BLUETOOTH > 0
#include "wcp.h"
#include "ble_client.h"
#include "ble_update.h"
#include "if_ble_config.h"
#include "if_apkg_code.h"
#include "if_gw.h"
#endif

struct beingConfigIOT{
    bool ConfigFlag;
    unsigned short Config_Sn;
    unsigned short Config_Id;
}g_beingConfigIOT = {
    .ConfigFlag = false,
    .Config_Sn = 0,
    .Config_Id = 0
};

unsigned long g_ts[IOTDEV_MAX_NUM] = {0};  // 记录最新收到心跳时的时间
QueueHandle_t xMessageQueueHandle = NULL;
unsigned char g_slaver_status = 0;
static bool g_bDataSend_TimerArr = false;
static bool g_bErrorSend_TimerArr = false;
bool g_DataUpdateFlag = false;
bool g_bErrorUpdateFlag = false;
extern TimerHandle_t xSensorTimer;
extern TimerHandle_t xSensorDataFreshTimer;
SemaphoreHandle_t xSemaphoreWaitResp = NULL;
SemaphoreHandle_t xSemaphoreDataResp = NULL;

/* Private macros ------------------------------------------------------------*/
// Task info
#define WIT_LED_TASK_STK (configMINIMAL_STACK_SIZE * 3)
#define WIT_LED_TASK_PRIORITY (tskIDLE_PRIORITY + 4)
#define WIT_KEY_TASK_STK (configMINIMAL_STACK_SIZE * 2)
#define WIT_KEY_TASK_PRIORITY (tskIDLE_PRIORITY + 3)
#define WIT_UDP_POLL_TASK_STK (configMINIMAL_STACK_SIZE * 4)
#define WIT_UDP_POLL_TASK_PRIORITY (tskIDLE_PRIORITY + 3)
#define WIT_UPDATE_POLL_TASK_STK (configMINIMAL_STACK_SIZE * 4)
#define WIT_UPDATE_POLL_TASK_PRIORITY (tskIDLE_PRIORITY + 3)
#define WIT_MODBUS_SLAVE_POLL_STK (configMINIMAL_STACK_SIZE * 4)
#define WIT_MODBUS_SLAVE_POLL_PRIORITY (tskIDLE_PRIORITY + 4)
#define WIT_WIFI_RECONNECT_TASK_STK (configMINIMAL_STACK_SIZE * 4)
#define WIT_WIFI_RECONNECT_TASK_PRIORITY (tskIDLE_PRIORITY + 3)

#define WIT_DATA_LIST_EXT_FLASH_ADDR 0x700000
#define WIT_PER_DATA_STORE_MAX_COUNT 1024 // 每个特征存储数量
// Modbus info
#define WIT_MB_SLAVE_PORT 2
#define WIT_MB_MASTER_PORT 2
#define WIT_MB_BAUDRATE 115200
#define WIT_MB_PARITY MB_PAR_NONE
#define WIT_ACC_SENSOR_ID 1
#define WIT_ACC_MAX_WAIT_TIME (5 * 60 * 1000) // 5min

#if USE_WIT_PLATFORM > 0
#define WIT_MQTT_TOPIC_PERFIX "WIT"
#elif USE_WT_PLATFORM > 0
#define WIT_MQTT_TOPIC_PERFIX "WT"
#endif

#define WIT_MAX_ERROR_COUNT 6

#define WIT_UDPS_BUFFER_SIZE 256

// Acc sensor
#define WIT_MAX_ID_NUMBER 10000
#define WIT_MAX_WORKTIME 1051200 // 20 * 365 * 24 * 60

// Sensor MB Buffer
#define SENSOR_MB_MAX_BUFFER_SIZE 256

// define different acc 
#define ADXL357_FACTOR          (12.8f)
#define IIS3DWB_FACTOR          (0.488f)

// Sensor modbus info
#define WIT_MODBUS_PORT_COUNT 1
#define WIT_PER_PORT_SENSOR_COUNT (ACC_MAX_SENSOR_COUNT / WIT_MODBUS_PORT_COUNT)

// Auto threshold scale
#define WIT_AUTO_THRESHOLD_SCALE 0.1

#define WIT_MB_NORMAL_TIMEOUT 100
#define WIT_MB_UPDATE_TIMEOUT 5000

#define WIT_MB_CHECK_TIMES 2

#define WIT_TRUE 1
#define WIT_FALSE 0
/* Private types -------------------------------------------------------------*/

typedef enum
{
	SensorMode_Passive = 0,
	SensorMode_Auto,
	SensorMode_Active,
	SensorMode_Monitor
} SensorMode;

typedef enum
{
	Resp_Success = 0,
	Resp_ParmErr = 1,
	Resp_ParseErr = 2,
	Resp_None = 0xFF
} Response_Status;

// 数据 --- 阈值 --- 存储数据
// 网关数据

typedef struct
{
	Parameter Reserve;
} Gateway_Feature;

// 网关上发数据结构
typedef struct
{
	Gateway_Feature xGWFea;	   // 网关特征
	Sensor_Wave xSensWave;	   // 原始波形
	Sensor_Feature xSensFea;   // 波形特征
	unsigned long ulTimestamp; // 时间戳
} WIT_Data;

// UDP数据格式
typedef struct
{
	unsigned long ulWorkTime;
} WIT_UDPGWData;

typedef struct
{
	unsigned short usVersion;
	unsigned short usSensorTemp;
	unsigned long ulXVRMS;
	unsigned long ulYVRMS;
	unsigned long ulZVRMS;
	unsigned long ulXKur;
	unsigned long ulYKur;
	unsigned long ulZKur;
} WIT_UDPSensorData;

// 数据组记录
typedef struct
{
	bool bUsed;
	unsigned short usCount;
	unsigned long ulStartAddress;
	//	unsigned long 	ulCurrentAddress;
	//	unsigned long 	ulCount;
} DataListInfo;

#if CONFIG_W5100S_SPI_ETHERNET > 0

extern wiz_NetInfo gWIZNETINFO;
extern uint8_t ethBuf[ETH_MAX_BUF_SIZE];

bool sendmqtt = true;

#endif

#if CONFIG_ESP32_WIFI > 0
// WIFI 信息
typedef struct
{
	char cSSID[WIFI_MAX_NAME_LENGTH];	  // 当前SSID名称
	char cPassword[WIFI_MAX_NAME_LENGTH]; // 当前密码
	char cIP[WIFI_MAX_NAME_LENGTH];		  // 当前IP地址
	char cMAC[WIFI_MAX_NAME_LENGTH];	  // 当前MAC地址
	char cDHCP[WIFI_MAX_NAME_LENGTH];	  // 当前模式DHCP or STATIC
	char cMask[WIFI_MAX_NAME_LENGTH];	  // 当前子网掩码
	char cDNS[WIFI_MAX_NAME_LENGTH];	  // 当前DNS
	char cGateway[WIFI_MAX_NAME_LENGTH];  // 当前网关地址
} WIT_WIFIInfo;
#endif

// MQTT信息
typedef struct
{
	long lHandle;
	char cClientID[WIT_MQTT_MAX_NAME_LENGTH];
	// Pub topic
	char cPubDataTopic[WIT_MQTT_MAX_NAME_LENGTH];
	char cPubFeatureTopic[WIT_MQTT_MAX_NAME_LENGTH];
	char cPubUpdateResponseTopic[WIT_MQTT_MAX_NAME_LENGTH];
	char cPubCheckInTopic[WIT_MQTT_MAX_NAME_LENGTH];
	char cPubAlarmTopic[WIT_MQTT_MAX_NAME_LENGTH];
	char cPubConfigRespTopic[WIT_MQTT_MAX_NAME_LENGTH];
	char cPubWorkThrRespTopic[WIT_MQTT_MAX_NAME_LENGTH];
	char cPubSensorRespTopic[WIT_MQTT_MAX_NAME_LENGTH];
	char cPubPowerDownTopic[WIT_MQTT_MAX_NAME_LENGTH];
	char cPubThresholdTopic[WIT_MQTT_MAX_NAME_LENGTH];
	char cPubTimeRequestTopic[WIT_MQTT_MAX_NAME_LENGTH];
	char cPubIOTDataSendTopic[WIT_MQTT_MAX_NAME_LENGTH];
	char cPubIOTErrorSendTopic[WIT_MQTT_MAX_NAME_LENGTH];
#if CONFIG_BLUETOOTH > 0
	char cPubBleUpdateTopic[WIT_MQTT_MAX_NAME_LENGTH];
	char cPubBleUpdateRespTopic[WIT_MQTT_MAX_NAME_LENGTH];
	char cPubSenRegisterRespTopic[WIT_MQTT_MAX_NAME_LENGTH];
#endif
	char cPubErrorTopic[WIT_MQTT_MAX_NAME_LENGTH];
	char cPubRemoteCtrlResp[WIT_MQTT_MAX_NAME_LENGTH];
	char cPubExtConfigRespTopic[WIT_MQTT_MAX_NAME_LENGTH];
	char cPubVibAlarm[WIT_MQTT_MAX_NAME_LENGTH];
	char cPubBackDoorResp[WIT_MQTT_MAX_NAME_LENGTH];
	// Sub topic
	char cSubUpdateTopic[WIT_MQTT_MAX_NAME_LENGTH];
	char cSubConfigureTopic[WIT_MQTT_MAX_NAME_LENGTH];
	char cSubWorkDetectiveTopic[WIT_MQTT_MAX_NAME_LENGTH];
#if CONFIG_BLUETOOTH > 0
	char cSubBleRegisterTopic[WIT_MQTT_MAX_NAME_LENGTH];
	char cSubSensorUpdateTopic[WIT_MQTT_MAX_NAME_LENGTH];
#endif
	char cSubSyncTimeTopic[WIT_MQTT_MAX_NAME_LENGTH];
	char cSubTimeResponseTopic[WIT_MQTT_MAX_NAME_LENGTH];
	char cSubBackDoorTopic[WIT_MQTT_MAX_NAME_LENGTH];
	char cSubRemoteCtrl[WIT_MQTT_MAX_NAME_LENGTH];
	char cSubExtConfigureTopic[WIT_MQTT_MAX_NAME_LENGTH];
} WIT_MQTT;

// 传感器回复体
typedef struct
{
	unsigned char ucStatus;
	unsigned char ucMBData[SENSOR_MB_MAX_BUFFER_SIZE];
	unsigned char ucMBCnt;
} Sensor_Response;

// 网关回复体
typedef struct
{
	Response_Status eStatus;
	char cTaskID[24];
	char cAddr[4];
	char cBuffer[120];
	char cMessage[8];
	char *pcTopic;
	unsigned char ucCode;
} Gateway_Response;

// 故障告警
typedef struct
{
	Parameter xOtherChipError; // 其他芯片异常
} WIT_GWErrorParameter;

typedef struct
{
	Parameter xSensorExistError; // 传感器失联故障
	Parameter xShockChipError;	 // 振动芯片故障
	Parameter xNTCError;		 // NTC故障
	Parameter xOtherChipError;	 // 其他芯片故障
} WIT_SensorErrorParameter;

typedef struct
{
	unsigned char ucAlarmCount;
} AlarmListInfo;

// 掉电保存数据
typedef __attribute__((__aligned__(4))) struct
{
	unsigned long ulPackageID;
	unsigned long ulWorkMinutes;
	bool bSendingData;
	//	unsigned short	usLearningCount;		//废弃
	unsigned long ulConfirmValue;
} PVD_Data;
// -----------------------------------------------------------------------+
/* Private variables ---------------------------------------------------------*/
static WIT_Configure g_xWITConfig;
static WIT_Data g_xWITData;
static WIT_MQTT g_xWITMQTT;
extern dev_wave *gDevWave;
extern dev_feature *gDevFeature;

extern unsigned char *gMqttWaveData;
extern unsigned char *gMqttFeatureData;
#if CONFIG_BLUETOOTH
extern sSenInfo gSenInfo;
#endif
static TimerHandle_t g_xKeyTimer;
static TimerHandle_t g_xWorkTimer;
static TimerHandle_t g_xPushTimer;
static TimerHandle_t g_xDLICalTimer;
static TimerHandle_t g_xDataSendTimer;
static TimerHandle_t g_xErrorSendTimer;
static TimerHandle_t g_xCfgUpTimer;
static TimerHandle_t g_xSenConfigTimer;
static TimerHandle_t g_xThrUpTimer;
static TimerHandle_t g_xDLIUpTimer;
static TimerHandle_t g_xCfgQuickUpTimer;
static TimerHandle_t g_xGatewayFeatureTimer;
static TimerHandle_t g_xSensorFeatureTimer;
static TimerHandle_t g_xCheckInTimer;
static TimerHandle_t g_xWaveTimer;
static TimerHandle_t g_xAlarmTimer;
static bool g_bIsMotorWork[WIT_SENSOR_GROUP_MAX_NUM] = {false};
static bool g_bAlreadyCheckIn = false;
static unsigned char g_ucKeyTimes = 0;
#if CONFIG_RS485_RTU > 0
// 传感器特征使用标志
static unsigned short g_usBearingStatus = 0;
static unsigned short g_usGearStatus = 0;
static unsigned short g_usIsSetSpeed = 0;
// 传感器扩展特征使用标志
static unsigned short g_usExtInputStatus;
static unsigned short g_usExtOutputStatus;
static unsigned short g_usExtBearingStatus;
static unsigned short g_usExtGearStatus;
// 传感器风机和皮带使用标志
static unsigned short g_usFanStatus;
static unsigned short g_usBeltStatus;
static unsigned short g_usScrewStatus;
static unsigned short g_usMaleScrewStatus;
static unsigned short g_usFemaleScrewStatus;
static unsigned short g_usShaftStatus;
static unsigned short g_usSleeveStatus;
#endif
#if CONFIG_ESP32_WIFI > 0
static bool g_bGetSTAInfo = false;
static long g_lUDPSocketID = -1;
static unsigned char g_ucUDPSRespBuffer[WIT_UDPS_BUFFER_SIZE];
static WIT_WIFIInfo g_xCurrentWIFIInfo;
#endif
static PVD_Data g_xPVDData[6];
static Sensor_Response g_xSensorResponse = {0xFF, {0}, 0};
static Gateway_Response g_xGatewayResponse = {.eStatus = Resp_None};
static bool g_bDLISend = false;
static bool g_bDLICalculate = false;
static bool g_bDLIUpdated = false;
static bool g_bTimeSyncOver = false;
static bool g_bTimeSyncFlag = true;
static bool g_bUpdateConfigureArea = false;
static bool g_bSenorConfigArea = false;
static bool g_bUpdateThresholdArea = false;
static bool g_bUpdateDLIArea = false;
static bool g_bUpdateBackupArea = false;

bool g_bIsLowPower = true;

// Gateway and sensor parameter size
static unsigned long g_ulGatewayParameterSize = 0;
static unsigned long g_ulSensorParameterSize = 0;
static unsigned long g_ulTotalParameterSize = 0;

static unsigned long g_ulGatewayAlarmParameterSize = 0;
static unsigned long g_ulSensorAlarmParameterSize = 0;
static unsigned long g_ulTotalAlarmParameterSize = 0;

// Threshold pointer
static Threshold *g_pxGatewayThr = NULL;
static Threshold *g_pxSensorThr = NULL;
static Threshold *g_pxGlobalThr = NULL;

// Data list info
static DataListInfo *g_pxGatewayDLI = NULL;
static DataListInfo *g_pxSensorDLI = NULL;
static DataListInfo *g_pxGlobalDLI = NULL;

// Alarm list
static AlarmListInfo *g_pxGatewayALI = NULL;
static AlarmListInfo *g_pxSensorALI = NULL;
static AlarmListInfo *g_pxGlobalALI = NULL;

// Read index
bool g_bIsSensorExist[ACC_MAX_SENSOR_COUNT] = {false};

// Sensor semaphore
static SemaphoreHandle_t g_xMBSignal[WIT_MODBUS_PORT_COUNT];
static SemaphoreHandle_t g_xWITDataSignal;

// Error struct
static WIT_GWErrorParameter g_xGWErrParm;
static WIT_SensorErrorParameter g_xSrErrParm[ACC_MAX_SENSOR_COUNT];

// other info
static unsigned short g_usAccSamplingRate[48] = {0};
#if CONFIG_RS485_RTU > 0
	static unsigned short g_usSensorVersion;
	static unsigned long  g_ulSensorSerialNumber;
#endif
static unsigned long  g_ulUploadTheWaveform;
static unsigned long  g_ulUploadRunState;
#if CONFIG_BLUETOOTH > 0
static char g_cSensorSerialNumber[25][40];
static char g_cSensorType[25][20];
static char g_cSensorChildVersion[25][20];
static char g_cSensorLogoType[25][8];
static char g_cSensorVersion[25][8];
#endif
// Modbus Alarm
static unsigned char *g_pucMBAlarm;
static unsigned char *g_pucMBSensorAlarm;

#if CONFIG_ESP32_WIFI > 0
// UDP data
static WIT_UDPGWData g_xUDPGWData;
static WIT_UDPSensorData g_xUDPSensorData[ACC_MAX_SENSOR_COUNT];
#endif

// Modbus sensor id
static unsigned short g_usMBFeatureSensorID = 1;

#if CONFIG_EC200S_USB_4G > 0
static char g_cCCIDBuffer[24];
#endif

// 配置参数检测列表
unsigned long *g_pulConfigNumberPointerList[] =
	{
		&g_xWITConfig.ulModuleNameValue,
		&g_xWITConfig.ulConfigureValue,
		&g_xWITConfig.ulWIFIAndThrValue,
		&g_xWITConfig.ulMQTTInfoValue,
		&g_xWITConfig.ulLearningInfoValue,
		&g_xWITConfig.ulDelayInfoValue,
		&g_xWITConfig.ulCopyValueInfo
#if CONFIG_BLUETOOTH > 0
		,
		&g_xWITConfig.ulSenParmInfoValue
#endif
#if CONFIG_W5100S_SPI_ETHERNET > 0
        ,
		&g_xWITConfig.ulEthNetInfoValue
#endif
};

static const char *const g_pcImportantParameterName[] =
	{
		"Temp",
		"VRMS",
		"CrestFactor",
		"gSE"};

bool g_bCanSendGatewayFeature = true;
bool g_bCanSendSensorFeature = true;
bool g_bCanSendWave = true;

static bool g_bCanSendAlarm = true;
static bool g_bGWFeaHeart = true;
static bool g_bHeartPack[WIT_SENSOR_GROUP_MAX_NUM] = {true};
static bool g_bModuleError = false;

typedef enum
{
	SEL_X,
	SEL_Y,
	SEL_Z,
	SEL_MAX
} MotorActualSel;

#define SENSOR_NUM_MAX 6

speedSyncFSM spdSyncFSM = {
	.enable = 0,
	.mpsAddr = 1,
	.MotorARRate = 1,
	.state = 0};

static bool g_bRemoteCtrl = false;
static unsigned int cmdValue = 0x99;

/* Private functions ---------------------------------------------------------*/
static void WIT_LEDTask(void *pvParameter);
static void WIT_KeyTask(void *pvParameter);
static void WIT_EventPollTask(void *pvParameter);
#if CONFIG_ESP32_WIFI > 0
static void WIT_UDPPollTask(void *pvParameter);
static void WIT_WIFIReconnectTask(void *pvParameter);
#endif
static void WIT_ModbusSlavePollTask(void *pvParameter);
static void WIT_FeatureNameInit(void);
static void WIT_ModeAndValueInit(void);
static void WIT_ReadThrAndDLI(void);
static void WIT_MQTTInit(void);
static bool WIT_MQTTConnect(void);
static void WIT_MQTTConnectHandler(unsigned long ulMQTTID, bool bStatus);
static bool WIT_CheckIn(void);
static bool WIT_TimeRequest(void);
static bool WIT_SendGatewayFeatureToServer(unsigned char groupId, void (*pFuncHandler)(unsigned char, bool));
static bool WIT_SendSensorFeatureToServer(unsigned char groupId, unsigned char ucSensorAddr, void (*pFuncHandler)(unsigned char, bool));
static bool WIT_SendGatewayAlarm(unsigned char groupId, void (*pFuncHandler)(unsigned char, bool));
static bool WIT_AlarmDetective(unsigned char ucSensorAddr, Parameter *pxParm, Threshold *pxThr, unsigned char *pucMBAlarm, unsigned long ulParmCnt, bool bIsWork, bool bPeriodTimeout);
static unsigned long WIT_PackageFeatureIntoExtMemory(unsigned char groupId, unsigned char ucSensorAddr, Parameter *pxParm, unsigned long ulParmCnt, unsigned long *pulPackageID, unsigned long ulTimeStamp, unsigned long ulWorkTime,
													 void (*pFuncMemoryWrite)(unsigned long, unsigned char *, unsigned long), unsigned long ulMemoryWriteAddress);
static unsigned long WIT_PackageWaveDataIntoExtMemory(unsigned char ucSensorAddr, Sensor_Wave *pxWave, unsigned long ulPackageID, unsigned long ulTimeStamp,
													  void (*pFuncMemoryRead)(unsigned int, unsigned long, unsigned char *, unsigned long), void (*pFuncMemoryWrite)(unsigned long, unsigned char *, unsigned long),
													  unsigned long ulMemoryReadAddress, unsigned long ulMemoryWriteAddress);
static unsigned long WIT_PackageAlarmIntoExtMemory(unsigned char ucSensorAddr, Parameter *pxParm, unsigned long ulParmCnt, unsigned long *pulPackageID, unsigned long ulTimeStamp,
												   void (*pFuncMemoryWrite)(unsigned long, unsigned char *, unsigned long), unsigned long ulMemoryWriteAddress);
#if CONFIG_RS485_RTU > 0
static unsigned long WIT_PackageThresholdIntoExtMemory(unsigned char ucSensorAddr, Parameter *pxParm, Threshold *pxThr, DataListInfo *pxDLI, unsigned long ulParmCnt, unsigned long ulTimeStamp,
													   void (*pFuncMemoryWrite)(unsigned long, unsigned char *, unsigned long), unsigned long ulMemoryWriteAddress);
#endif
static void WIT_TimerHandler(TimerHandle_t xTimer);
static void WIT_GatewayUpdateHandler(MessageData *pxData);
#if CONFIG_BLUETOOTH > 0
static void WIT_BleSensorUpdateHandler(MessageData *pxData);
static void WIT_FeatureOperation(unsigned int uiAddr);
#endif
#if CONFIG_RS485_RTU > 0
static void WIT_SensorUpdateHandler(MessageData *pxData);
#endif
static void WIT_GatewayConfigureHandler(MessageData *pxData);
static void WIT_GatewayWorkDetectiveHandler(MessageData *pxData);
static void WIT_GatewaySyncTimeHandler(MessageData *pxData);
static void WIT_GatewayBackDoorHandler(MessageData *pxData);
static void WIT_GatewayRemoteCtrlHandler(MessageData *pxData);
#if CONFIG_BLUETOOTH > 0
static sGroupTable *getGroupFromGroupNo(char *monitorNo);
static void WIT_GatewaySenRegisterHandler(MessageData *pxData);
#endif
static void WIT_GatewayExtConfigHandler(MessageData *pxData);
#if CONFIG_ESP32_WIFI > 0
static void WIT_UDPSRequestHandler(unsigned char ucCommand, unsigned char *pucData, unsigned short usLen);
#endif
static void WIT_UpdateConfigureArea(void);
static void WIT_UpdateThresholdArea(void);
static void WIT_UpdateDLIArea(void);
#if CONFIG_RS485_RTU > 0
static void WIT_RestoreGWDataIntoExtFlash(void);
static void WIT_RestoreSensorDataIntoExtFlash(unsigned char ucSensorAddr);
#endif
static void WIT_CalculateThreshold(void);
#if CONFIG_RS485_RTU > 0
static bool WIT_SendThreshold(void);
// Sensor operation
static bool WIT_SensorOperation(unsigned char ucSensorAddr, void *pvOp, ...);
#endif
// Others
static void WIT_IncreasePackageIDAndBackUp(unsigned char groupId);
#if CONFIG_RS485_RTU > 0
static void WIT_CheckParameterIsUsed(void);
#endif
static void WIT_MQTTPublishHandler(unsigned char groupId, bool bStatus);
#if CONFIG_RS485_RTU > 0
static void WIT_IsSensorExist(void);
#endif
static bool WIT_SendErrorDataIfOccured(void);
#if CONFIG_ESP32_WIFI > 0
static void WIT_UDPStoreGWData(void);
static void WIT_UDPStoreSensorData(unsigned char ucSensorAddr, unsigned short usVersion);
#endif
static void WIT_AccEnable(unsigned char ucSensorAddr, unsigned char ucSensorID);
static unsigned char WIT_GetConfigureNumberCount(void);
static void WIT_PreTaskInit(void);
static bool WIT_IsNetBusy(void);
static bool WIT_IsImportantParameter(char *pcName);
static unsigned short WIT_GetFeatureMinLearnCount(void);
#if CONFIG_RS485_RTU > 0
static void WIT_UpdateFeatureName(unsigned char ucSensorAddr);
static void WIT_RS485Check(void);
#endif
static bool WIT_CheckMotorAddr(void);

static bool gErrorChanged = false;
static bool gGatewayAlarm = false, gSensorAlarm = false;

#if CONFIG_RS485_RTU > 0
static unsigned int lastActiveFlag = 0;
#endif

// my_iot function
void iot_Sensor_Init(void);
void Message_Deal_Start_Task(void);
void tcp_server_task_start(void);
bool WIT_SendIOTData(void);
bool WIT_SendIOTError(void);
bool WIT_SendGatewayFeature(void);
bool IOTConfigFunction();
void iot_Sensor_Print(void);

// 0: don't send feature
// 1: need send feature
// 2: need clear feature timer
// 3: need clear data and feature timer

static unsigned int amlVibAlarmAddr = 0;

static unsigned int ExConfigResponeFlag = 0;
static unsigned int ExConfigrespCode = 0;
static char ExConfigRespBuffer[32] = {0};

static bool g_bBackDoorResponse = false;
static bool g_bBackDoorReset = false;
static unsigned int g_bBackDoorCode = 0;
static char BackDoorRespBuffer[32] = {0};

/**
 * @brief  WIT Init
 * @param  None
 * @retval None
 */
void WIT_Init()
{
	// mqtt_update_init();
	pwr_init();
	vTaskDelay(100);
	// Init drivers
#if CONFIG_RS485_RTU > 0
	Acc_PinInit();
#endif
	flash_semphr_init();
	ram_init();
	LED_Init();
	Key_Init();
	pcf8563_init(I2C_NUM_0);
#if CONFIG_ESP32_WIFI > 0
	pwr_set(true, false);
#elif CONFIG_W5100S_SPI_ETHERNET > 0
	pwr_set(true, false);
	vTaskDelay(20);
#elif CONFIG_EC200S_USB_4G > 0
	pwr_set(true, true);
	vTaskDelay(20);
	Celluar_Init();
	Celluar_ModuleInit(&g_bModuleError);
#endif
	// Feature name init
	WIT_FeatureNameInit();
	// 从外部flash读取阈值以及数据存储配置
	WIT_ReadThrAndDLI();
	// Init mode and value
	// 从内部flash读取配置并进行配置
	WIT_ModeAndValueInit();
    //iot_Sensor_Print();
    // for (int i = 0; i < IOTDEV_MAX_NUM; i++)
    // {
    //     printf("my_IOTDev[%d] id_sn = %04x,%04x\n",i, g_xWITConfig.my_IOTDev[i].id, g_xWITConfig.my_IOTDev[i].sn);
    // }
    printf("threshold = %ld\n", g_xWITConfig.my_IOTThreshold);
    printf("period = %ld\n", g_xWITConfig.ulPeriod);


	if_sensor_config_init();
	g_xWITConfig.eGatewayMode = 1;

	// Config check
	if (g_xWITConfig.spdSel > 3)
	{
		g_xWITConfig.spdSel = 3;
	}

	// RS485 param check
#if CONFIG_RS485_RTU > 0
	WIT_RS485Check();
	// Check Sensor
	// 设置功能脚模式
	for (size_t i = 1; i <= ACC_MAX_SENSOR_COUNT; i++)
	{
		// 设置功能脚为modbus响应模式
		WIT_SensorOperation(i, Acc_SetPinFunctionToMB, WIT_ACC_SENSOR_ID);
		// 设置传感器模式为监控模式
		WIT_SensorOperation(i, Acc_SetMode, WIT_ACC_SENSOR_ID, SensorMode_Monitor);
	}
	vTaskDelay(10);
	WIT_IsSensorExist();

	vGatewayInit();
	sensorInit();
#endif
	// connect to server only in STA mode
	WIT_MQTTConnect();
    vTaskDelay(pdMS_TO_TICKS(100));
    tcp_server_task_start();
    vTaskDelay(pdMS_TO_TICKS(100));
    Message_Deal_Start_Task();
    
	// Check has fan monitor point
	if (g_xWITConfig.spdSyncEnable != 0)
	{
		if (WIT_CheckMotorAddr() == true)
		{
			spdSyncFSM.enable = true;
		}
	}
	else
	{
		spdSyncFSM.enable = false;
	}

	// create wifi task
#if CONFIG_ESP32_WIFI > 0
	xTaskCreate(WIT_UDPPollTask, "UDP Task", WIT_UDP_POLL_TASK_STK, NULL, WIT_UDP_POLL_TASK_PRIORITY, NULL);
	xTaskCreate(WIT_WIFIReconnectTask, "WIFI Task", WIT_WIFI_RECONNECT_TASK_STK, NULL, WIT_WIFI_RECONNECT_TASK_PRIORITY, NULL);
#endif
#if CONFIG_BLUETOOTH > 0
	// ble_client_init();
#endif
}

bool SensorCheckAmlVib(unsigned int *amlVibSensorAddr)
{
	for (size_t i = 0; i < ACC_MAX_SENSOR_COUNT; i++)
	{
		if (sensorAvailableGet(i) != true)
		{
			continue;
		}
#if CONFIG_RS485_RTU > 0
		unsigned short amlVib = 0;
		if (WIT_SensorOperation(i + 1, Acc_GetAmlVibAlarm, WIT_ACC_SENSOR_ID, &amlVib) == true)
		{
			if (amlVib == 1)
			{
				*amlVibSensorAddr = i + 1;
				return true;
			}
		}
#endif
	}

	return false;
}

bool SensorStatusSetIdle(unsigned int sensorAddr)
{
	unsigned char ucRetry = 5;
	unsigned short usReadData = 0;
	WIT_AccEnable(sensorAddr, WIT_ACC_SENSOR_ID);
	// read feature let sensor status to IDLE
	while (ucRetry > 0 && (eMBReadHoldingRegister(
							   ACC_ADDR_TO_PORT(sensorAddr),
							   WIT_ACC_SENSOR_ID,
							   0x03E8,
							   0x01,
							   &usReadData) != MB_EMASTERNOERR))
	{
		ucRetry--;
	}

	if (ucRetry > 0)
	{
		return true;
	}
	return false;
}

bool SensorSetCalFea(unsigned short isCal)
{
	bool ret = true;

	for (size_t i = 0; i < ACC_MAX_SENSOR_COUNT; i++)
	{
		if (sensorAvailableGet(i) != true)
		{
			continue;
		}
#if CONFIG_RS485_RTU > 0
		if (WIT_SensorOperation(i + 1, Acc_SetCalFea, WIT_ACC_SENSOR_ID, isCal) != true)
		{
			ret = false;
		}
#endif
	}
	return ret;
}

bool DeviceRunCheck(unsigned int sensorAddr)
{
#if CONFIG_RS485_RTU > 0
	unsigned int sensorIdx = sensorAddr - 1;
	// Check active flag
	unsigned short ActiveFlag = 0;
	if (WIT_SensorOperation(sensorAddr, Acc_GetActiveFlag, WIT_ACC_SENSOR_ID, &ActiveFlag) == true)
	{
		// 设备存在
		if (g_xSrErrParm[sensorIdx].xSensorExistError.bIsErrorOccured == true)
		{
			g_xSrErrParm[sensorIdx].xSensorExistError.ucErrorOccuredCount = 0;
			g_xSrErrParm[sensorIdx].xSensorExistError.bIsErrorOccured = false;
			g_xSrErrParm[sensorIdx].xSensorExistError.bIsErrorCleared = true;
			gErrorChanged = true;
		}

		if (ActiveFlag == true)
		{
			if (lastActiveFlag == 0)
				vTaskDelay(g_xWITConfig.ulDelayTime);
			lastActiveFlag = 1;
			return true;
		}
		else
		{
			lastActiveFlag = 0;
		}
	}
	else
	{
		if (g_xSrErrParm[sensorIdx].xSensorExistError.bIsErrorOccured == false)
		{
			// 设备不存在
			g_xSrErrParm[sensorIdx].xSensorExistError.ucErrorOccuredCount++;
			if (g_xSrErrParm[sensorIdx].xSensorExistError.ucErrorOccuredCount >= 100)
			{
				g_xSrErrParm[sensorIdx].xSensorExistError.bIsErrorOccured = true;
				gErrorChanged = true;
			}
		}
	}
#endif
	return false;
}

bool WIT_SendAmlVibAlarm(unsigned int sensorAddr)
{
	// publish anomaly vibration interval
	char s[64] = {0};
	sprintf(s, "[{\"addr\":\"%u\",\"id\":%lu,\"ts\":%lu,\"vrmsAlarm\":%d}]",
			sensorAddr,
			g_xWITConfig.ulPackageID,
			g_xWITData.ulTimestamp,
			1);
	if (MQTT_Publish(g_xWITMQTT.lHandle, g_xWITMQTT.cPubVibAlarm, (unsigned char *)s, strlen(s), false, false, QOS1))
	{
		return true;
	}

	return false;
}

void DeviceAlarmCheck(unsigned char groupId, unsigned int sensorAddr)
{
	g_bIsMotorWork[groupId] = false;
	if (g_bGWFeaHeart == true && g_bCanSendGatewayFeature == true && WIT_IsNetBusy() == false)
	{
		g_ulUploadRunState = WIT_FALSE;
		if (WIT_SendGatewayFeatureToServer(groupId, NULL) == true)
		{
			g_bGWFeaHeart = false;
		}
	}
	else
	{
		g_bGWFeaHeart = false;
	}

	// 上报网关预警
	if (gGatewayAlarm == true && g_bCanSendAlarm == true && g_xWITConfig.eGatewayMode == GatewayMode_Normal && WIT_IsNetBusy() == false)
	{
		if (WIT_SendGatewayAlarm(groupId, NULL) == true)
		{
			g_bCanSendAlarm = false;
			xTimerStart(g_xAlarmTimer, 1000);
		}
	}
}

bool ErrorStatusSend(unsigned int sensorAddr, unsigned int uiCode, unsigned long ts)
{
	bool bStatus = true;

	cJSON *pxRoot = NULL;
	cJSON *pxObj = NULL;
	unsigned char *pucSendBuffer = NULL;
	unsigned long ulBufferSize = 0;
	cJSON_bool bPackageStatus;
	char cSensorIDStr[4] = {0};

	pucSendBuffer = MQTT_SendBufferPointer(g_xWITMQTT.lHandle);
	ulBufferSize = MQTT_GetRestSendBufferSize(g_xWITMQTT.lHandle);

	if(WIT_IsNetBusy() == false)
	{
		pxRoot = cJSON_CreateArray();
		pxObj = cJSON_CreateObject();
		sprintf(cSensorIDStr, "%d", sensorAddr);
		cJSON_AddStringToObject(pxObj, "addr", cSensorIDStr);
		cJSON_AddNumberToObject(pxObj, "ts", ts);		
		cJSON_AddNumberToObject(pxObj, "SenCode", uiCode);
		
		cJSON_AddItemToArray(pxRoot, pxObj);
		bPackageStatus = cJSON_PrintPreallocated(pxRoot, (char *)pucSendBuffer, ulBufferSize, false);
		if (pxRoot != NULL)
		{
			cJSON_Delete(pxRoot);
			pxRoot = NULL;
		}

		if (bPackageStatus == false || MQTT_Publish(g_xWITMQTT.lHandle, g_xWITMQTT.cPubErrorTopic, pucSendBuffer, strlen((char *)pucSendBuffer), false, false, QOS1) != true)
		{
			bStatus = false;
		}	
		
	}
	return bStatus;
}

void ErrorSend(unsigned int sensorAddr)
{
	// 错误状态变化上发
	if (gErrorChanged == true && WIT_IsNetBusy() == false)
	{
		if (WIT_SendErrorDataIfOccured() == true)
		{
			gErrorChanged = false;
		}
	}
}

#if CONFIG_RS485_RTU > 0
void ReadSensorFeat(unsigned int sensorAddr)
{
	Parameter *pxSensorFeaPointer = (Parameter *)&g_xWITData.xSensFea;

	// clear sensor feature value
	for (int k = 0; k < g_ulSensorParameterSize; k++)
	{
		pxSensorFeaPointer->fValue = 0;
		pxSensorFeaPointer++;
	}

	// Get Feature
	WIT_SensorOperation(sensorAddr, Acc_GetFeature, WIT_ACC_SENSOR_ID,
						&g_xWITData.xSensFea, &g_usBearingStatus, &g_usGearStatus, &g_usIsSetSpeed,
						&g_usFanStatus, &g_usBeltStatus, &g_usScrewStatus, &g_usShaftStatus, &g_usMaleScrewStatus, &g_usFemaleScrewStatus, &g_usSleeveStatus,
						&g_xWITData.xSensFea.odr, &g_xWITData.xSensFea.sampSum, &g_xWITData.xSensFea.hpfEn, &g_xWITData.xSensFea.resolution);
	// Get Extend Feature
	WIT_SensorOperation(sensorAddr, Acc_GetExtendFeature, WIT_ACC_SENSOR_ID,
						&g_xWITData.xSensFea, &g_usExtBearingStatus, &g_usExtGearStatus, &g_usExtInputStatus, &g_usExtOutputStatus);
	// Get Actual Speed
	WIT_SensorOperation(sensorAddr, Acc_GetPredictedSpeed, WIT_ACC_SENSOR_ID, &g_xWITData.xSensFea);
	// get version and sampling rate
	WIT_SensorOperation(sensorAddr, Acc_GetVersion, WIT_ACC_SENSOR_ID, &g_usSensorVersion);
	WIT_SensorOperation(sensorAddr, Acc_GetSamplingRate, WIT_ACC_SENSOR_ID, g_usAccSamplingRate + (sensorAddr - 1));
	WIT_SensorOperation(sensorAddr, Acc_GetSerialNumber, WIT_ACC_SENSOR_ID, &g_ulSensorSerialNumber);

	// Store Feature to SRAM
	// SRAM_Write(sensorAddr, (unsigned char *)&g_xWITData.xSensFea, sizeof(Sensor_Feature));

	unsigned short ucRetry = 5;
	unsigned short usReadData = 0;
	WIT_AccEnable(sensorAddr, WIT_ACC_SENSOR_ID);
	while (ucRetry > 0 && (eMBReadHoldingRegister(ACC_ADDR_TO_PORT(sensorAddr), WIT_ACC_SENSOR_ID, 0x006E, 0x01, &usReadData) != MB_EMASTERNOERR))
	{
		ucRetry--;
	}
	// Store wave count
	g_xWITData.xSensWave.xWave.ulWaveCount[sensorAddr - 1] = usReadData;

#if CONFIG_ESP32_WIFI > 0
	WIT_UDPStoreSensorData(sensorAddr, g_usSensorVersion);
#endif
	// Check Parameter used status
	WIT_CheckParameterIsUsed();

	// 若为判断开机的Sensor地址，则进行电机是否运行的判断(废除)
	// 进入这里说明传感器已经过阈值振动
	g_bIsMotorWork[0] = true;

	// 工作中 存储网关特征
	if (g_bIsMotorWork[0] == true && g_xWITConfig.eGatewayMode == GatewayMode_Learning)
	{
		WIT_RestoreGWDataIntoExtFlash();
	}
	// 工作中 存储传感器特征
	if (g_bIsMotorWork[0] == true && g_xWITConfig.eGatewayMode == GatewayMode_Learning)
	{
		WIT_RestoreSensorDataIntoExtFlash(sensorAddr);
	}
}

void ReadSensorWave(unsigned int sensorAddr)
{
	// Read wave and feature
	WIT_SensorOperation(sensorAddr,
						Acc_GetWaveIntoExtMemory, WIT_ACC_SENSOR_ID, &g_xWITData.xSensWave, write_ram_wave, 0);
}
#endif

void AlarmThrCheck(unsigned int sensorAddr)
{
	// Sensor feature alarm detective
	Threshold *pxCurrentThr = g_pxSensorThr + (sensorAddr - 1) * g_ulSensorParameterSize;
	gSensorAlarm = WIT_AlarmDetective(sensorAddr,
									  (Parameter *)&g_xWITData.xSensFea,
									  pxCurrentThr,
									  g_pucMBSensorAlarm,
									  g_ulSensorParameterSize,
									  g_bIsMotorWork[0],
									  true);
}

void LearnThr(void)
{
	// 学习模式中
	if (g_xWITConfig.eGatewayMode != GatewayMode_Learning)
	{
		return;
	}
	// 获取当前学习数量
	g_xWITConfig.usCurrentLearningCount = WIT_GetFeatureMinLearnCount();

	// 计算阈值时间到或者学习完成 进行阈值更新操作
	if (g_bDLICalculate == true || g_xWITConfig.usCurrentLearningCount >= g_xWITConfig.usTotalLearningCount)
	{
		if (g_bDLIUpdated == true)
		{
			// 计算每一个特征阈值并更新
			WIT_CalculateThreshold();
			if (g_xWITConfig.bAutoThreshold == false)
			{
				// 自设定阈值标志启用
				g_xWITConfig.bAutoThreshold = true;
				xTimerStart(g_xCfgUpTimer, 1000);
			}
			// 更新阈值
			// 更新配置信息区域
			xTimerStart(g_xThrUpTimer, 1000);
			xTimerStart(g_xDLIUpTimer, 1000);
			// 清除标志
			g_bDLIUpdated = false;
		}

		g_bDLICalculate = false;
	}

	// 如果学习完成变为正常模式
	if (g_xWITConfig.usCurrentLearningCount >= g_xWITConfig.usTotalLearningCount)
	{
		// 强制发送最后一组学习数据
		g_bDLISend = true;
		// 切换为正常模式 时间周期修改
		g_xWITConfig.eGatewayMode = GatewayMode_Normal;
		// 切换为正常模式上发周期
		g_xWITConfig.ulPeriod = g_xWITConfig.ulFeaturePeriodCopy;
		g_xWITConfig.ulWavePeriod = g_xWITConfig.ulWavePeriodCopy;

		// 重启数据定时器
		xTimerChangePeriod(g_xGatewayFeatureTimer, g_xWITConfig.ulPeriod, 1000);
		xTimerChangePeriod(g_xSensorFeatureTimer, g_xWITConfig.ulPeriod, 1000);
		xTimerChangePeriod(g_xAlarmTimer, g_xWITConfig.ulPeriod, 1000);
		xTimerChangePeriod(g_xWaveTimer, g_xWITConfig.ulWavePeriod, 1000);

		// 写入配置
		xTimerStart(g_xCfgUpTimer, 1000);
	}
}

void WIT_Run()
{
	if (g_bAlreadyCheckIn == false && WIT_IsNetBusy() == false)
	{
#if CONFIG_RS485_RTU > 0
		WIT_IsSensorExist();
#endif
		if (WIT_CheckIn() == true)
		{
			xTimerStart(g_xCheckInTimer, 1000);
			g_bAlreadyCheckIn = true;
		}
		g_bTimeSyncOver = false;
	}
	// Time sync
	if (g_bTimeSyncFlag == true)
	{
		g_bTimeSyncFlag = false;
		g_bTimeSyncOver = false;
		xTimerChangePeriod(g_xGatewayFeatureTimer, (5 * 60 * 1000), 1000);
	}
	if (g_bTimeSyncOver == false && WIT_IsNetBusy() == false)
	{
		if (WIT_TimeRequest() == true)
		{
			vTaskDelay(1000);
		}
	}

    //Send IOT Data
    if (g_bDataSend_TimerArr == true && g_DataUpdateFlag == true) // The threshold control is missing.
	{
		if ((WIT_SendIOTData() == true) &&  (WIT_SendGatewayFeature() == true))
		{
            printf("Data Send success\n");
            g_DataUpdateFlag = false;
			g_bDataSend_TimerArr = false;
		}
        else{
            // printf("Send fail\n");
        }
	}

    //Send Error Data
    if (g_bErrorSend_TimerArr == true &&  g_bErrorUpdateFlag == true)
	{
		if (WIT_SendIOTError() == true )
		{
            printf("Error Send success\n");
            g_bErrorSend_TimerArr = false;
            g_bErrorUpdateFlag = false;
		}
        else{
            printf("Send fail\n");
        }
	}

}

/**
 * @brief  WIT LED task
 * @param  None
 * @retval None
 */
static void WIT_LEDTask(void *pvParameter)
{
	while (1)
	{
		if (g_bModuleError == false)
		{
#if CONFIG_ESP32_WIFI > 0
			if (g_xWITConfig.eOpMode == WIFI_STA)
			{
				if (MQTT_GetStatus(g_xWITMQTT.lHandle) == true)
				{
					LED_SetSTAStatus(true);
					vTaskDelay(1000 / portTICK_PERIOD_MS);
				}
				else if (WIFI_IsConnected() == true)
				{
					LED_STAToggle();
					vTaskDelay(200 / portTICK_PERIOD_MS);
				}
				else
				{
					LED_STAToggle();
					vTaskDelay(500 / portTICK_PERIOD_MS);
				}
			}
			else
			{
				LED_SetSTAStatus(false);
				if (g_ucKeyTimes == 0)
					LED_SetCOMStatus(true);
				vTaskDelay(1000 / portTICK_PERIOD_MS);
			}
#elif CONFIG_EC200S_USB_4G > 0
			if (MQTT_GetStatus(g_xWITMQTT.lHandle) == true)
			{
				// 连接MQTT成功
				LED_SetSTAStatus(true);
				vTaskDelay(1000 / portTICK_PERIOD_MS);
			}
			else if (Celluar_GetSocketStatus(g_xWITMQTT.lHandle) == true)
			{
				// 有网络 未连接MQTT
				LED_STAToggle();
				vTaskDelay(200 / portTICK_PERIOD_MS);
			}
			else if (Celluar_HasSIMCard() == true)
			{
				LED_STAToggle();
				vTaskDelay(500 / portTICK_PERIOD_MS);
			}
			else
			{
				// 无卡或其他
				LED_STAToggle();
				vTaskDelay(1000 / portTICK_PERIOD_MS);
			}
#elif CONFIG_W5100S_SPI_ETHERNET > 0
			if (MQTT_GetStatus(g_xWITMQTT.lHandle) == true)
			{
				// 连接MQTT成功
				LED_SetSTAStatus(true);
				vTaskDelay(1000 / portTICK_PERIOD_MS);
			}
			else if (ETHERNET_GetSocketStatus(g_xWITMQTT.lHandle) == true)
			{
				// 有网络 未连接MQTT
				LED_STAToggle();
				vTaskDelay(200 / portTICK_PERIOD_MS);
			}
			else
			{
				// 未入网
				LED_STAToggle();
				vTaskDelay(1000 / portTICK_PERIOD_MS);
			}
#endif
		}
		else
		{
			static bool bLEDStatus = false;
			bLEDStatus = (bool)!bLEDStatus;
			LED_SetSTAStatus(bLEDStatus);
			LED_SetCOMStatus(bLEDStatus);
			vTaskDelay(200 / portTICK_PERIOD_MS);
		}
	}
}

static void __Resetting_led(void)
{
	for(int i = 4; i <= 6; i++)
		gpio_set_level(i, 0);
	vTaskDelay(500);
	for(int i = 4; i <= 6; i++)
		gpio_set_level(i, 1);
	vTaskDelay(500);
	for(int i = 4; i <= 6; i++)
		gpio_set_level(i, 0);
	vTaskDelay(500);
	for(int i = 4; i <= 6; i++)
		gpio_set_level(i, 1);
	vTaskDelay(500);
}

static void WIT_KeyTask(void *pvParameter)
{
	#if CONFIG_W5100S_SPI_ETHERNET > 0
		char resetting_mac[] = {0x00, 0x08, 0xdc, 0x6f, 0x00, 0x88};
		char resetting_ip[] = {192, 168, 1, 1};
		char resetting_sn[] = {255, 255, 255, 0};
		char resetting_gw[] = {192, 168, 1, 1};
		bool resetting_dhcp = NETINFO_STATIC;
	#endif
	while (1)
	{  
		if (Key_Read() == false)
		{		
			if (g_xKeyTimer != NULL && xTimerIsTimerActive(g_xKeyTimer) == pdFALSE)
			{
				xTimerStart(g_xKeyTimer, 1000);
			}
		}
		else
		{
			if (g_xKeyTimer != NULL && xTimerIsTimerActive(g_xKeyTimer) == pdTRUE)
				xTimerStop(g_xKeyTimer, 1000);

			if (g_ucKeyTimes == 1)
			{
#if CONFIG_ESP32_WIFI > 0
				g_xWITConfig.eOpMode = WIFI_FORCE_AP;
				sprintf(g_xWITConfig.cAPName, "Witium-%08lu", g_xWITConfig.ulSerialNumber);
				WIT_UpdateConfigureArea();
				WIFI_ResetByPin();
				PWR_Restart();
#endif
			}
			else if (g_ucKeyTimes > 1 && g_ucKeyTimes < 3)
			{

				// Reset Modbus RTU param
				g_xWITConfig.ucModbusID = 1;
				g_xWITConfig.ulModbusBaudRate = 9600;
				g_xWITConfig.eModbusParity = MB_PAR_NONE;
				
				WIT_UpdateConfigureArea();
			}
			else if (g_ucKeyTimes >= 3)
			{
				#if CONFIG_W5100S_SPI_ETHERNET > 0	
					for (int i = 0; i < 4; i++)
					{
						g_xWITConfig.g_xConfigIp.ip[i] = resetting_ip[i];
						g_xWITConfig.g_xConfigIp.gw[i] = resetting_gw[i];
						g_xWITConfig.g_xConfigIp.sn[i] = resetting_sn[i];
					}
					for (int i = 0; i < 6; i++)
					{
						g_xWITConfig.g_xConfigIp.mac[i] = resetting_mac[i];
					}
					g_xWITConfig.g_xConfigIp.dhcp = resetting_dhcp;
				#endif
					g_xWITConfig.ulPort = 1883;
					g_xWITConfig.cTargetIP[0] = 116;
					g_xWITConfig.cTargetIP[1] = 62;
					g_xWITConfig.cTargetIP[2] = 127;
					g_xWITConfig.cTargetIP[3] = 242;
					WIT_UpdateConfigureArea();
				
				esp_restart(); // 软件复位
			}

			g_ucKeyTimes = 0;
		}

		if (g_ucKeyTimes == 1)
		{
#if CONFIG_ESP32_WIFI > 0
			// 200 + 200 ms toggle
			LED_COMToggle();
			vTaskDelay(200);
#endif
		}
		else if (g_ucKeyTimes > 1)
		{
			LED_COMToggle();
			vTaskDelay(50);
		}

		vTaskDelay(100);
	}
}

static void WIT_EventPollTask(void *pvParameter)
{
	while (1)
	{
		// 网关更新事件
		if (Update_GetStatus() == true)
		{
			unsigned char ucUpdateID;
			Update_ResponseStatus eStatus;
			char cUpdateResponse[24];

			eStatus = Update_Poll(&ucUpdateID);

			switch (eStatus)
			{
			case Response_Null:
				break;
			case Response_Success:
			case Response_Reset:
				sprintf(cUpdateResponse, "%d %08lu %d", ucUpdateID, g_xWITConfig.ulSerialNumber, 0);
				break;
			case Response_Error:
				sprintf(cUpdateResponse, "%d %08lu %d", ucUpdateID, g_xWITConfig.ulSerialNumber, 1);
				break;
			}

			if (eStatus != Response_Null)
			{
				if (MQTT_Publish(g_xWITMQTT.lHandle, g_xWITMQTT.cPubUpdateResponseTopic, (unsigned char *)cUpdateResponse, strlen(cUpdateResponse), false, false, QOS1))
				{
				}
			}

			if (eStatus == Response_Reset)
			{
				esp_restart(); // 软件复位
			}
		}
#if CONFIG_BLUETOOTH > 0
		// 蓝牙传感器更新
		if (Sensor_GetUpdateStatus() == true)
		{
			unsigned char ucUpdateID;
			Update_ResponseStatus eStatus;
			char cUpdateResponse[24];

			eStatus = BLE_Update_Poll(&ucUpdateID);

			switch (eStatus)
			{
			case Response_Null:
				break;
			case Response_Success:
				sprintf(cUpdateResponse, "%d %08lu %d", ucUpdateID, g_xWITConfig.ulSerialNumber, 0);
				break;
			case Response_Error:
				sprintf(cUpdateResponse, "%d %08lu %d", ucUpdateID, g_xWITConfig.ulSerialNumber, 1);
				break;
			case Response_Reset:
				g_bUpdateConfigureArea = true;
				sprintf(cUpdateResponse, "%d %08lu %d", ucUpdateID, g_xWITConfig.ulSerialNumber, 0);
				break;
			}
			if (eStatus != Response_Null)
			{
				MQTT_Publish(g_xWITMQTT.lHandle, g_xWITMQTT.cPubBleUpdateRespTopic, (unsigned char *)cUpdateResponse, strlen(cUpdateResponse), false, false, QOS1);
			}
		}
#endif
		// 传感器更新
		if (g_xSensorResponse.ucStatus != Resp_None)
		{
			unsigned short usSendLen = g_xSensorResponse.ucMBCnt * 2 + 1;
			g_xSensorResponse.ucMBData[0] = g_xSensorResponse.ucStatus;
			g_xSensorResponse.ucStatus = Resp_None;
			if (MQTT_Publish(g_xWITMQTT.lHandle, g_xWITMQTT.cPubSensorRespTopic, g_xSensorResponse.ucMBData, usSendLen, false, false, QOS1))
			{
			}
		}

		// 配置响应
		if (g_xGatewayResponse.eStatus != Resp_None)
		{
			if (g_xGatewayResponse.eStatus != Resp_Success)
			{
				g_xGatewayResponse.ucCode = 0;
				sprintf(g_xGatewayResponse.cMessage, "error");
			}
			else
			{
				g_xGatewayResponse.ucCode = 1;
				sprintf(g_xGatewayResponse.cMessage, "success");
			}
			ESP_LOGD("test", "enter config resp");
			sprintf(g_xGatewayResponse.cBuffer, "{\"taskId\":\"%s\",\"ss\":[{\"addr\":\"%s\",\"result\":{\"code\":%d,\"message\":\"%s\"}}]}",
					g_xGatewayResponse.cTaskID, g_xGatewayResponse.cAddr, g_xGatewayResponse.ucCode, g_xGatewayResponse.cMessage);
			g_xGatewayResponse.eStatus = Resp_None;
			if (MQTT_Publish(g_xWITMQTT.lHandle, g_xGatewayResponse.pcTopic, (unsigned char *)g_xGatewayResponse.cBuffer, strlen(g_xGatewayResponse.cBuffer), false, false, QOS1))
			{
			}
		}

		if (ExConfigResponeFlag == 1)
		{
			sprintf(ExConfigRespBuffer, "{\"respCode\":%d}", ExConfigrespCode);
			if (MQTT_Publish(g_xWITMQTT.lHandle, g_xWITMQTT.cPubExtConfigRespTopic, (unsigned char *)ExConfigRespBuffer, strlen(ExConfigRespBuffer), false, false, QOS1))
			{
			}
			ExConfigResponeFlag = 0;
		}
		if (g_bBackDoorResponse == true)
		{
			sprintf(BackDoorRespBuffer, "{\"code\":%d}", g_bBackDoorCode);
			if (MQTT_Publish(g_xWITMQTT.lHandle, g_xWITMQTT.cPubBackDoorResp, (unsigned char *)BackDoorRespBuffer, strlen(BackDoorRespBuffer), false, false, QOS1))
			{
			}
			g_bBackDoorResponse = false;
			if (g_bBackDoorReset == true)
			{
				g_bBackDoorReset = false;
				esp_restart(); // 软件复位
			}
		}

		// 数据更新事件
		if (g_bUpdateConfigureArea == true)
		{
			g_bUpdateConfigureArea = false;
			WIT_UpdateConfigureArea();
		#if CONFIG_W5100S_SPI_ETHERNET > 0	
			if(get_pending_bit())
				esp_restart(); 
		#endif
		}

		if (g_bUpdateThresholdArea == true)
		{
			g_bUpdateThresholdArea = false;
			WIT_UpdateThresholdArea();
		}
#if CONFIG_BLUETOOTH > 0
		if (g_bSenorConfigArea == true)
		{
			g_bSenorConfigArea = false;
			printf("enter write flash config ....\r\n");
			if_falsh_write_config();
		}
#endif
		if (g_bUpdateDLIArea == true)
		{
			g_bUpdateDLIArea = false;
			WIT_UpdateDLIArea();
		}
		if (g_bUpdateBackupArea == true)
		{
			g_bUpdateBackupArea = false;
		}

        // if(g_beingConfigIOT.ConfigFlag == true)
        // {
        //     IOTConfigFunction();
        //     g_beingConfigIOT.ConfigFlag = false;
        // }

		vTaskDelay(50);
	}
}

#if CONFIG_ESP32_WIFI > 0
/**
 * @brief  WIT UDP task
 * @param  None
 * @retval None
 */
static void WIT_UDPPollTask(void *pvParameter)
{
	while (1)
	{
		UDPS_Poll();

		vTaskDelay(100);
	}
}

static void WIT_WIFIReconnectTask(void *pvParameter)
{
	while (1)
	{
		if (g_xWITConfig.eOpMode == WIFI_STA)
		{
			// WIFI 掉线处理
			if (MQTT_GetStatus(g_xWITMQTT.lHandle) == false)
			{
				static unsigned char s_ucError = 0;
				static unsigned char s_ucResetCount = 0;
				s_ucError++;
				if (WIFI_IsConnected() == true && pFuncWIFIMQTTReconnect != NULL)
				{
					if (g_bGetSTAInfo == false)
					{
						unsigned char ucRetry;

						ucRetry = 5;
						while (ucRetry > 0)
						{
							if (WIFI_GetCurrentRunMode() == WIFI_Mode_Trans)
								WIFI_EnterIntoConfigMode();
							if (WIFI_GetSTANetInfo(g_xCurrentWIFIInfo.cDHCP, g_xCurrentWIFIInfo.cIP, g_xCurrentWIFIInfo.cMask, g_xCurrentWIFIInfo.cGateway, g_xCurrentWIFIInfo.cDNS, WIFI_MAX_NAME_LENGTH) == WIFI_OK)
							{
								g_bGetSTAInfo = true;
								break;
							}

							vTaskDelay(500);
							ucRetry--;
						}

						WIFI_EnterIntoTransMode();
					}

					if (pFuncWIFIMQTTReconnect(g_xWITMQTT.lHandle) == true)
					{
						s_ucError = 0;
						s_ucResetCount = 0;
					}
				}
				else if (WIFI_IsConnected() == false)
				{
					g_bGetSTAInfo = false;
				}

				if (s_ucError >= WIT_MAX_ERROR_COUNT)
				{
					s_ucError = 0;
					s_ucResetCount++;
					WIFI_ResetByPin();
					WIFI_ChangeMode(WIFI_Mode_Trans);
					vTaskDelay(4000);

					if (s_ucResetCount >= 3)
					{
						WIFI_OpMode eOpMode;

						s_ucResetCount = 0;
						// 测试中出现过串口无反应配置全部出厂设置的情况
						//  Enter into wifi config mode
						if (WIFI_GetCurrentRunMode() == WIFI_Mode_Trans)
							WIFI_EnterIntoConfigMode();
						// Set pre SSID and PWD
						if (WIFI_GetCurrentOpMode(&eOpMode) == WIFI_OK)
						{
							char cAPSSID[36];

							if (WIFI_GetSSIDAndPassword(WIFI_AP, cAPSSID, sizeof(cAPSSID), NULL, sizeof(cAPSSID)) == WIFI_OK)
							{
								if (eOpMode != WIFI_STA || strcmp(cAPSSID, g_xWITConfig.cAPName) != 0)
								{
									if (strlen(g_xWITConfig.xWIFI.cSSID) != 0)
									{
										if (WIFI_SetSSIDAndPassword(WIFI_STA, g_xWITConfig.xWIFI.cSSID, g_xWITConfig.xWIFI.cPassword) == WIFI_OK)
										{
											WIFI_SetSSIDAndPassword(WIFI_AP, g_xWITConfig.cAPName, "None");
										}
									}
								}
							}
						}
						// Set to STA mode
						WIFI_SetToSTAMode(g_xWITMQTT.lHandle);
						// UDP Init
						UDPS_Init(g_lUDPSocketID);
						// Set other function
						WIFI_Preparation(g_xWITMQTT.lHandle, g_lUDPSocketID);
						// Reset
						WIFI_RestartModule();
					}
				}

				vTaskDelay(1000);
			}
		}

		vTaskDelay(100);
	}
}
#endif

/**
 * @brief  WIT MB task
 * @param  None
 * @retval None
 */
static void WIT_ModbusSlavePollTask(void *pvParameter)
{
	while (1)
	{
		eMBPoll(WIT_MB_SLAVE_PORT);
		MBUpdate_Poll();
		vTaskDelay(1);
	}
}
/**
 * @brief  WIT feature name init
 * @param  None
 * @retval None
 */
static void WIT_FeatureNameInit()
{
	int i, j;
	char cAxis[4];
	Sensor_Feature *pxSenFea = &g_xWITData.xSensFea;
	Feature *pxFea = NULL;
	ExtendFeature *pxExtFea = NULL;
	Parameter *pxParm = NULL;
	unsigned char ucIndex = 0;

	// Sensor Feature Name Init
	pxSenFea->xActualSpd.bIsUsed = true;
	sprintf(pxSenFea->xActualSpd.cName, "actualSpd");
	pxSenFea->temp.bIsUsed = true;
	sprintf(pxSenFea->temp.cName, "temp");

	for (i = 0; i < 3; i++)
	{
		if (i == 0)
		{
			sprintf(cAxis, "x");
			pxFea = &pxSenFea->x;
			pxExtFea = &pxSenFea->xExt;
		}
		else if (i == 1)
		{
			sprintf(cAxis, "y");
			pxFea = &pxSenFea->y;
			pxExtFea = &pxSenFea->yExt;
		}
		else if (i == 2)
		{
			sprintf(cAxis, "z");
			pxFea = &pxSenFea->z;
			pxExtFea = &pxSenFea->zExt;
		}

		// 基本特征
		pxFea->xMean.bIsUsed = true;
		pxFea->xRMS.bIsUsed = true;
		pxFea->xHighFreqRMS.bIsUsed = true;
		pxFea->xStd.bIsUsed = true;
		pxFea->xVar.bIsUsed = true;
		pxFea->xPeak.bIsUsed = true;
		pxFea->xCrestFactor.bIsUsed = true;
		pxFea->xKurtosis.bIsUsed = true;
		pxFea->xVRMS.bIsUsed = true;
		sprintf(pxFea->xMean.cName, "%sMean", cAxis);
		sprintf(pxFea->xRMS.cName, "%sRMS", cAxis);
		sprintf(pxFea->xHighFreqRMS.cName, "%sHighFreqRMS", cAxis);
		sprintf(pxFea->xStd.cName, "%sStd", cAxis);
		sprintf(pxFea->xVar.cName, "%sVar", cAxis);
		sprintf(pxFea->xPeak.cName, "%sPeak", cAxis);
		sprintf(pxFea->xCrestFactor.cName, "%sCrestFactor", cAxis);
		sprintf(pxFea->xKurtosis.cName, "%sKurtosis", cAxis);
		sprintf(pxFea->xVRMS.cName, "%sVRMS", cAxis);
#if CONFIG_RS485_RTU > 0
		sprintf(pxFea->xFreq1x.cName, "%sFreq1x", cAxis);
		sprintf(pxFea->xFreq2x.cName, "%sFreq2x", cAxis);
		sprintf(pxFea->xFreq3x.cName, "%sFreq3x", cAxis);
#endif
#if CONFIG_BLUETOOTH > 0
		sprintf(pxFea->xFreq05x.cName, "%sFreq0.5x", cAxis);
		sprintf(pxFea->xFreq10x.cName, "%sFreq1x", cAxis);
		sprintf(pxFea->xFreq15x.cName, "%sFreq1.5x", cAxis);
		sprintf(pxFea->xFreq20x.cName, "%sFreq2x", cAxis);
		sprintf(pxFea->xFreq25x.cName, "%sFreq2.5x", cAxis);
		sprintf(pxFea->xFreq30x.cName, "%sFreq3x", cAxis);
		sprintf(pxFea->xFreq35x.cName, "%sFreq3.5x", cAxis);
		sprintf(pxFea->xFreq40x.cName, "%sFreq4x", cAxis);
		sprintf(pxFea->xFreq45x.cName, "%sFreq4.5x", cAxis);
		sprintf(pxFea->xFreq50x.cName, "%sFreq5x", cAxis);
		sprintf(pxFea->xFreq55x.cName, "%sFreq5.5x", cAxis);
		sprintf(pxFea->xFreq60x.cName, "%sFreq6x", cAxis);
		sprintf(pxFea->xFreq65x.cName, "%sFreq6.5x", cAxis);
		sprintf(pxFea->xFreq70x.cName, "%sFreq7x", cAxis);
		sprintf(pxFea->xFreq75x.cName, "%sFreq7.5x", cAxis);
		sprintf(pxFea->xFreq80x.cName, "%sFreq8x", cAxis);
		sprintf(pxFea->xFreq85x.cName, "%sFreq8.5x", cAxis);
		sprintf(pxFea->xFreq90x.cName, "%sFreq9x", cAxis);
		sprintf(pxFea->xFreq95x.cName, "%sFreq9.5x", cAxis);
		sprintf(pxFea->xFreq100x.cName, "%sFreq10x", cAxis);
#endif
		// 扩展特征
		pxExtFea->xBaseExtFea.xHighFreqPeakToPeak.bIsUsed = true;

		sprintf(pxExtFea->xBaseExtFea.xHighFreqPeakToPeak.cName, "%sgSE", cAxis);
		sprintf(pxExtFea->xBaseExtFea.xInputEnvEnergy.cName, "%sgseA1", cAxis);
		sprintf(pxExtFea->xBaseExtFea.xOutputEnvEnergy.cName, "%sseA2", cAxis);
		// 轴承
		for (j = 0; j < ACC_MAX_BEARING_COUNT; j++)
		{
			sprintf(pxFea->xBearingFea[j].xFreqBO.cName, "%sFreqBO%d", cAxis, j + 1);
			sprintf(pxFea->xBearingFea[j].xFreqBI.cName, "%sFreqBI%d", cAxis, j + 1);
			sprintf(pxFea->xBearingFea[j].xFreqBB.cName, "%sFreqBB%d", cAxis, j + 1);
			sprintf(pxFea->xBearingFea[j].xFreqBF.cName, "%sFreqBF%d", cAxis, j + 1);
			// 轴承扩展特征
			sprintf(pxExtFea->xBearingExtFea[j].xEnvEnergy.cName, "%sseB%d", cAxis, j + 1);
			sprintf(pxExtFea->xBearingExtFea[j].xEnv2Energy.cName, "%sse2B%d", cAxis, j + 1);
			sprintf(pxExtFea->xBearingExtFea[j].xEnv3Energy.cName, "%sse3B%d", cAxis, j + 1);
		}
		// 齿轮
		for (j = 0; j < ACC_MAX_GEAR_COUNT; j++)
		{
			sprintf(pxFea->xGearFea[j].xFreq1xGMF.cName, "%sFreq1xGMF%d", cAxis, j + 1);
			sprintf(pxFea->xGearFea[j].xFreq2xGMF.cName, "%sFreq2xGMF%d", cAxis, j + 1);
			sprintf(pxFea->xGearFea[j].xFreq3xGMF.cName, "%sFreq3xGMF%d", cAxis, j + 1);
			// 齿轮扩展特征
			sprintf(pxExtFea->xGearExtFea[j].x1xSideEnergy.cName, "%sSide1xEnergy%d", cAxis, j + 1);
			sprintf(pxExtFea->xGearExtFea[j].x2xSideEnergy.cName, "%sSide2xEnergy%d", cAxis, j + 1);
			sprintf(pxExtFea->xGearExtFea[j].x3xSideEnergy.cName, "%sSide3xEnergy%d", cAxis, j + 1);
			sprintf(pxExtFea->xGearExtFea[j].xEnvEnergy.cName, "%sseG%d", cAxis, j + 1);
		}
		// 风机
		sprintf(pxFea->xFan.xSPM1x.cName, "%sspm1xF1", cAxis);
		sprintf(pxFea->xFan.xSPM2x.cName, "%sspm2xF1", cAxis);
		sprintf(pxFea->xFan.xSPM3x.cName, "%sspm3xF1", cAxis);
		// 皮带
		sprintf(pxFea->xBelt.xBelt1x.cName, "%sbelt1xF1", cAxis);
		sprintf(pxFea->xBelt.xSelfVirb.cName, "%sAmbeltVirF1", cAxis);
		// Screw
		sprintf(pxFea->xScrew.xScrew1x.cName, "%sscrew1xF1", cAxis);
		sprintf(pxFea->xScrew.xScrew2x.cName, "%sscrew2xF1", cAxis);
		sprintf(pxFea->xScrew.xScrew3x.cName, "%sscrew3xF1", cAxis);
		// Male Screw
		sprintf(pxFea->xMaleScrew.xMaleScrew1x.cName, "%smalescrew1xF1", cAxis);
		sprintf(pxFea->xMaleScrew.xMaleScrew2x.cName, "%smalescrew2xF1", cAxis);
		sprintf(pxFea->xMaleScrew.xMaleScrew3x.cName, "%smalescrew3xF1", cAxis);
		// Female Screw
		sprintf(pxFea->xFemaleScrew.xFemaleScrew1x.cName, "%sfemalescrew1xF1", cAxis);
		sprintf(pxFea->xFemaleScrew.xFemaleScrew2x.cName, "%sfemalescrew2xF1", cAxis);
		sprintf(pxFea->xFemaleScrew.xFemaleScrew3x.cName, "%sfemalescrew3xF1", cAxis);
		// Shaft
		sprintf(pxFea->xShaft.xShaft1x.cName, "%sshaft1xF1", cAxis);
		sprintf(pxFea->xShaft.xShaft2x.cName, "%sshaft2xF1", cAxis);
		sprintf(pxFea->xShaft.xShaft3x.cName, "%sshaft3xF1", cAxis);
		// Sleeve
		sprintf(pxFea->xSleeve.xShaftA05xFn.cName, "%sshaftA05xFn", cAxis);
		sprintf(pxFea->xSleeve.xShaftA1xFn.cName, "%sshaftA1xFn", cAxis);
		sprintf(pxFea->xSleeve.xShaftA2xFn.cName, "%sshaftA2xFn", cAxis);
		sprintf(pxFea->xSleeve.xShaftA3xFn.cName, "%sshaftA3xFn", cAxis);
		sprintf(pxFea->xSleeve.xShaftA4xFn.cName, "%sshaftA4xFn", cAxis);
		sprintf(pxFea->xSleeve.xShaftA5xFn.cName, "%sshaftA5xFn", cAxis);
		sprintf(pxFea->xSleeve.xShaftA6xFn.cName, "%sshaftA6xFn", cAxis);
		sprintf(pxFea->xSleeve.xShaftA7xFn.cName, "%sshaftA7xFn", cAxis);
		sprintf(pxFea->xSleeve.xShaftA8xFn.cName, "%sshaftA8xFn", cAxis);
		sprintf(pxFea->xSleeve.xShaftA9xFn.cName, "%sshaftA9xFn", cAxis);
		sprintf(pxFea->xSleeve.xShaftA10xFn.cName, "%sshaftA10xFn", cAxis);
		sprintf(pxFea->xSleeve.xmaxFreqA05xFn.cName, "%smaxFreqAt05xn", cAxis);
		// PowerFrequency
		sprintf(pxFea->xPowerFrequency.xNaAmFreq.cName, "%sLineFrequency", cAxis);
		sprintf(pxFea->xPowerFrequency.xEnergy.cName, "%spowerFrequency", cAxis);
	}
	// Gateway error parameter
	g_xGWErrParm.xOtherChipError.bIsUsed = true;
	sprintf(g_xGWErrParm.xOtherChipError.cName, "errGMod");

	for (i = 0; i < ACC_MAX_SENSOR_COUNT; i++)
	{
		// Sensor error parameter
		g_xSrErrParm[i].xSensorExistError.bIsUsed = true;
		g_xSrErrParm[i].xShockChipError.bIsUsed = true;
		g_xSrErrParm[i].xNTCError.bIsUsed = true;
		g_xSrErrParm[i].xOtherChipError.bIsUsed = true;
		sprintf(g_xSrErrParm[i].xSensorExistError.cName, "errSen");
		sprintf(g_xSrErrParm[i].xShockChipError.cName, "errSck");
		sprintf(g_xSrErrParm[i].xNTCError.cName, "errNTC");
		sprintf(g_xSrErrParm[i].xOtherChipError.cName, "errSMod");
	}

	// GW and sensor parameter size
	g_ulGatewayParameterSize = sizeof(Gateway_Feature) / sizeof(Parameter) - 1;
	g_ulSensorParameterSize = sizeof(Sensor_Feature) / sizeof(Parameter);
	g_ulTotalParameterSize = g_ulGatewayParameterSize + ACC_MAX_SENSOR_COUNT * g_ulSensorParameterSize;

	// Create gateway and sensor threshold
	g_pxGlobalThr = (Threshold *)malloc(g_ulTotalParameterSize * sizeof(Threshold));
	g_pxGatewayThr = g_pxGlobalThr;
	g_pxSensorThr = g_pxGlobalThr + g_ulGatewayParameterSize;

	// Create gateway data list info
	g_pxGlobalDLI = (DataListInfo *)malloc(g_ulTotalParameterSize * sizeof(DataListInfo));
	g_pxGatewayDLI = g_pxGlobalDLI;
	g_pxSensorDLI = g_pxGlobalDLI + g_ulGatewayParameterSize;

	// Create modbus alarm buffer
	g_pucMBAlarm = (unsigned char *)malloc((g_ulGatewayParameterSize + g_ulSensorParameterSize) * sizeof(unsigned char));
	g_pucMBSensorAlarm = g_pucMBAlarm + g_ulGatewayParameterSize;

	// alarm list info
	pxParm = (Parameter *)&g_xWITData.xGWFea;

	ucIndex = 0;
	for (i = 0; i < g_ulGatewayParameterSize; i++)
	{
		if (WIT_IsImportantParameter(pxParm->cName) == true)
		{
			pxParm->ucAlarmAddressIndex = ucIndex;
			g_ulGatewayAlarmParameterSize++;
			ucIndex++;
		}
		pxParm++;
	}

	pxParm = (Parameter *)&g_xWITData.xSensFea;
	ucIndex = 0;
	for (i = 0; i < g_ulSensorParameterSize; i++)
	{
		if (WIT_IsImportantParameter(pxParm->cName) == true)
		{
			pxParm->ucAlarmAddressIndex = ucIndex;
			g_ulSensorAlarmParameterSize++;
			ucIndex++;
		}
		pxParm++;
	}

	g_ulTotalAlarmParameterSize = g_ulGatewayAlarmParameterSize + ACC_MAX_SENSOR_COUNT * g_ulSensorAlarmParameterSize;
	g_pxGlobalALI = (AlarmListInfo *)malloc(g_ulTotalAlarmParameterSize * sizeof(AlarmListInfo));
	g_pxGatewayALI = g_pxGlobalALI;
	g_pxSensorALI = g_pxGlobalALI + g_ulGatewayAlarmParameterSize;
	memset(g_pxGlobalALI, 0, g_ulTotalAlarmParameterSize * sizeof(AlarmListInfo));
}

/**
 * @brief  WIT init g_xWITConfig
 * @param  None
 * @retval None
 */
static void WIT_ModeAndValueInit()
{
#if CONFIG_ESP32_WIFI > 0
	bool bNeedGetInitialWIFIInfo = false;
#endif
	bool bNeedUpdateConfigArea = false;
	unsigned char ucCorrectCnt = 0, ucBkCorrectCnt = 0;

	// FLASH备份检查与更新
	read_flash(E_GW_CONFIG, &g_xWITConfig, sizeof(g_xWITConfig));
	ucCorrectCnt = WIT_GetConfigureNumberCount();
	read_flash(E_GWBP_CONFIG, &g_xWITConfig, sizeof(g_xWITConfig));
	ucBkCorrectCnt = WIT_GetConfigureNumberCount();

	ESP_LOGD("test", "ucCorrectCnt = %d, ucBkCorrectCnt = %d", ucCorrectCnt, ucBkCorrectCnt);
	if (ucCorrectCnt != ucBkCorrectCnt)
	{
		if (ucCorrectCnt > ucBkCorrectCnt)
		{
			// 备份区域不完成 读取配置区域
			read_flash(E_GW_CONFIG, &g_xWITConfig, sizeof(g_xWITConfig));
			// 更新备份区域
			write_flash(E_GWBP_CONFIG, &g_xWITConfig, sizeof(g_xWITConfig));
		}
		else
		{
			// 配置区域数据不完整，更新配置区域
			write_flash(E_GW_CONFIG, &g_xWITConfig, sizeof(g_xWITConfig));
		}
	}

 	// g_xWITConfig.cTargetIP[0] = 112;
	// g_xWITConfig.cTargetIP[1] = 17;
	// g_xWITConfig.cTargetIP[2] = 107;
	// g_xWITConfig.cTargetIP[3] = 240;
    // strcpy(g_xWITConfig.cMQTTBroker, "112.17.107.240");
	// strcpy(g_xWITConfig.cMQTTPort, "1883");
	// bNeedUpdateConfigArea = true;
	// strcpy(g_xWITConfig.cMQTTBroker, "www.witium.com.cn");
	// strcpy(g_xWITConfig.cMQTTPort, "12883");
	// strcpy(g_xWITConfig.cMQTTBroker, "116.62.127.242");
	// strcpy(g_xWITConfig.cMQTTPort, "1883");
	// bNeedUpdateConfigArea = true;
	//  g_xWITConfig.ulPort = 345;
     g_xWITConfig.ulSerialNumber = 20240108;
	 g_xWITConfig.ulPort = 12883;
	 g_xWITConfig.cTargetIP[0] = 222;
	 g_xWITConfig.cTargetIP[1] = 71;
     g_xWITConfig.cTargetIP[2] = 96;
	 g_xWITConfig.cTargetIP[3] = 91;
    //  g_xWITConfig.my_IOTDev[0].addr = 1;
    //  g_xWITConfig.my_IOTDev[0].id = 0x003f;
    //  g_xWITConfig.my_IOTDev[0].sn = 0x0001;
    //  g_xWITConfig.my_IOTDev[0].bIsTimeout = false;
    //  g_xWITConfig.my_IOTDev[0].bIsUsed = true;

    iot_Sensor_Init(); // 清空传感器配置
     
	//  sprintf(g_xWITConfig.cAPName, "Witium-%08lu", g_xWITConfig.ulSerialNumber);
    //  bNeedUpdateConfigArea = true;
	//  sprintf(g_xWITConfig.cAPName, "Witium-%08lu", g_xWITConfig.ulSerialNumber);
	for (int i = 0; i < WIT_SENSOR_GROUP_MAX_NUM; i++)
	{
		g_xWITConfig.xGroupTable[i].ulOperation = 0;
		g_xWITConfig.xGroupTable[i].ulWavePushEnable = 1;
		g_xWITConfig.xGroupTable[i].ulFeaPushEnable = 1;
	}
	// WIT_UpdateConfigureArea();
	// g_xWITConfig.ulConfigureValue = 0;

	// Check ID and motor work time
	// 1.Firstly check backup area

	read_flash(E_PVD_STORE, g_xPVDData, sizeof(g_xPVDData));
	for (size_t i = 0; i < 6; i++)
	{
		// 备份域读写
		if (g_xPVDData[i].ulConfirmValue == WIT_CONFIGURE_VALUE)
		{
			if (g_xPVDData[i].bSendingData == true)
			{
				g_xPVDData[i].bSendingData = false;
				g_xPVDData[i].ulPackageID++;
			}

			g_xWITConfig.xGroupTable[i].packid = g_xPVDData[i].ulPackageID;
			g_xWITConfig.xGroupTable[i].ulWorkMinutes = g_xPVDData[i].ulWorkMinutes;
			g_xWITConfig.spdSel = 3;
		}
		else
		{
			// 2.Secondly check flash
			read_flash(E_PVD_STORE, &g_xPVDData, sizeof(PVD_Data));
			if (g_xPVDData[i].ulConfirmValue == WIT_CONFIGURE_VALUE)
			{
				if (g_xPVDData[i].bSendingData == true)
				{
					g_xPVDData[i].bSendingData = false;
					g_xPVDData[i].ulPackageID++;
				}

				g_xWITConfig.xGroupTable[i].packid = g_xPVDData[i].ulPackageID;
				g_xWITConfig.xGroupTable[i].ulWorkMinutes = g_xPVDData[i].ulWorkMinutes;
			}
		}
	}
	write_flash(E_PVD_STORE, g_xPVDData, sizeof(g_xPVDData));

	if (g_xWITConfig.ulConfigureValue != WIT_CONFIGURE_VALUE)
	{
		unsigned long ulSN = g_xWITConfig.ulSerialNumber;

		if (ulSN == 0xFFFFFFFF)
			ulSN = 0;
		memset((unsigned char *)&g_xWITConfig, 0, sizeof(WIT_Configure));
#if CONFIG_ESP32_WIFI > 0
		g_xWITConfig.eOpMode = WIFI_FORCE_AP;
#endif
		g_xWITConfig.ulSerialNumber = ulSN;
		g_xWITConfig.ulPeriod = WIT_NORMAL_FEATURE_PERIOD;
		g_xWITConfig.bConfigUpdated = false;
#if CONFIG_ESP32_WIFI > 0
		strcpy(g_xWITConfig.xWIFI.cDHCP, "DHCP");
#if USE_WIT_PLATFORM > 0
		sprintf(g_xWITConfig.cAPName, "JCI-%08lu", g_xWITConfig.ulSerialNumber);
#else
		sprintf(g_xWITConfig.cAPName, "Witium-%08lu", g_xWITConfig.ulSerialNumber);
#endif
#endif
		g_xWITConfig.ulConfigureValue = WIT_CONFIGURE_VALUE;
		bNeedUpdateConfigArea = true;
	}

	if (g_xWITConfig.ulWIFIAndThrValue != WIT_CONFIGURE_VALUE)
	{
		g_xWITConfig.ulInputSpeed = 1500;
		g_xWITConfig.bAutoThreshold = false;
#if CONFIG_ESP32_WIFI > 0
		bNeedGetInitialWIFIInfo = true;
#endif
		memset(&g_xWITConfig.xWorkDet, 0xFF, sizeof(WorkDetective));
		g_xWITConfig.xWorkDet.ucSensorAddr = 1;
		g_xWITConfig.xWorkDet.xXVarWorkThr.fHighLimit = 80;
		g_xWITConfig.xWorkDet.ulWorkMinutes = 0;
		g_xWITConfig.ulWIFIAndThrValue = WIT_CONFIGURE_VALUE;
		bNeedUpdateConfigArea = true;
	}

	if (g_xWITConfig.ulMQTTInfoValue != WIT_CONFIGURE_VALUE)
	{
#if USE_WIT_PLATFORM > 0
		sprintf(g_xWITConfig.cMQTTBroker, "47.96.79.210");
		sprintf(g_xWITConfig.cMQTTPort, "1883");
		sprintf(g_xWITConfig.cMQTTUser, "witcd");
		sprintf(g_xWITConfig.cMQTTPassword, "Witium37774020");
		g_xWITConfig.ulPort = 1883;
		g_xWITConfig.cTargetIP[0] = 47;
		g_xWITConfig.cTargetIP[1] = 96;
		g_xWITConfig.cTargetIP[2] = 79;
		g_xWITConfig.cTargetIP[2] = 210;
#elif USE_WT_PLATFORM > 0
		sprintf(g_xWITConfig.cMQTTBroker, "116.62.127.242");
		sprintf(g_xWITConfig.cMQTTPort, "1883");
		sprintf(g_xWITConfig.cMQTTUser, "witcd");
		sprintf(g_xWITConfig.cMQTTPassword, "Witium37774020");
		g_xWITConfig.ulPort = 1883;
		g_xWITConfig.cTargetIP[0] = 116;
		g_xWITConfig.cTargetIP[1] = 62;
		g_xWITConfig.cTargetIP[2] = 127;
		g_xWITConfig.cTargetIP[3] = 242;
#endif
		g_xWITConfig.ulMQTTInfoValue = WIT_CONFIGURE_VALUE;
		bNeedUpdateConfigArea = true;
	}

	if (g_xWITConfig.ulModuleNameValue != WIT_CONFIGURE_VALUE)
	{
#if USE_WIT_PLATFORM > 0
		sprintf(g_xWITConfig.cModuleName, "JCIG1X40%s", MODULE_TYPE_STR);
#elif USE_WT_PLATFORM > 0
		sprintf(g_xWITConfig.cModuleName, "WTGB8N%s", MODULE_TYPE_STR);
#endif
		g_xWITConfig.ulModuleNameValue = WIT_CONFIGURE_VALUE;
		bNeedUpdateConfigArea = true;
	}
	if (g_xWITConfig.ulLearningInfoValue != WIT_CONFIGURE_VALUE)
	{
		g_xWITConfig.eGatewayMode = GatewayMode_Normal;
		g_xWITConfig.bAutoThreshold = false;
		g_xWITConfig.usTotalLearningCount = WIT_PER_DATA_STORE_MAX_COUNT;
		g_xWITConfig.usCurrentLearningCount = 0;
		g_xWITConfig.ulWavePeriod = WIT_NORMAL_WAVE_PERIOD;
		g_xWITConfig.ucModbusID = 0x01;
		g_xWITConfig.ulModbusBaudRate = 9600;
		g_xWITConfig.eModbusParity = MB_PAR_NONE;
		g_xWITConfig.ulLearningInfoValue = WIT_CONFIGURE_VALUE;
		bNeedUpdateConfigArea = true;
	}
	// Fixed total count
	if (g_xWITConfig.usTotalLearningCount != WIT_PER_DATA_STORE_MAX_COUNT)
	{
		g_xWITConfig.usTotalLearningCount = WIT_PER_DATA_STORE_MAX_COUNT;
		bNeedUpdateConfigArea = true;
	}

	if (g_xWITConfig.ulDelayInfoValue != WIT_CONFIGURE_VALUE)
	{
		g_xWITConfig.ulDelayTime = 1000;
		g_xWITConfig.ulDelayInfoValue = WIT_CONFIGURE_VALUE;
		bNeedUpdateConfigArea = true;
	}

	if (g_xWITConfig.ulCopyValueInfo != WIT_CONFIGURE_VALUE)
	{
		g_xWITConfig.ulFeaturePeriodCopy = g_xWITConfig.ulPeriod;
		g_xWITConfig.ulWavePeriodCopy = g_xWITConfig.ulWavePeriod;
		g_xWITConfig.ulCopyValueInfo = WIT_CONFIGURE_VALUE;
		bNeedUpdateConfigArea = true;
	}
#if CONFIG_BLUETOOTH > 0
	if (g_xWITConfig.ulSenParmInfoValue != WIT_CONFIGURE_VALUE)
	{
		for (int i = 0; i < WIT_SENSOR_GROUP_MAX_NUM * WIT_SENSOR_MAX_CONNECT_NUM; i++)
		{
			memset(g_xWITConfig.xSenTable[i].cName, 0, sizeof(g_xWITConfig.xSenTable[i].cName));
			memset(g_xWITConfig.xSenTable[i].cMac, 0, sizeof(g_xWITConfig.xSenTable[i].cMac));
			g_xWITConfig.xSenTable[i].ulAddr = 0;
			g_xWITConfig.xSenTable[i].ulGroup = 0;
			g_xWITConfig.xSenTable[i].ulUseEnable = 0;
			g_xWITConfig.xSenTable[i].ulConfigEnable = 0;
		}
		for (int i = 0; i < WIT_SENSOR_GROUP_MAX_NUM; i++)
		{
			g_xWITConfig.xGroupTable[i].ulGroup = i + 1;
			g_xWITConfig.xGroupTable[i].ulGUseEnable = 0;
			g_xWITConfig.xGroupTable[i].ulPeriod = 3000;
			g_xWITConfig.xGroupTable[i].ulFeaPeriod = 3000;
			g_xWITConfig.xGroupTable[i].ulConnectNum = 0;
			g_xWITConfig.xGroupTable[i].ulConnectTime = i * 150;
			g_xWITConfig.xGroupTable[i].ulWavePushEnable = 0;
			g_xWITConfig.xGroupTable[i].ulFeaPushEnable = 0;
			g_xWITConfig.xGroupTable[i].packid = 0;
			sprintf(g_xWITConfig.xGroupTable[i].groupNo, "%s%d", "RM1111", i);
			// memset(g_xWITConfig.xGroupTable[i].groupNo, "RM11111", sizeof(g_xWITConfig.xGroupTable[i].groupNo));
		}
		g_xWITConfig.xGroupTable[0].ulGUseEnable = 1;

		g_xWITConfig.ulBleBinSize = 0;
		g_xWITConfig.ulSenParmInfoValue = WIT_CONFIGURE_VALUE;
		bNeedUpdateConfigArea = true;
	}
#endif
#if CONFIG_W5100S_SPI_ETHERNET > 0
	if (g_xWITConfig.ulEthNetInfoValue != WIT_CONFIGURE_VALUE)
	{
		g_xWITConfig.g_xConfigIp.mac[0] = 0x00;
		g_xWITConfig.g_xConfigIp.mac[1] = 0x08;
		g_xWITConfig.g_xConfigIp.mac[2] = 0xdc;
		g_xWITConfig.g_xConfigIp.mac[3] = 0x6f;
		g_xWITConfig.g_xConfigIp.mac[4] = 0x00;
		g_xWITConfig.g_xConfigIp.mac[5] = 0x88;

		g_xWITConfig.g_xConfigIp.ip[0] = 192;
		g_xWITConfig.g_xConfigIp.ip[1] = 168;
		g_xWITConfig.g_xConfigIp.ip[2] = 1;
		g_xWITConfig.g_xConfigIp.ip[3] = 100;

		g_xWITConfig.g_xConfigIp.gw[0] = 192;
		g_xWITConfig.g_xConfigIp.gw[1] = 168;
		g_xWITConfig.g_xConfigIp.gw[2] = 1;
		g_xWITConfig.g_xConfigIp.gw[3] = 1;

		g_xWITConfig.g_xConfigIp.sn[0] = 255;
		g_xWITConfig.g_xConfigIp.sn[1] = 255;
		g_xWITConfig.g_xConfigIp.sn[2] = 255;
		g_xWITConfig.g_xConfigIp.sn[3] = 0;

		g_xWITConfig.g_xConfigIp.dns[0] = 114;
		g_xWITConfig.g_xConfigIp.dns[1] = 114;
		g_xWITConfig.g_xConfigIp.dns[2] = 114;
		g_xWITConfig.g_xConfigIp.dns[3] = 114;
		g_xWITConfig.g_xConfigIp.dhcp = NETINFO_STATIC;
		g_xWITConfig.ulEthNetInfoValue = WIT_CONFIGURE_VALUE;
		bNeedUpdateConfigArea = true;
	}
	set_wiz_NetInfo(&g_xWITConfig.g_xConfigIp);
#endif
#if USE_WIT_PLATFORM > 0 && CONFIG_ESP32_WIFI > 0
	// 把之前辉度的机器更新成杰牌后前缀显示JCI
	if (strstr(g_xWITConfig.cAPName, "Witium-") != NULL)
	{
		sprintf(g_xWITConfig.cAPName, "JCI-%08lu", g_xWITConfig.ulSerialNumber);
		bNeedUpdateConfigArea = true;
	}
#endif

#if WIT_FIXED_MODULE_NAME > 0
	if (strcmp(g_xWITConfig.cModuleName, WIT_MODULE_NAME) != 0)
	{
		strcpy(g_xWITConfig.cModuleName, WIT_MODULE_NAME);
		bNeedUpdateConfigArea = true;
	}
#endif
#if CONFIG_W5100S_SPI_ETHERNET > 0
	ETHERNET_Init(&g_bModuleError, NULL);
#endif
	// 获取当前学习数量
	g_xWITConfig.usCurrentLearningCount = WIT_GetFeatureMinLearnCount();
	WIT_PreTaskInit();

#if CONFIG_ESP32_WIFI > 0
	// wait for wifi
	vTaskDelay(4000);
	// Enter into wifi config mode
	if (WIFI_GetCurrentRunMode() == WIFI_Mode_Trans)
		WIFI_EnterIntoConfigMode();
	if (bNeedGetInitialWIFIInfo == true)
	{
		unsigned char ucRetry = 5;

		while (ucRetry--)
		{
			// Get current STA ssid and password
			if (WIFI_GetSSIDAndPassword(WIFI_STA, g_xWITConfig.xWIFI.cSSID, WIFI_MAX_NAME_LENGTH, g_xWITConfig.xWIFI.cPassword, WIFI_MAX_NAME_LENGTH) == WIFI_OK)
				break;
		}
		bNeedGetInitialWIFIInfo = false;
	}

	// Check mode AP or STA
	{
		unsigned char ucRetry = 5;
		WIFI_OpMode eOpMode;

		// Check Mode
		while (ucRetry > 0 && g_xWITConfig.eOpMode != WIFI_FORCE_AP)
		{
			if (WIFI_GetCurrentOpMode(&eOpMode) == WIFI_OK)
			{
				if (eOpMode != g_xWITConfig.eOpMode)
				{
					g_xWITConfig.eOpMode = WIFI_STA;
					bNeedUpdateConfigArea = true;
				}

				break;
			}

			ucRetry--;
			vTaskDelay(200);
		}

		g_bModuleError = ucRetry > 0 ? false : true;
	}

	// 配置成功变为STA模式
	if (g_xWITConfig.bConfigUpdated == true && g_xWITConfig.eOpMode == WIFI_AP)
	{
		g_xWITConfig.eOpMode = WIFI_STA;
		bNeedUpdateConfigArea = true;
	}

	// 强制AP模式
	if (g_xWITConfig.eOpMode == WIFI_FORCE_AP)
	{
		g_xWITConfig.bConfigUpdated = false;
		g_xWITConfig.eOpMode = WIFI_AP;
		bNeedUpdateConfigArea = true;
	}
#endif

	// Create led task
	xTaskCreate(WIT_LEDTask, "LED Task", WIT_LED_TASK_STK, NULL, WIT_LED_TASK_PRIORITY, NULL);

	// #if CONFIG_EC200S_USB_4G  > 0
	// 	Celluar_ModuleInit(&g_bModuleError);
	// #endif
	// MQTT Init
	WIT_MQTTInit();

#if CONFIG_ESP32_WIFI > 0
	// WIFI Configure
	{
		unsigned char ucRetry = 5;

		if (g_xWITConfig.eOpMode == WIFI_AP)
		{
			while (ucRetry > 0 && (WIFI_SetToAPMode(g_xWITMQTT.lHandle, g_xWITConfig.cAPName) != true))
			{
				ucRetry--;
				vTaskDelay(1000);
			}

			sprintf(g_xCurrentWIFIInfo.cIP, "192.168.1.1");
			sprintf(g_xCurrentWIFIInfo.cDHCP, "DHCP");
			sprintf(g_xCurrentWIFIInfo.cGateway, "192.168.1.1");
			sprintf(g_xCurrentWIFIInfo.cMask, "255.255.255.0");
			sprintf(g_xCurrentWIFIInfo.cDNS, "0.0.0.0");
		}
		else if (g_xWITConfig.eOpMode == WIFI_STA)
		{
			while (ucRetry > 0 && (WIFI_SetToSTAMode(g_xWITMQTT.lHandle) != true))
			{
				ucRetry--;
				vTaskDelay(1000);
			}
		}

		// Enter into wifi config mode
		if (WIFI_GetCurrentRunMode() == WIFI_Mode_Trans)
			WIFI_EnterIntoConfigMode();

		// UART1 Set to UDP Server mode
		g_lUDPSocketID = UDPS_Init(g_lUDPSocketID);
		UDPS_SetRequestHandler(WIT_UDPSRequestHandler);

		if (g_xWITConfig.bConfigUpdated == true)
		{
			WIFI_IPMode eIPMode = WIFI_DHCP;
			if (strstr(g_xWITConfig.xWIFI.cDHCP, "STATIC") != NULL)
				eIPMode = WIFI_Static;
			else
				eIPMode = WIFI_DHCP;

			// 设置wifi名称和密码
			if (WIFI_SetSSIDAndPassword(WIFI_STA, g_xWITConfig.xWIFI.cSSID, g_xWITConfig.xWIFI.cPassword) == WIFI_OK)
			{
				// 设置DHCP或静态地址
				if (WIFI_SetSTANetInfo(eIPMode, g_xWITConfig.xWIFI.cIP, g_xWITConfig.xWIFI.cMask, g_xWITConfig.xWIFI.cGateway, g_xWITConfig.xWIFI.cDNS) == WIFI_OK)
				{
					g_xWITConfig.bConfigUpdated = false;
				}
			}
		}

		ucRetry = 5;
		// 获取WIFI信息
		while (ucRetry > 0 && WIFI_GetSSIDAndPassword(WIFI_STA, g_xCurrentWIFIInfo.cSSID, WIFI_MAX_NAME_LENGTH, g_xCurrentWIFIInfo.cPassword, WIFI_MAX_NAME_LENGTH) != WIFI_OK)
		{
			vTaskDelay(1000);
			ucRetry--;
		}
		if (ucRetry > 0)
		{
			if (strcmp(g_xCurrentWIFIInfo.cSSID, g_xWITConfig.xWIFI.cSSID) != 0 ||
				strcmp(g_xCurrentWIFIInfo.cPassword, g_xWITConfig.xWIFI.cPassword) != 0)
			{
				char cAPSSID[36];

				ucRetry = 5;
				// 查看AP名称
				while (ucRetry > 0 && WIFI_GetSSIDAndPassword(WIFI_AP, cAPSSID, sizeof(cAPSSID), NULL, sizeof(cAPSSID)) != WIFI_OK)
				{
					vTaskDelay(1000);
					ucRetry--;
				}

				if (ucRetry > 0)
				{
					// 更新SSID名称
					if (strcmp(cAPSSID, g_xWITConfig.cAPName) == 0)
					{
						strcpy(g_xWITConfig.xWIFI.cSSID, g_xCurrentWIFIInfo.cSSID);
						strcpy(g_xWITConfig.xWIFI.cPassword, g_xCurrentWIFIInfo.cPassword);
						bNeedUpdateConfigArea = true;
					}
				}
			}
		}

		// 获取MAC信息
		ucRetry = 5;
		while (ucRetry > 0 && WIFI_GetMAC(g_xCurrentWIFIInfo.cMAC, WIFI_MAX_NAME_LENGTH) != WIFI_OK)
		{
			vTaskDelay(1000);
			ucRetry--;
		}
		// Set other function
		WIFI_Preparation(g_xWITMQTT.lHandle, g_lUDPSocketID);
		// Restart module
		WIFI_RestartModule();
	}

	// enter into trans mode
	if (WIFI_GetCurrentRunMode() == WIFI_Mode_Config)
		WIFI_EnterIntoTransMode();
#endif
	if (bNeedUpdateConfigArea == true)
	{
		bNeedUpdateConfigArea = false;
		WIT_UpdateConfigureArea();
	}

	for(int x = 0; x < 6; x++)
	{
		g_xPVDData[x].ulPackageID = g_xWITConfig.xGroupTable[x].packid;
		g_xPVDData[x].ulWorkMinutes = g_xWITConfig.xGroupTable[x].ulWorkMinutes;
		g_xPVDData[x].bSendingData = false;
		g_xPVDData[x].ulConfirmValue = WIT_CONFIGURE_VALUE;
	}
	// Set pvd value

	ESP_LOGI("test", "g_xWITConfig.ulSerialNumber %ld", g_xWITConfig.ulSerialNumber);
	ESP_LOGI("test", "g_xWITConfig.ulModuleNameValue %s", g_xWITConfig.cModuleName);
}

/**
 * @brief 从外部FLASH中读取阈值和储存信息
 * @param  None
 * @retval None
 */
static void WIT_ReadThrAndDLI()
{
	unsigned long ulValue;
	int i, j;

	// Read Thr Flag
	read_flash(E_THRESHOLD_FLAG, &ulValue, sizeof(unsigned long));
	if (ulValue != WIT_THRESHOLD_VLAUE)
	{
		memset(g_pxGlobalThr, 0xFF, sizeof(Threshold) * g_ulTotalParameterSize);
		write_flash(E_THRESHOLD_STORE, g_pxGlobalThr, g_ulTotalParameterSize * sizeof(Threshold));
		ulValue = WIT_THRESHOLD_VLAUE;
		write_flash(E_THRESHOLD_FLAG, &ulValue, sizeof(unsigned long));
	}
	else
	{
		read_flash(E_THRESHOLD_STORE, g_pxGlobalThr, g_ulTotalParameterSize * sizeof(Threshold));
	}

	// Read DLI Flag
	read_flash(E_FEA_HISTORY_FLAG, &ulValue, sizeof(unsigned long));
	if (ulValue != WIT_HISTORY_DATA_VALUE)
	{
		DataListInfo *pxPointer = NULL;
		unsigned long ulStoreAddress = WIT_DATA_LIST_EXT_FLASH_ADDR;

		// 网关首地址 - 1M | 传感器1首地址 - 2M | 传感器2首地址 - 3M | 传感器N首地址 - (N + 1)M
		pxPointer = g_pxGatewayDLI;
		for (i = 0; i < g_ulGatewayParameterSize; i++)
		{
			pxPointer->bUsed = false;
			pxPointer->usCount = 0;
			pxPointer->ulStartAddress = ulStoreAddress;
			ulStoreAddress += WIT_PER_DATA_STORE_MAX_COUNT * sizeof(float);
			pxPointer++;
		}

		pxPointer = g_pxSensorDLI;
		for (i = 0; i < ACC_MAX_SENSOR_COUNT; i++)
		{
			pxPointer = g_pxSensorDLI + i * g_ulSensorParameterSize;
			ulStoreAddress = (i + 2) * 1024 * 1024;

			for (j = 0; j < g_ulSensorParameterSize; j++)
			{
				pxPointer->bUsed = false;
				pxPointer->usCount = 0;
				pxPointer->ulStartAddress = ulStoreAddress;
				ulStoreAddress += WIT_PER_DATA_STORE_MAX_COUNT * sizeof(float);
				pxPointer++;
			}
		}
		write_flash(E_FEA_HISTORY_STORE, g_pxGatewayDLI, g_ulTotalParameterSize * sizeof(DataListInfo));
		ulValue = WIT_HISTORY_DATA_VALUE;
		read_flash(E_FEA_HISTORY_FLAG, &ulValue, sizeof(unsigned long));
	}
	else
	{
		write_flash(E_FEA_HISTORY_STORE, g_pxGatewayDLI, g_ulTotalParameterSize * sizeof(DataListInfo));
	}
}

/**
 * @brief  WIT MQTT Init
 * @param  None
 * @retval None
 */
static void WIT_MQTTInit()
{
#if USE_WIT_PLATFORM > 0
	// Client ID
	sprintf(g_xWITMQTT.cClientID, "%s_%08lu", g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber);
#elif USE_WT_PLATFORM > 0
	// Client ID
	sprintf(g_xWITMQTT.cClientID, "%s_%08lu", g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber);
#endif
	// Publish topic
	sprintf(g_xWITMQTT.cPubDataTopic, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "Data");
	sprintf(g_xWITMQTT.cPubFeatureTopic, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "Feature");
	sprintf(g_xWITMQTT.cPubUpdateResponseTopic, "%s/%s/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, "Update/Response");
	sprintf(g_xWITMQTT.cPubCheckInTopic, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "CheckIn");
	sprintf(g_xWITMQTT.cPubIOTDataSendTopic, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "opData");
	sprintf(g_xWITMQTT.cPubIOTErrorSendTopic, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "Error");
	sprintf(g_xWITMQTT.cPubAlarmTopic, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "Alarm");
	sprintf(g_xWITMQTT.cPubConfigRespTopic, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "ConfigResponse");
	sprintf(g_xWITMQTT.cPubWorkThrRespTopic, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "ConfigWorkThrResponse");
	sprintf(g_xWITMQTT.cPubSensorRespTopic, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "SensorResp");
	sprintf(g_xWITMQTT.cPubPowerDownTopic, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "PowerDown");
	sprintf(g_xWITMQTT.cPubThresholdTopic, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "Threshold");
	sprintf(g_xWITMQTT.cPubTimeRequestTopic, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "Request");
#if CONFIG_BLUETOOTH > 0
	sprintf(g_xWITMQTT.cPubBleUpdateTopic, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "BleUpdateResult");
	sprintf(g_xWITMQTT.cPubBleUpdateRespTopic, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "BleUpdateResp");
	sprintf(g_xWITMQTT.cPubSenRegisterRespTopic, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "SenRegisterResp");
#endif
	sprintf(g_xWITMQTT.cPubErrorTopic, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "Error");
	sprintf(g_xWITMQTT.cPubRemoteCtrlResp, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "RemoteCtrlResp");
	sprintf(g_xWITMQTT.cPubExtConfigRespTopic, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "ExtConfigResp");
	sprintf(g_xWITMQTT.cPubVibAlarm, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "VibAlarm");
	sprintf(g_xWITMQTT.cPubBackDoorResp, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "BackDoorResp");
	// Subscribe topic
	sprintf(g_xWITMQTT.cSubUpdateTopic, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "Update");
	sprintf(g_xWITMQTT.cSubConfigureTopic, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "Config");
	sprintf(g_xWITMQTT.cSubWorkDetectiveTopic, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "ConfigWorkThr");
#if CONFIG_BLUETOOTH > 0
	sprintf(g_xWITMQTT.cSubBleRegisterTopic, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "SenRegister");
	sprintf(g_xWITMQTT.cSubSensorUpdateTopic, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "BleSensorUpdate");
#endif
#if CONFIG_RS485_RTU > 0
	sprintf(g_xWITMQTT.cSubSensorUpdateTopic, "%s/%s/%08lu/+/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "Update");
#endif
	sprintf(g_xWITMQTT.cSubSyncTimeTopic, "%s/%s/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, "Time");
	sprintf(g_xWITMQTT.cSubTimeResponseTopic, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "Time");
	sprintf(g_xWITMQTT.cSubBackDoorTopic, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "BackDoor");
	sprintf(g_xWITMQTT.cSubRemoteCtrl, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "RemoteCtrl");
	sprintf(g_xWITMQTT.cSubExtConfigureTopic, "%s/%s/%08lu/%s", WIT_MQTT_TOPIC_PERFIX, g_xWITConfig.cModuleName, g_xWITConfig.ulSerialNumber, "ExtConfig");
	// MQTT sokcet id
	g_xWITMQTT.lHandle = MQTT_ClientInit(g_xWITMQTT.cClientID, g_xWITConfig.cMQTTBroker, g_xWITConfig.cMQTTPort, g_xWITConfig.cMQTTUser, g_xWITConfig.cMQTTPassword, WIT_MQTT_KEEP_ALIVE_TIME, WIT_MQTT_TIMEOUT);
	printf("g_xWITMQTT.lHandle %ld\r\n", g_xWITMQTT.lHandle);
}

/**
 * @brief  WIT MQTT connect
 * @param  None
 * @retval None
 */
static bool WIT_MQTTConnect()
{
#if CONFIG_ESP32_WIFI > 0
	// Check if connect to wifi
	if (g_xWITConfig.eOpMode == WIFI_STA)
	{
		unsigned char ucRetry = 10;

		while (ucRetry > 0 && WIFI_IsConnected() == false)
		{
			ucRetry--;
			vTaskDelay(100);
		}

		if (WIFI_IsConnected() == true)
		{
			ucRetry = 5;
			while (ucRetry > 0)
			{
				if (WIFI_GetCurrentRunMode() == WIFI_Mode_Trans)
					WIFI_EnterIntoConfigMode();
				if (WIFI_GetSTANetInfo(g_xCurrentWIFIInfo.cDHCP, g_xCurrentWIFIInfo.cIP, g_xCurrentWIFIInfo.cMask, g_xCurrentWIFIInfo.cGateway, g_xCurrentWIFIInfo.cDNS, WIFI_MAX_NAME_LENGTH) == WIFI_OK)
				{
					g_bGetSTAInfo = true;
					break;
				}

				vTaskDelay(500);
				ucRetry--;
			}

			WIFI_EnterIntoTransMode();
		}
		return MQTT_Connect(g_xWITMQTT.lHandle, WIT_MQTTConnectHandler, g_xWITConfig.cTargetIP, g_xWITConfig.ulPort);
	}

	return false;
#else
	return MQTT_Connect(g_xWITMQTT.lHandle, WIT_MQTTConnectHandler, g_xWITConfig.cTargetIP, g_xWITConfig.ulPort);
#endif
}

/**
 * @brief  WIT MQTT connect callback
 * @param  None
 * @retval None
 */
static void WIT_MQTTConnectHandler(unsigned long ulMQTTID, bool bStatus)
{
	if (bStatus == true)
	{
		// do something
		MQTT_Subscribe(ulMQTTID, g_xWITMQTT.cSubUpdateTopic, QOS0, WIT_GatewayUpdateHandler);
#if CONFIG_BLUETOOTH > 0
		MQTT_Subscribe(ulMQTTID, g_xWITMQTT.cSubSensorUpdateTopic, QOS0, WIT_BleSensorUpdateHandler);
#endif
#if CONFIG_RS485_RTU > 0
		MQTT_Subscribe(ulMQTTID, g_xWITMQTT.cSubSensorUpdateTopic, QOS0, WIT_SensorUpdateHandler);
#endif
		MQTT_Subscribe(ulMQTTID, g_xWITMQTT.cSubSyncTimeTopic, QOS0, WIT_GatewaySyncTimeHandler);
		MQTT_Subscribe(ulMQTTID, g_xWITMQTT.cSubTimeResponseTopic, QOS0, WIT_GatewaySyncTimeHandler);
		MQTT_Subscribe(ulMQTTID, g_xWITMQTT.cSubBackDoorTopic, QOS0, WIT_GatewayBackDoorHandler);
		MQTT_Subscribe(ulMQTTID, g_xWITMQTT.cSubWorkDetectiveTopic, QOS0, WIT_GatewayWorkDetectiveHandler);
		MQTT_Subscribe(ulMQTTID, g_xWITMQTT.cSubRemoteCtrl, QOS0, WIT_GatewayRemoteCtrlHandler);
		MQTT_Subscribe(ulMQTTID, g_xWITMQTT.cSubExtConfigureTopic, QOS0, WIT_GatewayExtConfigHandler);
#if CONFIG_BLUETOOTH > 0
		MQTT_Subscribe(ulMQTTID, g_xWITMQTT.cSubBleRegisterTopic, QOS0, WIT_GatewaySenRegisterHandler);
#endif
		// Wait for work dective package
		vTaskDelay(100);
		MQTT_Subscribe(ulMQTTID, g_xWITMQTT.cSubConfigureTopic, QOS0, WIT_GatewayConfigureHandler);
		// Wait for configure package
		vTaskDelay(100);
	}
}


bool WIT_SendIOTData(void)
{
    cJSON *pxRoot = NULL;
	cJSON *pxObj = NULL;
	unsigned char *pucSendBuffer = NULL;
	unsigned long ulBufferSize = 0;
	bool bStatus = true;
	cJSON_bool bPackageStatus = false;

	pucSendBuffer = MQTT_SendBufferPointer(g_xWITMQTT.lHandle);
	ulBufferSize = MQTT_GetRestSendBufferSize(g_xWITMQTT.lHandle);
	pxRoot = cJSON_CreateArray();
	pxObj = cJSON_CreateObject();
    char addr[8];
    unsigned long now_ts = RTC_InternalGetTimeStamp();

    for(int i = 0; i < IOTDEV_MAX_NUM; i++)
    {
        if(g_xWITConfig.my_IOTDev[i].bIsUsed == true && g_xWITConfig.my_IOTDev[i].bIsTimeout == false)
        {
            sprintf(addr, "%d", i+1);
	        cJSON_AddStringToObject(pxObj, "addr", addr);
            cJSON_AddNumberToObject(pxObj, "iot1", g_xWITConfig.my_IOTDev[i].rope_value / 10.0);
            cJSON_AddNumberToObject(pxObj, "ts", now_ts);
        }
    }
	cJSON_AddItemToArray(pxRoot, pxObj);

	bPackageStatus = cJSON_PrintPreallocated(pxRoot, (char *)pucSendBuffer, ulBufferSize, false);
	if (pxRoot != NULL)
	{
		cJSON_Delete(pxRoot);
		pxRoot = NULL;
	}


	if (bPackageStatus == false || MQTT_Publish(g_xWITMQTT.lHandle, g_xWITMQTT.cPubIOTDataSendTopic, pucSendBuffer, strlen((char *)pucSendBuffer), false, false, QOS1) != true)
	{
		bStatus = false;
	}
	return bStatus;
}

bool WIT_SendIOTError(void)
{
    cJSON *pxRoot = NULL;
	cJSON *pxObj = NULL;
	unsigned char *pucSendBuffer = NULL;
	unsigned long ulBufferSize = 0;
	bool bStatus = true;
	cJSON_bool bPackageStatus = false;

	pucSendBuffer = MQTT_SendBufferPointer(g_xWITMQTT.lHandle);
	ulBufferSize = MQTT_GetRestSendBufferSize(g_xWITMQTT.lHandle);
	pxRoot = cJSON_CreateArray();
    char addr[8];
    unsigned long now_ts = RTC_InternalGetTimeStamp();

    for(int i = 0; i < IOTDEV_MAX_NUM; i++)
    {
        if(g_xWITConfig.my_IOTDev[i].bIsUsed == true && g_xWITConfig.my_IOTDev[i].bIsTimeout == true)
        {
	        pxObj = cJSON_CreateObject();
            sprintf(addr, "%d", i);
	        cJSON_AddStringToObject(pxObj, "addr", addr);
            cJSON_AddNumberToObject(pxObj, "ts", now_ts);
	        cJSON_AddItemToArray(pxRoot, pxObj);
        }
    }

	bPackageStatus = cJSON_PrintPreallocated(pxRoot, (char *)pucSendBuffer, ulBufferSize, false);
	if (pxRoot != NULL)
	{
		cJSON_Delete(pxRoot);
		pxRoot = NULL;
	}

	if (bPackageStatus == false || MQTT_Publish(g_xWITMQTT.lHandle, g_xWITMQTT.cPubIOTErrorSendTopic, pucSendBuffer, strlen((char *)pucSendBuffer), false, false, QOS1) != true)
	{
		bStatus = false;
	}

	return bStatus;
}


bool WIT_SendGatewayFeature(void)
{
    cJSON *pxRoot = NULL;
	cJSON *pxObj = NULL;
	unsigned char *pucSendBuffer = NULL;
	unsigned long ulBufferSize = 0;
	cJSON_bool bPackageStatus = false;
	bool bStatus = true;

	pucSendBuffer = MQTT_SendBufferPointer(g_xWITMQTT.lHandle);
	ulBufferSize = MQTT_GetRestSendBufferSize(g_xWITMQTT.lHandle);
	pxRoot = cJSON_CreateArray();
	pxObj = cJSON_CreateObject();
    unsigned long now_ts = RTC_InternalGetTimeStamp();
    unsigned long worktime = xTaskGetTickCount() / 1000 / 60;
    unsigned long online_num = getIotConnectNum();

    cJSON_AddStringToObject(pxObj, "addr", "0");
    cJSON_AddNumberToObject(pxObj, "ts", now_ts);
    cJSON_AddNumberToObject(pxObj, "period", g_xWITConfig.ulPeriod);
    cJSON_AddNumberToObject(pxObj, "exist", online_num);
    cJSON_AddNumberToObject(pxObj, "workTime", worktime);
	cJSON_AddItemToArray(pxRoot, pxObj);

	bPackageStatus = cJSON_PrintPreallocated(pxRoot, (char *)pucSendBuffer, ulBufferSize, false);
	if (pxRoot != NULL)
	{
		cJSON_Delete(pxRoot);
		pxRoot = NULL;
	}


	if (bPackageStatus == false || MQTT_Publish(g_xWITMQTT.lHandle, g_xWITMQTT.cPubFeatureTopic, pucSendBuffer, strlen((char *)pucSendBuffer), false, false, QOS1) != true)
	{
		bStatus = false;
	}
	return bStatus;
}

/**
 * @brief  WIT check in
 * @param  None
 * @retval None
 */
static bool WIT_CheckIn()
{
	cJSON *pxRoot = NULL;
	cJSON *pxObj = NULL;
	char cSecondaryCode[10];
#if CONFIG_W5100S_SPI_ETHERNET > 0	
	char cBuf[20] = {0};
#endif
	unsigned char *pucSendBuffer = NULL;
	unsigned long ulBufferSize = 0;
	bool bStatus = true;
	cJSON_bool bPackageStatus = false;
#if CONFIG_EC200S_USB_4G > 0
	unsigned char ucSignal = 0;
	unsigned char ucRetry = 5;
#endif

	pucSendBuffer = MQTT_SendBufferPointer(g_xWITMQTT.lHandle);
	ulBufferSize = MQTT_GetRestSendBufferSize(g_xWITMQTT.lHandle);
	pxRoot = cJSON_CreateArray();
	pxObj = cJSON_CreateObject();

	cJSON_AddStringToObject(pxObj, "addr", "0");
#if CONFIG_EC200S_USB_4G > 0
	// Check signal
	pFuncCelluar_GetSignal(&ucSignal, NULL);
	cJSON_AddNumberToObject(pxObj, "signal", ucSignal);
	// Get ccid
	while (ucRetry--)
	{
		if (pFuncCelluar_GetCCID(g_cCCIDBuffer, sizeof(g_cCCIDBuffer)) == Celluar_OK)
			break;
	}
	cJSON_AddStringToObject(pxObj, "ccid", g_cCCIDBuffer);
#endif
#if CONFIG_W5100S_SPI_ETHERNET > 0	
    sprintf(cBuf, "%02x:%02x:%02x:%02x:%02x:%02x ", g_xWITConfig.g_xConfigIp.mac[0], g_xWITConfig.g_xConfigIp.mac[1], g_xWITConfig.g_xConfigIp.mac[2],
		g_xWITConfig.g_xConfigIp.mac[3], g_xWITConfig.g_xConfigIp.mac[4], g_xWITConfig.g_xConfigIp.mac[5]);
    cJSON_AddStringToObject(pxObj, "eth_mac", (char *)cBuf);
	sprintf(cBuf, "%d.%d.%d.%d", g_xWITConfig.g_xConfigIp.ip[0], g_xWITConfig.g_xConfigIp.ip[1], g_xWITConfig.g_xConfigIp.ip[2], g_xWITConfig.g_xConfigIp.ip[3]);
	cJSON_AddStringToObject(pxObj, "eth_ip", (char *)cBuf);
	sprintf(cBuf, "%d.%d.%d.%d", g_xWITConfig.g_xConfigIp.gw[0], g_xWITConfig.g_xConfigIp.gw[1], g_xWITConfig.g_xConfigIp.gw[2], g_xWITConfig.g_xConfigIp.gw[3]);
	cJSON_AddStringToObject(pxObj, "eth_gw", (char *)cBuf);
	sprintf(cBuf, "%d.%d.%d.%d", g_xWITConfig.g_xConfigIp.sn[0], g_xWITConfig.g_xConfigIp.sn[1], g_xWITConfig.g_xConfigIp.sn[2], g_xWITConfig.g_xConfigIp.sn[3]);
	cJSON_AddStringToObject(pxObj, "eth_sub", (char *)cBuf);
#endif
	cJSON_AddStringToObject(pxObj, "version", WIT_VERSION);
	cJSON_AddStringToObject(pxObj, "hardware", "1.0");
	sprintf(cSecondaryCode, "%s%s%s%s%s%s", WIT_ERP_SYSTEM, WIT_PCB_VERSION, WIT_VERSION_NUMBER, WIT_MCU_MODEL, WIT_PT00_STATUS, WIT_BQ_MODEL);

	cJSON_AddStringToObject(pxObj, "subHCode", cSecondaryCode);
#if CONFIG_RS485_RTU > 0
	cJSON_AddNumberToObject(pxObj, "exist", sensorsAvailableStateGet());
#endif
#if CONFIG_BLUETOOTH > 0
	cJSON_AddNumberToObject(pxObj, "exist", getBLEConnectNum());
#endif
	cJSON_AddItemToArray(pxRoot, pxObj);

	bPackageStatus = cJSON_PrintPreallocated(pxRoot, (char *)pucSendBuffer, ulBufferSize, false);
	if (pxRoot != NULL)
	{
		cJSON_Delete(pxRoot);
		pxRoot = NULL;
	}

	if (bPackageStatus == false || MQTT_Publish(g_xWITMQTT.lHandle, g_xWITMQTT.cPubCheckInTopic, pucSendBuffer, strlen((char *)pucSendBuffer), false, false, QOS1) != true)
	{
		bStatus = false;
	}

	return bStatus;
}

static bool WIT_TimeRequest()
{
	cJSON *pxRoot = NULL;
	unsigned char *pucSendBuffer = NULL;
	unsigned long ulBufferSize = 0;
	bool bStatus = true;
	cJSON_bool bPackageStatus = false;

	pucSendBuffer = MQTT_SendBufferPointer(g_xWITMQTT.lHandle);
	ulBufferSize = MQTT_GetRestSendBufferSize(g_xWITMQTT.lHandle);
	pxRoot = cJSON_CreateObject();

	bPackageStatus = cJSON_PrintPreallocated(pxRoot, (char *)pucSendBuffer, ulBufferSize, false);
	if (pxRoot != NULL)
	{
		cJSON_Delete(pxRoot);
		pxRoot = NULL;
	}

	if (bPackageStatus == false || MQTT_Publish(g_xWITMQTT.lHandle, g_xWITMQTT.cPubTimeRequestTopic, pucSendBuffer, strlen((char *)pucSendBuffer), false, false, QOS1) != true)
	{
		bStatus = false;
	}

	return bStatus;
}

/**
 * @brief  package and send gateway feature to server
 * @param  None
 * @retval None
 */
static bool WIT_SendGatewayFeatureToServer(unsigned char groupId, void (*pFuncHandler)(unsigned char, bool))
{
	unsigned long ulLength;
	bool bStatus = true;
	bool bPublishStatus = false;

	if (g_ucKeyTimes == 0
#if CONFIG_ESP32_WIFI > 0
		&& g_xWITConfig.eOpMode == WIFI_STA
#endif
	)
		LED_SetCOMStatus(true);
	MQTT_ChangeTimeout(g_xWITMQTT.lHandle, WIT_MQTT_DATA_TIMEOUT);
	ulLength = WIT_PackageFeatureIntoExtMemory(groupId, 0, (Parameter *)&g_xWITData.xGWFea, g_ulGatewayParameterSize, pFuncHandler ? &g_xWITConfig.xGroupTable[groupId].packid : NULL, pFuncHandler ? g_xWITData.ulTimestamp : RTC_InternalGetTimeStamp(),
											   g_xWITConfig.xGroupTable[groupId].ulWorkMinutes, write_mqtt_feature_data, 0);
	if ((bPublishStatus = MQTT_PublishFromExtMemory(g_xWITMQTT.lHandle, g_xWITMQTT.cPubFeatureTopic, ulLength, false, false, QOS1, read_mqtt_feature_data, 0)) == false)
	{
		bStatus = false;
	}

	if (pFuncHandler != NULL)
		pFuncHandler(groupId, bPublishStatus);

	if (g_ucKeyTimes == 0
#if CONFIG_ESP32_WIFI > 0
		&& g_xWITConfig.eOpMode == WIFI_STA
#endif
	)
		LED_SetCOMStatus(false);

	MQTT_ChangeTimeout(g_xWITMQTT.lHandle, WIT_MQTT_TIMEOUT);
	return bStatus;
}

/**
 * @brief  package and send sensor feature to server
 * @param  None
 * @retval None
 */
static bool WIT_SendSensorFeatureToServer(unsigned char groupId, unsigned char ucSensorAddr, void (*pFuncHandler)(unsigned char, bool))
{
	unsigned long ulLength;
	bool bStatus = true;
	bool bPublishStatus = false;

	// 锁g_xWITData
	xSemaphoreTake(g_xWITDataSignal, portMAX_DELAY);
// 更新轴承齿轮别名
#if CONFIG_RS485_RTU > 0
	WIT_UpdateFeatureName(ucSensorAddr);
#endif
	if (g_ucKeyTimes == 0
#if CONFIG_ESP32_WIFI > 0
		&& g_xWITConfig.eOpMode == WIFI_STA
#endif
	)
		LED_SetCOMStatus(true);
	MQTT_ChangeTimeout(g_xWITMQTT.lHandle, WIT_MQTT_DATA_TIMEOUT);
	ulLength = WIT_PackageFeatureIntoExtMemory(groupId, ucSensorAddr, (Parameter *)&g_xWITData.xSensFea, g_ulSensorParameterSize, &g_xWITConfig.xGroupTable[groupId].packid, g_xWITData.ulTimestamp,
											   g_xWITConfig.xGroupTable[groupId].ulWorkMinutes, write_mqtt_feature_data, 0);
	xSemaphoreGive(g_xWITDataSignal);
	if ((bPublishStatus = MQTT_PublishFromExtMemory(g_xWITMQTT.lHandle, g_xWITMQTT.cPubFeatureTopic, ulLength, false, false, QOS1, read_mqtt_feature_data, 0)) == false)
	{
		bStatus = false;
	}

	if (pFuncHandler != NULL)
		pFuncHandler(groupId, bPublishStatus);

	if (g_ucKeyTimes == 0
#if CONFIG_ESP32_WIFI > 0
		&& g_xWITConfig.eOpMode == WIFI_STA
#endif
	)
		LED_SetCOMStatus(false);
	MQTT_ChangeTimeout(g_xWITMQTT.lHandle, WIT_MQTT_TIMEOUT);
	return bStatus;
}

/**
 * @brief  package and send wave to server
 * @param  None
 * @retval None
 */
bool WIT_SendWaveToServer(unsigned char groupId, unsigned char ucSensorAddr, void (*pFuncHandler)(unsigned char, bool))
{
	unsigned long ulLength;
	bool bStatus = true;
	char cSensorIDStr[4];
	bool bPublishStatus = false;

	sprintf(cSensorIDStr, "%d", ucSensorAddr);

	if (g_ucKeyTimes == 0
#if CONFIG_ESP32_WIFI > 0
		&& g_xWITConfig.eOpMode == WIFI_STA
#endif
	)
		LED_SetCOMStatus(true);
	MQTT_ChangeTimeout(g_xWITMQTT.lHandle, WIT_MQTT_DATA_TIMEOUT);
	// Wave data
	ulLength = WIT_PackageWaveDataIntoExtMemory(ucSensorAddr, &g_xWITData.xSensWave, g_xWITConfig.xGroupTable[groupId].packid, g_xWITData.ulTimestamp, read_ram_wave, write_mqtt_wave_data, 0, 0);
	if ((bPublishStatus = MQTT_PublishFromExtMemory(g_xWITMQTT.lHandle, g_xWITMQTT.cPubDataTopic, ulLength, false, false, QOS1, read_mqtt_wave_data, 0)) == false)
	{
		bStatus = false;
	}

	if (pFuncHandler != NULL)
		pFuncHandler(groupId, bPublishStatus);

	if (g_ucKeyTimes == 0
#if CONFIG_ESP32_WIFI > 0
		&& g_xWITConfig.eOpMode == WIFI_STA
#endif
	)
		LED_SetCOMStatus(false);
	MQTT_ChangeTimeout(g_xWITMQTT.lHandle, WIT_MQTT_TIMEOUT);
	return bStatus;
}

/**
 * @brief  package and send gateway alarm data to server
 * @param  None
 * @retval None
 */
static bool WIT_SendGatewayAlarm(unsigned char groupId, void (*pFuncHandler)(unsigned char, bool))
{
	unsigned long ulLength;
	bool bAlarm = false;
	bool bStatus = true;
	bool bPublishStatus = false;

	// Check gateway alarm
	ulLength = WIT_PackageAlarmIntoExtMemory(0, (Parameter *)&g_xWITData.xGWFea, g_ulGatewayParameterSize, pFuncHandler ? &g_xWITConfig.ulPackageID : NULL, pFuncHandler ? g_xWITData.ulTimestamp : RTC_InternalGetTimeStamp(), write_mqtt_feature_data, 0);
	if (ulLength > 0)
	{
		bAlarm = true;
		if ((bPublishStatus = MQTT_PublishFromExtMemory(g_xWITMQTT.lHandle, g_xWITMQTT.cPubFeatureTopic, ulLength, false, false, QOS1, read_mqtt_feature_data, 0)) == false)
		{
			bStatus = false;
		}
	}

	if (bAlarm == true)
	{
		if (pFuncHandler != NULL)
			pFuncHandler(groupId, bPublishStatus);
	}

	return bStatus;
}

// 预警逻辑: abc温度一组 油温一组 xyz各个特征一组 间隔时间内一组数据不重复预警
static bool WIT_AlarmDetective(unsigned char ucSensorAddr, Parameter *pxParm, Threshold *pxThr, unsigned char *pucMBAlarm, unsigned long ulParmCnt, bool bIsWork, bool bPeriodTimeout)
{
	int i;
	bool bStatus = false;
	AlarmListInfo *pxAlarm;

	if (pxThr == NULL)
		return false;

	if (ucSensorAddr == 0)
	{
		pxAlarm = g_pxGatewayALI;
	}
	else
	{
		pxAlarm = g_pxSensorALI + (ucSensorAddr - 1) * g_ulSensorAlarmParameterSize * sizeof(AlarmListInfo);
	}

	for (i = 0; i < ulParmCnt; i++)
	{
		bool bHighAlarm = false, bLowAlarm = false;

		pxParm->bIsHighLimitAlarm = false;
		pxParm->bIsLowLimitAlarm = false;
		if ((g_xWITConfig.eGatewayMode != GatewayMode_Learning) || (g_xWITConfig.eGatewayMode == GatewayMode_Learning && bIsWork == true))
		{
		}

		if (bHighAlarm == true || bLowAlarm == true)
		{
			*pucMBAlarm |= (1 << ucSensorAddr);

			if (WIT_IsImportantParameter(pxParm->cName) == true)
			{
				// 预警计数
				if (pxAlarm[pxParm->ucAlarmAddressIndex].ucAlarmCount < 255)
					pxAlarm[pxParm->ucAlarmAddressIndex].ucAlarmCount++;

				if (pxAlarm[pxParm->ucAlarmAddressIndex].ucAlarmCount >= WIT_ALARM_COUNT_THRESHOLD)
				{
					pxParm->bIsHighLimitAlarm = true;
					pxParm->bIsLowLimitAlarm = true;
					bStatus = true;
				}
			}
		}
		else
		{
			*pucMBAlarm &= ~(1 << ucSensorAddr);

			// 预警清空
			pxAlarm[pxParm->ucAlarmAddressIndex].ucAlarmCount = 0;
		}

		pxParm++;
		pxThr++;
		pucMBAlarm++;
	}

	return bStatus;
}

/**
 * @brief  Package feature data into external memory
 * @param  None
 * @retval length
 */
static unsigned long WIT_PackageFeatureIntoExtMemory(unsigned char groupId, unsigned char ucSensorAddr, Parameter *pxParm, unsigned long ulParmCnt, unsigned long *pulPackageID, unsigned long ulTimeStamp, unsigned long ulWorkTime,
													 void (*pFuncMemoryWrite)(unsigned long, unsigned char *, unsigned long), unsigned long ulMemoryWriteAddress)
{
	char cStr[512];
	unsigned long ulStrIndex = 0;
	unsigned long ulLength = 0;
	unsigned long ulWriteHeader = ulMemoryWriteAddress;
	int i;
	uint32_t sensorExistState = 0;

#if CONFIG_BLUETOOTH > 0
	unsigned char usPoint = 0;
	sGroupTable *pxGroupTable = NULL;
	usPoint = ucSensorAddr;

	pxGroupTable = getGroupFromGroup(groupId + 1);
	sensorExistState = pxGroupTable->ulConnectNum;
#endif
#if CONFIG_RS485_RTU > 0
	sensorExistState = sensorsAvailableStateGet();
#endif
#if CONFIG_EC200S_USB_4G > 0
    unsigned char celluarSignal = 0;
	pFuncCelluar_GetSignal(&celluarSignal, NULL);
#endif
	if (pFuncMemoryWrite == NULL)
		return 0;

#if CONFIG_BLUETOOTH > 0
	sprintf(cStr, "[{\"addr\":\"%d\",", usPoint);
#endif
#if CONFIG_RS485_RTU > 0
	sprintf(cStr, "[{\"addr\":\"%d\",", ucSensorAddr);
#endif
	ulStrIndex = strlen(cStr);
	if (pulPackageID != NULL)
	{
		sprintf(cStr + ulStrIndex, "\"id\":%lu,", *pulPackageID);
		ulStrIndex = strlen(cStr);
	}

	// Head
	if (ucSensorAddr != 0)
	{
		unsigned short highTempProtect = 2;
#if CONFIG_RS485_RTU > 0
		WIT_SensorOperation(ucSensorAddr, Acc_GetHighTempProtect, WIT_ACC_SENSOR_ID, &highTempProtect);
#endif
		sprintf(cStr + ulStrIndex,
				"\"groupId\":%ld,\"groupNo\":\"%s\",\"ts\":%lu,\"sr\":%d,\"sc\":%lu,"
#if CONFIG_BLUETOOTH > 0
				"\"sn\":%s,\"type\":\"%s\",\"subHCode\":\"%s\",\"logo\":\"%s\",\"version\":\"%d.%d.%d\","
#endif
#if CONFIG_RS485_RTU > 0
				"\"sn\":%lu,\"ver\":%d,\"version\":\"%x.%x\","
#endif
				"\"hasReal\":%lu,\"hpfEn\":%u,\"resolution\":%.2f,\"highTempProtect\":%d",
				pxGroupTable->ulGroup, pxGroupTable->groupNo, ulTimeStamp, g_usAccSamplingRate[ucSensorAddr - 1], g_xWITData.xSensWave.xWave.ulWaveCount[ucSensorAddr - 1],
#if CONFIG_BLUETOOTH > 0
				g_cSensorSerialNumber[usPoint - 1], g_cSensorType[usPoint - 1], g_cSensorChildVersion[usPoint - 1], g_cSensorLogoType[usPoint - 1], g_cSensorVersion[usPoint - 1][0], g_cSensorVersion[usPoint - 1][1], g_cSensorVersion[usPoint - 1][2],
#endif
#if CONFIG_RS485_RTU > 0
				g_ulSensorSerialNumber, g_usSensorVersion, g_usSensorVersion >> 8, g_usSensorVersion & 0x00ff,
#endif
				g_ulUploadTheWaveform, g_xWITData.xSensFea.hpfEn, (g_xWITData.xSensFea.resolution / 100.0),
				highTempProtect);
	}
	else
	{
		sprintf(cStr + ulStrIndex, "\"groupId\":%ld,\"groupNo\":\"%s\",\"ts\":%lu,\"version\":\"%s\","
#if CONFIG_EC200S_USB_4G > 0
								   "\"ccid\":\"%s\","
								   "\"signal\":\"%d\","
#endif
								   "\"exist\":%u,\"workTime\":%lu,\"mode\":%d,\"learn\":%d,\"total\":%d,\"period\":%lu,\"wPeriod\":%lu,\"delay\":%lu,\"run\":%lu,\"spdSel\":%lu",
				pxGroupTable->ulGroup, pxGroupTable->groupNo, ulTimeStamp, WIT_VERSION,
#if CONFIG_EC200S_USB_4G > 0
				g_cCCIDBuffer,
				celluarSignal,
#endif
				(int)sensorExistState, ulWorkTime,
				g_xWITConfig.eGatewayMode, g_xWITConfig.usCurrentLearningCount, g_xWITConfig.usTotalLearningCount,
				pxGroupTable->ulFeaPeriod, pxGroupTable->ulPeriod, pxGroupTable->ulWorkMinutes, g_ulUploadRunState, g_xWITConfig.spdSel
				// debug used
		);
	}
	pFuncMemoryWrite(ulMemoryWriteAddress, (unsigned char *)cStr, strlen(cStr));
	ulMemoryWriteAddress += strlen(cStr);
	if (pxParm != NULL)
	{
		for (i = 0; i < ulParmCnt; i++)
		{
			if (pxParm->bIsUsed == true)
			{
				sprintf(cStr, ",\"%s\":%.2f", pxParm->cName, pxParm->fValue);
				pFuncMemoryWrite(ulMemoryWriteAddress, (unsigned char *)cStr, strlen(cStr));
				ulMemoryWriteAddress += strlen(cStr);
			}
			pxParm++;
		}
	}
	// Tail
	pFuncMemoryWrite(ulMemoryWriteAddress, (unsigned char *)"}]", strlen("}]"));
	ulMemoryWriteAddress += strlen("}]");
	ulLength = ulMemoryWriteAddress - ulWriteHeader;
	return ulLength;
}

/**
 * @brief  Package wave data into external memory
 * @param  None
 * @retval length
 */
static unsigned long WIT_PackageWaveDataIntoExtMemory(unsigned char ucSensorAddr, Sensor_Wave *pxWave, unsigned long ulPackageID, unsigned long ulTimeStamp,
													  void (*pFuncMemoryRead)(unsigned int, unsigned long, unsigned char *, unsigned long), void (*pFuncMemoryWrite)(unsigned long, unsigned char *, unsigned long),
													  unsigned long ulMemoryReadAddress, unsigned long ulMemoryWriteAddress)
{
	char cStr[256];
	unsigned long ulLength = 0;
	unsigned long ulWriteHeader = ulMemoryWriteAddress;
	unsigned long ulReadHeader = ulMemoryReadAddress;
	unsigned long ulTotalSize;
	unsigned char usPoint = 0;
	int i, j;
	if (pFuncMemoryWrite == NULL)
		return 0;

	usPoint = ucSensorAddr;

	// #if CONFIG_BLUETOOTH > 0
	// 	usPoint = gSenInfo.xDevInfo[ucSensorAddr - 1].addr;
	// #endif
	// Head
	sprintf(cStr, "[{\"addr\":\"%d\",\"id\":%lu,\"ts\":%lu"
		#if CONFIG_BLUETOOTH > 0
			",\"sn\":%s,\"type\":\"%s\",\"subHCode\":\"%s\",\"logo\":\"%s\",\"version\":\"%d.%d.%d\",\"sr\":%d,\"sc\":%lu,\"rssi\":%d,\"gtms\":%d"
		#endif
			,usPoint, ulPackageID, ulTimeStamp
		#if CONFIG_BLUETOOTH > 0
			,g_cSensorSerialNumber[usPoint - 1], g_cSensorType[usPoint - 1], g_cSensorChildVersion[usPoint - 1], g_cSensorLogoType[usPoint - 1]
			,g_cSensorVersion[usPoint - 1][0], g_cSensorVersion[usPoint - 1][1], g_cSensorVersion[usPoint - 1][2]
			,g_usAccSamplingRate[usPoint - 1], g_xWITData.xSensWave.xWave.ulWaveCount[usPoint - 1], gSenInfo.xDevInfo[usPoint - 1].rssi
			,gSenInfo.xDevInfo[usPoint - 1].gtms
		#endif
	);

	pFuncMemoryWrite(ulMemoryWriteAddress, (unsigned char *)cStr, strlen(cStr));
	ulMemoryWriteAddress += strlen(cStr);
	for (i = 0; i < ACC_AXIS_COUNT; i++)
	{
		bool bFlag = true;
        if( strstr(g_cSensorType[usPoint - 1], "3C") != NULL)
        {
		    ulMemoryReadAddress = ulReadHeader + i * pxWave->xWave.ulWaveCount[ucSensorAddr - 1] * 2;
        }
        else
        {
		    ulMemoryReadAddress = ulReadHeader + i * pxWave->xWave.ulWaveCount[ucSensorAddr - 1] * 4;
        }
		ESP_LOGD("wit", "ulMemoryReadAddress = %d", (int)ulMemoryReadAddress);
		// Axis head
		if (i == 0)
		{
			pFuncMemoryWrite(ulMemoryWriteAddress, (unsigned char *)",\"x\":[", strlen(",\"x\":["));
			ulMemoryWriteAddress += strlen(",\"x\":[");
		}
		else if (i == 1)
		{
			pFuncMemoryWrite(ulMemoryWriteAddress, (unsigned char *)",\"y\":[", strlen(",\"y\":["));
			ulMemoryWriteAddress += strlen(",\"y\":[");
		}
		else if (i == 2)
		{
			pFuncMemoryWrite(ulMemoryWriteAddress, (unsigned char *)",\"z\":[", strlen(",\"z\":["));
			ulMemoryWriteAddress += strlen(",\"z\":[");
		}

		ulTotalSize = pxWave->xWave.ulWaveCount[ucSensorAddr - 1];

        if( strstr(g_cSensorType[usPoint - 1], "3C") != NULL)
        {
            for (j = 0; j < ulTotalSize; j++)
            {
                short sWave;
                float fWave = 0.0f;
                pFuncMemoryRead((ucSensorAddr - 1) % 24, ulMemoryReadAddress + (j * 2), ((unsigned char *)&sWave), 2);
                fWave = sWave * IIS3DWB_FACTOR;
                // if(fWave < -10000)
                // 	ESP_LOGD("text","sWave %d", sWave);
                if (bFlag == true)
                {
                    bFlag = false;
                    fWave = ((float)sWave) * IIS3DWB_FACTOR;
                    sprintf(cStr, "%.2f", fWave);
                    pFuncMemoryWrite(ulMemoryWriteAddress, (unsigned char *)cStr, strlen(cStr));
                    ulMemoryWriteAddress += strlen(cStr);
                }
                else
                {
                    sprintf(cStr, ",%.2f", fWave);
                    pFuncMemoryWrite(ulMemoryWriteAddress, (unsigned char *)cStr, strlen(cStr));
                    ulMemoryWriteAddress += strlen(cStr);
                }
            }
        }
        else
        {
            for (j = 0; j < ulTotalSize; j++)
            {
                long sWave;
                float fWave = 0.0f;
                pFuncMemoryRead((ucSensorAddr - 1) % 24, ulMemoryReadAddress + (j * 4), ((unsigned char *)&sWave), 4);
                fWave = sWave / ADXL357_FACTOR;
                if (bFlag == true)
                {
                    bFlag = false;
                    fWave = sWave / ADXL357_FACTOR;
                    sprintf(cStr, "%.2f", fWave);
                    pFuncMemoryWrite(ulMemoryWriteAddress, (unsigned char *)cStr, strlen(cStr));
                    ulMemoryWriteAddress += strlen(cStr);
                }
                else
                {
                    sprintf(cStr, ",%.2f", fWave);
                    pFuncMemoryWrite(ulMemoryWriteAddress, (unsigned char *)cStr, strlen(cStr));
                    ulMemoryWriteAddress += strlen(cStr);
                }
            }
        }
            
		// Axis tail
		pFuncMemoryWrite(ulMemoryWriteAddress, (unsigned char *)"]", strlen("]"));
		ulMemoryWriteAddress += strlen("]");
    }
    // Tail
	pFuncMemoryWrite(ulMemoryWriteAddress, (unsigned char *)"}]", strlen("}]"));
	ulMemoryWriteAddress += strlen("}]");

	ulLength = ulMemoryWriteAddress - ulWriteHeader;
	ESP_LOGD("wit", "mqtt size = %d", (int)ulLength);
	return ulLength;
}

/**
 * @brief  Package feature data into external memory
 * @param  None
 * @retval length
 */
static unsigned long WIT_PackageAlarmIntoExtMemory(unsigned char ucSensorAddr, Parameter *pxParm, unsigned long ulParmCnt, unsigned long *pulPackageID, unsigned long ulTimeStamp,
												   void (*pFuncMemoryWrite)(unsigned long, unsigned char *, unsigned long), unsigned long ulMemoryWriteAddress)
{
	char cStr[64];
	unsigned long ulLength = 0;
	unsigned long ulWriteHeader = ulMemoryWriteAddress;
	int i;
	bool bHeadPackaged = false;
	AlarmListInfo *pxAlarm;

	if (pFuncMemoryWrite == NULL)
		return 0;

	if (ucSensorAddr == 0)
	{
		pxAlarm = g_pxGatewayALI;
	}
	else
	{
		pxAlarm = g_pxSensorALI + (ucSensorAddr - 1) * g_ulSensorAlarmParameterSize * sizeof(AlarmListInfo);
	}
	if (pxParm != NULL)
	{
		for (i = 0; i < ulParmCnt; i++)
		{
			if (pxParm->bIsHighLimitAlarm == true ||
				pxParm->bIsLowLimitAlarm == true)
			{
				// 预警数量清0
				pxAlarm[pxParm->ucAlarmAddressIndex].ucAlarmCount = 0;

				if (bHeadPackaged == false)
				{
					// Head
					if (pulPackageID != NULL)
						sprintf(cStr, "[{\"addr\":\"%d\",\"id\":%lu,\"ts\":%lu", ucSensorAddr, *pulPackageID, ulTimeStamp);
					else
						sprintf(cStr, "[{\"addr\":\"%d\",\"ts\":%lu", ucSensorAddr, ulTimeStamp);
					pFuncMemoryWrite(ulMemoryWriteAddress, (unsigned char *)cStr, strlen(cStr));
					ulMemoryWriteAddress += strlen(cStr);
					bHeadPackaged = true;
				}

				sprintf(cStr, ",\"%s\":1", pxParm->cName);
				pFuncMemoryWrite(ulMemoryWriteAddress, (unsigned char *)cStr, strlen(cStr));
				ulMemoryWriteAddress += strlen(cStr);
			}
			pxParm++;
		}
	}
	// Tail
	pFuncMemoryWrite(ulMemoryWriteAddress, (unsigned char *)"}]", strlen("}]"));
	ulMemoryWriteAddress += strlen("}]");

	ulLength = ulMemoryWriteAddress - ulWriteHeader;

	return ulLength;
}

#if CONFIG_RS485_RTU > 0
static unsigned long WIT_PackageThresholdIntoExtMemory(unsigned char ucSensorAddr, Parameter *pxParm, Threshold *pxThr, DataListInfo *pxDLI, unsigned long ulParmCnt, unsigned long ulTimeStamp,
													   void (*pFuncMemoryWrite)(unsigned long, unsigned char *, unsigned long), unsigned long ulMemoryWriteAddress)
{
	char cStr[64];
	unsigned long ulLength = 0;
	unsigned long ulWriteHeader = ulMemoryWriteAddress;
	int i;
	unsigned char ucNoThr[] = {0xFF, 0xFF, 0xFF, 0xFF};

	if (pFuncMemoryWrite == NULL)
		return 0;

	// Head
	sprintf(cStr, "[{\"addr\":\"%d\"", ucSensorAddr);
	pFuncMemoryWrite(ulMemoryWriteAddress, (unsigned char *)cStr, strlen(cStr));
	ulMemoryWriteAddress += strlen(cStr);

	for (i = 0; i < ulParmCnt; i++)
	{
		if (pxDLI->bUsed == true)
		{
			if (memcmp(&pxThr->fHighLimit, ucNoThr, sizeof(float)) != 0 &&
				memcmp(&pxThr->fLowLimit, ucNoThr, sizeof(float)) != 0)
			{
				sprintf(cStr, ",\"%s\":{\"h\":%.2f,\"l\":%.2f}", pxParm->cName, pxThr->fHighLimit, pxThr->fLowLimit);
				pFuncMemoryWrite(ulMemoryWriteAddress, (unsigned char *)cStr, strlen(cStr));
				ulMemoryWriteAddress += strlen(cStr);
			}
		}
		pxParm++;
		pxThr++;
		pxDLI++;
	}
	// Tail
	pFuncMemoryWrite(ulMemoryWriteAddress, (unsigned char *)"}]", strlen("}]"));
	ulMemoryWriteAddress += strlen("}]");

	ulLength = ulMemoryWriteAddress - ulWriteHeader;

	return ulLength;
}
#endif

/**
 * @brief  FreeRTOS timer handler
 * @param  xTimer
 * @retval None
 */
static void WIT_TimerHandler(TimerHandle_t xTimer)
{
	if (xTimer == g_xKeyTimer)
	{
		g_ucKeyTimes++;
		ESP_LOGD("timer", "g_ucKeyTimes %d", g_ucKeyTimes);
		if(g_ucKeyTimes >= 3)
		{
			__Resetting_led();
		}
	}
	else if (xTimer == g_xWorkTimer)
	{
		for (int i = 0; i < WIT_SENSOR_GROUP_MAX_NUM; i++)
		{
			if (g_bIsMotorWork[i] == true)
			{
				g_xWITConfig.xGroupTable[i].ulWorkMinutes++;
			}
		}
	}
	else if (xTimer == g_xDLICalTimer)
	{
		g_bDLISend = true;
		g_bDLICalculate = true;
	}
    else if (xTimer == g_xDataSendTimer)
	{
		g_bDataSend_TimerArr = true;
	}
    else if(xTimer == g_xErrorSendTimer)
    {
		g_bErrorSend_TimerArr = true;
    }
	else if (xTimer == g_xCfgUpTimer || xTimer == g_xCfgQuickUpTimer)
	{
		g_bUpdateConfigureArea = true;
	}
	else if (xTimer == g_xThrUpTimer)
	{
		g_bUpdateThresholdArea = true;
	}
	else if (xTimer == g_xSenConfigTimer)
	{
		g_bSenorConfigArea = true;
	}
	else if (xTimer == g_xDLIUpTimer)
	{
		g_bUpdateDLIArea = true;
	}
	else if (xTimer == g_xGatewayFeatureTimer)
	{
		g_bTimeSyncFlag = true;
		// g_bGWFeaHeart = true;
	}
	else if (xTimer == g_xSensorFeatureTimer)
	{
		g_bCanSendGatewayFeature = true;
		g_bCanSendSensorFeature = true;
	}
	else if (xTimer == g_xWaveTimer)
	{
		g_bCanSendWave = true;
	}
	else if (xTimer == g_xAlarmTimer)
	{
		g_bCanSendAlarm = true;
	}
	else if (xTimer == g_xCheckInTimer)
	{
		g_bAlreadyCheckIn = false;
	}
	else if (xTimer == g_xPushTimer)
	{
		for (int i = 0; i < WIT_SENSOR_GROUP_MAX_NUM; i++)
		{
			if (g_xWITConfig.xGroupTable[i].ulGUseEnable == 1)
			{
				g_xWITConfig.xGroupTable[i].ulPeriodCnt++;
				g_xWITConfig.xGroupTable[i].ulFeaPeriodCnt++;
				if (g_xWITConfig.xGroupTable[i].ulPeriodCnt >= g_xWITConfig.xGroupTable[i].ulPeriod)
				{
					g_xWITConfig.xGroupTable[i].ulWavePushEnable = 1;
					g_xWITConfig.xGroupTable[i].ulPeriodCnt = 0;
				}
				if (g_xWITConfig.xGroupTable[i].ulFeaPeriodCnt >= g_xWITConfig.xGroupTable[i].ulFeaPeriod)
				{
					g_bHeartPack[i] = true;
					g_xWITConfig.xGroupTable[i].ulFeaPushEnable = 1;
					g_xWITConfig.xGroupTable[i].ulFeaPeriodCnt = 0;
				}
			}
		}
	}
}

/**
 * @brief  Gateway update handler
 * @param  xTimer
 * @retval None
 */
static void WIT_GatewayUpdateHandler(MessageData *pxData)
{
	if (UDP_GetUpdateStatus() == false)
		Update_GatewayHandler(pxData->message->payload, pxData->message->payloadlen);
}

#if CONFIG_BLUETOOTH > 0
static void WIT_BleSensorUpdateHandler(MessageData *pxData)
{
	if (UDP_GetUpdateStatus() == false)
		Sensor_UpdateHandler(pxData->message->payload, pxData->message->payloadlen);
}
#endif
#if CONFIG_RS485_RTU > 0
static void WIT_SensorUpdateHandler(MessageData *pxData)
{
	unsigned char ucSensorAddr;
	char *pcAddr = pxData->topicName->lenstring.data;
	unsigned char ucFindTimes = 3;

	if (UDP_GetSensorUpdateStatus() == true)
	{
		g_xSensorResponse.ucStatus = MB_EIO;
		g_xSensorResponse.ucMBCnt = 0;

		return;
	}

	while (ucFindTimes--)
	{
		if ((pcAddr = strstr(pcAddr, "/")) != NULL)
		{
			pcAddr += 1;
		}
		else
		{
			break;
		}
	}

	if (pcAddr != NULL)
	{
		// Get address
		ucSensorAddr = atol(pcAddr);

		if (ucSensorAddr <= ACC_MAX_SENSOR_COUNT && ucSensorAddr != 0)
		{
			xSemaphoreTake(g_xMBSignal[(ucSensorAddr - 1) / WIT_PER_PORT_SENSOR_COUNT], portMAX_DELAY);
			vMBSetTimeout(WIT_MB_UPDATE_TIMEOUT);
			WIT_AccEnable(ucSensorAddr, WIT_ACC_SENSOR_ID);
			g_xSensorResponse.ucStatus = Sensor_UpdateHandler(ucSensorAddr, WIT_ACC_SENSOR_ID, pxData->message->payload, pxData->message->payloadlen,
															  g_xSensorResponse.ucMBData + 1, &g_xSensorResponse.ucMBCnt);
			vMBSetTimeout(WIT_MB_NORMAL_TIMEOUT);
			if (Sensor_GetFinishedStatus() == true)
			{
				// Wait for sensor reset
				vTaskDelay(5000);
				WIT_AccEnable(ucSensorAddr, WIT_ACC_SENSOR_ID);
			}
			xSemaphoreGive(g_xMBSignal[(ucSensorAddr - 1) / WIT_PER_PORT_SENSOR_COUNT]);
		}
		else
		{
			g_xSensorResponse.ucStatus = MB_EINVAL;
			g_xSensorResponse.ucMBCnt = 0;
		}
	}
}
#endif




#if 1

// 解析字符串"12345678"为id(0x1234)和sn(0x5678)
void stringIdSnToShorts(const char* str, unsigned short* id, unsigned short* sn)
{
    char _id[5];
    char _sn[5]; 

    if(strlen(str) != 8)
    {
        printf("stringIdSnToShorts Fail");
        *id = 0;
        *sn = 0;
        return;
    }

    for(uint8_t i = 0; i < 8; i++)
    {
        if( i < 4)
            _id[i] = str[i];
        else
            _sn [i - 4] = str[i];
    }
    _id[4] = '\0'; 
    _sn[4] = '\0';

    *id = (unsigned short)strtol(_id, NULL, 16); // ID是前4个字符转换得到的值
    *sn = (unsigned short)strtol(_sn, NULL, 16); // SN是后4个字符转换得到的值
    
    // printf("id = %04x, sn = %04x\n", *id, *sn);
}

static void WIT_GatewayConfigureHandler(MessageData *pxData)
{
	DJSON xRoot;
	DJSON xSSArray;
	DJSON xSSObj;
	DJSON xCfgArray;
	DJSON xCfgObj;
    DJSON xDevArray;
	DJSON xDevObj;
	DJSON xItem;

    unsigned long ulSSArraySize = 0;
	unsigned char ucAddr = 0;
	char cAddr[8] = {0};
    char cDev[16] = {0};
    uint8_t dev_num = 0;
	bool bStatus = true;
    bool bNeedUpdateFTimer = false;
	bool bNeedUpdateCfg = false;

	xRoot.pcItemPointer = pxData->message->payload;
	xRoot.ulItemLength = pxData->message->payloadlen;
	xRoot.eItemType = DJSON_Object;

    // printf("WIT_GATEWAYCONFIG\n");

	// Get taskId
	memset(g_xGatewayResponse.cTaskID, 0, sizeof(g_xGatewayResponse.cTaskID));
	xItem = DJSON_GetObjectItem(&xRoot, "taskId");
	DJSON_GetValueString(&xItem, g_xGatewayResponse.cTaskID, sizeof(g_xGatewayResponse.cTaskID));

	// Get ss array
	xSSArray = DJSON_GetObjectItem(&xRoot, "ss");
	ulSSArraySize = DJSON_GetArraySize(&xSSArray);

    if (ulSSArraySize == 0 || xSSArray.eItemType != DJSON_Array)
	{
		bStatus = false;
		goto judge_status;
	}
	// Get addr
	xSSObj = DJSON_GetArrayItem(&xSSArray, 0);
	xItem = DJSON_GetObjectItem(&xSSObj, "addr");

	if (xItem.eItemType != DJSON_String)
	{
		bStatus = false;
		goto judge_status;
	}
	DJSON_GetValueString(&xItem, cAddr, sizeof(cAddr));

    // Get cfg
	xCfgArray = DJSON_GetObjectItem(&xSSObj, "cfg");
	if (xCfgArray.eItemType != DJSON_Array)
	{
		bStatus = false;
		goto judge_status;
	}

	xCfgObj = DJSON_GetArrayItem(&xCfgArray, 0);
	strncpy(g_xGatewayResponse.cAddr, cAddr, 4);
    ucAddr = atoi(cAddr);
	if (ucAddr <= IOTDEV_MAX_NUM)
	{
		Parameter *pxParm;
		Threshold *pxThr;
		// 网关配置
		if (ucAddr == 0)
		{
			// 数据与特征发送周期
			xItem = DJSON_GetObjectItem(&xCfgObj, "period");
			if (xItem.eItemType == DJSON_Number)
			{
				unsigned long ulSetPeriod;

				ulSetPeriod = DJSON_GetValueInt(&xItem) * 1000; // MS   -> S
				g_xWITConfig.ulPeriod = ulSetPeriod;
				g_xWITConfig.ulFeaturePeriodCopy = ulSetPeriod;
				bNeedUpdateFTimer = true;
				bNeedUpdateCfg = true;
			}

            // IOT阈值设定
            xItem = DJSON_GetObjectItem(&xCfgObj, "threshold");
			if (xItem.eItemType == DJSON_Number)
			{
				unsigned long ulSetThreshold;

				ulSetThreshold = DJSON_GetValueDouble(&xItem) * 10; // 4% 下发 40，0x00 0x28 
                if(ulSetThreshold > 1000 || ulSetThreshold < 0)
                {
                    printf("Set Threshold Error.\n");
                    bStatus = false;
                    goto judge_status;
                }
                printf("Threshold %ld\n", ulSetThreshold);

                if(IOTConfigFunction())
                {
                   bStatus = true;
                   printf("IOTConfigFunction true\n");
                }
                else
                {
                    bStatus = false;
                    goto judge_status;
                }
				// g_beingConfigIOT.ConfigFlag = true;
				g_xWITConfig.my_IOTThreshold = ulSetThreshold;
                bNeedUpdateCfg = true;
			}


			// 解析设备[id --- sn,id]
            xDevArray = DJSON_GetObjectItem(&xCfgObj, "dev");
            dev_num = DJSON_GetArraySize(&xDevArray); // 计算设备数量
            // printf("dev has %d\n", dev_num);
            if(dev_num > IOTDEV_MAX_NUM)
            {
                dev_num = IOTDEV_MAX_NUM;
                printf("dev_num over than IOTDEV_MAX_NUM\n");
                bStatus = false;
                goto judge_status;
            }
            if (xDevArray.eItemType == DJSON_Array)
            {
                for(int i = 0; i < dev_num; i++)
                {
                    xDevObj = DJSON_GetArrayItem(&xDevArray, i);
                    if (xDevObj.eItemType == DJSON_Object)
                    {
                        xItem = DJSON_GetObjectItem(&xDevObj, "addr");
                        if (xItem.eItemType == DJSON_String)
			            {
	                        DJSON_GetValueString(&xItem, cAddr, sizeof(cAddr));
                            g_xWITConfig.my_IOTDev[i].addr = atoi(cAddr);
                        }

                        xItem = DJSON_GetObjectItem(&xDevObj, "id");// 解析sn,id
                        if (xItem.eItemType == DJSON_String)
			            {
	                        DJSON_GetValueString(&xItem, cDev, sizeof(cDev));
                            stringIdSnToShorts(cDev, &g_xWITConfig.my_IOTDev[i].id, &g_xWITConfig.my_IOTDev[i].sn);
                            if(0 == g_xWITConfig.my_IOTDev[i].id || 0 == g_xWITConfig.my_IOTDev[i].sn)
                            {
                                printf("Config id/sn Fail\n");
                                goto judge_status;
                            }
                            printf("ID: 0X%04X; SN: 0X%04X\n", g_xWITConfig.my_IOTDev[i].id, g_xWITConfig.my_IOTDev[i].sn);
                            g_xWITConfig.my_IOTDev[i].bIsUsed = true;
                            g_xWITConfig.my_IOTDev[i].bIsTimeout = true;
                            g_xWITConfig.my_IOTDev[i].bIsPlugIn = false; // 此处是否插入标志为false
                            g_xWITConfig.my_IOTDev[i].rope_value = 0;
                            printf("dev[%d] Configured\n", i);
                        }
                    }
                }
            }
        }

        // 更新定时器
		if (bNeedUpdateFTimer == true)
		{
			xTimerChangePeriod(g_xDataSendTimer, g_xWITConfig.ulPeriod, 1000);
			xTimerChangePeriod(xSensorDataFreshTimer, g_xWITConfig.ulPeriod, 1000);
            printf("xTimerChangePeriod g_xDataSendTimer %ld\n", g_xWITConfig.ulPeriod);
		}

        // 更新数据 开始定时器
		if (bNeedUpdateCfg == true)
			xTimerStart(g_xCfgUpTimer, 1000);

    }

judge_status:

	g_xGatewayResponse.pcTopic = g_xWITMQTT.cPubConfigRespTopic;
	if (bStatus == true)
	{
		g_xGatewayResponse.eStatus = Resp_Success;
	}
	else
	{
		g_xGatewayResponse.eStatus = Resp_ParmErr;
	}
}

#else
#if CONFIG_RS485_RTU > 0
static void WIT_GatewayConfigureHandler(MessageData *pxData)
{
	DJSON xRoot;
	DJSON xSSArray;
	DJSON xSSObj;
	DJSON xCfgArray;
	DJSON xCfgObj;
	DJSON xLimit;
	DJSON xBearingObj;
	DJSON xGearObj;
	DJSON xArray;
	DJSON xItem;
	unsigned long ulSSArraySize;
	unsigned char ucAddr;
	char cAddr[8];
	int j;
	bool bStatus = true;
	bool bNeedUpdateDLI = false;
	bool bNeedUpdateCfg = false;
	bool bNeedUpdateThr = false;
	bool bNeedUpdateFTimer = false;
	bool bNeedUpdateWTimer = false;
	static bool s_bUseThrFromPlatform = false;

	xRoot.pcItemPointer = pxData->message->payload;
	xRoot.ulItemLength = pxData->message->payloadlen;
	xRoot.eItemType = DJSON_Object;

	// Get taskId
	memset(g_xGatewayResponse.cTaskID, 0, sizeof(g_xGatewayResponse.cTaskID));
	xItem = DJSON_GetObjectItem(&xRoot, "taskId");
	DJSON_GetValueString(&xItem, g_xGatewayResponse.cTaskID, sizeof(g_xGatewayResponse.cTaskID));

	// Get ss array
	xSSArray = DJSON_GetObjectItem(&xRoot, "ss");
	ulSSArraySize = DJSON_GetArraySize(&xSSArray);

	if (ulSSArraySize == 0 || xSSArray.eItemType != DJSON_Array)
	{
		bStatus = false;
		goto json_parse_end;
	}
	// Get addr
	xSSObj = DJSON_GetArrayItem(&xSSArray, 0);
	xItem = DJSON_GetObjectItem(&xSSObj, "addr");
	if (xItem.eItemType != DJSON_String)
	{
		bStatus = false;
		goto json_parse_end;
	}
	DJSON_GetValueString(&xItem, cAddr, sizeof(cAddr));

	// Get cfg
	xCfgArray = DJSON_GetObjectItem(&xSSObj, "cfg");
	if (xCfgArray.eItemType != DJSON_Array)
	{
		bStatus = false;
		goto json_parse_end;
	}

	xCfgObj = DJSON_GetArrayItem(&xCfgArray, 0);
	// 获取配置地址 0代表网关自身 1-6代表传感器 all代表全部
	strncpy(g_xGatewayResponse.cAddr, cAddr, 4);
	if (strcmp(cAddr, "all") == 0)
	{
		// 发送广播地址配置全部
		ucAddr = 0xFF;
	}
	else
	{
		ucAddr = atoi(cAddr);
	}

	// 检查地址是否在范围内
	if (ucAddr <= ACC_MAX_SENSOR_COUNT)
	{
		Parameter *pxParm;
		Threshold *pxThr;
		// 网关配置
		if (ucAddr == 0)
		{
			// 振动读取延时
			xItem = DJSON_GetObjectItem(&xCfgObj, "delay");
			if (xItem.eItemType == DJSON_Number)
			{
				unsigned long ulDelay = 0;

				ulDelay = DJSON_GetValueInt(&xItem);
				if (ulDelay != g_xWITConfig.ulDelayTime)
				{
					g_xWITConfig.ulDelayTime = ulDelay;
					bNeedUpdateCfg = true;
				}
			}
			// 特征发送周期
			xItem = DJSON_GetObjectItem(&xCfgObj, "period");
			if (xItem.eItemType == DJSON_Number)
			{
				unsigned long ulSetPeriod;

				ulSetPeriod = DJSON_GetValueInt(&xItem) * 1000;
				g_xWITConfig.ulPeriod = ulSetPeriod;
				g_xWITConfig.ulFeaturePeriodCopy = ulSetPeriod;
				bNeedUpdateFTimer = true;
				bNeedUpdateCfg = true;
			}

			// 原始波形发送周期
			xItem = DJSON_GetObjectItem(&xCfgObj, "wPeriod");
			if (xItem.eItemType == DJSON_Number)
			{
				unsigned long ulSetPeriod;

				ulSetPeriod = DJSON_GetValueInt(&xItem) * 1000;
				if (ulSetPeriod <= WIT_MAX_TIME)
				{
					g_xWITConfig.ulWavePeriod = ulSetPeriod;
					g_xWITConfig.ulWavePeriodCopy = ulSetPeriod;
					bNeedUpdateWTimer = true;
					bNeedUpdateCfg = true;
				}
			}

			// 学习模式下 wPeriod = period
			if (g_xWITConfig.eGatewayMode == GatewayMode_Learning && bNeedUpdateCfg == true)
			{
				g_xWITConfig.ulWavePeriod = g_xWITConfig.ulPeriod;
			}

			xItem = DJSON_GetObjectItem(&xCfgObj, "spdSel");
			if (xItem.eItemType == DJSON_Number)
			{
				unsigned short spdsel = DJSON_GetValueInt(&xItem) % 4;

				// Set motor actral sel
				if (WIT_SensorOperation(spdSyncFSM.mpsAddr,
										Acc_SetMotorActualSel,
										WIT_ACC_SENSOR_ID,
										spdsel) == false)
				{
					bStatus = false;
				}

				g_xWITConfig.spdSel = spdsel;
				g_xWITConfig.spdSyncEnable = 1;
			}
			else
			{
				// Disable motor speed sync
				spdSyncFSM.enable = 0;
				g_xWITConfig.spdSyncEnable = 0;
			}

			if (g_xWITConfig.eGatewayMode != GatewayMode_Learning)
			{
				// 是否启用平台阈值
				xItem = DJSON_GetObjectItem(&xCfgObj, "autoThreshold");
				if (xItem.eItemType == DJSON_Number)
				{
					if (DJSON_GetValueInt(&xItem) == 0)
					{
						DataListInfo *pxDLIPointer = g_pxGlobalDLI;
						// s_bUseThrFromPlatform = true;
						// 关闭设备自设定阈值状态位
						if (g_xWITConfig.bAutoThreshold == true)
						{
							g_xWITConfig.bAutoThreshold = false;
							bNeedUpdateCfg = true;
						}
						// Clear DLI info and use threshold from platform
						g_xWITConfig.usCurrentLearningCount = WIT_GetFeatureMinLearnCount();
						if (g_xWITConfig.usCurrentLearningCount != 0)
						{
							g_xWITConfig.usCurrentLearningCount = 0;
							// DLI count reset
							for (j = 0; j < g_ulTotalParameterSize; j++)
							{
								pxDLIPointer->usCount = 0;
								pxDLIPointer++;
							}
							bNeedUpdateDLI = true;
						}
					}
					else
					{
						s_bUseThrFromPlatform = false;
					}
				}
			}
		}
		else if (g_bIsSensorExist[ucAddr - 1] == true)
		{
			unsigned char ucNumber, ucArrSize, ucIndex;

			ucIndex = ucAddr - 1;

			// 传感器配置
			//  ODR Config
			DJSON xObject = DJSON_GetObjectItem(&xCfgObj, "odr");
			if (xObject.eItemType == DJSON_Number)
			{
				unsigned short odr = ((unsigned short)DJSON_GetValueInt(&xObject)) % 6;
				if (WIT_SensorOperation(ucAddr, Acc_SetOdr, WIT_ACC_SENSOR_ID, odr) == false)
				{
					bStatus = false;
				}
			}
			// Sample sum Config
			xObject = DJSON_GetObjectItem(&xCfgObj, "sampSum");
			if (xObject.eItemType == DJSON_Number)
			{
				unsigned short sampSum = ((unsigned short)DJSON_GetValueInt(&xObject));
				if (sampSum < 2048)
				{
					sampSum = 2048;
				}
				if (WIT_SensorOperation(ucAddr, Acc_SetSampSum, WIT_ACC_SENSOR_ID, sampSum) == false)
				{
					bStatus = false;
				}
			}
			// HPF enable Config
			xObject = DJSON_GetObjectItem(&xCfgObj, "hpfEn");
			if (xObject.eItemType == DJSON_Number)
			{
				unsigned short hpfEn = ((unsigned short)DJSON_GetValueInt(&xObject)) % 2;
				if (WIT_SensorOperation(ucAddr, Acc_SetHpfEn, WIT_ACC_SENSOR_ID, hpfEn) == false)
				{
					bStatus = false;
				}
			}
			// anomaly vibration threshold
			xObject = DJSON_GetObjectItem(&xCfgObj, "amlVibTh");
			if (xObject.eItemType == DJSON_Number)
			{
				unsigned short amlVibTh = (unsigned short)DJSON_GetValueInt(&xObject);
				if (WIT_SensorOperation(ucAddr, Acc_SetAmlVibTh, WIT_ACC_SENSOR_ID, amlVibTh) == false)
				{
					bStatus = false;
				}
			}
			// anomaly vibration interval
			xObject = DJSON_GetObjectItem(&xCfgObj, "amlVibInterval");
			if (xObject.eItemType == DJSON_Number)
			{
				unsigned short amlVibInterval = (unsigned short)DJSON_GetValueInt(&xObject);
				if (WIT_SensorOperation(ucAddr, Acc_SetAmlVibInterval, WIT_ACC_SENSOR_ID, amlVibInterval) == false)
				{
					bStatus = false;
				}
			}

			// anomaly vibration keep cnt
			xObject = DJSON_GetObjectItem(&xCfgObj, "amlVibKeepCnt");
			if (xObject.eItemType == DJSON_Number)
			{
				unsigned short amlVibKeepCnt = ((unsigned short)DJSON_GetValueInt(&xObject)) % 100;
				if (WIT_SensorOperation(ucAddr, Acc_SetAmlVibKeepCnt, WIT_ACC_SENSOR_ID, amlVibKeepCnt) == false)
				{
					bStatus = false;
				}
			}

			// sensor PSRAM high temperature protect feature
			xObject = DJSON_GetObjectItem(&xCfgObj, "highTempProtect");
			if (xObject.eItemType == DJSON_Number)
			{
				unsigned short highTempProtect = (unsigned short)DJSON_GetValueInt(&xObject);
				highTempProtect %= 2;
				if (WIT_SensorOperation(ucAddr, Acc_SetHighTempProtect, WIT_ACC_SENSOR_ID, highTempProtect) == false)
				{
					bStatus = false;
				}
			}
			// 轴承配置
			// 清除传感器原先配置
			if (WIT_SensorOperation(ucAddr, Acc_ClearBearingStatus, WIT_ACC_SENSOR_ID) == false)
				bStatus = false;
			if (WIT_SensorOperation(ucAddr, Acc_ClearBearingExtendStauts, WIT_ACC_SENSOR_ID) == false)
				bStatus = false;
			xArray = DJSON_GetObjectItem(&xCfgObj, "bearing");
			if (xArray.eItemType == DJSON_Array)
			{
				Bearing xBearingInfo = {0};
				Bearing_ExtendConfig xBearingExtCfg = {0};

				ucArrSize = DJSON_GetArraySize(&xArray);

				for (j = 0; j < ucArrSize; j++)
				{
					xBearingObj = DJSON_GetArrayItem(&xArray, j);

					if (xBearingObj.eItemType == DJSON_Object)
					{
						memset(&xBearingInfo, 0, sizeof(Bearing));
						// num
						xItem = DJSON_GetObjectItem(&xBearingObj, "num");
						if (xItem.eItemType == DJSON_Number)
						{
							unsigned char ucBIStatus = 0, ucBFStatus = 0;
							unsigned char ucExtCfgStatus = 0;

							ucNumber = DJSON_GetValueInt(&xItem);

							xItem = DJSON_GetObjectItem(&xBearingObj, "ballD");
							if (xItem.eItemType == DJSON_Number)
							{
								xBearingInfo.fBallDim = DJSON_GetValueDouble(&xItem);
								ucBIStatus |= 0x01;
							}

							xItem = DJSON_GetObjectItem(&xBearingObj, "pcd");
							if (xItem.eItemType == DJSON_Number)
							{
								xBearingInfo.fPCD = DJSON_GetValueDouble(&xItem);
								ucBIStatus |= 0x02;
							}

							xItem = DJSON_GetObjectItem(&xBearingObj, "speed");
							if (xItem.eItemType == DJSON_Number)
							{
								xBearingInfo.fRotateSpeed = DJSON_GetValueDouble(&xItem);
								ucBIStatus |= 0x04;
							}

							xItem = DJSON_GetObjectItem(&xBearingObj, "count");
							if (xItem.eItemType == DJSON_Number)
							{
								xBearingInfo.fBallCount = DJSON_GetValueDouble(&xItem);
								ucBIStatus |= 0x08;
							}

							xItem = DJSON_GetObjectItem(&xBearingObj, "angle");
							if (xItem.eItemType == DJSON_Number)
							{
								xBearingInfo.fWorkAngle = DJSON_GetValueDouble(&xItem);
								ucBIStatus |= 0x10;
							}

							xItem = DJSON_GetObjectItem(&xBearingObj, "BPFO");
							if (xItem.eItemType == DJSON_Number)
							{
								xBearingInfo.fBPFO = DJSON_GetValueDouble(&xItem);
								ucBFStatus |= 0x01;
							}

							xItem = DJSON_GetObjectItem(&xBearingObj, "BPFI");
							if (xItem.eItemType == DJSON_Number)
							{
								xBearingInfo.fBPFI = DJSON_GetValueDouble(&xItem);
								ucBFStatus |= 0x02;
							}

							xItem = DJSON_GetObjectItem(&xBearingObj, "BSF");
							if (xItem.eItemType == DJSON_Number)
							{
								xBearingInfo.fBSF = DJSON_GetValueDouble(&xItem);
								ucBFStatus |= 0x04;
							}

							xItem = DJSON_GetObjectItem(&xBearingObj, "FTF");
							if (xItem.eItemType == DJSON_Number)
							{
								xBearingInfo.fFTF = DJSON_GetValueDouble(&xItem);
								ucBFStatus |= 0x08;
							}

							xItem = DJSON_GetObjectItem(&xBearingObj, "sFreq");
							if (xItem.eItemType == DJSON_Number)
							{
								xBearingExtCfg.usStartFreq = DJSON_GetValueDouble(&xItem);
								ucExtCfgStatus |= 0x01;
							}

							xItem = DJSON_GetObjectItem(&xBearingObj, "eFreq");
							if (xItem.eItemType == DJSON_Number)
							{
								xBearingExtCfg.usStopFreq = DJSON_GetValueDouble(&xItem);
								ucExtCfgStatus |= 0x02;
							}

							xBearingExtCfg.usAlias = ucNumber;

							// 通过Modbus设置此轴承信息
							if (WIT_SensorOperation(ucAddr, Acc_SetBearingAlias, WIT_ACC_SENSOR_ID, j, xBearingExtCfg.usAlias) == false)
								bStatus = false;
							if (ucBIStatus == 0x1F && WIT_SensorOperation(ucAddr, Acc_SetBearingInfo, WIT_ACC_SENSOR_ID, j, &xBearingInfo) == false)
								bStatus = false;
							if (ucBFStatus == 0x0F && WIT_SensorOperation(ucAddr, Acc_SetBearingFrequency, WIT_ACC_SENSOR_ID, j, &xBearingInfo) == false)
								bStatus = false;
							if (ucExtCfgStatus == 0x03 && WIT_SensorOperation(ucAddr, Acc_SetBearingExtendConfig, WIT_ACC_SENSOR_ID, j, &xBearingExtCfg) == false)
								bStatus = false;
						}
					}
				}
			}

			// 齿轮配置
			// 清除传感器原先配置
			if (WIT_SensorOperation(ucAddr, Acc_ClearGearStatus, WIT_ACC_SENSOR_ID) == false)
				bStatus = false;
			if (WIT_SensorOperation(ucAddr, Acc_ClearGearExtendStauts, WIT_ACC_SENSOR_ID) == false)
				bStatus = false;
			xArray = DJSON_GetObjectItem(&xCfgObj, "gear");
			if (xArray.eItemType == DJSON_Array)
			{
				Gear xGearInfo = {0};
				Gear_ExtendConfig xGearExtCfg = {0};

				ucArrSize = DJSON_GetArraySize(&xArray);

				for (j = 0; j < ucArrSize; j++)
				{
					xGearObj = DJSON_GetArrayItem(&xArray, j);

					if (xGearObj.eItemType == DJSON_Object)
					{
						// num
						xItem = DJSON_GetObjectItem(&xGearObj, "num");
						if (xItem.eItemType == DJSON_Number)
						{
							unsigned char ucGIStatus = 0, ucGFStatus = 0;
							unsigned char ucExtCfgStatus = 0;

							ucNumber = DJSON_GetValueInt(&xItem);
							memset(&xGearInfo, 0, sizeof(Gear));

							xItem = DJSON_GetObjectItem(&xGearObj, "sGT");
							if (xItem.eItemType == DJSON_Number)
							{
								xGearInfo.fTeethCount = DJSON_GetValueDouble(&xItem);
								ucGIStatus |= 0x01;
							}

							xItem = DJSON_GetObjectItem(&xGearObj, "sGS");
							if (xItem.eItemType == DJSON_Number)
							{
								xGearInfo.fRotateSpeed = DJSON_GetValueDouble(&xItem);
								ucGIStatus |= 0x02;
							}

							xItem = DJSON_GetObjectItem(&xGearObj, "GMF");
							if (xItem.eItemType == DJSON_Number)
							{
								xGearInfo.f1xGMF = DJSON_GetValueDouble(&xItem);
								xGearInfo.f2xGMF = xGearInfo.f1xGMF * 2;
								xGearInfo.f3xGMF = xGearInfo.f1xGMF * 3;
								ucGFStatus |= 0x07;
							}

							xItem = DJSON_GetObjectItem(&xGearObj, "sFreq");
							if (xItem.eItemType == DJSON_Number)
							{
								xGearExtCfg.usStartFreq = DJSON_GetValueDouble(&xItem);
								ucExtCfgStatus |= 0x01;
							}

							xItem = DJSON_GetObjectItem(&xGearObj, "eFreq");
							if (xItem.eItemType == DJSON_Number)
							{
								xGearExtCfg.usStopFreq = DJSON_GetValueDouble(&xItem);
								ucExtCfgStatus |= 0x02;
							}

							xItem = DJSON_GetObjectItem(&xGearObj, "sideRange");
							if (xItem.eItemType == DJSON_Number)
							{
								xGearExtCfg.usSideRange = DJSON_GetValueDouble(&xItem);
								ucExtCfgStatus |= 0x04;
							}
							xGearExtCfg.usAlias = ucNumber;

							// 通过Modbus设置此齿轮信息
							if (WIT_SensorOperation(ucAddr, Acc_SetGearAlias, WIT_ACC_SENSOR_ID, j, xGearExtCfg.usAlias) == false)
								bStatus = false;
							if (ucGIStatus == 0x03 && WIT_SensorOperation(ucAddr, Acc_SetGearInfo, WIT_ACC_SENSOR_ID, j, &xGearInfo) == false)
								bStatus = false;
							if (ucGFStatus == 0x07 && WIT_SensorOperation(ucAddr, Acc_SetGearFrequency, WIT_ACC_SENSOR_ID, j, &xGearInfo) == false)
								bStatus = false;
							if (ucExtCfgStatus == 0x07 && WIT_SensorOperation(ucAddr, Acc_SetGearExtendConfig, WIT_ACC_SENSOR_ID, j, &xGearExtCfg) == false)
								bStatus = false;
						}
					}
				}
			}

			// 清除风机配置
			if (WIT_SensorOperation(ucAddr, Acc_ClearFanStatus, WIT_ACC_SENSOR_ID) == false)
				bStatus = false;
			xObject = DJSON_GetObjectItem(&xCfgObj, "fan");
			if (xObject.eItemType == DJSON_Object)
			{
				xItem = DJSON_GetObjectItem(&xObject, "freq");
				if (xItem.eItemType == DJSON_Number)
				{
					Fan xFanCfg;

					xFanCfg.fPassFrequency = DJSON_GetValueDouble(&xItem);
					if (WIT_SensorOperation(ucAddr, Acc_SetFanConfigure, WIT_ACC_SENSOR_ID, &xFanCfg) == false)
						bStatus = false;
				}

				xItem = DJSON_GetObjectItem(&xObject, "num");
				if (xItem.eItemType == DJSON_Number)
				{
					unsigned short usFanAlias;

					usFanAlias = DJSON_GetValueInt(&xItem);
					if (WIT_SensorOperation(ucAddr, Acc_SetFanAlias, WIT_ACC_SENSOR_ID, usFanAlias) == false)
						bStatus = false;
				}
			}

			// 清除皮带配置
			if (WIT_SensorOperation(ucAddr, Acc_ClearBeltStatus, WIT_ACC_SENSOR_ID) == false)
				bStatus = false;
			xObject = DJSON_GetObjectItem(&xCfgObj, "belt");
			if (xObject.eItemType == DJSON_Object)
			{
				xItem = DJSON_GetObjectItem(&xObject, "freq");
				if (xItem.eItemType == DJSON_Number)
				{
					Fan xBeltCfg;

					xBeltCfg.fPassFrequency = DJSON_GetValueDouble(&xItem);
					if (WIT_SensorOperation(ucAddr, Acc_SetBeltConfigure, WIT_ACC_SENSOR_ID, &xBeltCfg) == false)
						bStatus = false;
				}
				xItem = DJSON_GetObjectItem(&xObject, "num");
				if (xItem.eItemType == DJSON_Number)
				{
					unsigned short usBeltAlias;

					usBeltAlias = DJSON_GetValueInt(&xItem);
					if (WIT_SensorOperation(ucAddr, Acc_SetBeltAlias, WIT_ACC_SENSOR_ID, usBeltAlias) == false)
						bStatus = false;
				}
			}
			// Clean Screw
			if (WIT_SensorOperation(ucAddr, Acc_ClearScrewStatus, WIT_ACC_SENSOR_ID) == false)
				bStatus = false;
			xObject = DJSON_GetObjectItem(&xCfgObj, "screw");
			if (xObject.eItemType == DJSON_Object)
			{
				xItem = DJSON_GetObjectItem(&xObject, "freq");
				if (xItem.eItemType == DJSON_Number)
				{
					Screw xScrewCfg;
					xScrewCfg.fPassFrequency = DJSON_GetValueDouble(&xItem);
					if (WIT_SensorOperation(ucAddr, Acc_SetScrewConfigure, WIT_ACC_SENSOR_ID, &xScrewCfg) == false)
						bStatus = false;
				}
				xItem = DJSON_GetObjectItem(&xObject, "num");
				if (xItem.eItemType == DJSON_Number)
				{
					unsigned short usScrewAlias;

					usScrewAlias = DJSON_GetValueInt(&xItem);
					if (WIT_SensorOperation(ucAddr, Acc_SetScrewAlias, WIT_ACC_SENSOR_ID, usScrewAlias) == false)
						bStatus = false;
				}
			}
			// Clean  Male Screw
			if (WIT_SensorOperation(ucAddr, Acc_ClearMaleScrewStatus, WIT_ACC_SENSOR_ID) == false)
				bStatus = false;
			xObject = DJSON_GetObjectItem(&xCfgObj, "malescrew");
			if (xObject.eItemType == DJSON_Object)
			{
				xItem = DJSON_GetObjectItem(&xObject, "freq");
				if (xItem.eItemType == DJSON_Number)
				{
					MaleScrew xMaleScrewCfg;
					xMaleScrewCfg.fPassFrequency = DJSON_GetValueDouble(&xItem);
					if (WIT_SensorOperation(ucAddr, Acc_SetMaleScrewConfigure, WIT_ACC_SENSOR_ID, &xMaleScrewCfg) == false)
						bStatus = false;
				}
				xItem = DJSON_GetObjectItem(&xObject, "num");
				if (xItem.eItemType == DJSON_Number)
				{
					unsigned short usMaleScrewAlias;

					usMaleScrewAlias = DJSON_GetValueInt(&xItem);
					if (WIT_SensorOperation(ucAddr, Acc_SetMaleScrewAlias, WIT_ACC_SENSOR_ID, usMaleScrewAlias) == false)
						bStatus = false;
				}
			}
			//	 Clean  Female Screw
			if (WIT_SensorOperation(ucAddr, Acc_ClearFemaleScrewStatus, WIT_ACC_SENSOR_ID) == false)
				bStatus = false;
			xObject = DJSON_GetObjectItem(&xCfgObj, "femalescrew");
			if (xObject.eItemType == DJSON_Object)
			{
				xItem = DJSON_GetObjectItem(&xObject, "freq");
				if (xItem.eItemType == DJSON_Number)
				{
					FemaleScrew xFemaleScrewCfg;
					xFemaleScrewCfg.fPassFrequency = DJSON_GetValueDouble(&xItem);
					if (WIT_SensorOperation(ucAddr, Acc_SetFemaleScrewConfigure, WIT_ACC_SENSOR_ID, &xFemaleScrewCfg) == false)
						bStatus = false;
				}
				xItem = DJSON_GetObjectItem(&xObject, "num");
				if (xItem.eItemType == DJSON_Number)
				{
					unsigned short usFemaleScrewAlias;

					usFemaleScrewAlias = DJSON_GetValueInt(&xItem);
					if (WIT_SensorOperation(ucAddr, Acc_SetFemaleScrewAlias, WIT_ACC_SENSOR_ID, usFemaleScrewAlias) == false)
						bStatus = false;
				}
			}
			if (WIT_SensorOperation(ucAddr, Acc_ClearShaftStatus, WIT_ACC_SENSOR_ID) == false)
				bStatus = false;
			xObject = DJSON_GetObjectItem(&xCfgObj, "shaft");
			if (xObject.eItemType == DJSON_Object)
			{
				xItem = DJSON_GetObjectItem(&xObject, "freq");
				if (xItem.eItemType == DJSON_Number)
				{
					Shaft xShaftCfg;
					xShaftCfg.fPassFrequency = DJSON_GetValueDouble(&xItem);
					if (WIT_SensorOperation(ucAddr, Acc_SetShaftConfigure, WIT_ACC_SENSOR_ID, &xShaftCfg) == false)
						bStatus = false;
				}
				xItem = DJSON_GetObjectItem(&xObject, "num");
				if (xItem.eItemType == DJSON_Number)
				{
					unsigned short usShaftAlias;

					usShaftAlias = DJSON_GetValueInt(&xItem);
					if (WIT_SensorOperation(ucAddr, Acc_SetShaftAlias, WIT_ACC_SENSOR_ID, usShaftAlias) == false)
						bStatus = false;
				}
			} // sleeve config
			if (WIT_SensorOperation(ucAddr, Acc_ClearSleeveStatus, WIT_ACC_SENSOR_ID) == false)
				bStatus = false;
			xObject = DJSON_GetObjectItem(&xCfgObj, "sleeveBearing");
			if (xObject.eItemType == DJSON_Object)
			{
				xItem = DJSON_GetObjectItem(&xObject, "freq");
				if (xItem.eItemType == DJSON_Number)
				{
					Sleeve xSleeveCfg;
					xSleeveCfg.fRotOfAxesFrequency = DJSON_GetValueDouble(&xItem);
					if (WIT_SensorOperation(ucAddr, Acc_SetSleeveConfigure, WIT_ACC_SENSOR_ID, &xSleeveCfg) == false)
						bStatus = false;
				}
				xItem = DJSON_GetObjectItem(&xObject, "num");
				if (xItem.eItemType == DJSON_Number)
				{
					unsigned short usSleeveAlias;

					usSleeveAlias = DJSON_GetValueInt(&xItem);
					if (WIT_SensorOperation(ucAddr, Acc_SetSleeveAlias, WIT_ACC_SENSOR_ID, usSleeveAlias) == false)
						bStatus = false;
				}
			}
			// 阈值配置
			{
				Base_ExtendConfig xBaseExtCfg;
				if (WIT_SensorOperation(ucAddr, Acc_ClearSpeedStatus, WIT_ACC_SENSOR_ID) == false)
					bStatus = false;
				// 输入轴转速
				xItem = DJSON_GetObjectItem(&xCfgObj, "inputSpeed");
				if (xItem.eItemType == DJSON_Number)
				{
					// 转速更新 (无用)
					g_xWITConfig.ulInputSpeed = DJSON_GetValueInt(&xItem);

					// 通过Modbus更新传感器输入轴转速
					if (WIT_SensorOperation(ucAddr, Acc_SetSpeed, WIT_ACC_SENSOR_ID, g_xWITConfig.ulInputSpeed) == false)
						bStatus = false;

					// Set motor point sensor addr
					spdSyncFSM.mpsAddr = ucAddr;
					spdSyncFSM.enable = 1;
				}
				// power frequency
				xItem = DJSON_GetObjectItem(&xCfgObj, "powerFrequency");
				if (xItem.eItemType == DJSON_Number)
				{
					unsigned short usPowerFrequency = DJSON_GetValueInt(&xItem) * 100;
					if (WIT_SensorOperation(ucAddr, Acc_SetPowerFrequency, WIT_ACC_SENSOR_ID, usPowerFrequency) == false)
						bStatus = false;
				}
				// 输入轴包络起始频率
				// 清除状态
				if (WIT_SensorOperation(ucAddr, Acc_ClearBaseExtendStauts, WIT_ACC_SENSOR_ID) == false)
					bStatus = false;
				xItem = DJSON_GetObjectItem(&xCfgObj, "inSFreq");
				if (xItem.eItemType == DJSON_Number)
				{
					xBaseExtCfg.usInputStartFreq = DJSON_GetValueInt(&xItem);
				}
				else
				{
					xBaseExtCfg.usInputStartFreq = 0;
				}

				xItem = DJSON_GetObjectItem(&xCfgObj, "inEFreq");
				if (xItem.eItemType == DJSON_Number)
				{
					xBaseExtCfg.usInputStopFreq = DJSON_GetValueInt(&xItem);
				}
				else
				{
					xBaseExtCfg.usInputStopFreq = 0;
				}

				xItem = DJSON_GetObjectItem(&xCfgObj, "outSFreq");
				if (xItem.eItemType == DJSON_Number)
				{
					xBaseExtCfg.usOutputStartFreq = DJSON_GetValueInt(&xItem);
				}
				else
				{
					xBaseExtCfg.usOutputStartFreq = 0;
				}

				xItem = DJSON_GetObjectItem(&xCfgObj, "outEFreq");
				if (xItem.eItemType == DJSON_Number)
				{
					xBaseExtCfg.usOutputStopFreq = DJSON_GetValueInt(&xItem);
				}
				else
				{
					xBaseExtCfg.usOutputStopFreq = 0;
				}
				// 设置包络频率
				if (WIT_SensorOperation(ucAddr, Acc_SetBaseExtendConfig, WIT_ACC_SENSOR_ID, &xBaseExtCfg) == false)
					bStatus = false;

				if (g_xWITConfig.eGatewayMode != GatewayMode_Learning && s_bUseThrFromPlatform == true)
				{
					Threshold xSetThr;

					// 锁g_xWITData
					xSemaphoreTake(g_xWITDataSignal, portMAX_DELAY);
					// 更新轴承齿轮别名
					WIT_UpdateFeatureName(ucAddr);

					pxParm = (Parameter *)&g_xWITData.xSensFea;
					pxThr = g_pxSensorThr + (ucIndex++) * g_ulSensorParameterSize;
					for (j = 0; j < g_ulSensorParameterSize; j++)
					{
						memset(&xSetThr, 0xFF, sizeof(Threshold));
						xLimit = DJSON_GetObjectItem(&xCfgObj, pxParm->cName);

						if (xLimit.eItemType == DJSON_Object)
						{
							xItem = DJSON_GetObjectItem(&xLimit, "h");
							if (xItem.eItemType == DJSON_Number)
							{
								xSetThr.fHighLimit = DJSON_GetValueDouble(&xItem);
							}

							xItem = DJSON_GetObjectItem(&xLimit, "l");
							if (xItem.eItemType == DJSON_Number)
							{
								xSetThr.fLowLimit = DJSON_GetValueDouble(&xItem);
							}
						}

						if (memcmp(pxThr, &xSetThr, sizeof(Threshold)) != 0)
						{
							memcpy(pxThr, &xSetThr, sizeof(Threshold));
							bNeedUpdateThr = true;
						}

						pxParm++;
						pxThr++;
					}

					xSemaphoreGive(g_xWITDataSignal);
				}
			}
		}
		else
		{
			bStatus = false;
			goto invalid;
		}

		// 更新定时器
		if (bNeedUpdateFTimer == true)
		{
			xTimerChangePeriod(g_xGatewayFeatureTimer, g_xWITConfig.ulPeriod, 1000);
			xTimerChangePeriod(g_xSensorFeatureTimer, g_xWITConfig.ulPeriod, 1000);
			xTimerChangePeriod(g_xAlarmTimer, g_xWITConfig.ulPeriod, 1000);
		}

		if (bNeedUpdateWTimer == true)
		{
			xTimerChangePeriod(g_xWaveTimer, g_xWITConfig.ulWavePeriod, 1000);
		}

		// 更新数据 开始定时器
		if (bNeedUpdateCfg == true)
			xTimerStart(g_xCfgUpTimer, 1000);
		if (bNeedUpdateThr == true)
			xTimerStart(g_xThrUpTimer, 1000);
		// 更新数据区域
		if (bNeedUpdateDLI == true)
			xTimerStart(g_xDLIUpTimer, 1000);
	}

invalid:
	g_xGatewayResponse.pcTopic = g_xWITMQTT.cPubConfigRespTopic;
	if (bStatus == true)
	{
		g_xGatewayResponse.eStatus = Resp_Success;
	}
	else
	{
		g_xGatewayResponse.eStatus = Resp_ParmErr;
	}

json_parse_end:
{
}
}
#endif
#if CONFIG_BLUETOOTH > 0
static void WIT_GatewayConfigureHandler(MessageData *pxData)
{
	DJSON xRoot;
	DJSON xSSArray;
	DJSON xSSObj;
	DJSON xCfgArray;
	DJSON xCfgObj;
	DJSON xLimit;
	DJSON xBearingObj;
	DJSON xGearObj;
	DJSON xArray;
	DJSON xItem;
	unsigned long ulSSArraySize;
	unsigned long ulcfgArraySize;
	unsigned long ulOtherArraySize;

	unsigned char ucAddr = 0;
	char cAddr[8];
	char other_buf[30] = {0};
	int i = 0;
	int j = 0;
	bool bStatus = true;
	bool bNeedUpdateDLI = false;
	bool bNeedUpdateCfg = false;
	bool bNeedUpdateThr = false;
	bool bNeedUpdateFTimer = false;
	bool bNeedUpdateWTimer = false;
	bool bNeedUpdateSenConfig = false;
	static bool s_bUseThrFromPlatform = false;

	xRoot.pcItemPointer = pxData->message->payload;
	xRoot.ulItemLength = pxData->message->payloadlen;
	xRoot.eItemType = DJSON_Object;
	// Get taskId
	memset(g_xGatewayResponse.cTaskID, 0, sizeof(g_xGatewayResponse.cTaskID));
	xItem = DJSON_GetObjectItem(&xRoot, "taskId");
	DJSON_GetValueString(&xItem, g_xGatewayResponse.cTaskID, sizeof(g_xGatewayResponse.cTaskID));

	// Get ss array
	xSSArray = DJSON_GetObjectItem(&xRoot, "ss");
	ulSSArraySize = DJSON_GetArraySize(&xSSArray);

	if (ulSSArraySize == 0 || xSSArray.eItemType != DJSON_Array)
	{
		bStatus = false;
		goto json_parse_end;
	}
	// Get addr
	xSSObj = DJSON_GetArrayItem(&xSSArray, 0);
	xItem = DJSON_GetObjectItem(&xSSObj, "addr");
	if (xItem.eItemType != DJSON_String)
	{
		bStatus = false;
		goto json_parse_end;
	}
	DJSON_GetValueString(&xItem, cAddr, sizeof(cAddr));
	strncpy(g_xGatewayResponse.cAddr, cAddr, 4);
	ucAddr = atoi(cAddr);
	printf("ucAddr %d\r\n", ucAddr);
	if (ucAddr > 30)
	{
		bStatus = false;
		goto json_parse_end;
	}
	// Get cfg
	xCfgArray = DJSON_GetObjectItem(&xSSObj, "cfg");
	ulcfgArraySize = DJSON_GetArraySize(&xCfgArray);
	if (ulcfgArraySize == 0 || xCfgArray.eItemType != DJSON_Array)
	{
		bStatus = false;
		goto json_parse_end;
	}
	if (ucAddr == 0)
		if_clear_multi_gateway_data();
	for (i = 0; i < ulcfgArraySize; i++)
	{
		xCfgObj = DJSON_GetArrayIItem(&xCfgArray, i);
		// 检查地址是否在范围内
		if (ucAddr <= 23)
		{
			Parameter *pxParm;
			Threshold *pxThr;
			unsigned long moterPos = 0;
			// 网关配置
			if (ucAddr == 0)
			{
				xItem = DJSON_GetObjectItem(&xCfgObj, "groupNo");
				if (xItem.eItemType != DJSON_String)
				{
					bStatus = false;
					goto json_parse_end;
				}
				DJSON_GetValueString(&xItem, g_xWITConfig.xGroupTable[i].groupNo, sizeof(g_xWITConfig.xGroupTable[i].groupNo));
				g_xWITConfig.xGroupTable[i].ulGroup = i + 1;
				printf("groupNo %s...\r\n", g_xWITConfig.xGroupTable[i].groupNo);
				xItem = DJSON_GetObjectItem(&xCfgObj, "motorPos");
				if (xItem.eItemType == DJSON_Number)
				{
					moterPos = DJSON_GetValueInt(&xItem);
					if (moterPos == 0 || moterPos == 26)
					{
						bStatus = false;
						goto invalid;
					}
					if (g_xWITConfig.xSenTable[moterPos - 1].ulAddr != moterPos)
					{
						bStatus = false;
						goto invalid;
					}
					g_xWITConfig.xSenTable[moterPos - 1].ulUseEnable = 1;
					g_xWITConfig.xSenTable[moterPos - 1].ulGroup = i + 1;
					g_xWITConfig.xSenTable[moterPos - 1].ulVirAddr = 1;
					g_xWITConfig.xGroupTable[i].ulConnectNum = g_xWITConfig.xGroupTable[i].ulConnectNum | (1 << ((moterPos - 1) % 6));
					g_xWITConfig.xGroupTable[i].ulConnectCnt++;
					g_xWITConfig.xGroupTable[i].ulGUseEnable = 1;
					g_xWITConfig.xGroupTable[i].ulGroup = i + 1;
					printf("moterPos %ld...\r\n", moterPos);
				}
				else
				{
					bStatus = false;
					goto invalid;
				}

				// g_xWITConfig.xSenTable[moterPos].ulUseEnable = 1;
				// 执行操作
				xItem = DJSON_GetObjectItem(&xCfgObj, "otherAddr");
				ulOtherArraySize = DJSON_GetArrayInt(&xItem, other_buf);
				if (xItem.eItemType == DJSON_Array)
				{
					for (int z = 0; z < ulOtherArraySize; z++)
					{
						if (other_buf[z] == 0 || other_buf[z] == 23)
						{
							bStatus = false;
							goto invalid;
						}
						if (g_xWITConfig.xSenTable[other_buf[z] - 1].ulAddr != other_buf[z])
						{
							bStatus = false;
							goto invalid;
						}
						g_xWITConfig.xSenTable[other_buf[z] - 1].ulUseEnable = 1;
						g_xWITConfig.xSenTable[other_buf[z] - 1].ulGroup = i + 1;
						g_xWITConfig.xSenTable[other_buf[z] - 1].ulVirAddr = other_buf[z] - moterPos + 1;
						g_xWITConfig.xGroupTable[i].ulConnectNum = g_xWITConfig.xGroupTable[i].ulConnectNum | (1 << ((other_buf[z] - 1) % 6));
						g_xWITConfig.xGroupTable[i].ulConnectCnt++;
						g_xWITConfig.xGroupTable[i].ulGUseEnable = 1;
						g_xWITConfig.xGroupTable[i].ulGroup = i + 1;
						printf("ulVirAddr %ld...\r\n", g_xWITConfig.xSenTable[other_buf[z] - 1].ulVirAddr);
						printf("other_buf[%d] %d...\r\n", z, other_buf[z]);

					}
				}
				else
				{
					bStatus = false;
					goto invalid;
				}

				ESP_LOGD("ulConnectNum", "%ld", g_xWITConfig.xGroupTable[i].ulConnectNum);
				ESP_LOGD("ulConnectCnt", "%ld", g_xWITConfig.xGroupTable[i].ulConnectCnt);
				// 振动读取延时
				xItem = DJSON_GetObjectItem(&xCfgObj, "delay");
				if (xItem.eItemType == DJSON_Number)
				{
					unsigned long ulDelay = 0;

					ulDelay = DJSON_GetValueInt(&xItem);
					if (ulDelay != g_xWITConfig.ulDelayTime)
					{
						g_xWITConfig.ulDelayTime = ulDelay;
						bNeedUpdateCfg = true;
					}
				}
				// 特征发送周期
				xItem = DJSON_GetObjectItem(&xCfgObj, "period");
				if (xItem.eItemType == DJSON_Number)
				{
					unsigned long ulSetPeriod;

					ulSetPeriod = DJSON_GetValueInt(&xItem) * 1000;
					g_xWITConfig.xGroupTable[i].ulFeaPeriod = ulSetPeriod / 1000;
					// g_xWITConfig.ulPeriod = ulSetPeriod;
					// g_xWITConfig.ulFeaturePeriodCopy = ulSetPeriod;
					bNeedUpdateFTimer = true;
					bNeedUpdateCfg = true;
				}

				// 原始波形发送周期
				xItem = DJSON_GetObjectItem(&xCfgObj, "wPeriod");
				if (xItem.eItemType == DJSON_Number)
				{
					unsigned long ulSetPeriod;

					ulSetPeriod = DJSON_GetValueInt(&xItem) * 1000;
					if (ulSetPeriod <= WIT_MAX_TIME)
					{
						// g_xWITConfig.ulWavePeriod = ulSetPeriod;
						// g_xWITConfig.ulWavePeriodCopy = ulSetPeriod;
						g_xWITConfig.xGroupTable[i].ulPeriod = ulSetPeriod / 1000;
						bNeedUpdateWTimer = true;
						bNeedUpdateCfg = true;
					}
				}

				// 学习模式下 wPeriod = period
				if (g_xWITConfig.eGatewayMode == GatewayMode_Learning && bNeedUpdateCfg == true)
				{
					g_xWITConfig.ulWavePeriod = g_xWITConfig.ulPeriod;
				}

				xItem = DJSON_GetObjectItem(&xCfgObj, "spdSel");
				if (xItem.eItemType == DJSON_Number)
				{
					unsigned short spdsel = DJSON_GetValueInt(&xItem) % 4;

					// Set motor actral sel
					g_xWITConfig.spdSel = spdsel;
					g_xWITConfig.spdSyncEnable = 1;
				}
				else
				{
					// Disable motor speed sync
					spdSyncFSM.enable = 0;
					g_xWITConfig.spdSyncEnable = 0;
				}
			}
			else if (ucAddr > 0 && ucAddr <= 30)
			{
				unsigned char ucNumber, ucArrSize, ucIndex;
				Sensor_Config *pSenConfig = NULL;
				ucIndex = ucAddr - 1;
				printf("ucIndex %d\r\n", ucIndex);
				pSenConfig = getSensorConfig(ucIndex);
				printf("config address %p\r\n", pSenConfig);
				pSenConfig->ulAddr = ucAddr;
				// 传感器配置
				//  ODR Config
				DJSON xObject = DJSON_GetObjectItem(&xCfgObj, "odr");
				if (xObject.eItemType == DJSON_Number)
				{
					unsigned char odr = ((unsigned char)DJSON_GetValueInt(&xObject)) % 6;
					pSenConfig->odr = odr;
					printf("%d: pSenConfig->odr %d\r\n", ucAddr, pSenConfig->odr);
				}
				// Sample sum Config
				xObject = DJSON_GetObjectItem(&xCfgObj, "sampSum");
				if (xObject.eItemType == DJSON_Number)
				{
					unsigned short sampSum = ((unsigned short)DJSON_GetValueInt(&xObject));
					if (sampSum < 2048)
					{
						sampSum = 2048;
					}
					pSenConfig->usSamplingCount = sampSum;
					printf("%d: pSenConfig->usSamplingCount %d\r\n", ucAddr, pSenConfig->usSamplingCount);
				}
				// HPF enable Config
				xObject = DJSON_GetObjectItem(&xCfgObj, "hpfEn");
				if (xObject.eItemType == DJSON_Number)
				{
					// ((unsigned short)DJSON_GetValueInt(&xObject)) % 2;
				}
				// anomaly vibration threshold
				xObject = DJSON_GetObjectItem(&xCfgObj, "amlVibTh");
				if (xObject.eItemType == DJSON_Number)
				{
					unsigned short amlVibTh = (unsigned short)DJSON_GetValueInt(&xObject);
					pSenConfig->amlVibTh = amlVibTh;
					printf("%d: pSenConfig->amlVibTh %d\r\n", ucAddr, pSenConfig->amlVibTh);
				}
				// anomaly vibration interval
				xObject = DJSON_GetObjectItem(&xCfgObj, "amlVibInterval");
				if (xObject.eItemType == DJSON_Number)
				{
					unsigned short amlVibInterval = (unsigned short)DJSON_GetValueInt(&xObject);
					pSenConfig->amlVibInterval = amlVibInterval;
					printf("%d: pSenConfig->amlVibInterval %d\r\n", ucAddr, pSenConfig->amlVibInterval);
				}
				// anomaly vibration keep cnt
				xObject = DJSON_GetObjectItem(&xCfgObj, "amlVibKeepCnt");
				if (xObject.eItemType == DJSON_Number)
				{
					unsigned short amlVibKeepCnt = ((unsigned short)DJSON_GetValueInt(&xObject)) % 100;
					pSenConfig->amlVibKeepCnt = amlVibKeepCnt;
					printf("%d: pSenConfig->amlVibKeepCnt %d\r\n", ucAddr, pSenConfig->amlVibKeepCnt);
				}

				// sensor PSRAM high temperature protect feature
				xObject = DJSON_GetObjectItem(&xCfgObj, "highTempProtect");
				if (xObject.eItemType == DJSON_Number)
				{
					unsigned short highTempProtect = (unsigned short)DJSON_GetValueInt(&xObject);
					pSenConfig->highTempProtect = highTempProtect;
					printf("%d: pSenConfig->highTempProtect %d\r\n", ucAddr, pSenConfig->highTempProtect);
				}
				// 清除轴承原先配置
				memset(&pSenConfig->xBearing, 0, SENSOR_BEARING_LOGITIC_MAX_SIZE * sizeof(Bearing));
				// 轴承标志清空
				for (int iIndex = 0; iIndex < SENSOR_BEARING_MAX_COUNT; iIndex++)
				{
					pSenConfig->bBearingEnabled[iIndex] = false;
					pSenConfig->bBearingExtendEnable[iIndex] = false;
					pSenConfig->xBaseExtendConfig.usBearingExtendFlag = 0;
				}
				// 轴承配置
				xArray = DJSON_GetObjectItem(&xCfgObj, "bearing");
				if (xArray.eItemType == DJSON_Array)
				{
					ucArrSize = DJSON_GetArraySize(&xArray);
					for (j = 0; j < ucArrSize; j++)
					{
						xBearingObj = DJSON_GetArrayItem(&xArray, j);

						if (xBearingObj.eItemType == DJSON_Object)
						{
							// num
							xItem = DJSON_GetObjectItem(&xBearingObj, "num");
							if (xItem.eItemType == DJSON_Number)
							{
								ucNumber = DJSON_GetValueInt(&xItem);
								if (ucNumber > 0)
								{
									pSenConfig->xBearingExtendConfig[j].usAlias = ucNumber;
									printf("%d: BearingExtendConfig.usAlias %d\r\n", ucAddr, pSenConfig->xBearingExtendConfig[j].usAlias);
								}
								xItem = DJSON_GetObjectItem(&xBearingObj, "speed");
								if (xItem.eItemType == DJSON_Number)
								{
									float fSpeed = 0.0;
									fSpeed = DJSON_GetValueDouble(&xItem);
									if (ucNumber > 0)
									{
										pSenConfig->xBearing[j].fRotateSpeed = fSpeed;
										printf("%d: Bearing.fRotateSpeed %f\r\n", ucAddr, pSenConfig->xBearing[j].fRotateSpeed);
									}
								}
								xItem = DJSON_GetObjectItem(&xBearingObj, "BPFO");
								if (xItem.eItemType == DJSON_Number)
								{
									float fBPFO = 0.0;
									fBPFO = DJSON_GetValueDouble(&xItem);
									if (ucNumber > 0)
									{
										pSenConfig->xBearing[j].fBPFO = fBPFO;
										printf("%d: Bearing.fBPFO %f\r\n", ucAddr, pSenConfig->xBearing[j].fBPFO);
									}
								}
								xItem = DJSON_GetObjectItem(&xBearingObj, "BPFI");
								if (xItem.eItemType == DJSON_Number)
								{
									float fBPFI = 0.0;
									fBPFI = DJSON_GetValueDouble(&xItem);
									if (ucNumber > 0)
									{
										pSenConfig->xBearing[j].fBPFI = fBPFI;
										printf("%d: Bearing.fBPFI %f\r\n", ucAddr, pSenConfig->xBearing[j].fBPFI);
									}
								}
								xItem = DJSON_GetObjectItem(&xBearingObj, "BSF");
								if (xItem.eItemType == DJSON_Number)
								{
									float fBSF = 0.0;
									fBSF = DJSON_GetValueDouble(&xItem);
									if (ucNumber > 0)
									{
										pSenConfig->xBearing[j].fBSF = fBSF;
										printf("%d: Bearing.fBSF %f\r\n", ucAddr, pSenConfig->xBearing[j].fBSF);
									}
								}

								xItem = DJSON_GetObjectItem(&xBearingObj, "FTF");
								if (xItem.eItemType == DJSON_Number)
								{
									float fFTF = 0.0;
									fFTF = DJSON_GetValueDouble(&xItem);
									if (ucNumber > 0)
									{
										pSenConfig->xBearing[j].fFTF = fFTF;
										pSenConfig->bBearingEnabled[j] = true;
										printf("%d: Bearing.fFTF %d\r\n", ucAddr, pSenConfig->bBearingEnabled[j]);
										printf("%d: bBearingEnabled %d\r\n", ucAddr, pSenConfig->bBearingEnabled[j]);
									}
								}
								xItem = DJSON_GetObjectItem(&xBearingObj, "sFreq");
								if (xItem.eItemType == DJSON_Number)
								{
									float fsFreq = 0.0;
									fsFreq = DJSON_GetValueDouble(&xItem);
									if (ucNumber > 0)
									{
										pSenConfig->xBearingExtendConfig[j].usStartFreq = ((unsigned short)(fsFreq * 10));
										printf("%d: bear usStartFreq %d\r\n", ucAddr, pSenConfig->xBearingExtendConfig[j].usStartFreq);
									}
								}

								xItem = DJSON_GetObjectItem(&xBearingObj, "eFreq");
								if (xItem.eItemType == DJSON_Number)
								{
									float feFreq = 0.0;
									feFreq = DJSON_GetValueDouble(&xItem);
									if (ucNumber > 0)
									{
										pSenConfig->xBearingExtendConfig[j].usStopFreq = ((unsigned short)(feFreq * 10));
										pSenConfig->bBearingExtendEnable[j] = true;
										printf("%d: bear usStopFreq %d\r\n", ucAddr, pSenConfig->xBearingExtendConfig[j].usStopFreq);
										printf("%d: bear bBearingExtendEnable %d\r\n", ucAddr, pSenConfig->bBearingExtendEnable[j]);
									}
								}
								pSenConfig->xBaseExtendConfig.usBearingExtendFlag |= (0x0001 << j);
								printf("%d: bear usBearingExtendFlag %d\r\n", ucAddr, pSenConfig->xBaseExtendConfig.usBearingExtendFlag);
							}
						}
					}
				}
				// 清除齿轮配置
				memset(&pSenConfig->xGear, 0, SENSOR_GEAR_LOGITIC_MAX_SIZE * sizeof(Gear));
				// 齿轮标志清空
				for (int iIndex = 0; iIndex < SENSOR_GEAR_MAX_COUNT; iIndex++)
				{
					pSenConfig->bGearEnabled[iIndex] = false;
					pSenConfig->bGearExtendEnable[iIndex] = false;
					pSenConfig->xBaseExtendConfig.usGearExtendFlag = 0;
				}
				// 齿轮配置
				xArray = DJSON_GetObjectItem(&xCfgObj, "gear");
				if (xArray.eItemType == DJSON_Array)
				{
					ucArrSize = DJSON_GetArraySize(&xArray);
					for (j = 0; j < ucArrSize; j++)
					{
						xGearObj = DJSON_GetArrayItem(&xArray, j);
						if (xGearObj.eItemType == DJSON_Object)
						{
							// num
							xItem = DJSON_GetObjectItem(&xGearObj, "num");
							if (xItem.eItemType == DJSON_Number)
							{
								ucNumber = DJSON_GetValueInt(&xItem);
								if (ucNumber > 0)
								{
									pSenConfig->xGearExtendConfig[j].usAlias = ucNumber;
									printf("%d: gear usAlias %d\r\n", ucAddr, pSenConfig->xGearExtendConfig[j].usAlias);
								}
								xItem = DJSON_GetObjectItem(&xGearObj, "sGT");
								if (xItem.eItemType == DJSON_Number)
								{
									float fsGT = 0.0;
									fsGT = DJSON_GetValueDouble(&xItem);
									if (ucNumber > 0)
									{
										pSenConfig->xGear[j].fTeethCount = fsGT;
										printf("%d: gear fTeethCount %f\r\n", ucAddr, pSenConfig->xGear[j].fTeethCount);
									}
								}
								xItem = DJSON_GetObjectItem(&xGearObj, "sGS");
								if (xItem.eItemType == DJSON_Number)
								{
									float fsGS = 0.0;
									fsGS = DJSON_GetValueDouble(&xItem);
									if (ucNumber > 0)
									{
										pSenConfig->xGear[j].fRotateSpeed = fsGS;
										printf("%d: gear fRotateSpeed %f\r\n", ucAddr, pSenConfig->xGear[j - 1].fRotateSpeed);
									}
								}
								xItem = DJSON_GetObjectItem(&xGearObj, "GMF");
								if (xItem.eItemType == DJSON_Number)
								{
									float fGMS = 0.0;
									fGMS = DJSON_GetValueDouble(&xItem);
									if (ucNumber > 0)
									{
										pSenConfig->xGear[j].f1xGMF = fGMS;
										pSenConfig->xGear[j].f2xGMF = 2 * fGMS;
										pSenConfig->xGear[j].f3xGMF = 3 * fGMS;
										pSenConfig->bGearEnabled[j] = true;
										printf("%d: gear f1xGMF %f\r\n", ucAddr, pSenConfig->xGear[j].f1xGMF);
										printf("%d: gear bGearEnabled %d\r\n", ucAddr, pSenConfig->bGearEnabled[j]);
									}
								}
								xItem = DJSON_GetObjectItem(&xGearObj, "sFreq");
								if (xItem.eItemType == DJSON_Number)
								{
									float fsFreq = 0.0;
									fsFreq = DJSON_GetValueDouble(&xItem);
									if (ucNumber > 0)
									{
										pSenConfig->xGearExtendConfig[j].usStartFreq = ((unsigned short)(fsFreq * 10));
										printf(" %d: gear usStartFreq %d\r\n", ucAddr, pSenConfig->xGearExtendConfig[j].usStartFreq);
									}
								}
								xItem = DJSON_GetObjectItem(&xGearObj, "eFreq");
								if (xItem.eItemType == DJSON_Number)
								{
									float feFreq = 0.0;
									feFreq = DJSON_GetValueDouble(&xItem);
									if (ucNumber > 0)
									{
										pSenConfig->xGearExtendConfig[j].usStopFreq = ((unsigned short)(feFreq * 10));
										printf("%d: gear usStopFreq %d\r\n", ucAddr, pSenConfig->xGearExtendConfig[j].usStopFreq);
									}
								}

								xItem = DJSON_GetObjectItem(&xGearObj, "sideRange");
								if (xItem.eItemType == DJSON_Number)
								{
									float fsideRange = 0.0;
									fsideRange = DJSON_GetValueDouble(&xItem);
									if (ucNumber > 0)
									{
										pSenConfig->xGearExtendConfig[j].usSideRange = ((unsigned short)(fsideRange * 10));
										pSenConfig->bGearExtendEnable[j] = true;
										printf("%d: gear usSideRange %d\r\n", ucAddr, pSenConfig->xGearExtendConfig[j].usSideRange);
									}
								}
							}
							pSenConfig->xBaseExtendConfig.usGearExtendFlag |= (0x0001 << j);
							printf("%d: gear uusGearExtendFlag %d\r\n", ucAddr, pSenConfig->xBaseExtendConfig.usGearExtendFlag);
						}
					}
				}

				pSenConfig->bFanEnable = false;
				// 风机配置
				xObject = DJSON_GetObjectItem(&xCfgObj, "fan");
				if (xObject.eItemType == DJSON_Object)
				{
					xItem = DJSON_GetObjectItem(&xObject, "freq");
					if (xItem.eItemType == DJSON_Number)
					{
						pSenConfig->xFan.fPassFrequency = DJSON_GetValueDouble(&xItem);
						printf("%d: pSenConfig->xFan.fPassFrequency %f\r\n", ucAddr, pSenConfig->xFan.fPassFrequency);
					}

					xItem = DJSON_GetObjectItem(&xObject, "num");
					if (xItem.eItemType == DJSON_Number)
					{
						pSenConfig->usFanAlias = DJSON_GetValueInt(&xItem);
						printf("%d: pSenConfig->usFanAlias %d\r\n", ucAddr, pSenConfig->usFanAlias);
					}
					pSenConfig->bFanEnable = true;
					printf("%d: pSenConfig->bFanEnable %d\r\n", ucAddr, pSenConfig->bFanEnable);
				}

				// 皮带配置
				pSenConfig->bBeltEnable = false;
				xObject = DJSON_GetObjectItem(&xCfgObj, "belt");
				if (xObject.eItemType == DJSON_Object)
				{
					xItem = DJSON_GetObjectItem(&xObject, "freq");
					if (xItem.eItemType == DJSON_Number)
					{
						pSenConfig->xBelt.fPassFrequency = DJSON_GetValueDouble(&xItem);
						printf("%d: xBelt.fPassFrequency %f\r\n", ucAddr, pSenConfig->xBelt.fPassFrequency);
					}

					xItem = DJSON_GetObjectItem(&xObject, "selfVirbFreq");
					if (xItem.eItemType == DJSON_Number)
					{
						pSenConfig->xBelt.fSelfVirbFreq = DJSON_GetValueDouble(&xItem);
						printf("%d: xBelt.fSelfVirbFreq %f\r\n", ucAddr, pSenConfig->xBelt.fSelfVirbFreq);
					}

					xItem = DJSON_GetObjectItem(&xObject, "num");
					if (xItem.eItemType == DJSON_Number)
					{
						pSenConfig->usBeltAlias = DJSON_GetValueInt(&xItem);
						printf("%d: xBelt.sBeltAlias %d\r\n", ucAddr, pSenConfig->usBeltAlias);
					}
					pSenConfig->bBeltEnable = true;
				}
				// Clean Screw
				pSenConfig->bScrewEnable = false;
				xObject = DJSON_GetObjectItem(&xCfgObj, "screw");
				if (xObject.eItemType == DJSON_Object)
				{
					xItem = DJSON_GetObjectItem(&xObject, "freq");
					if (xItem.eItemType == DJSON_Number)
					{
						pSenConfig->xScrew.fPassFrequency = DJSON_GetValueDouble(&xItem);
						printf("%d: xScrew.fPassFrequency %f\r\n", ucAddr, pSenConfig->xScrew.fPassFrequency);
					}
					xItem = DJSON_GetObjectItem(&xObject, "num");
					if (xItem.eItemType == DJSON_Number)
					{
						pSenConfig->usScrewAlias = DJSON_GetValueInt(&xItem);
						printf("%d: xScrew.usScrewAlias %d\r\n", ucAddr, pSenConfig->usScrewAlias);
					}
					pSenConfig->bScrewEnable = true;
				}
				// Clean  Male Screw
				pSenConfig->bMaleScrewEnable = false;
				xObject = DJSON_GetObjectItem(&xCfgObj, "malescrew");
				if (xObject.eItemType == DJSON_Object)
				{
					xItem = DJSON_GetObjectItem(&xObject, "freq");
					if (xItem.eItemType == DJSON_Number)
					{
						pSenConfig->xMaleScrew.fPassFrequency = DJSON_GetValueDouble(&xItem);
						printf("%d: xMaleScrew.fPassFrequency %f\r\n", ucAddr, pSenConfig->xMaleScrew.fPassFrequency);
					}
					xItem = DJSON_GetObjectItem(&xObject, "num");
					if (xItem.eItemType == DJSON_Number)
					{
						pSenConfig->usMaleScrewAlias = DJSON_GetValueInt(&xItem);
						printf("%d: xMaleScrew.usMaleScrewAlias %d\r\n", ucAddr, pSenConfig->usMaleScrewAlias);
					}
					pSenConfig->bMaleScrewEnable = true;
				}
				//	 Clean  Female Screw
				pSenConfig->bFemaleScrewEnable = false;
				xObject = DJSON_GetObjectItem(&xCfgObj, "femalescrew");
				if (xObject.eItemType == DJSON_Object)
				{
					xItem = DJSON_GetObjectItem(&xObject, "freq");
					if (xItem.eItemType == DJSON_Number)
					{
						pSenConfig->xFemaleScrew.fPassFrequency = DJSON_GetValueDouble(&xItem);
						printf("%d: xFemaleScrew.fPassFrequency %f\r\n", ucAddr, pSenConfig->xFemaleScrew.fPassFrequency);
					}
					xItem = DJSON_GetObjectItem(&xObject, "num");
					if (xItem.eItemType == DJSON_Number)
					{
						pSenConfig->usFemaleScrewAlias = DJSON_GetValueInt(&xItem);
						printf("%d: xFemaleScrew.usFemaleScrewAlias %d\r\n", ucAddr, pSenConfig->usFemaleScrewAlias);
					}
					pSenConfig->bFemaleScrewEnable = true;
				}
				// clear shaft
				pSenConfig->bShaftEnable = false;
				xObject = DJSON_GetObjectItem(&xCfgObj, "shaft");
				if (xObject.eItemType == DJSON_Object)
				{
					xItem = DJSON_GetObjectItem(&xObject, "freq");
					if (xItem.eItemType == DJSON_Number)
					{
						pSenConfig->xShaft.fPassFrequency = DJSON_GetValueDouble(&xItem);
						printf("%d: xShaft.fPassFrequency %f\r\n", ucAddr, pSenConfig->xShaft.fPassFrequency);
					}
					xItem = DJSON_GetObjectItem(&xObject, "num");
					if (xItem.eItemType == DJSON_Number)
					{
						pSenConfig->usShaftAlias = DJSON_GetValueInt(&xItem);
						printf("%d: xShaft.usShaftAlias %d\r\n", ucAddr, pSenConfig->usShaftAlias);
					}
					pSenConfig->bShaftEnable = true;
				}
				// sleeve config
				pSenConfig->bSleeveEnable = false;
				xObject = DJSON_GetObjectItem(&xCfgObj, "sleeveBearing");
				if (xObject.eItemType == DJSON_Object)
				{
					xItem = DJSON_GetObjectItem(&xObject, "freq");
					if (xItem.eItemType == DJSON_Number)
					{
						pSenConfig->xSleeve.fRotOfAxesFrequency = DJSON_GetValueDouble(&xItem);
					}
					xItem = DJSON_GetObjectItem(&xObject, "num");
					if (xItem.eItemType == DJSON_Number)
					{
						pSenConfig->usSleeveAlias = DJSON_GetValueInt(&xItem);
					}
					pSenConfig->bSleeveEnable = true;
				}
				// 输入轴转速
				pSenConfig->bSetSpeed = false;
				pSenConfig->ulSpeed = 1500;
				xItem = DJSON_GetObjectItem(&xCfgObj, "inputSpeed");
				if (xItem.eItemType == DJSON_Number)
				{

					g_xWITConfig.ulInputSpeed = DJSON_GetValueInt(&xItem);
					pSenConfig->ulSpeed = g_xWITConfig.ulInputSpeed;
					pSenConfig->bSetSpeed = true;
					printf("%d: ulInputSpeed %ld\r\n", ucAddr, pSenConfig->ulSpeed);
					printf("%d: bSetSpeed %d\r\n", ucAddr, pSenConfig->bSetSpeed);
				}
				// power frequency
				pSenConfig->xPwrFreq.bEnable = false;
				xItem = DJSON_GetObjectItem(&xCfgObj, "powerFrequency");
				if (xItem.eItemType == DJSON_Number)
				{
					pSenConfig->xPwrFreq.fPassFrequency = DJSON_GetValueInt(&xItem) * 100;
					pSenConfig->xPwrFreq.bEnable = true;
				}
				// 输入轴包络起始频率
				// 清除状态
				xItem = DJSON_GetObjectItem(&xCfgObj, "inSFreq");
				if (xItem.eItemType == DJSON_Number)
				{
					pSenConfig->xBaseExtendConfig.usInputStartFreq = DJSON_GetValueInt(&xItem);
					printf("%d: InputStartFreq %d\r\n", ucAddr, pSenConfig->xBaseExtendConfig.usInputStartFreq);
				}
				else
				{
					pSenConfig->xBaseExtendConfig.usInputStartFreq = 0;
				}

				xItem = DJSON_GetObjectItem(&xCfgObj, "inEFreq");
				if (xItem.eItemType == DJSON_Number)
				{
					pSenConfig->xBaseExtendConfig.usInputStopFreq = DJSON_GetValueInt(&xItem);
					pSenConfig->xBaseExtendConfig.usInputSpeedEnable = 1;
					printf("%d: InputStopFreq %d\r\n", ucAddr, pSenConfig->xBaseExtendConfig.usInputStopFreq);
				}
				else
				{
					pSenConfig->xBaseExtendConfig.usInputStopFreq = 0;
				}

				xItem = DJSON_GetObjectItem(&xCfgObj, "outSFreq");
				if (xItem.eItemType == DJSON_Number)
				{
					pSenConfig->xBaseExtendConfig.usOutputStartFreq = DJSON_GetValueInt(&xItem);
					printf("%d: OutputStartFreq %d\r\n", ucAddr, pSenConfig->xBaseExtendConfig.usOutputStartFreq);
				}
				else
				{
					pSenConfig->xBaseExtendConfig.usOutputStartFreq = 0;
				}

				xItem = DJSON_GetObjectItem(&xCfgObj, "outEFreq");
				if (xItem.eItemType == DJSON_Number)
				{
					pSenConfig->xBaseExtendConfig.usOutputStopFreq = DJSON_GetValueInt(&xItem);
					printf("%d: OutputStartFreq %d\r\n", ucAddr, pSenConfig->xBaseExtendConfig.usOutputStartFreq);
					pSenConfig->xBaseExtendConfig.usOutputSpeedEnable = 1;
				}
				else
				{
					pSenConfig->xBaseExtendConfig.usOutputStopFreq = 0;
				}

				bNeedUpdateSenConfig = true;
				// 阈值配置
				if (g_xWITConfig.eGatewayMode != GatewayMode_Learning && s_bUseThrFromPlatform == true)
				{
					Threshold xSetThr;

					// 锁g_xWITData
					xSemaphoreTake(g_xWITDataSignal, portMAX_DELAY);

					pxParm = (Parameter *)&g_xWITData.xSensFea;
					pxThr = g_pxSensorThr + (ucIndex++) * g_ulSensorParameterSize;
					for (j = 0; j < g_ulSensorParameterSize; j++)
					{
						memset(&xSetThr, 0xFF, sizeof(Threshold));
						xLimit = DJSON_GetObjectItem(&xCfgObj, pxParm->cName);

						if (xLimit.eItemType == DJSON_Object)
						{
							xItem = DJSON_GetObjectItem(&xLimit, "h");
							if (xItem.eItemType == DJSON_Number)
							{
								xSetThr.fHighLimit = DJSON_GetValueDouble(&xItem);
							}

							xItem = DJSON_GetObjectItem(&xLimit, "l");
							if (xItem.eItemType == DJSON_Number)
							{
								xSetThr.fLowLimit = DJSON_GetValueDouble(&xItem);
							}
						}

						if (memcmp(pxThr, &xSetThr, sizeof(Threshold)) != 0)
						{
							memcpy(pxThr, &xSetThr, sizeof(Threshold));
							bNeedUpdateThr = true;
						}

						pxParm++;
						pxThr++;
					}

					xSemaphoreGive(g_xWITDataSignal);
				}
			}
			else
			{
				bStatus = false;
				goto invalid;
			}

			// 更新定时器
			if (bNeedUpdateFTimer == true)
			{
				g_xWITConfig.xGroupTable[i].ulWavePushEnable = 1;
			}

			if (bNeedUpdateWTimer == true)
			{
				g_xWITConfig.xGroupTable[i].ulFeaPushEnable = 1;
			}
		}
		// 更新数据 开始定时器
		if (bNeedUpdateCfg == true)
			xTimerStart(g_xCfgUpTimer, 1000);
		if (bNeedUpdateThr == true)
			xTimerStart(g_xThrUpTimer, 1000);
		// 更新数据区域
		if (bNeedUpdateDLI == true)
			xTimerStart(g_xDLIUpTimer, 1000);
		if (bNeedUpdateSenConfig == true)
		{
			WIT_SetAddrConfigure(ucAddr, 1);
			xTimerStart(g_xSenConfigTimer, 1000);
		}
	}

invalid:
	g_xGatewayResponse.pcTopic = g_xWITMQTT.cPubConfigRespTopic;
	if (bStatus == true)
	{
		g_xGatewayResponse.eStatus = Resp_Success;
	}
	else
	{
		g_xGatewayResponse.eStatus = Resp_ParmErr;
	}
json_parse_end:
{
}
}
#endif
#endif

static void WIT_GatewayWorkDetectiveHandler(MessageData *pxData)
{
	DJSON xRoot;
	DJSON xItem;
	DJSON xSSArray;
	DJSON xSSObj;
	DJSON xCfgArray;
	DJSON xCfg;

	Sensor_Config *pSenConfig = NULL;
	sGroupTable *pxGroup = NULL;
	char groupNo[32];
	bool bStatus = true;
	bool bNeedUpdate = false;

	unsigned int ulcfgArraySize;
	unsigned int ulSSArraySize;

	xRoot.pcItemPointer = pxData->message->payload;
	xRoot.ulItemLength = pxData->message->payloadlen;
	xRoot.eItemType = DJSON_Object;

	g_xGatewayResponse.pcTopic = g_xWITMQTT.cPubWorkThrRespTopic;
	// Get taskId
	memset(g_xGatewayResponse.cTaskID, 0, sizeof(g_xGatewayResponse.cTaskID));
	xItem = DJSON_GetObjectItem(&xRoot, "taskId");
	DJSON_GetValueString(&xItem, g_xGatewayResponse.cTaskID, sizeof(g_xGatewayResponse.cTaskID));

	// Get ss array
	xSSArray = DJSON_GetObjectItem(&xRoot, "ss");
	ulSSArraySize = DJSON_GetArraySize(&xSSArray);
	if (ulSSArraySize == 0 || xSSArray.eItemType != DJSON_Array)
	{
		bStatus = false;
		goto end;
	}
	// Get addr
	xSSObj = DJSON_GetArrayItem(&xSSArray, 0);
	xItem = DJSON_GetObjectItem(&xSSObj, "addr");
	if (xItem.eItemType != DJSON_String)
	{
		bStatus = false;
		goto end;
	}
	// Get cfg
	xCfgArray = DJSON_GetObjectItem(&xSSObj, "cfg");
	ulcfgArraySize = DJSON_GetArraySize(&xCfgArray);
	if (ulcfgArraySize == 0 || xCfgArray.eItemType != DJSON_Array)
	{
		bStatus = false;
		goto end;
	}
	for (int i = 0; i < ulcfgArraySize; i++)
	{
		xCfg = DJSON_GetArrayIItem(&xCfgArray, i);
		xItem = DJSON_GetObjectItem(&xCfg, "groupNo");
		DJSON_GetValueString(&xItem, groupNo, sizeof(groupNo));
#if CONFIG_BLUETOOTH > 0
		pxGroup = getGroupFromGroupNo(groupNo);
#endif
		if (pxGroup == NULL)
		{
			bStatus = false;
			goto end;
		}
		xItem = DJSON_GetObjectItem(&xCfg, "xVar");
		if (xItem.eItemType != DJSON_Number)
		{
			bStatus = false;
			goto end;
		}
		float value;
		value = DJSON_GetValueDouble(&xItem);
#if CONFIG_BLUETOOTH > 0
		for (int z = 0; z < WIT_SENSOR_MAX_CONNECT_NUM * WIT_SENSOR_GROUP_MAX_NUM; z++)
		{
			if (g_xWITConfig.xSenTable[z].ulGroup == pxGroup->ulGroup)
			{
				pSenConfig = getSensorConfig(z);
				pSenConfig->usxVar = (unsigned short)value;
			}
		}
#endif

#if CONFIG_RS485_RTU > 0
		int j;

		if (value != gw->xWorkDet.xXVarWorkThr.fHighLimit)
		{
			gw->xWorkDet.xXVarWorkThr.fHighLimit = value;
			bNeedUpdate = true;
		}
		pxGroup = getGroupFromGroupNo(char *groupNo);
		for (j = 1; j <= ACC_MAX_SENSOR_COUNT; j++)
		{
			if (gw->OtherAddr[j])
			{
				JIE_SensorOperation(j, Acc_SetShockThreshold, MB_ADDRESS_BROADCAST, (short)gw->xWorkDet.xXVarWorkThr.fHighLimit);
			}
		}
		vTaskDelay(50);
#endif
	}
#if CONFIG_RS485_RTU > 0
	if (bNeedUpdate)
	{
		xTimerStart(g_xCfgUpTimer, 1000); // 保存配置内容
	}
#endif
#if CONFIG_BLUETOOTH > 0
	if (bNeedUpdate)
	{
		xTimerStart(g_xSenConfigTimer, 1000);
	}
#endif
end:
	if (bStatus == true)
	{
		g_xGatewayResponse.eStatus = Resp_Success;
	}
	else
	{
		g_xGatewayResponse.eStatus = Resp_ParmErr;
	}
}

static void WIT_GatewayRemoteCtrlHandler(MessageData *pxData)
{
	DJSON xRoot;
	DJSON xItem;

	xRoot.pcItemPointer = pxData->message->payload;
	xRoot.ulItemLength = pxData->message->payloadlen;
	xRoot.eItemType = DJSON_Object;

	xItem = DJSON_GetObjectItem(&xRoot, "command");
	if (xItem.eItemType == DJSON_Number)
	{
		double value = DJSON_GetValueDouble(&xItem);
		unsigned int cmd = (unsigned int)value;

		cmdValue = cmd;
	}

	g_bRemoteCtrl = true;
}

#if CONFIG_BLUETOOTH > 0
static void WIT_GatewaySenRegisterHandler(MessageData *pxData)
{
	bool bStatus = true;
	bool bNeedUpdateCfg = true;
	DJSON xRoot;
	DJSON xSSArray;
	DJSON xSSObj;
	DJSON xCfgArray;
	DJSON xCfgObj;
	DJSON xItem;
	char cAddr[4];

	unsigned long ulSSArraySize;
	unsigned long ulcfgArraySize;

	xRoot.pcItemPointer = pxData->message->payload;
	xRoot.ulItemLength = pxData->message->payloadlen;
	xRoot.eItemType = DJSON_Object;
	memset(g_xGatewayResponse.cTaskID, 0, sizeof(g_xGatewayResponse.cTaskID));

	xItem = DJSON_GetObjectItem(&xRoot, "taskId");
	DJSON_GetValueString(&xItem, g_xGatewayResponse.cTaskID, sizeof(g_xGatewayResponse.cTaskID));

	// Get ss array
	xSSArray = DJSON_GetObjectItem(&xRoot, "ss");
	ulSSArraySize = DJSON_GetArraySize(&xSSArray);

	if (ulSSArraySize == 0 || xSSArray.eItemType != DJSON_Array)
	{
		bStatus = false;
		bNeedUpdateCfg = false;
		goto register_error;
	}

	xSSObj = DJSON_GetArrayItem(&xSSArray, 0);
	xItem = DJSON_GetObjectItem(&xSSObj, "addr");
	if (xItem.eItemType != DJSON_String)
	{
		bStatus = false;
		bNeedUpdateCfg = false;
		goto register_error;
	}

	DJSON_GetValueString(&xItem, cAddr, sizeof(cAddr));
	strncpy(g_xGatewayResponse.cAddr, cAddr, 4);

	xCfgArray = DJSON_GetObjectItem(&xSSObj, "cfg");
	ulcfgArraySize = DJSON_GetArraySize(&xCfgArray);
	if (ulcfgArraySize == 0 || xCfgArray.eItemType != DJSON_Array)
	{
		bStatus = false;
		bNeedUpdateCfg = false;
		goto register_error;
	}
	memset(g_xWITConfig.xSenTable, 0, sizeof(g_xWITConfig.xSenTable));
	if_clear_multi_gateway_data();
	for (int i = 0; i < ulcfgArraySize; i++)
	{
		unsigned int uiAddr = 0;
		xCfgObj = DJSON_GetArrayIItem(&xCfgArray, i);
		xItem = DJSON_GetObjectItem(&xCfgObj, "addr");
		if (xItem.eItemType != DJSON_Number)
		{
			bStatus = false;
			bNeedUpdateCfg = false;
			goto register_error;
		}
		uiAddr = DJSON_GetValueInt(&xItem);
		g_xWITConfig.xSenTable[uiAddr - 1].ulAddr = uiAddr;
		xItem = DJSON_GetObjectItem(&xCfgObj, "SenName");
		if (xItem.eItemType != DJSON_String)
		{
			bStatus = false;
			bNeedUpdateCfg = false;
			goto register_error;
		}
		DJSON_GetValueString(&xItem, g_xWITConfig.xSenTable[uiAddr - 1].cName, sizeof(g_xWITConfig.xSenTable[uiAddr - 1].cName));
		ESP_LOGD("SenRegister","addr %d , name %s", uiAddr, g_xWITConfig.xSenTable[uiAddr - 1].cName);
		xItem = DJSON_GetObjectItem(&xCfgObj, "SenMac");
		if (xItem.eItemType != DJSON_String)
		{
			bStatus = false;
			bNeedUpdateCfg = false;
			goto register_error;
		}
		DJSON_GetValueString(&xItem, g_xWITConfig.xSenTable[uiAddr - 1].cMac, sizeof(g_xWITConfig.xSenTable[uiAddr - 1].cMac));
		g_xWITConfig.xSenTable[uiAddr - 1].ulUseEnable = 1;
	}
	if (bNeedUpdateCfg == true)
	{
		xTimerStart(g_xCfgUpTimer, 1000);
	}
	else
	{
		read_flash(E_GW_CONFIG, &g_xWITConfig, sizeof(g_xWITConfig));
	}
register_error:
	g_xGatewayResponse.pcTopic = g_xWITMQTT.cPubSenRegisterRespTopic;
	if (bStatus == true)
	{
		g_xGatewayResponse.eStatus = Resp_Success;
	}
	else
	{
		g_xGatewayResponse.eStatus = Resp_ParmErr;
	}
}
#endif

static void WIT_GatewayExtConfigHandler(MessageData *pxData)
{
	DJSON xRoot;

	xRoot.pcItemPointer = pxData->message->payload;
	xRoot.ulItemLength = pxData->message->payloadlen;
	xRoot.eItemType = DJSON_Object;
	do
	{
		DJSON addrItem = DJSON_GetObjectItem(&xRoot, "addr");
		if (addrItem.eItemType != DJSON_Number)
			break;

		unsigned char addr = DJSON_GetValueInt(&addrItem);
		if (addr == 0)
		{
			// Gateway config
			ExConfigrespCode = 1;
			break;
		}
		else if (addr > 6)
		{
			// invailed addr
			ExConfigrespCode = 1;
			break;
		}

		// sensor config
		DJSON configItem = DJSON_GetObjectItem(&xRoot, "config");
		if (configItem.eItemType != DJSON_Object)
		{
			ExConfigrespCode = 2;
			break;
		}
		// ODR Config
		DJSON xObject = DJSON_GetObjectItem(&configItem, "odr");
		if (xObject.eItemType == DJSON_Number)
		{
#if CONFIG_RS485_RTU > 0
			unsigned short odr = ((unsigned short)DJSON_GetValueInt(&xObject)) % 6;
			if (WIT_SensorOperation(addr, Acc_SetOdr, WIT_ACC_SENSOR_ID, odr) == false)
			{
				ExConfigrespCode = 3;
				break;
			}
#endif
		}
		// Sample sum Config
		xObject = DJSON_GetObjectItem(&configItem, "sampSum");
		if (xObject.eItemType == DJSON_Number)
		{
			unsigned short sampSum = ((unsigned short)DJSON_GetValueInt(&xObject));
			if (sampSum < 2048)
			{
				sampSum = 2048;
			}
#if CONFIG_RS485_RTU > 0
			if (WIT_SensorOperation(addr, Acc_SetSampSum, WIT_ACC_SENSOR_ID, sampSum) == false)
			{
				ExConfigrespCode = 4;
				break;
			}
#endif
		}
		// HPF enable Config
		xObject = DJSON_GetObjectItem(&configItem, "hpfEn");
		if (xObject.eItemType == DJSON_Number)
		{
#if CONFIG_RS485_RTU > 0
			unsigned short hpfEn = ((unsigned short)DJSON_GetValueInt(&xObject)) % 2;
			if (WIT_SensorOperation(addr, Acc_SetHpfEn, WIT_ACC_SENSOR_ID, hpfEn) == false)
			{
				ExConfigrespCode = 5;
				break;
			}
#endif
		}
		// anomaly vibration threshold
		xObject = DJSON_GetObjectItem(&configItem, "amlVibTh");
		if (xObject.eItemType == DJSON_Number)
		{
#if CONFIG_RS485_RTU > 0
			unsigned short amlVibTh = (unsigned short)DJSON_GetValueInt(&xObject);
			if (WIT_SensorOperation(addr, Acc_SetAmlVibTh, WIT_ACC_SENSOR_ID, amlVibTh) == false)
			{
				ExConfigrespCode = 6;
				break;
			}
#endif
		}
		// anomaly vibration interval
		xObject = DJSON_GetObjectItem(&configItem, "amlVibInterval");
		if (xObject.eItemType == DJSON_Number)
		{
#if CONFIG_RS485_RTU > 0
			unsigned short amlVibInterval = (unsigned short)DJSON_GetValueInt(&xObject);
			if (WIT_SensorOperation(addr, Acc_SetAmlVibInterval, WIT_ACC_SENSOR_ID, amlVibInterval) == false)
			{
				ExConfigrespCode = 7;
				break;
			}
#endif
		}

		// anomaly vibration keep cnt
		xObject = DJSON_GetObjectItem(&configItem, "amlVibKeepCnt");
		if (xObject.eItemType == DJSON_Number)
		{
#if CONFIG_RS485_RTU > 0
			unsigned short amlVibKeepCnt = ((unsigned short)DJSON_GetValueInt(&xObject)) % 100;
			if (WIT_SensorOperation(addr, Acc_SetAmlVibKeepCnt, WIT_ACC_SENSOR_ID, amlVibKeepCnt) == false)
			{
				ExConfigrespCode = 8;
				break;
			}
#endif
		}

		// sensor PSRAM high temperature protect feature
		xObject = DJSON_GetObjectItem(&configItem, "highTempProtect");
		if (xObject.eItemType == DJSON_Number)
		{
#if CONFIG_RS485_RTU > 0
			unsigned short highTempProtect = (unsigned short)DJSON_GetValueInt(&xObject);
			highTempProtect %= 2;
			if (WIT_SensorOperation(addr, Acc_SetHighTempProtect, WIT_ACC_SENSOR_ID, highTempProtect) == false)
			{
				ExConfigrespCode = 9;
				break;
			}
#endif
		}

		ExConfigrespCode = 200;
	} while (0);

	ExConfigResponeFlag = 1;
	return;
}

static void WIT_GatewaySyncTimeHandler(MessageData *pxData)
{
	DJSON xRoot;
	DJSON xItem;

	xRoot.pcItemPointer = pxData->message->payload;
	xRoot.ulItemLength = pxData->message->payloadlen;
	xRoot.eItemType = DJSON_Object;
	xItem = DJSON_GetObjectItem(&xRoot, "ts");
	if (xItem.eItemType == DJSON_Number)
	{
		double timestamp;
		// unsigned long ullasttime = 0;
		// ullasttime = RTC_InternalGetTimeStamp();
		timestamp = DJSON_GetValueInt(&xItem);
		RTC_InternalSetTimeStamp(timestamp);
		// #if CONFIG_BLUETOOTH > 0
		// 	if_calibration_time(ullasttime);
		// 	ble_set_CheckTime(true);
		// #endif
		g_bTimeSyncOver = true;
	}
}

static void WIT_GatewayBackDoorHandler(MessageData *pxData)
{
	DJSON xRoot;
	DJSON xItem;

	unsigned long ulOtherArraySize = 0;
	char other_buf[WIT_SENSOR_GROUP_MAX_NUM * WIT_SENSOR_MAX_CONNECT_NUM] = {0};

	xRoot.pcItemPointer = pxData->message->payload;
	xRoot.ulItemLength = pxData->message->payloadlen;
	xRoot.eItemType = DJSON_Object;

	xItem = DJSON_GetObjectItem(&xRoot, "cmd");
	if (xItem.eItemType == DJSON_Number)
	{
		int cmd;
		cmd = DJSON_GetValueInt(&xItem);
		if (cmd == 1)
		{
			g_bBackDoorReset = true;
			g_bBackDoorCode = 1;
		}
		if (cmd == 3)
		{
			clear_sensor_updata_flag();
			g_bBackDoorCode = 3;
		}
	}
	xItem = DJSON_GetObjectItem(&xRoot, "WorkTime");
	if (xItem.eItemType == DJSON_Number)
	{
		unsigned long worktime = 0;
		worktime = DJSON_GetValueInt(&xItem);
		g_xWITConfig.xWorkDet.ulWorkMinutes = worktime;
		g_bUpdateBackupArea = true;
		g_bBackDoorCode = 2;
	}
	xItem = DJSON_GetObjectItem(&xRoot, "updateAddr");
	ulOtherArraySize = DJSON_GetArrayInt(&xItem, other_buf);
	if (xItem.eItemType == DJSON_Array)
	{
		for (int i = 0; i < ulOtherArraySize; i++)
		{
			printf("addr %d\r\n", other_buf[i]);
			if (other_buf[i] == 0)
			{
				continue;
			}
			for (int j = 0; j < WIT_SENSOR_GROUP_MAX_NUM * WIT_SENSOR_MAX_CONNECT_NUM; j++)
			{
				if (g_xWITConfig.xSenTable[j].ulAddr == other_buf[i])
				{
					g_xWITConfig.xSenTable[j].ulUpdate = 1;
				}	
			}
		}
		g_bBackDoorCode = 4;
		WIT_SetUpdateConfigureAreaFlag(true);
	}
	g_bBackDoorResponse = true;
}

#if CONFIG_ESP32_WIFI > 0
/**
 * @brief  UDP server request handler
 * @param
 * @retval None
 */
static void WIT_UDPSRequestHandler(unsigned char ucCommand, unsigned char *pucData, unsigned short usLen)
{
	unsigned char ucRespError = UDPS_SUCCESS_CODE;
	unsigned short usRespLen = 0;
	bool bReset = false;

	if ((ucCommand & UDPS_WRITE_BIT) == UDPS_WRITE_BIT)
	{
		unsigned short usTemp;

		// 写操作
		switch (ucCommand & 0x7F)
		{
		case UDPS_WIFI_INFO_COMMAND:
		{
			if (usLen >= 2)
			{
				usTemp = strlen((char *)pucData) + 1;
				if (strlen((char *)pucData) < WIFI_MAX_NAME_LENGTH &&
					strlen((char *)(pucData + usTemp)) < WIFI_MAX_NAME_LENGTH)
				{
					// SSID
					strcpy(g_xWITConfig.xWIFI.cSSID, (char *)pucData);

					// Password
					if (strlen((char *)(pucData + usTemp)) == 0)
					{
						strcpy(g_xWITConfig.xWIFI.cPassword, "NONE");
					}
					else
					{
						strcpy(g_xWITConfig.xWIFI.cPassword, (char *)(pucData + usTemp));
					}
					g_xWITConfig.bConfigUpdated = true;
					// Update config
					xTimerStart(g_xCfgQuickUpTimer, 1000);
				}
				else
				{
					ucRespError = UDPS_ERROR_PARM_CODE;
				}
			}
			else
			{
				ucRespError = UDPS_ERROR_PARM_CODE;
			}
		}
		break;
		case UDPS_NET_INFO_COMMAND:
		{
			if (usLen >= 13)
			{
				// IP
				sprintf(g_xWITConfig.xWIFI.cIP, "%d.%d.%d.%d", pucData[0], pucData[1], pucData[2], pucData[3]);
				// MAC(不可设置)

				// DHCP
				if (pucData[4] == 0x00)
				{
					sprintf(g_xWITConfig.xWIFI.cDHCP, "STATIC");
				}
				else
				{
					sprintf(g_xWITConfig.xWIFI.cDHCP, "DHCP");
				}
				// Mask
				sprintf(g_xWITConfig.xWIFI.cMask, "%d.%d.%d.%d", pucData[5], pucData[6], pucData[7], pucData[8]);
				// Gateway
				sprintf(g_xWITConfig.xWIFI.cGateway, "%d.%d.%d.%d", pucData[9], pucData[10], pucData[11], pucData[12]);
				strcpy(g_xWITConfig.xWIFI.cDNS, g_xWITConfig.xWIFI.cGateway);

				g_xWITConfig.bConfigUpdated = true;
				// Update config
				xTimerStart(g_xCfgQuickUpTimer, 1000);
			}
			else
			{
				ucRespError = UDPS_ERROR_PARM_CODE;
			}
		}
		break;
		case UDPS_RESET_COMMAND:
		{
			bReset = true;
		}
		break;
		case UDPS_IMG_UPDATE_COMMAND:
		{
			unsigned char ucPackageNumber;

			if (Update_GetStatus() == false)
			{
				if (usLen >= 18)
				{
					static unsigned char s_ucTotalPackageNumber = 0;
					unsigned char *pucUpdateData = NULL;
					unsigned short usUpdateLen = 0;

					ucPackageNumber = pucData[0];

					if (ucPackageNumber == 0)
					{
						s_ucTotalPackageNumber = pucData[1];
					}

					pucUpdateData = pucData + 1;
					usUpdateLen = usLen - 1;
					// Check length
					if ((ucPackageNumber == 0 && usUpdateLen == 16) || (ucPackageNumber != 0 && usUpdateLen == 1024))
					{
						if (UDP_Update_DataHandler(ucPackageNumber, pucUpdateData, usLen - 2) == false)
						{
							ucRespError = UDPS_ERROR_FAIL_CODE;
						}
						else
						{
							if (s_ucTotalPackageNumber == ucPackageNumber && s_ucTotalPackageNumber > 0)
							{
								bReset = true;
							}
						}
					}
					else
					{
						ucRespError = UDPS_ERROR_PARM_CODE;
					}
				}
				else
				{
					ucRespError = UDPS_ERROR_PARM_CODE;
				}
			}
			else
			{
				ucRespError = UDPS_ERROR_FAIL_CODE;
			}
		}
		break;
		case UDPS_SENSOR_IMG_UPDATE_COMMAND:
		{
			if (Sensor_GetUpdateStatus() == false)
			{
				if (usLen >= 21)
				{
					static unsigned char s_ucSensorAddr;
					unsigned short usPackageNumber;
					unsigned char *pucUpdateData = NULL;
					unsigned short usUpdateLen = 0;

					usPackageNumber = (unsigned short)(pucData[0] << 8) | pucData[1];
					if (usPackageNumber == 0)
					{
						s_ucSensorAddr = pucData[4];
					}
					pucUpdateData = pucData + 2;
					usUpdateLen = usLen - 2;

					xSemaphoreTake(g_xMBSignal[(s_ucSensorAddr - 1) / WIT_PER_PORT_SENSOR_COUNT], portMAX_DELAY);
					// Enabel acc
					WIT_AccEnable(s_ucSensorAddr, WIT_ACC_SENSOR_ID);
					if ((s_ucSensorAddr > 0 && s_ucSensorAddr <= 6) &&
						((usPackageNumber == 0 && usUpdateLen == 19) || (usPackageNumber != 0 && usUpdateLen == 240)))
					{
						if (UDP_SensorUpdateDataHandler(WIT_ACC_SENSOR_ID, usPackageNumber, pucUpdateData, usUpdateLen) == false)
						{
							ucRespError = UDPS_ERROR_FAIL_CODE;
						}
						else
						{
						}
					}
					else
					{
						ucRespError = UDPS_ERROR_PARM_CODE;
					}

					xSemaphoreGive(g_xMBSignal[(s_ucSensorAddr - 1) / WIT_PER_PORT_SENSOR_COUNT]);
				}
			}
			else
			{
				ucRespError = UDPS_ERROR_FAIL_CODE;
			}
		}
		break;
		default:
		{
			ucRespError = UDPS_ERROR_COMMAND_CODE;
		}
		break;
		}

		if (ucRespError == UDPS_SUCCESS_CODE)
			g_ucUDPSRespBuffer[usRespLen++] = UDPS_SUCCESS_CODE;
	}
	else
	{
		// 读操作
		switch (ucCommand & 0x7F)
		{
		case UDPS_MODULE_INFO_COMMAND:
		{
			// Module name
			strcpy((char *)g_ucUDPSRespBuffer, g_xWITConfig.cModuleName);
			usRespLen += strlen(g_xWITConfig.cModuleName) + 1;
			// Serial number
			g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xWITConfig.ulSerialNumber >> 24);
			g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xWITConfig.ulSerialNumber >> 16);
			g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xWITConfig.ulSerialNumber >> 8);
			g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xWITConfig.ulSerialNumber);
			// Version
			g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(WIT_VERSION_HEX >> 8);
			g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(WIT_VERSION_HEX);
		}
		break;
		case UDPS_WIFI_INFO_COMMAND:
		{
			strcpy((char *)g_ucUDPSRespBuffer, g_xCurrentWIFIInfo.cSSID);
			usRespLen += strlen(g_xCurrentWIFIInfo.cSSID) + 1;
			strcpy((char *)(g_ucUDPSRespBuffer + usRespLen), g_xCurrentWIFIInfo.cPassword);
			usRespLen += strlen(g_xCurrentWIFIInfo.cPassword) + 1;
		}
		break;
		case UDPS_NET_INFO_COMMAND:
		{
			int iTemp[6], i;

			// IP
			sscanf(g_xCurrentWIFIInfo.cIP, "%d.%d.%d.%d", &iTemp[0], &iTemp[1], &iTemp[2], &iTemp[3]);
			for (i = 0; i < 4; i++)
			{
				g_ucUDPSRespBuffer[usRespLen++] = iTemp[i];
			}
			// MAC
			sscanf(g_xCurrentWIFIInfo.cMAC, "%02X%02X%02X%02X%02X%02X", &iTemp[0], &iTemp[1], &iTemp[2], &iTemp[3], &iTemp[4], &iTemp[5]);
			for (i = 0; i < 6; i++)
			{
				g_ucUDPSRespBuffer[usRespLen++] = iTemp[i];
			}
			// DHCP
			if (strstr(g_xCurrentWIFIInfo.cDHCP, "STATIC") != NULL)
			{
				g_ucUDPSRespBuffer[usRespLen++] = 0x00;
			}
			else
			{
				g_ucUDPSRespBuffer[usRespLen++] = 0x01;
			}
			// MASK
			sscanf(g_xCurrentWIFIInfo.cMask, "%d.%d.%d.%d", &iTemp[0], &iTemp[1], &iTemp[2], &iTemp[3]);
			for (i = 0; i < 4; i++)
			{
				g_ucUDPSRespBuffer[usRespLen++] = iTemp[i];
			}
			// Gateway
			sscanf(g_xCurrentWIFIInfo.cGateway, "%d.%d.%d.%d", &iTemp[0], &iTemp[1], &iTemp[2], &iTemp[3]);
			for (i = 0; i < 4; i++)
			{
				g_ucUDPSRespBuffer[usRespLen++] = iTemp[i];
			}
		}
		break;
		case UDPS_DATA_INFO_COMMAND:
		{
			if (usLen == 0 || usLen == 1)
			{
				int i;

				// 网关数据
				g_ucUDPSRespBuffer[usRespLen++] = 0x00;
				g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(0);
				g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(0);
				g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(0);
				g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(0);
				g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(0);
				g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(0);
				g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(0);
				g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(0);
				g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPGWData.ulWorkTime >> 24);
				g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPGWData.ulWorkTime >> 16);
				g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPGWData.ulWorkTime >> 8);
				g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPGWData.ulWorkTime);
				// 传感器数据
				for (i = 0; i < ACC_MAX_SENSOR_COUNT; i++)
				{
					if (usLen == 1 && (i + 1) != pucData[0])
						continue;

					if (g_bIsSensorExist[i] == true)
					{
						g_ucUDPSRespBuffer[usRespLen++] = i + 1;
						g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPSensorData[i].usVersion >> 8);
						g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPSensorData[i].usVersion);
						g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPSensorData[i].usSensorTemp >> 8);
						g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPSensorData[i].usSensorTemp);
						g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPSensorData[i].ulXVRMS >> 24);
						g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPSensorData[i].ulXVRMS >> 16);
						g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPSensorData[i].ulXVRMS >> 8);
						g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPSensorData[i].ulXVRMS);
						g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPSensorData[i].ulYVRMS >> 24);
						g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPSensorData[i].ulYVRMS >> 16);
						g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPSensorData[i].ulYVRMS >> 8);
						g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPSensorData[i].ulYVRMS);
						g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPSensorData[i].ulZVRMS >> 24);
						g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPSensorData[i].ulZVRMS >> 16);
						g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPSensorData[i].ulZVRMS >> 8);
						g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPSensorData[i].ulZVRMS);
						g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPSensorData[i].ulXKur >> 24);
						g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPSensorData[i].ulXKur >> 16);
						g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPSensorData[i].ulXKur >> 8);
						g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPSensorData[i].ulXKur);
						g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPSensorData[i].ulYKur >> 24);
						g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPSensorData[i].ulYKur >> 16);
						g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPSensorData[i].ulYKur >> 8);
						g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPSensorData[i].ulYKur);
						g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPSensorData[i].ulZKur >> 24);
						g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPSensorData[i].ulZKur >> 16);
						g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPSensorData[i].ulZKur >> 8);
						g_ucUDPSRespBuffer[usRespLen++] = (unsigned char)(g_xUDPSensorData[i].ulZKur);
					}
				}
			}
			else
			{
				ucRespError = UDPS_ERROR_PARM_CODE;
			}
		}
		break;
		case UDPS_GET_SENSOR_EXIST_STATUS:
		{
			int i;
			unsigned char ucSensorExistStatus = 0;

			for (i = 0; i < ACC_MAX_SENSOR_COUNT; i++)
			{
				if (g_bIsSensorExist[i] == true)
				{
					ucSensorExistStatus |= (1 << i);
				}
			}

			g_ucUDPSRespBuffer[usRespLen++] = ucSensorExistStatus;
		}
		break;
		default:
		{
			ucRespError = UDPS_ERROR_COMMAND_CODE;
		}
		break;
		}
	}

	UDPS_SendData(g_lUDPSocketID, ucRespError, ucCommand, g_ucUDPSRespBuffer, usRespLen);

	// Check reset
	if (bReset == true)
	{
		vTaskDelay(2000);
		PWR_Restart();
	}
}
#endif

static void WIT_UpdateConfigureArea()
{
	write_flash(E_GW_CONFIG, &g_xWITConfig, sizeof(g_xWITConfig));
	write_flash(E_GWBP_CONFIG, &g_xWITConfig, sizeof(g_xWITConfig));
}

static void WIT_UpdateThresholdArea()
{
	write_flash(E_THRESHOLD_STORE, g_pxGlobalThr, g_ulTotalParameterSize * sizeof(Threshold));
}

static void WIT_UpdateDLIArea()
{
	write_flash(E_FEA_HISTORY_STORE, g_pxGlobalDLI, g_ulTotalParameterSize * sizeof(DataListInfo));
}

#if CONFIG_RS485_RTU > 0
static void WIT_RestoreGWDataIntoExtFlash()
{
	if (g_xWITConfig.eGatewayMode == GatewayMode_Learning)
	{
		write_flash(E_FEATURE_STORE, &g_xWITData.xGWFea, sizeof(g_xWITData.xGWFea));
	}
}

static void WIT_RestoreSensorDataIntoExtFlash(unsigned char ucSensorAddr)
{
	if (g_xWITConfig.eGatewayMode == GatewayMode_Learning)
	{
		switch (ucSensorAddr)
		{
		case 1:
			write_flash(E_SEN_FEA1_STORE, &g_xWITData.xSensFea, sizeof(g_xWITData.xSensFea));
			break;
		case 2:
			write_flash(E_SEN_FEA2_STORE, &g_xWITData.xSensFea, sizeof(g_xWITData.xSensFea));
			break;
		case 3:
			write_flash(E_SEN_FEA3_STORE, &g_xWITData.xSensFea, sizeof(g_xWITData.xSensFea));
			break;
		case 4:
			write_flash(E_SEN_FEA4_STORE, &g_xWITData.xSensFea, sizeof(g_xWITData.xSensFea));
			break;
		case 5:
			write_flash(E_SEN_FEA5_STORE, &g_xWITData.xSensFea, sizeof(g_xWITData.xSensFea));
			break;
		case 6:
			write_flash(E_SEN_FEA6_STORE, &g_xWITData.xSensFea, sizeof(g_xWITData.xSensFea));
			break;
		default:
			break;
		}
	}
}
#endif

static void WIT_CalculateThreshold()
{
	int i, j;
	Threshold *pxThrPointer = NULL;
	DataListInfo *pxDLIPointer = NULL;

	pxThrPointer = g_pxGatewayThr;
	pxDLIPointer = g_pxGatewayDLI;

	// Gateway Threshold
	for (i = 0; i < g_ulGatewayParameterSize && g_xWITConfig.eGatewayMode == GatewayMode_Learning; i++)
	{
		if (pxDLIPointer->bUsed == true)
		{
			// CalculateAdaptiveThreshold(NULL, pxDLIPointer->usCount, &pxThrPointer->fHighLimit, &pxThrPointer->fLowLimit,
			// 						   W25Qxx_Read, pxDLIPointer->ulStartAddress, WIT_AUTO_THRESHOLD_SCALE);
		}
		pxThrPointer++;
		pxDLIPointer++;
	}

	// Sensor Threshold
	for (i = 0; i < ACC_MAX_SENSOR_COUNT; i++)
	{
		if (g_bIsSensorExist[i] == false)
			continue;

		pxThrPointer = g_pxSensorThr + i * g_ulSensorParameterSize;
		pxDLIPointer = g_pxSensorDLI + i * g_ulSensorParameterSize;

		for (j = 0; j < g_ulSensorParameterSize && g_xWITConfig.eGatewayMode == GatewayMode_Learning; j++)
		{
			if (pxDLIPointer->bUsed == true)
			{
				// CalculateAdaptiveThreshold(NULL, pxDLIPointer->usCount, &pxThrPointer->fHighLimit, &pxThrPointer->fLowLimit,
				// 						   W25Qxx_Read, pxDLIPointer->ulStartAddress, WIT_AUTO_THRESHOLD_SCALE);
			}
			pxThrPointer++;
			pxDLIPointer++;
		}
	}
}

#if CONFIG_RS485_RTU > 0
static bool WIT_SendThreshold()
{
	unsigned long ulLength;
	int i;
	// pointer
	Parameter *pxParmPointer = NULL;
	Threshold *pxThrPointer = NULL;
	DataListInfo *pxDLIPointer = NULL;
	unsigned long ulParmCount;
	bool bStatus = true;

	for (i = 0; i <= ACC_MAX_SENSOR_COUNT; i++)
	{
		if (i == 0)
		{
			pxParmPointer = (Parameter *)&g_xWITData.xGWFea;
			pxThrPointer = g_pxGatewayThr;
			pxDLIPointer = g_pxGatewayDLI;
			ulParmCount = g_ulGatewayParameterSize;
		}
		else
		{
			if (g_bIsSensorExist[i - 1] == false)
				continue;

			pxParmPointer = (Parameter *)&g_xWITData.xSensFea;
			pxThrPointer = g_pxSensorThr + (i - 1) * g_ulSensorParameterSize;
			pxDLIPointer = g_pxSensorDLI + (i - 1) * g_ulSensorParameterSize;
			ulParmCount = g_ulSensorParameterSize;
		}

		// 锁g_xWITData
		xSemaphoreTake(g_xWITDataSignal, portMAX_DELAY);
		// 更新轴承齿轮别名
		if (i != 0)
			WIT_UpdateFeatureName(i);
		ulLength = WIT_PackageThresholdIntoExtMemory(i, pxParmPointer, pxThrPointer, pxDLIPointer, ulParmCount, RTC_InternalGetTimeStamp(), write_mqtt_feature_data, 0);
		xSemaphoreGive(g_xWITDataSignal);
		if (MQTT_PublishFromExtMemory(g_xWITMQTT.lHandle, g_xWITMQTT.cPubFeatureTopic, ulLength, false, false, QOS1, read_mqtt_feature_data, 0) == false)
		{
			bStatus = false;
		}
	}

	return bStatus;
}
// Sensor operation
static bool WIT_SensorOperation(unsigned char ucSensorAddr, void *pvOp, ...)
{
	bool bStatus;
	va_list xVaList;
	unsigned char ucSensorID = 0, ucPort = 0;

	if (pvOp == NULL || ucSensorAddr <= 0 || ucSensorAddr > ACC_MAX_SENSOR_COUNT)
		return false;

	// 信号量 锁 2个485对应不同信号量
	xSemaphoreTake(g_xMBSignal[(ucSensorAddr - 1) / WIT_PER_PORT_SENSOR_COUNT], portMAX_DELAY);
	va_start(xVaList, pvOp);
	// Parameter - sensor id
	ucSensorID = va_arg(xVaList, int);
	ucPort = ACC_ADDR_TO_PORT(ucSensorAddr);
	// Enabel acc
	WIT_AccEnable(ucSensorAddr, ucSensorID);

	if (pvOp == Acc_GetWaveIntoExtMemory)
	{
		void *pvWave = NULL;
		void (*pvWrFunc)(unsigned long, unsigned char *, unsigned long);
		unsigned long ulAddr;

		pvWave = va_arg(xVaList, void *);
		pvWrFunc = va_arg(xVaList, void (*)(unsigned long, unsigned char *, unsigned long));
		ulAddr = va_arg(xVaList, unsigned long);
		bStatus = Acc_GetWaveIntoExtMemory(ucPort, ucSensorID, pvWave, pvWrFunc, ulAddr);
	}
	else if (pvOp == Acc_GetFeature)
	{
		void *pvFeature = NULL, *pvBS = NULL, *pvGS = NULL, *pvSS = NULL, *pvFanS = NULL, *pvBeltS = NULL, *pvScrewS = NULL, *pvShaftS = NULL, *pvMaleScrewS = NULL, *pvFemaleScrewS = NULL, *pvSleeveS = NULL, *pvOdr = NULL, *pvSampSum = NULL, *pvHpfEn = NULL, *pvResolution = NULL;

		pvFeature = va_arg(xVaList, void *);
		pvBS = va_arg(xVaList, void *);
		pvGS = va_arg(xVaList, void *);
		pvSS = va_arg(xVaList, void *);
		pvFanS = va_arg(xVaList, void *);
		pvBeltS = va_arg(xVaList, void *);
		pvScrewS = va_arg(xVaList, void *);
		pvShaftS = va_arg(xVaList, void *);
		pvMaleScrewS = va_arg(xVaList, void *);
		pvFemaleScrewS = va_arg(xVaList, void *);
		pvSleeveS = va_arg(xVaList, void *);
		pvOdr = va_arg(xVaList, void *);
		pvSampSum = va_arg(xVaList, void *);
		pvHpfEn = va_arg(xVaList, void *);
		pvResolution = va_arg(xVaList, void *);

		bStatus = Acc_GetFeature(ucPort, ucSensorID, pvFeature, pvBS, pvGS, pvSS, pvFanS, pvBeltS, pvScrewS, pvShaftS, pvMaleScrewS, pvFemaleScrewS, pvSleeveS, pvOdr, pvSampSum, pvHpfEn, pvResolution);
	}
	else if (pvOp == Acc_GetExtendFeature)
	{
		void *pvExtFeature = NULL, *pvExtBS = NULL, *pvExtGS = NULL, *pvExtIS = NULL, *pvExtOS = NULL;

		pvExtFeature = va_arg(xVaList, void *);
		pvExtBS = va_arg(xVaList, void *);
		pvExtGS = va_arg(xVaList, void *);
		pvExtIS = va_arg(xVaList, void *);
		pvExtOS = va_arg(xVaList, void *);

		bStatus = Acc_GetExtendFeature(ucPort, ucSensorID, pvExtFeature, pvExtBS, pvExtGS, pvExtIS, pvExtOS);
	}
	else if (pvOp == Acc_SetBearingInfo || pvOp == Acc_SetBearingFrequency)
	{
		unsigned char ucNumber;
		void *pvComponent = NULL;

		ucNumber = va_arg(xVaList, int);
		pvComponent = va_arg(xVaList, void *);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned char, Bearing *))pvOp)(ucPort, ucSensorID, ucNumber, pvComponent);
	}
	else if (pvOp == Acc_SetGearInfo || pvOp == Acc_SetGearFrequency)
	{
		unsigned char ucNumber;
		void *pvComponent = NULL;

		ucNumber = va_arg(xVaList, int);
		pvComponent = va_arg(xVaList, void *);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned char, Gear *))pvOp)(ucPort, ucSensorID, ucNumber, pvComponent);
	}
	else if (pvOp == Acc_SetSpeed)
	{
		unsigned long ulSpeed;

		ulSpeed = va_arg(xVaList, unsigned long);
		bStatus = Acc_SetSpeed(ucPort, ucSensorID, ulSpeed);
	}
	else if (pvOp == Acc_SetPowerFrequency)
	{
		unsigned short usSpowerFrequency;

		usSpowerFrequency = va_arg(xVaList, unsigned long);
		bStatus = Acc_SetPowerFrequency(ucPort, ucSensorID, usSpowerFrequency);
	}
	else if (pvOp == Acc_GetVersion || pvOp == Acc_GetSensorStatus || pvOp == Acc_GetSamplingRate || pvOp == Acc_GetActiveFlag ||
			 pvOp == Acc_GetMode)
	{
		unsigned short *pusParm;

		pusParm = va_arg(xVaList, unsigned short *);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned short *))pvOp)(ucPort, ucSensorID, pusParm);
	}
	else if (pvOp == Acc_SetMode || pvOp == Acc_SetShockThreshold)
	{
		unsigned short usSetValue;

		usSetValue = va_arg(xVaList, int);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned short))pvOp)(ucPort, ucSensorID, usSetValue);
	}
	else if (pvOp == Acc_SetBaseExtendConfig)
	{
		void *pvExtCfgPointer = NULL;

		pvExtCfgPointer = va_arg(xVaList, void *);
		bStatus = ((bool (*)(unsigned char, unsigned char, Base_ExtendConfig *))pvOp)(ucPort, ucSensorID, pvExtCfgPointer);
	}
	else if (pvOp == Acc_SetBearingExtendConfig)
	{
		unsigned char ucNumber;
		void *pvExtCfgPointer = NULL;

		ucNumber = va_arg(xVaList, int);
		pvExtCfgPointer = va_arg(xVaList, void *);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned char, Bearing_ExtendConfig *))pvOp)(ucPort, ucSensorID, ucNumber, pvExtCfgPointer);
	}
	else if (pvOp == Acc_SetGearExtendConfig)
	{
		unsigned char ucNumber;
		void *pvExtCfgPointer = NULL;

		ucNumber = va_arg(xVaList, int);
		pvExtCfgPointer = va_arg(xVaList, void *);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned char, Gear_ExtendConfig *))pvOp)(ucPort, ucSensorID, ucNumber, pvExtCfgPointer);
	}
	else if (pvOp == Acc_GetBearingAlias || pvOp == Acc_GetGearAlias)
	{
		unsigned char ucNumber;
		unsigned short *pusAlias;

		ucNumber = va_arg(xVaList, int);
		pusAlias = va_arg(xVaList, unsigned short *);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned char, unsigned short *))pvOp)(ucPort, ucSensorID, ucNumber, pusAlias);
	}
	else if (pvOp == Acc_SetBearingAlias || pvOp == Acc_SetGearAlias)
	{
		unsigned char ucNumber;
		unsigned short usAlias;

		ucNumber = va_arg(xVaList, int);
		usAlias = va_arg(xVaList, int);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned char, unsigned short))pvOp)(ucPort, ucSensorID, ucNumber, usAlias);
	}
	else if (pvOp == Acc_SetFanConfigure)
	{
		void *pvCfgPointer = NULL;

		pvCfgPointer = va_arg(xVaList, void *);
		bStatus = ((bool (*)(unsigned char, unsigned char, Fan *))pvOp)(ucPort, ucSensorID, pvCfgPointer);
	}
	else if (pvOp == Acc_SetBeltConfigure)
	{
		void *pvCfgPointer = NULL;

		pvCfgPointer = va_arg(xVaList, void *);
		bStatus = ((bool (*)(unsigned char, unsigned char, Belt *))pvOp)(ucPort, ucSensorID, pvCfgPointer);
	}
	else if (pvOp == Acc_SetScrewConfigure)
	{
		void *pvCfgPointer = NULL;

		pvCfgPointer = va_arg(xVaList, void *);
		bStatus = ((bool (*)(unsigned char, unsigned char, Screw *))pvOp)(ucPort, ucSensorID, pvCfgPointer);
	}
	else if (pvOp == Acc_SetMaleScrewConfigure)
	{
		void *pvCfgPointer = NULL;

		pvCfgPointer = va_arg(xVaList, void *);
		bStatus = ((bool (*)(unsigned char, unsigned char, MaleScrew *))pvOp)(ucPort, ucSensorID, pvCfgPointer);
	}
	else if (pvOp == Acc_SetFemaleScrewConfigure)
	{
		void *pvCfgPointer = NULL;

		pvCfgPointer = va_arg(xVaList, void *);
		bStatus = ((bool (*)(unsigned char, unsigned char, FemaleScrew *))pvOp)(ucPort, ucSensorID, pvCfgPointer);
	}
	else if (pvOp == Acc_SetShaftConfigure)
	{
		void *pvCfgPointer = NULL;

		pvCfgPointer = va_arg(xVaList, void *);
		bStatus = ((bool (*)(unsigned char, unsigned char, Shaft *))pvOp)(ucPort, ucSensorID, pvCfgPointer);
	}
	else if (pvOp == Acc_SetSleeveConfigure)
	{
		void *pvCfgPointer = NULL;

		pvCfgPointer = va_arg(xVaList, void *);
		bStatus = ((bool (*)(unsigned char, unsigned char, Sleeve *))pvOp)(ucPort, ucSensorID, pvCfgPointer);
	}
	else if (pvOp == Acc_GetSerialNumber)
	{
		unsigned long *pulSN = NULL;

		pulSN = va_arg(xVaList, unsigned long *);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned long *))pvOp)(ucPort, ucSensorID, pulSN);
	}
	else if (pvOp == Acc_SetFanAlias || pvOp == Acc_SetBeltAlias)
	{
		unsigned short usAlias;

		usAlias = va_arg(xVaList, int);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned short))pvOp)(ucPort, ucSensorID, usAlias);
	}
	else if (pvOp == Acc_GetFanAlias || pvOp == Acc_GetBeltAlias)
	{
		unsigned short *pusAlias;

		pusAlias = va_arg(xVaList, unsigned short *);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned short *))pvOp)(ucPort, ucSensorID, pusAlias);
	}
	else if (pvOp == Acc_SetScrewAlias)
	{
		unsigned short usAlias;

		usAlias = va_arg(xVaList, int);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned short))pvOp)(ucPort, ucSensorID, usAlias);
	}
	else if (pvOp == Acc_GetScrewAlias)
	{
		unsigned short *pusAlias;

		pusAlias = va_arg(xVaList, unsigned short *);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned short *))pvOp)(ucPort, ucSensorID, pusAlias);
	}
	else if (pvOp == Acc_SetMaleScrewAlias)
	{
		unsigned short usAlias;

		usAlias = va_arg(xVaList, int);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned short))pvOp)(ucPort, ucSensorID, usAlias);
	}
	else if (pvOp == Acc_GetMaleScrewAlias)
	{
		unsigned short *pusAlias;

		pusAlias = va_arg(xVaList, unsigned short *);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned short *))pvOp)(ucPort, ucSensorID, pusAlias);
	}
	else if (pvOp == Acc_SetFemaleScrewAlias)
	{
		unsigned short usAlias;

		usAlias = va_arg(xVaList, int);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned short))pvOp)(ucPort, ucSensorID, usAlias);
	}
	else if (pvOp == Acc_GetFemaleScrewAlias)
	{
		unsigned short *pusAlias;

		pusAlias = va_arg(xVaList, unsigned short *);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned short *))pvOp)(ucPort, ucSensorID, pusAlias);
	}
	else if (pvOp == Acc_SetShaftAlias)
	{
		unsigned short usAlias;

		usAlias = va_arg(xVaList, int);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned short))pvOp)(ucPort, ucSensorID, usAlias);
	}
	else if (pvOp == Acc_GetShaftAlias)
	{
		unsigned short *pusAlias;

		pusAlias = va_arg(xVaList, unsigned short *);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned short *))pvOp)(ucPort, ucSensorID, pusAlias);
	}
	else if (pvOp == Acc_SetSleeveAlias)
	{
		unsigned short usAlias;

		usAlias = va_arg(xVaList, int);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned short))pvOp)(ucPort, ucSensorID, usAlias);
	}
	else if (pvOp == Acc_GetSleeveAlias)
	{
		unsigned short *pusAlias;

		pusAlias = va_arg(xVaList, unsigned short *);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned short *))pvOp)(ucPort, ucSensorID, pusAlias);
	}
	else if (pvOp == Acc_GetPredictedSpeed)
	{
		unsigned long *pulSP = NULL;

		pulSP = va_arg(xVaList, unsigned long *);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned long *))pvOp)(ucPort, ucSensorID, pulSP);
	}
	// fActual speed set and get
	else if (pvOp == Acc_SetMotorARRate)
	{
		unsigned long fActual = 0;

		fActual = va_arg(xVaList, unsigned long);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned long))pvOp)(ucPort, ucSensorID, fActual);
	}
	else if (pvOp == Acc_GetMotorARRate)
	{
		unsigned long *fActual = NULL;

		fActual = va_arg(xVaList, unsigned long *);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned long *))pvOp)(ucPort, ucSensorID, fActual);
	}
	// Motor actual speed select
	else if (pvOp == Acc_SetMotorActualSel)
	{
		unsigned short sel = 0;

		sel = va_arg(xVaList, int);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned short))pvOp)(ucPort, ucSensorID, sel);
	}
	else if (pvOp == Acc_GetMotorActualSel)
	{
		unsigned short *sel = NULL;

		sel = va_arg(xVaList, unsigned short *);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned short *))pvOp)(ucPort, ucSensorID, sel);
	}
	else if (pvOp == Acc_SetOdr)
	{
		unsigned short odr = 0;

		odr = va_arg(xVaList, int);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned short))pvOp)(ucPort, ucSensorID, odr);
	}
	else if (pvOp == Acc_SetSampSum)
	{
		unsigned short sampSum = 0;

		sampSum = va_arg(xVaList, int);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned short))pvOp)(ucPort, ucSensorID, sampSum);
	}
	else if (pvOp == Acc_SetHpfEn)
	{
		unsigned short hpfEn = 0;

		hpfEn = va_arg(xVaList, int);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned short))pvOp)(ucPort, ucSensorID, hpfEn);
	}
	// Set anomal vibration threshold
	else if (pvOp == Acc_SetAmlVibTh)
	{
		unsigned short AmlVibTh = 0;

		AmlVibTh = va_arg(xVaList, int);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned short))pvOp)(ucPort, ucSensorID, AmlVibTh);
	}
	// Set anomal vibration interval
	else if (pvOp == Acc_SetAmlVibInterval)
	{
		unsigned short AmlVibInterval = 0;

		AmlVibInterval = va_arg(xVaList, int);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned short))pvOp)(ucPort, ucSensorID, AmlVibInterval);
	}
	// set calculate feature
	else if (pvOp == Acc_SetCalFea)
	{
		unsigned short uscalFea = 0;

		uscalFea = va_arg(xVaList, int);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned short))pvOp)(ucPort, ucSensorID, uscalFea);
	}
	// sensor PSRAM high temperature protect feature
	else if (pvOp == Acc_GetHighTempProtect)
	{
		unsigned short *highTempProtect = NULL;

		highTempProtect = va_arg(xVaList, unsigned short *);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned short *))pvOp)(ucPort, ucSensorID, highTempProtect);
	}
	// sensor PSRAM high temperature protect feature
	else if (pvOp == Acc_SetHighTempProtect)
	{
		unsigned short highTempProtect = 0;

		highTempProtect = va_arg(xVaList, int);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned short))pvOp)(ucPort, ucSensorID, highTempProtect);
	}
	else if (pvOp == Acc_GetAmlVibAlarm)
	{
		unsigned short *amlvib = NULL;

		amlvib = va_arg(xVaList, unsigned short *);
		bStatus = ((bool (*)(unsigned char, unsigned char, unsigned short *))pvOp)(ucPort, ucSensorID, amlvib);
	}
	else
	{
		bStatus = ((bool (*)(unsigned char, unsigned char))pvOp)(ucPort, ucSensorID);
	}

	va_end(xVaList);
	// 信号量 释放
	xSemaphoreGive(g_xMBSignal[(ucSensorAddr - 1) / WIT_PER_PORT_SENSOR_COUNT]);

	return bStatus;
}
#endif

static void WIT_IncreasePackageIDAndBackUp(unsigned char groupId)
{
	g_xWITConfig.xGroupTable[groupId].packid++;
	// ID 过5W清0
	if (g_xWITConfig.xGroupTable[groupId].packid > WIT_MAX_ID_NUMBER)
		g_xWITConfig.xGroupTable[groupId].packid = 0;

	// 改为掉电存储 增加FLASH寿命，先存储到后备寄存器
	g_xPVDData[groupId].ulPackageID = g_xWITConfig.xGroupTable[groupId].packid;
	g_xPVDData[groupId].ulWorkMinutes = g_xWITConfig.xGroupTable[groupId].ulWorkMinutes;
	g_xPVDData[groupId].bSendingData = false;
}

#if CONFIG_RS485_RTU > 0
static void WIT_CheckParameterIsUsed()
{
	int i, j, k;
	Parameter *pxParm;
	Parameter *pxActSpd;
	// Frequcency
	pxActSpd = &g_xWITData.xSensFea.xActualSpd;
	if (g_usIsSetSpeed == true && pxActSpd->fValue > 0)
		pxActSpd->bIsUsed = true;
	else
		pxActSpd->bIsUsed = false;
	for (i = 0; i < ACC_AXIS_COUNT; i++)
	{
		Feature *pxFea = NULL;
		ExtendFeature *pxExtFea = NULL;

		pxFea = &g_xWITData.xSensFea.x + i;
		pxExtFea = &g_xWITData.xSensFea.xExt + i;
		pxParm = (Parameter *)&pxFea->xFreq1x;
		// Frequcency
		for (j = 0; j < ACC_FREQUENCY_FEATURE_COUNT; j++)
		{
			if (g_usIsSetSpeed == true)
				pxParm->bIsUsed = true;
			else
				pxParm->bIsUsed = false;
			pxParm++;
		}

		// Bearing feature
		for (j = 0; j < ACC_MAX_BEARING_COUNT; j++)
		{
			pxParm = (Parameter *)&pxFea->xBearingFea[j];
			for (k = 0; k < sizeof(Bearing_Parameter) / sizeof(Parameter); k++)
			{
				if (g_usBearingStatus & (1 << j))
					pxParm->bIsUsed = true;
				else
					pxParm->bIsUsed = false;
				pxParm++;
			}
		}

		// Gear feature
		for (j = 0; j < ACC_MAX_GEAR_COUNT; j++)
		{
			pxParm = (Parameter *)&pxFea->xGearFea[j];
			for (k = 0; k < sizeof(Gear_Parameter) / sizeof(Parameter); k++)
			{
				if (g_usGearStatus & (1 << j))
					pxParm->bIsUsed = true;
				else
					pxParm->bIsUsed = false;
				pxParm++;
			}
		}

		// Extend base feature
		pxParm = (Parameter *)&pxExtFea->xBaseExtFea.xInputEnvEnergy;
		if (g_usExtInputStatus == true)
		{
			pxParm->bIsUsed = true;
		}
		else
		{
			pxParm->bIsUsed = false;
		}

		pxParm = (Parameter *)&pxExtFea->xBaseExtFea.xOutputEnvEnergy;
		if (g_usExtOutputStatus == true)
		{
			pxParm->bIsUsed = true;
		}
		else
		{
			pxParm->bIsUsed = false;
		}

		// Extend bearing feature
		for (j = 0; j < ACC_MAX_BEARING_COUNT; j++)
		{
			pxParm = (Parameter *)&pxExtFea->xBearingExtFea[j];
			for (k = 0; k < sizeof(Bearing_ExtendParameter) / sizeof(Parameter); k++)
			{
				if (g_usExtBearingStatus & (1 << j))
					pxParm->bIsUsed = true;
				else
					pxParm->bIsUsed = false;
				pxParm++;
			}
		}

		// Extend gear feature
		for (j = 0; j < ACC_MAX_GEAR_COUNT; j++)
		{
			pxParm = (Parameter *)&pxExtFea->xGearExtFea[j];
			for (k = 0; k < sizeof(Gear_ExtendParameter) / sizeof(Parameter); k++)
			{
				if (g_usExtGearStatus & (1 << j))
					pxParm->bIsUsed = true;
				else
					pxParm->bIsUsed = false;
				pxParm++;
			}
		}

		// fan
		pxParm = (Parameter *)&pxFea->xFan;
		for (k = 0; k < sizeof(Fan_Parameter) / sizeof(Parameter); k++)
		{
			if (g_usFanStatus == true)
				pxParm->bIsUsed = true;
			else
				pxParm->bIsUsed = false;
			pxParm++;
		}
		// belt
		pxParm = (Parameter *)&pxFea->xBelt;
		for (k = 0; k < sizeof(Belt_Parameter) / sizeof(Parameter); k++)
		{
			if (g_usBeltStatus == true)
				pxParm->bIsUsed = true;
			else
				pxParm->bIsUsed = false;
			pxParm++;
		}
		// Screw
		pxParm = (Parameter *)&pxFea->xScrew;
		for (k = 0; k < sizeof(Screw_Parameter) / sizeof(Parameter); k++)
		{
			if (g_usScrewStatus == true)
				pxParm->bIsUsed = true;
			else
				pxParm->bIsUsed = false;
			pxParm++;
		}
		// MaleScrew
		pxParm = (Parameter *)&pxFea->xMaleScrew;
		for (k = 0; k < sizeof(MaleScrew_Parameter) / sizeof(Parameter); k++)
		{
			if (g_usMaleScrewStatus == true)
				pxParm->bIsUsed = true;
			else
				pxParm->bIsUsed = false;
			pxParm++;
		}
		// FemaleScrew
		pxParm = (Parameter *)&pxFea->xFemaleScrew;
		for (k = 0; k < sizeof(FemaleScrew_Parameter) / sizeof(Parameter); k++)
		{
			if (g_usFemaleScrewStatus == true)
				pxParm->bIsUsed = true;
			else
				pxParm->bIsUsed = false;
			pxParm++;
		}
		// Shaft
		pxParm = (Parameter *)&pxFea->xShaft;
		for (k = 0; k < sizeof(Shaft_Parameter) / sizeof(Parameter); k++)
		{
			if (g_usShaftStatus == true)
				pxParm->bIsUsed = true;
			else
				pxParm->bIsUsed = false;
			pxParm++;
		}
		// Sleeve
		pxParm = (Parameter *)&pxFea->xSleeve;
		for (k = 0; k < sizeof(Sleeve_Parameter) / sizeof(Parameter); k++)
		{
			if (g_usSleeveStatus == true)
				pxParm->bIsUsed = true;
			else
				pxParm->bIsUsed = false;
			pxParm++;
		}
		// Power Frequency
		pxParm = (Parameter *)&pxFea->xPowerFrequency;
		for (k = 0; k < sizeof(PowerFrequency_Parameter) / sizeof(Parameter); k++)
		{
			if (g_usSleeveStatus == true)
				pxParm->bIsUsed = true;
			else
				pxParm->bIsUsed = false;
			pxParm++;
		}
	}
}
#endif

static void WIT_MQTTPublishHandler(unsigned char groupId, bool bStatus)
{
	if (bStatus == true)
	{
		if (g_xPVDData[groupId].bSendingData == false)
		{
			g_xPVDData[groupId].bSendingData = true;
			write_flash(E_PVD_STORE, g_xPVDData, sizeof(g_xPVDData));
			// ESP_LOGD("test1","bSendingData: %d, groupId %d", g_xPVDData[groupId].bSendingData, groupId);
		}
	}
}

#if CONFIG_RS485_RTU > 0
static void WIT_IsSensorExist(void)
{
	// 上电自动检测哪几路接了传感器设备
	int i = 0, j = 0;

	// Set 100ms for querying
	vMBSetTimeout(WIT_MB_NORMAL_TIMEOUT);

	for (j = 0; j < WIT_MB_CHECK_TIMES; j++)
	{
		for (i = 1; i <= ACC_MAX_SENSOR_COUNT; i++)
		{
			if (WIT_SensorOperation(i, Acc_GetSensorStatus, WIT_ACC_SENSOR_ID, NULL) == true)
			{
				g_bIsSensorExist[i - 1] = true;
			}
			else
			{
				g_bIsSensorExist[i - 1] = false;
			}

			vTaskDelay(2);
		}
	}
}
#endif

static bool WIT_SendErrorDataIfOccured()
{
	bool bStatus = true;
	Parameter *pxParm;
	unsigned long ulParmCnt;
	int i, j;
	cJSON *pxRoot = NULL;
	cJSON *pxObj = NULL;
	bool bHasError = false;
	unsigned char *pucSendBuffer = NULL;
	unsigned long ulBufferSize = 0;
	char cSensorIDStr[4];
	cJSON_bool bPackageStatus;
	bool bPublishStatus = false;
	unsigned long ulErrorTs;

	ulErrorTs = RTC_InternalGetTimeStamp();
	// Get mqtt buffer info
	pucSendBuffer = MQTT_SendBufferPointer(g_xWITMQTT.lHandle);
	ulBufferSize = MQTT_GetRestSendBufferSize(g_xWITMQTT.lHandle);

	// Gateway error
	pxParm = (Parameter *)&g_xGWErrParm;
	ulParmCnt = sizeof(WIT_GWErrorParameter) / sizeof(Parameter);
	for (i = 0; i < ulParmCnt; i++)
	{
		if (pxParm->bIsErrorOccured == true || pxParm->bIsErrorCleared == true)
		{
			if (bHasError == false)
			{
				bHasError = true;
				pxRoot = cJSON_CreateArray();
				pxObj = cJSON_CreateObject();
				cJSON_AddStringToObject(pxObj, "addr", "0");
				cJSON_AddNumberToObject(pxObj, "ts", ulErrorTs);
			}

			if (pxParm->bIsErrorOccured == true)
				cJSON_AddNumberToObject(pxObj, pxParm->cName, 1);
			else
				cJSON_AddNumberToObject(pxObj, pxParm->cName, 0);
		}

		pxParm++;
	}

	if (bHasError == true)
	{
		cJSON_AddItemToArray(pxRoot, pxObj);
		bPackageStatus = cJSON_PrintPreallocated(pxRoot, (char *)pucSendBuffer, ulBufferSize, false);
		if (pxRoot != NULL)
		{
			cJSON_Delete(pxRoot);
			pxRoot = NULL;
		}

		if (bPackageStatus == false || (bPublishStatus = MQTT_Publish(g_xWITMQTT.lHandle, g_xWITMQTT.cPubErrorTopic, pucSendBuffer, strlen((char *)pucSendBuffer), false, false, QOS1)) != true)
		{
			bStatus = false;
		}

		if (bPublishStatus == true)
		{
			pxParm = (Parameter *)&g_xGWErrParm;
			for (i = 0; i < ulParmCnt; i++)
			{
				pxParm->bIsErrorCleared = false;
				pxParm++;
			}
		}
	}

	// Sensor error
	ulParmCnt = sizeof(WIT_SensorErrorParameter) / sizeof(Parameter);
	for (i = 0; i < ACC_MAX_SENSOR_COUNT; i++)
	{
		if (g_bIsSensorExist[i] == true)
		{
			bHasError = false;
			pxParm = (Parameter *)&g_xSrErrParm[i];
			sprintf(cSensorIDStr, "%d", i + 1);
			for (j = 0; j < ulParmCnt; j++)
			{
				if (pxParm->bIsErrorOccured == true || pxParm->bIsErrorCleared == true)
				{
					if (bHasError == false)
					{
						bHasError = true;
						pxRoot = cJSON_CreateArray();
						pxObj = cJSON_CreateObject();
						cJSON_AddStringToObject(pxObj, "addr", cSensorIDStr);
						cJSON_AddNumberToObject(pxObj, "ts", ulErrorTs);
					}

					if (pxParm->bIsErrorOccured == true)
						cJSON_AddNumberToObject(pxObj, pxParm->cName, 1);
					else
						cJSON_AddNumberToObject(pxObj, pxParm->cName, 0);
				}

				pxParm++;
			}

			if (bHasError == true)
			{
				cJSON_AddItemToArray(pxRoot, pxObj);
				bPackageStatus = cJSON_PrintPreallocated(pxRoot, (char *)pucSendBuffer, ulBufferSize, false);
				if (pxRoot != NULL)
				{
					cJSON_Delete(pxRoot);
					pxRoot = NULL;
				}

				if (bPackageStatus == false || (bPublishStatus = MQTT_Publish(g_xWITMQTT.lHandle, g_xWITMQTT.cPubErrorTopic, pucSendBuffer, strlen((char *)pucSendBuffer), false, false, QOS1)) != true)
				{
					bStatus = false;
				}

				if (bPublishStatus == true)
				{
					if (bPublishStatus == true)
					{
						pxParm = (Parameter *)&g_xSrErrParm[i];
						for (j = 0; j < ulParmCnt; j++)
						{
							pxParm->bIsErrorCleared = false;
							pxParm++;
						}
					}
				}
			}
		}
	}

	return bStatus;
}

#if CONFIG_ESP32_WIFI > 0
static void WIT_UDPStoreGWData()
{
	g_xUDPGWData.ulWorkTime = g_xWITConfig.xWorkDet.ulWorkMinutes;
}

static void WIT_UDPStoreSensorData(unsigned char ucSensorAddr, unsigned short usVersion)
{
	unsigned char ucIndex = ucSensorAddr - 1;

	g_xUDPSensorData[ucIndex].usVersion = usVersion;
	g_xUDPSensorData[ucIndex].usSensorTemp = (short)(g_xWITData.xSensFea.temp.fValue * 100);
	g_xUDPSensorData[ucIndex].ulXVRMS = (unsigned long)(g_xWITData.xSensFea.x.xVRMS.fValue * 100);
	g_xUDPSensorData[ucIndex].ulYVRMS = (unsigned long)(g_xWITData.xSensFea.y.xVRMS.fValue * 100);
	g_xUDPSensorData[ucIndex].ulZVRMS = (unsigned long)(g_xWITData.xSensFea.z.xVRMS.fValue * 100);
	g_xUDPSensorData[ucIndex].ulXKur = (unsigned long)(g_xWITData.xSensFea.x.xKurtosis.fValue * 100);
	g_xUDPSensorData[ucIndex].ulYKur = (unsigned long)(g_xWITData.xSensFea.y.xKurtosis.fValue * 100);
	g_xUDPSensorData[ucIndex].ulZKur = (unsigned long)(g_xWITData.xSensFea.z.xKurtosis.fValue * 100);
}
#endif

// unsigned char s_ucSensorStatus = 0;

static void WIT_AccEnable(unsigned char ucSensorAddr, unsigned char ucSensorID)
{
	static unsigned char s_ucSensorStatus = 0;

	if (ucSensorID == 0)
	{
		s_ucSensorStatus = 0x3F;
	}
	else
	{
		s_ucSensorStatus = ucSensorAddr - 1;
	}

	Acc_Enable(s_ucSensorStatus);

	// 上升沿稳定50ms-使能 下降沿后稳定50ms-失能
	vTaskDelay(150);
}

static unsigned char WIT_GetConfigureNumberCount(void)
{
	int i;
	unsigned char ucCnt = 0;

	for (i = 0; i < sizeof(g_pulConfigNumberPointerList) / 4; i++)
	{
		if (*g_pulConfigNumberPointerList[i] == WIT_CONFIGURE_VALUE)
			ucCnt++;
	}

	return ucCnt;
}

static void WIT_PreTaskInit(void)
{
	int i;
	// modbus init
	// eMBInit(MB_Master, 0x1, WIT_MB_MASTER_PORT, WIT_MB_BAUDRATE, WIT_MB_PARITY);
	// eMBEnable(WIT_MB_MASTER_PORT);
	// WIT_MB_SLAVE_PORT
	eMBInit(MB_RTU, g_xWITConfig.ucModbusID, WIT_MB_SLAVE_PORT, g_xWITConfig.ulModbusBaudRate, g_xWITConfig.eModbusParity);
	eMBEnable(WIT_MB_SLAVE_PORT);

	// Create timer
	g_xWorkTimer = xTimerCreate("Work Timer", 60000, pdTRUE, NULL, WIT_TimerHandler);
	if (g_xWorkTimer != NULL)
		xTimerStart(g_xWorkTimer, 1000);
	g_xPushTimer = xTimerCreate("Push Timer", 1000, pdTRUE, NULL, WIT_TimerHandler);
	if (g_xPushTimer != NULL)
		xTimerStart(g_xPushTimer, 1000);
	g_xDLICalTimer = xTimerCreate("DLI Timer", WIT_DATA_LIST_RESTORE_TIME, pdTRUE, NULL, WIT_TimerHandler);
	if (g_xDLICalTimer != NULL)
		xTimerStart(g_xDLICalTimer, 1000);
    g_xDataSendTimer = xTimerCreate("IOT DataSend Timer", 10000, pdTRUE, NULL, WIT_TimerHandler);
    if (g_xDataSendTimer != NULL)
		xTimerStart(g_xDataSendTimer, 5000);
    g_xErrorSendTimer = xTimerCreate("IOT ErrorSend Timer", 10000, pdTRUE, NULL, WIT_TimerHandler);
    if (g_xErrorSendTimer!= NULL)
		xTimerStart(g_xErrorSendTimer, 5000);


	g_xCfgUpTimer = xTimerCreate("Up1 Timer", WIT_DATA_UPDATE_IDLE_TIME, pdFALSE, NULL, WIT_TimerHandler);
	g_xThrUpTimer = xTimerCreate("Up2 Timer", WIT_DATA_UPDATE_IDLE_TIME, pdFALSE, NULL, WIT_TimerHandler);
	g_xDLIUpTimer = xTimerCreate("Up3 Timer", WIT_DATA_UPDATE_IDLE_TIME, pdFALSE, NULL, WIT_TimerHandler);
	g_xCfgQuickUpTimer = xTimerCreate("Up4 Timer", WIT_DATA_UPDATE_QUICT_TIME, pdFALSE, NULL, WIT_TimerHandler);
	g_xSenConfigTimer = xTimerCreate("Up5 Timer", WIT_DATA_UPDATE_IDLE_TIME, pdFALSE, NULL, WIT_TimerHandler);

	// data send timer
	g_xGatewayFeatureTimer = xTimerCreate("GF Timer", (5 * 60 * 1000), pdFALSE, NULL, WIT_TimerHandler);
	g_xCheckInTimer = xTimerCreate("CI Timer", WIT_DATA_UPDATE_CHECKIN_TIME, pdFALSE, NULL, WIT_TimerHandler);
	if (g_xGatewayFeatureTimer != NULL)
		xTimerStart(g_xGatewayFeatureTimer, 1000);
	g_xSensorFeatureTimer = xTimerCreate("SF Timer", g_xWITConfig.ulPeriod, pdFALSE, NULL, WIT_TimerHandler);
	g_xWaveTimer = xTimerCreate("W Timer", g_xWITConfig.ulWavePeriod, pdFALSE, NULL, WIT_TimerHandler);
	g_xAlarmTimer = xTimerCreate("A Timer", g_xWITConfig.ulPeriod, pdFALSE, NULL, WIT_TimerHandler);

	// Create signal
	for (i = 0; i < WIT_MODBUS_PORT_COUNT; i++)
	{
		g_xMBSignal[i] = xSemaphoreCreateBinary();
		xSemaphoreGive(g_xMBSignal[i]);
	}

	// 修改WIT Data中的轴承 齿轮名称时使用
	g_xWITDataSignal = xSemaphoreCreateBinary();
	xSemaphoreGive(g_xWITDataSignal);

	// Create task
	g_xKeyTimer = xTimerCreate("Key Timer", WIT_KEY_RESET_TIME, pdFALSE, NULL, WIT_TimerHandler);
	xTaskCreate(WIT_KeyTask, "Key Task", WIT_KEY_TASK_STK, NULL, WIT_KEY_TASK_PRIORITY, NULL);
	xTaskCreate(WIT_ModbusSlavePollTask, "MB Task", WIT_MODBUS_SLAVE_POLL_STK, NULL, WIT_MODBUS_SLAVE_POLL_PRIORITY, NULL);
	xTaskCreate(WIT_EventPollTask, "EV Task", WIT_UPDATE_POLL_TASK_STK, NULL, WIT_UPDATE_POLL_TASK_PRIORITY, NULL);
}

static bool WIT_IsNetBusy()
{
	if (MQTT_GetStatus(g_xWITMQTT.lHandle) == false || Update_GetStatus() == true || Sensor_GetUpdateStatus() == true)
		return true;

	return false;
}

static bool WIT_IsImportantParameter(char *pcName)
{
	int i;

	for (i = 0; i < sizeof(g_pcImportantParameterName) / 4; i++)
	{
		if (strncmp(pcName + 1, g_pcImportantParameterName[i], strlen(g_pcImportantParameterName[i])) == 0)
		{
			return true;
		}
	}

	return false;
}

static unsigned short WIT_GetFeatureMinLearnCount(void)
{
	unsigned short usMinFeaLearnCnt = 0;
	DataListInfo *pxDLIPointer = g_pxGlobalDLI;
	int i;
	bool bFirstData = false;

	for (i = 0; i < g_ulTotalParameterSize; i++)
	{
		if (pxDLIPointer->bUsed == true)
		{
			if (bFirstData == false)
			{
				if (pxDLIPointer->usCount != 0)
				{
					usMinFeaLearnCnt = pxDLIPointer->usCount;
					bFirstData = true;
				}
			}
			else
			{
				if (pxDLIPointer->usCount < usMinFeaLearnCnt)
				{
					usMinFeaLearnCnt = pxDLIPointer->usCount;
				}
			}
		}
		pxDLIPointer++;
	}

	return usMinFeaLearnCnt;
}

unsigned short WIT_GetGWAndSensorFeatureCount()
{
	return (g_ulGatewayParameterSize + g_ulSensorParameterSize);
}

#if CONFIG_RS485_RTU > 0
static void WIT_UpdateFeatureName(unsigned char ucSensorAddr)
{
	int i, j;
	char cAxis[4];
	Sensor_Feature *pxSenFea = &g_xWITData.xSensFea;

	Feature *pxFea = NULL;
	ExtendFeature *pxExtFea = NULL;

	// Bearing change name
	for (i = 0; i < ACC_MAX_BEARING_COUNT; i++)
	{
		unsigned short usAlias;

		// Get number alias
		if (WIT_SensorOperation(ucSensorAddr, Acc_GetBearingAlias, WIT_ACC_SENSOR_ID, i, &usAlias) == true)
		{
			for (j = 0; j < 3; j++)
			{
				if (j == 0)
				{
					sprintf(cAxis, "x");
					pxFea = &pxSenFea->x;
					pxExtFea = &pxSenFea->xExt;
				}
				else if (j == 1)
				{
					sprintf(cAxis, "y");
					pxFea = &pxSenFea->y;
					pxExtFea = &pxSenFea->yExt;
				}
				else if (j == 2)
				{
					sprintf(cAxis, "z");
					pxFea = &pxSenFea->z;
					pxExtFea = &pxSenFea->zExt;
				}

				sprintf(pxFea->xBearingFea[i].xFreqBO.cName, "%sFreqBO%d", cAxis, usAlias);
				sprintf(pxFea->xBearingFea[i].xFreqBI.cName, "%sFreqBI%d", cAxis, usAlias);
				sprintf(pxFea->xBearingFea[i].xFreqBB.cName, "%sFreqBB%d", cAxis, usAlias);
				sprintf(pxFea->xBearingFea[i].xFreqBF.cName, "%sFreqBF%d", cAxis, usAlias);
				// 轴承扩展特征
				sprintf(pxExtFea->xBearingExtFea[i].xEnvEnergy.cName, "%sseB%d", cAxis, usAlias);
			}
		}
	}

	// Gear change name
	for (i = 0; i < ACC_MAX_GEAR_COUNT; i++)
	{
		unsigned short usAlias;

		// Get number alias
		if (WIT_SensorOperation(ucSensorAddr, Acc_GetGearAlias, WIT_ACC_SENSOR_ID, i, &usAlias) == true)
		{
			for (j = 0; j < 3; j++)
			{
				if (j == 0)
				{
					sprintf(cAxis, "x");
					pxFea = &pxSenFea->x;
					pxExtFea = &pxSenFea->xExt;
				}
				else if (j == 1)
				{
					sprintf(cAxis, "y");
					pxFea = &pxSenFea->y;
					pxExtFea = &pxSenFea->yExt;
				}
				else if (j == 2)
				{
					sprintf(cAxis, "z");
					pxFea = &pxSenFea->z;
					pxExtFea = &pxSenFea->zExt;
				}

				sprintf(pxFea->xGearFea[i].xFreq1xGMF.cName, "%sFreq1xGMF%d", cAxis, usAlias);
				sprintf(pxFea->xGearFea[i].xFreq2xGMF.cName, "%sFreq2xGMF%d", cAxis, usAlias);
				sprintf(pxFea->xGearFea[i].xFreq3xGMF.cName, "%sFreq3xGMF%d", cAxis, usAlias);
				// 齿轮扩展特征
				sprintf(pxExtFea->xGearExtFea[i].x1xSideEnergy.cName, "%sSide1xEnergy%d", cAxis, usAlias);
				sprintf(pxExtFea->xGearExtFea[i].x2xSideEnergy.cName, "%sSide2xEnergy%d", cAxis, usAlias);
				sprintf(pxExtFea->xGearExtFea[i].x3xSideEnergy.cName, "%sSide3xEnergy%d", cAxis, usAlias);
				sprintf(pxExtFea->xGearExtFea[i].xEnvEnergy.cName, "%sseG%d", cAxis, usAlias);
			}
		}
	}

	// Get fan alias
	{
		unsigned short usAlias;

		if (WIT_SensorOperation(ucSensorAddr, Acc_GetFanAlias, WIT_ACC_SENSOR_ID, &usAlias) == true)
		{
			for (j = 0; j < 3; j++)
			{
				if (j == 0)
				{
					sprintf(cAxis, "x");
					pxFea = &pxSenFea->x;
					pxExtFea = &pxSenFea->xExt;
				}
				else if (j == 1)
				{
					sprintf(cAxis, "y");
					pxFea = &pxSenFea->y;
					pxExtFea = &pxSenFea->yExt;
				}
				else if (j == 2)
				{
					sprintf(cAxis, "z");
					pxFea = &pxSenFea->z;
					pxExtFea = &pxSenFea->zExt;
				}

				sprintf(pxFea->xFan.xSPM1x.cName, "%sspm1xF%d", cAxis, usAlias);
				sprintf(pxFea->xFan.xSPM2x.cName, "%sspm2xF%d", cAxis, usAlias);
				sprintf(pxFea->xFan.xSPM3x.cName, "%sspm3xF%d", cAxis, usAlias);
			}
		}
	}

	// Get belt alias
	{
		unsigned short usAlias;

		if (WIT_SensorOperation(ucSensorAddr, Acc_GetBeltAlias, WIT_ACC_SENSOR_ID, &usAlias) == true)
		{
			for (j = 0; j < 3; j++)
			{
				if (j == 0)
				{
					sprintf(cAxis, "x");
					pxFea = &pxSenFea->x;
					pxExtFea = &pxSenFea->xExt;
				}
				else if (j == 1)
				{
					sprintf(cAxis, "y");
					pxFea = &pxSenFea->y;
					pxExtFea = &pxSenFea->yExt;
				}
				else if (j == 2)
				{
					sprintf(cAxis, "z");
					pxFea = &pxSenFea->z;
					pxExtFea = &pxSenFea->zExt;
				}

				sprintf(pxFea->xBelt.xBelt1x.cName, "%sbelt1xF%d", cAxis, usAlias);
			}
		}
	}
	// Get Screw alias
	{
		unsigned short usAlias;

		if (WIT_SensorOperation(ucSensorAddr, Acc_GetScrewAlias, WIT_ACC_SENSOR_ID, &usAlias) == true)
		{
			for (j = 0; j < 3; j++)
			{
				if (j == 0)
				{
					sprintf(cAxis, "x");
					pxFea = &pxSenFea->x;
					pxExtFea = &pxSenFea->xExt;
				}
				else if (j == 1)
				{
					sprintf(cAxis, "y");
					pxFea = &pxSenFea->y;
					pxExtFea = &pxSenFea->yExt;
				}
				else if (j == 2)
				{
					sprintf(cAxis, "z");
					pxFea = &pxSenFea->z;
					pxExtFea = &pxSenFea->zExt;
				}

				sprintf(pxFea->xScrew.xScrew1x.cName, "%sscrew1xF%d", cAxis, usAlias);
				sprintf(pxFea->xScrew.xScrew2x.cName, "%sscrew2xF%d", cAxis, usAlias);
				sprintf(pxFea->xScrew.xScrew3x.cName, "%sscrew3xF%d", cAxis, usAlias);
			}
		}
	}
	// Get MaleScrew alias
	{
		unsigned short usAlias;

		if (WIT_SensorOperation(ucSensorAddr, Acc_GetMaleScrewAlias, WIT_ACC_SENSOR_ID, &usAlias) == true)
		{
			for (j = 0; j < 3; j++)
			{
				if (j == 0)
				{
					sprintf(cAxis, "x");
					pxFea = &pxSenFea->x;
					pxExtFea = &pxSenFea->xExt;
				}
				else if (j == 1)
				{
					sprintf(cAxis, "y");
					pxFea = &pxSenFea->y;
					pxExtFea = &pxSenFea->yExt;
				}
				else if (j == 2)
				{
					sprintf(cAxis, "z");
					pxFea = &pxSenFea->z;
					pxExtFea = &pxSenFea->zExt;
				}

				sprintf(pxFea->xMaleScrew.xMaleScrew1x.cName, "%smalescrew1xF%d", cAxis, usAlias);
				sprintf(pxFea->xMaleScrew.xMaleScrew2x.cName, "%smalescrew2xF%d", cAxis, usAlias);
				sprintf(pxFea->xMaleScrew.xMaleScrew3x.cName, "%smalescrew3xF%d", cAxis, usAlias);
			}
		}
	}
	// Get FemaleScrew alias
	{
		unsigned short usAlias;

		if (WIT_SensorOperation(ucSensorAddr, Acc_GetFemaleScrewAlias, WIT_ACC_SENSOR_ID, &usAlias) == true)
		{
			for (j = 0; j < 3; j++)
			{
				if (j == 0)
				{
					sprintf(cAxis, "x");
					pxFea = &pxSenFea->x;
					pxExtFea = &pxSenFea->xExt;
				}
				else if (j == 1)
				{
					sprintf(cAxis, "y");
					pxFea = &pxSenFea->y;
					pxExtFea = &pxSenFea->yExt;
				}
				else if (j == 2)
				{
					sprintf(cAxis, "z");
					pxFea = &pxSenFea->z;
					pxExtFea = &pxSenFea->zExt;
				}

				sprintf(pxFea->xFemaleScrew.xFemaleScrew1x.cName, "%sfemalescrew1xF%d", cAxis, usAlias);
				sprintf(pxFea->xFemaleScrew.xFemaleScrew2x.cName, "%sfemalescrew2xF%d", cAxis, usAlias);
				sprintf(pxFea->xFemaleScrew.xFemaleScrew3x.cName, "%sfemalescrew3xF%d", cAxis, usAlias);
			}
		}
	}
	// Get Shaft alias
	{
		unsigned short usAlias;

		if (WIT_SensorOperation(ucSensorAddr, Acc_GetShaftAlias, WIT_ACC_SENSOR_ID, &usAlias) == true)
		{
			for (j = 0; j < 3; j++)
			{
				if (j == 0)
				{
					sprintf(cAxis, "x");
					pxFea = &pxSenFea->x;
					pxExtFea = &pxSenFea->xExt;
				}
				else if (j == 1)
				{
					sprintf(cAxis, "y");
					pxFea = &pxSenFea->y;
					pxExtFea = &pxSenFea->yExt;
				}
				else if (j == 2)
				{
					sprintf(cAxis, "z");
					pxFea = &pxSenFea->z;
					pxExtFea = &pxSenFea->zExt;
				}

				sprintf(pxFea->xShaft.xShaft1x.cName, "%sshaft1xF%d", cAxis, usAlias);
				sprintf(pxFea->xShaft.xShaft2x.cName, "%sshaft2xF%d", cAxis, usAlias);
				sprintf(pxFea->xShaft.xShaft3x.cName, "%sshaft3xF%d", cAxis, usAlias);
			}
		}
	}
	// Get Sleeve alias
	{
		unsigned short usAlias;

		if (WIT_SensorOperation(ucSensorAddr, Acc_GetSleeveAlias, WIT_ACC_SENSOR_ID, &usAlias) == true)
		{
			for (j = 0; j < 3; j++)
			{
				if (j == 0)
				{
					sprintf(cAxis, "x");
					pxFea = &pxSenFea->x;
					pxExtFea = &pxSenFea->xExt;
				}
				else if (j == 1)
				{
					sprintf(cAxis, "y");
					pxFea = &pxSenFea->y;
					pxExtFea = &pxSenFea->yExt;
				}
				else if (j == 2)
				{
					sprintf(cAxis, "z");
					pxFea = &pxSenFea->z;
					pxExtFea = &pxSenFea->zExt;
				}
				sprintf(pxFea->xSleeve.xShaftA05xFn.cName, "%sshaftA05xF%d", cAxis, usAlias);
				sprintf(pxFea->xSleeve.xShaftA1xFn.cName, "%sshaftA1xF%d", cAxis, usAlias);
				sprintf(pxFea->xSleeve.xShaftA2xFn.cName, "%sshaftA2xF%d", cAxis, usAlias);
				sprintf(pxFea->xSleeve.xShaftA3xFn.cName, "%sshaftA3xF%d", cAxis, usAlias);
				sprintf(pxFea->xSleeve.xShaftA4xFn.cName, "%sshaftA4xF%d", cAxis, usAlias);
				sprintf(pxFea->xSleeve.xShaftA5xFn.cName, "%sshaftA5xF%d", cAxis, usAlias);
				sprintf(pxFea->xSleeve.xShaftA6xFn.cName, "%sshaftA6xF%d", cAxis, usAlias);
				sprintf(pxFea->xSleeve.xShaftA7xFn.cName, "%sshaftA7xF%d", cAxis, usAlias);
				sprintf(pxFea->xSleeve.xShaftA8xFn.cName, "%sshaftA8xF%d", cAxis, usAlias);
				sprintf(pxFea->xSleeve.xShaftA9xFn.cName, "%sshaftA9xF%d", cAxis, usAlias);
				sprintf(pxFea->xSleeve.xShaftA10xFn.cName, "%sshaftA10xF%d", cAxis, usAlias);
				sprintf(pxFea->xSleeve.xmaxFreqA05xFn.cName, "%smaxFreqAt05x%d", cAxis, usAlias);
			}
		}
	}
}

void WIT_RS485Check()
{
	unsigned char ucModbusID = g_xWITConfig.ucModbusID;
	unsigned long ulSetBaudRate = g_xWITConfig.ulModbusBaudRate;
	unsigned char uceModbusParity = g_xWITConfig.eModbusParity;

	if (ucModbusID >= 1 && ucModbusID <= 247)
	{
		if (ulSetBaudRate != 1200 || ulSetBaudRate != 2400 ||
			ulSetBaudRate != 4800 || ulSetBaudRate != 9600 ||
			ulSetBaudRate != 14400 || ulSetBaudRate != 19200 ||
			ulSetBaudRate != 38400 || ulSetBaudRate != 56000 ||
			ulSetBaudRate != 57600 || ulSetBaudRate != 115200)
		{
			if (uceModbusParity <= 2)
			{
				return;
			}
		}
	}
	g_xWITConfig.ucModbusID = 1;
	g_xWITConfig.ulModbusBaudRate = 9600;
	g_xWITConfig.eModbusParity = MB_PAR_NONE;

	WIT_UpdateConfigureArea();
}
#endif

bool WIT_CheckMotorAddr(void)
{
	unsigned short data;
	// Scan all sensor has fan
	for (size_t i = 1; i <= ACC_MAX_SENSOR_COUNT; i++)
	{
		data = 0;
		unsigned int port = ACC_ADDR_TO_PORT(i);

		WIT_AccEnable(i, WIT_ACC_SENSOR_ID);

		if (eMBReadHoldingRegister(port, WIT_ACC_SENSOR_ID, 0x37D3, 0x01, &data) == MB_EMASTERNOERR)
		{
			if (data == 1)
			{
				spdSyncFSM.mpsAddr = i;
// Set motor actral sel
#if CONFIG_RS485_RTU > 0
				if (WIT_SensorOperation(i, Acc_SetMotorActualSel, WIT_ACC_SENSOR_ID, (unsigned short)g_xWITConfig.spdSel) == true)
				{
					return true;
				}
#endif
			}
		}
	}

	return false;
}

/* --------------------------- Modbus ---------------------------*/
// MB Readx
unsigned short WIT_MB_GetSerialNumber(unsigned short usAddress)
{
	return g_xWITConfig.ulSerialNumber >> (16 * (WIT_MB_SN_ADDR + WIT_MB_SN_COUNT - 1 - usAddress));
}

unsigned short WIT_MB_GetVersion()
{
	return WIT_VERSION_HEX;
}

unsigned short WIT_MB_GetModuleName(unsigned short usAddress)
{
	if (usAddress - WIT_MB_MODULE_NAME_ADDR < (strlen(g_xWITConfig.cModuleName) + 1) / 2)
	{
		return ((unsigned short *)(&g_xWITConfig.cModuleName))[usAddress - WIT_MB_MODULE_NAME_ADDR];
	}
	else
	{
		return 0;
	}
}

unsigned short WIT_MB_GetMQTTBroker(unsigned short usAddress)
{
	int index = usAddress - WIT_MB_MQTT_BROKER_ADDR;

	return ((unsigned short *)g_xWITConfig.cMQTTBroker)[index];
}

unsigned short WIT_MB_GetMQTTPort()
{
	return atol(g_xWITConfig.cMQTTPort);
}

unsigned short WIT_MB_GetMQTTUser(unsigned short usAddress)
{
	int index = usAddress - WIT_MB_MQTT_USER_ADDR;

	return ((unsigned short *)g_xWITConfig.cMQTTUser)[index];
}

unsigned short WIT_MB_GetMQTTPassword(unsigned short usAddress)
{
	int index = usAddress - WIT_MB_MQTT_PASSWORD_ADDR;

	return ((unsigned short *)g_xWITConfig.cMQTTPassword)[index];
}

unsigned short WIT_MB_GetAlarm(unsigned short usAddress)
{
	unsigned short usIndex;

	usIndex = usAddress - WIT_MB_ALARM_ADDR;

	return g_pucMBAlarm[usIndex];
}

unsigned short WIT_MB_GetSensorExistStatus()
{
	return (unsigned short)(sensorsAvailableStateGet());
}

unsigned short WIT_MB_GetSensorID(void)
{
	return g_usMBFeatureSensorID;
}

unsigned short WIT_MB_GetSensorTemperture(void)
{
	return 0;
}

unsigned short WIT_MB_GetSensorFeature(unsigned short usAddress)
{
	return 0;
}

unsigned short WIT_MB_GetNumberSensorFeature(unsigned short usAddress)
{
	return 0;
}

unsigned short WIT_MB_GetID(void)
{
	return g_xWITConfig.ucModbusID;
}

unsigned short WIT_MB_GetBaudRate(unsigned short usAddress)
{
	if (usAddress == WIT_MB_BAUD_RATE_ADDRESS)
	{
		return (unsigned short)(g_xWITConfig.ulModbusBaudRate >> 16);
	}
	else
	{
		return (unsigned short)(g_xWITConfig.ulModbusBaudRate);
	}
}

unsigned short WIT_MB_GetParity(void)
{
	return g_xWITConfig.eModbusParity;
}

// MB Write
void WIT_MB_SetMQTTBroker(unsigned short usAddress, unsigned short usValue)
{
	int index = usAddress - WIT_MB_MQTT_BROKER_ADDR;

	((unsigned short *)g_xWITConfig.cMQTTBroker)[index] = usValue;

	if ((unsigned char)usValue == 0x00 || (unsigned char)(usValue >> 8) == 0x00)
	{
		xTimerStart(g_xCfgQuickUpTimer, 1000);
	}
}

void WIT_MB_SetMQTTPort(unsigned short usValue)
{
	sprintf(g_xWITConfig.cMQTTPort, "%d", usValue);

	xTimerStart(g_xCfgQuickUpTimer, 1000);
}

void WIT_MB_SetMQTTUser(unsigned short usAddress, unsigned short usValue)
{
	int index = usAddress - WIT_MB_MQTT_USER_ADDR;

	((unsigned short *)g_xWITConfig.cMQTTUser)[index] = usValue;

	if ((unsigned char)usValue == 0x00 || (unsigned char)(usValue >> 8) == 0x00)
	{
		xTimerStart(g_xCfgQuickUpTimer, 1000);
	}
}

void WIT_MB_SetMQTTPassword(unsigned short usAddress, unsigned short usValue)
{
	int index = usAddress - WIT_MB_MQTT_PASSWORD_ADDR;

	((unsigned short *)g_xWITConfig.cMQTTPassword)[index] = usValue;

	if ((unsigned char)usValue == 0x00 || (unsigned char)(usValue >> 8) == 0x00)
	{
		xTimerStart(g_xCfgQuickUpTimer, 1000);
	}
}

void WIT_MB_SetSensorID(unsigned short usValue)
{
	if (usValue > 0 && usValue <= ACC_MAX_SENSOR_COUNT)
		g_usMBFeatureSensorID = usValue;
}

void WIT_MB_SetChannelCount(unsigned short usValue)
{
	if (usValue > 0 && usValue <= ACC_MAX_SENSOR_COUNT)
	{
		if (usValue == 1 || usValue == 2 || usValue == 4 || usValue == 6)
		{
#if USE_WIT_PLATFORM > 0
			sprintf(g_xWITConfig.cModuleName, "JCIG2%d40%s", usValue, MODULE_TYPE_STR);
#elif USE_WT_PLATFORM > 0
			sprintf(g_xWITConfig.cModuleName, "WTG90%d%s", usValue, MODULE_TYPE_STR);
#endif
			xTimerStart(g_xCfgQuickUpTimer, 1000);
		}
	}
}

void WIT_MB_SetID(unsigned short usValue)
{
	if (usValue >= 1 && usValue <= 247)
	{
		if (g_xWITConfig.ucModbusID != usValue)
		{
			g_xWITConfig.ucModbusID = usValue;
			xTimerStart(g_xCfgQuickUpTimer, 1000);
		}
	}
}

void WIT_MB_SetBaudRate(unsigned short usAddress, unsigned short usValue)
{
	static unsigned long ulSetBaudRate = 0;

	if (usAddress == WIT_MB_BAUD_RATE_ADDRESS)
	{
		ulSetBaudRate = usValue;
		ulSetBaudRate <<= 16;
	}
	else
	{
		ulSetBaudRate |= usValue;

		if (ulSetBaudRate == 1200 || ulSetBaudRate == 2400 || ulSetBaudRate == 4800 || ulSetBaudRate == 9600 ||
			ulSetBaudRate == 14400 || ulSetBaudRate == 19200 || ulSetBaudRate == 38400 || ulSetBaudRate == 56000 ||
			ulSetBaudRate == 57600 || ulSetBaudRate == 115200)
		{
			if (g_xWITConfig.ulModbusBaudRate != ulSetBaudRate)
			{
				g_xWITConfig.ulModbusBaudRate = ulSetBaudRate;
				xTimerStart(g_xCfgQuickUpTimer, 1000);
			}
		}
	}
}

void WIT_MB_SetParity(unsigned short usValue)
{
	if (usValue <= 2)
	{
		if (g_xWITConfig.eModbusParity != (eMBParity)(usValue))
		{
			g_xWITConfig.eModbusParity = (eMBParity)(usValue);
			xTimerStart(g_xCfgQuickUpTimer, 1000);
		}
	}
}

bool g_bFlagGwFeat = false;
bool g_bFlagSenFeat = false;
bool g_bFlagSenWave = false;

bool WIT2FSM_SensorStateGet(uint32_t addr, uint16_t *state)
{
#if CONFIG_RS485_RTU > 0
	return WIT_SensorOperation(addr,
							   Acc_GetSensorStatus,
							   WIT_ACC_SENSOR_ID,
							   state);
#endif
	return false;
}

bool WIT2FSM_SendHeartToServer(unsigned char groupId)
{
	g_xWITData.ulTimestamp = RTC_InternalGetTimeStamp();

	g_ulUploadRunState = g_bIsMotorWork[groupId];
#if CONFIG_RS485_RTU > 0
	if (g_bGWFeaHeart == true)
	{
		// Need send gateway feature
		g_bGWFeaHeart = false;
		xTimerStart(g_xGatewayFeatureTimer, 1000);
		if (g_xPVDData[groupId].bSendingData == true)
		{
			WIT_IncreasePackageIDAndBackUp(groupId);
		}
		uint32_t tryCount = 0;
		while (tryCount < 5)
		{
			// Send Gateway feature addr=0
			if (WIT_SendGatewayFeatureToServer(groupId, WIT_MQTTPublishHandler) == true)
			{
				// Send success
				return true;
			}
			else
			{
				// Send failed
				tryCount++;
			}
			return false;
		}
	}
#endif
#if CONFIG_BLUETOOTH > 0
	if (g_bHeartPack[groupId] == true)
	{
		// Need send gateway feature
		g_bHeartPack[groupId] = false;
		if (g_xPVDData[groupId].bSendingData == true)
		{
			WIT_IncreasePackageIDAndBackUp(groupId);
		}
		uint32_t tryCount = 0;
		while (tryCount < 5)
		{
			// Send Gateway feature addr=0
			if (WIT_SendGatewayFeatureToServer(groupId, WIT_MQTTPublishHandler) == true)
			{
				// Send success
				return true;
			}
			else
			{
				// Send failed
				tryCount++;
			}
			return false;
		}
	}
#endif
	else
	{
		return true;
	}

	return false;
}

bool WIT2FSM_SendError(void)
{
	// Error status changed
	if (gErrorChanged == true)
	{
		if (WIT_IsNetBusy() == false)
		{
			if (WIT_SendErrorDataIfOccured() == true)
			{
				// send error message success
				gErrorChanged = false;
				return true;
			}
			else
			{
				// send error message failed
				return false;
			}
		}
		else
		{
			// Net busy
			return false;
		}
	}
	else
	{
		return false;
	}
}

bool WIT2FSM_SendAmlVibAlarm(void)
{
	if (WIT_SendAmlVibAlarm(amlVibAlarmAddr) == true)
	{
		// Send success
		return true;
	}
	else
	{
		// Send failed
		return true;
	}
}

bool WIT2FSM_SendGatewayFeatureToServer(unsigned char groupId)
{
	g_ulUploadRunState = g_bIsMotorWork[groupId];
#if CONFIG_BLUETOOTH > 0
	uint32_t tryCount = 0;
	// if (g_xPVDData[groupId].bSendingData == true)
	// {
	// 	WIT_IncreasePackageIDAndBackUp(groupId);
	// }
	while (tryCount < 5)
	{
		if (WIT_IsNetBusy() == false)
		{
			printf("ent...............\r\n");
			// Send Gateway feature addr=0
			if (WIT_SendGatewayFeatureToServer(groupId, WIT_MQTTPublishHandler) == true)
			{
				// Send success
				return true;
			}
		}
		vTaskDelay(5 / portTICK_PERIOD_MS);
		// Send failed
		tryCount++;
	}
	return true;
#endif
#if CONFIG_RS485_RTU > 0
	if (g_bFlagGwFeat == true)
	{
		// Need send gateway feature
		g_bGWFeaHeart = false;
		xTimerStart(g_xGatewayFeatureTimer, 1000);
		uint32_t tryCount = 0;
		while (tryCount < 5)
		{
			// Send Gateway feature addr=0
			if (WIT_SendGatewayFeatureToServer(groupId, WIT_MQTTPublishHandler) == true)
			{
				// Send success
				return true;
			}
			else
			{
				// Send failed
				tryCount++;
			}
		}
	}
	else
	{
		return true;
	}
#endif
	return false;
}

void WIT2FSM_AddGatewayID(unsigned char groupId)
{
	if (g_xPVDData[groupId].bSendingData == true)
	{
		WIT_IncreasePackageIDAndBackUp(groupId);
	}
}

bool WIT2FSM_SendSensorFeatureToServer(unsigned char groupId, unsigned char ucSensorAddr)
{
#if CONFIG_BLUETOOTH > 0
	uint32_t tryCount = 0;
	while (tryCount < 5)
	{
		if (WIT_IsNetBusy() == false)
		{
			// Send Gateway feature addr=0
			if (WIT_SendSensorFeatureToServer(groupId, ucSensorAddr, WIT_MQTTPublishHandler) == true)
			{
				// Send success
				return true;
			}
		}
		// Send failed
		tryCount++;
	}
#endif
#if CONFIG_RS485_RTU > 0
	if (g_bFlagSenFeat == true)
	{
#if CONFIG_RS485_RTU > 0
		ReadSensorFeat(ucSensorAddr);
#endif
		// Need send sensor feature
		uint32_t tryCount = 0;
		while (tryCount < 5)
		{
			// Send Gateway feature addr=0
			if (WIT_SendSensorFeatureToServer(groupId, ucSensorAddr, WIT_MQTTPublishHandler) == true)
			{
				// Send success
				return true;
			}
			else
			{
				// Send failed
				tryCount++;
			}
		}
	}
	else
	{
		return true;
	}
#endif
	return false;
}

bool WIT2FSM_SendWaveToServer(unsigned char groupId, unsigned char ucSensorAddr)
{
#if CONFIG_BLUETOOTH > 0
	uint32_t tryCount = 0;
	while (tryCount < 5)
	{
		// Send Gateway feature addr=0
		if (WIT_SendWaveToServer(groupId, ucSensorAddr, WIT_MQTTPublishHandler) == true)
		{
			// Send success
			return true;
		}
		else
		{
			// Send failed
			tryCount++;
		}
	}
#endif
#if CONFIG_RS485_RTU > 0
	if (g_bFlagSenWave == true)
	{
#if CONFIG_RS485_RTU > 0
		ReadSensorWave(ucSensorAddr);
#endif
		// Need send sensor wave
		uint32_t tryCount = 0;
		while (tryCount < 5)
		{
			// Send Gateway feature addr=0
			if (WIT_SendWaveToServer(groupId, ucSensorAddr, WIT_MQTTPublishHandler) == true)
			{
				// Send success
				return true;
			}
			else
			{
				// Send failed
				tryCount++;
			}
		}
	}
	else
	{
		return true;
	}
#endif
	return false;
}

bool WIT2FSM_DeviceRunCheck(void)
{
	for (size_t i = 0; i < SENSOR_NUM_MAX; i++)
	{
		if (sensorAvailableGet(i) != true)
		{
			continue;
		}
		if (DeviceRunCheck(i + 1) == true)
		{
			g_bIsMotorWork[0] = true;
			return true;
		}
	}
	g_bIsMotorWork[0] = false;
	return false;
}

bool WIT2FSM_NeedSend(void)
{
	g_bFlagGwFeat = g_bCanSendGatewayFeature;
	g_bFlagSenFeat = g_bCanSendSensorFeature;
	g_bFlagSenWave = g_bCanSendWave;

	if (g_bCanSendWave == true)
	{
		g_bFlagGwFeat = true;
		g_bFlagSenFeat = true;
		g_ulUploadTheWaveform = 1;
		return true;
	}

	if (g_bFlagSenFeat == true)
	{
		g_bFlagGwFeat = true;
		return true;
	}
	else
	{
		return false;
	}
}

bool WIT2FSM_StartCollect(void)
{
	SensorSetCalFea(1);
	g_xWITData.ulTimestamp = RTC_InternalGetTimeStamp();

	// Modbus broadcast start collecting
	for (size_t i = 1; i <= ACC_MAX_SENSOR_COUNT; i++)
	{
#if CONFIG_RS485_RTU > 0
		unsigned long *vp = (unsigned long *)(&spdSyncFSM.MotorARRate);
		// Set sensor fActual
		WIT_SensorOperation(i, Acc_SetMotorARRate, WIT_ACC_SENSOR_ID, *vp);
		WIT_SensorOperation(i, Acc_StartCollecting, WIT_ACC_SENSOR_ID);
#endif
	}

	return true;
}

bool WIT2FSM_MotorSpeedSync(speedSyncFSM *fsm)
{
	// Now sensor number is motor monitor point sensor
	// Read fActual speed
	unsigned long vp = 0;
	float MotorARRate = -1;
#if CONFIG_RS485_RTU > 0
	if (WIT_SensorOperation(fsm->mpsAddr, Acc_GetMotorARRate, WIT_ACC_SENSOR_ID, &vp) == false)
	{
		return false;
	}
#endif
	MotorARRate = *((float *)&vp);
	// MotorARRate Scale and Check
	if (MotorARRate > 4000 || MotorARRate < 0)
	{
		// Invailed data
		return false;
	}

	// MotorARRate need update
	if (fsm->MotorARRate == MotorARRate)
	{
		return false;
	}

	fsm->MotorARRate = MotorARRate;
	// set recalculete feature register
	for (int i = 1; i <= ACC_MAX_SENSOR_COUNT; i += 1)
	{
		if (g_bIsSensorExist[i - 1] == false)
		{
			continue;
		}
#if CONFIG_RS485_RTU > 0
		unsigned long *vp = (unsigned long *)(&fsm->MotorARRate);

		if (WIT_SensorOperation(i, Acc_SetMotorARRate, WIT_ACC_SENSOR_ID, *vp) != true)
			return false;
		if (WIT_SensorOperation(i, Acc_SetRecalculete, WIT_ACC_SENSOR_ID, 1) != true)
			return false;
#endif
	}

	return true;
}

bool WIT2FSM_ClearVarible(void)
{
	g_bIsMotorWork[0] = false;

	if (g_bFlagSenFeat == true)
	{
		g_bCanSendGatewayFeature = false;
		g_bCanSendSensorFeature = false;
		g_bFlagGwFeat = false;
		xTimerStart(g_xSensorFeatureTimer, 1000);
		xTimerStart(g_xGatewayFeatureTimer, 1000);
	}

	if (g_bFlagSenWave == true)
	{
		g_bCanSendWave = false;
		xTimerStart(g_xWaveTimer, 1000);
	}

	g_ulUploadTheWaveform = 0;

	if (g_xPVDData[0].bSendingData == true)
	{
		WIT_IncreasePackageIDAndBackUp(0);
	}

	return true;
}

bool WIT2FSM_AmlVibCheck(void)
{
	// Check anomaly vibration
	if (SensorCheckAmlVib(&amlVibAlarmAddr) == true)
	{
		// Has anomaly vibration
		g_bFlagGwFeat = true;
		g_bFlagSenFeat = true;
		g_bFlagSenWave = true;
		g_ulUploadTheWaveform = 1;
		return true;
	}
	else
	{
		// Has not anomaly vibration
		return false;
	}
}

bool WIT2FSM_MqttPub(unsigned char *s)
{
	return MQTT_Publish(g_xWITMQTT.lHandle, g_xWITMQTT.cPubRemoteCtrlResp, (unsigned char *)s, strlen((const char *)s), false, false, QOS1);
}

bool WIT2FSM_GetSpdSyncConfig(void)
{
	return g_xWITConfig.spdSyncEnable;
}

#if CONFIG_BLUETOOTH
WIT_Configure *getGWConfigure(void)
{
	return &g_xWITConfig;
}

unsigned long *WIT_GetWITDataTime(void)
{
	return &g_xWITData.ulTimestamp;
}

sSenTable *getSensorTable(int index)
{
	return &g_xWITConfig.xSenTable[index];
}

sSenTable *getSensorFromAddr(int addr)
{
	if(addr == 0)
		return NULL;
	for (int i = 0; i < WIT_SENSOR_MAX_CONNECT_NUM * WIT_SENSOR_GROUP_MAX_NUM; i++)
	{
		if (addr == g_xWITConfig.xSenTable[i].ulAddr)
			return &g_xWITConfig.xSenTable[i];
	}
	return NULL;
}

sGroupTable *getGroupTable(int index)
{
	return &g_xWITConfig.xGroupTable[index];
}

sGroupTable *getGroupFromGroup(int group)
{
	if(group == 0)
		return NULL;
	for (int i = 0; i < WIT_SENSOR_GROUP_MAX_NUM; i++)
	{
		if (group == g_xWITConfig.xGroupTable[i].ulGroup) 
			return &g_xWITConfig.xGroupTable[i];
	}
	return NULL;
}

void setDevWaveCount(unsigned char index, unsigned long count)
{
	g_xWITData.xSensWave.xWave.ulWaveCount[index] = count;
}

unsigned long getDevWaveCount(unsigned char index)
{
	return g_xWITData.xSensWave.xWave.ulWaveCount[index];
}

void setSamplesFrequency(unsigned char index, unsigned short freq)
{
	g_usAccSamplingRate[index] = freq;
}

void setBleBinSize(unsigned long size)
{
	g_xWITConfig.ulBleBinSize = size;
}

unsigned long getBleBinSize(void)
{
	return g_xWITConfig.ulBleBinSize;
}

void setUpdateFlag(unsigned long flag)
{
	for (int i = 0; i < WIT_SENSOR_GROUP_MAX_NUM * WIT_SENSOR_MAX_CONNECT_NUM; i++)
	{
		if (g_xWITConfig.xSenTable[i].ulUseEnable == 1)
			g_xWITConfig.xSenTable[i].ulUpdate = flag;
	}
}

void WIT_SetAddrConfigure(unsigned char ucAddr, unsigned long ulFlag)
{
	for (int i = 0; i < WIT_SENSOR_GROUP_MAX_NUM; i++)
	{
		if (g_xWITConfig.xSenTable[i].ulUseEnable == 1)
		{
			if (g_xWITConfig.xSenTable[i].ulAddr == ucAddr)
			{
				g_xWITConfig.xSenTable[i].ulConfigEnable = ulFlag;
			}
		}
	}
}

void WIT_SetUpdateConfigureAreaFlag(bool bflag)
{
	g_bUpdateConfigureArea = bflag;
}

static void WIT_FeatureOperation(unsigned int uiAddr)
{
	unsigned int uiAddress = 0;
	unsigned short usInputFlag = 0;
	unsigned short usOutputFlag = 0;
	unsigned char ucBearing = 0;
	unsigned short usExtendBearing = 0;
	unsigned char ucGear = 0;
	unsigned short usExtendGear = 0;
	unsigned char ucFanEnable = 0;
	unsigned char ucSpeedEnable = 0;
	unsigned char ucBeltEnable = 0;
	unsigned char ucScrewEnable = 0;
	unsigned char ucShaftEnable = 0;
	unsigned char ucFemaleEnable = 0;
	unsigned char ucMaleEnable = 0;
	unsigned char ucPwrFreqEnable = 0;
	unsigned long aliasaddr = 0;
	unsigned char num = 0;
	unsigned char flag = 0;

	unsigned char ucConfigFlag[SENSOR_PARTS_MAX_NUM] = {0};
	unsigned short usExtendConfigFlag[4] = {0};
	dev_feature *pFeatureData = NULL;
	pFeatureData = get_dev_fea(0);
	// 获取基本信息

	if (eWCPReadDataAddress(uiAddr, SENSOR_CHR_WRITE_HANDLER, SENSOR_DEVICE_SN_APKG_ADDR, SENSOR_DEVICE_INFO_APKG_NUM, pFeatureData->cFeature) == WCP_OK)
	{
		Acc_GetDeviceInfoData(pFeatureData->cFeature, g_cSensorSerialNumber[uiAddr], g_cSensorType[uiAddr], g_cSensorChildVersion[uiAddr], g_cSensorLogoType[uiAddr], g_cSensorVersion[uiAddr]);
	}
	// 采样频率
	if (eWCPReadDataAddress(uiAddr, SENSOR_CHR_WRITE_HANDLER, SENSOR_SAMPLING_NUM_APKG_ADDR, 2, (unsigned char *)&g_xWITData.xSensWave.xWave.ulWaveCount[uiAddr]) == WCP_OK)
	{
	
	}

	if (eWCPReadDataAddress(uiAddr, SENSOR_CHR_WRITE_HANDLER, SENSOR_FFT_RESOLUTION_APKG_ADDR, 5, (unsigned char *)&pFeatureData->cFeature) == WCP_OK)
	{
		if (strstr((char *)g_cSensorType, "BV3C") != NULL)
		{
			flag = 1;
		}
		else if (strstr((char *)g_cSensorType, "BV3A") != NULL)
		{
			flag = 2;
		}
		Acc_GetSensorData(pFeatureData->cFeature, flag, &g_usAccSamplingRate[uiAddr], &g_xWITData.xSensFea.resolution);
	}
	// 基本特征值
	if (eWCPReadDataAddress(uiAddr, SENSOR_CHR_WRITE_HANDLER, SENSOR_BASE_FEATURE_APKG_ADDR, 418, pFeatureData->cFeature) == WCP_OK)
	{
		Acc_GetSenBaseFeatureData(pFeatureData->cFeature, &g_xWITData.xSensFea);
	}
	// 获取特征值标志变量
	eWCPReadDataAddress(uiAddr, SENSOR_CHR_WRITE_HANDLER, SENSOR_BEARING_APKG_CONFIG_FLAG_ADDR, SENSOR_PARTS_MAX_NUM, ucConfigFlag);
	vTaskDelay(50 / portTICK_PERIOD_MS);
	eWCPReadDataAddress(uiAddr, SENSOR_CHR_WRITE_HANDLER, 28, 8, (unsigned char *)usExtendConfigFlag);
	vTaskDelay(100 / portTICK_PERIOD_MS);

	// 获取轴承特征值
	ucBearing = ucConfigFlag[0];
	usExtendBearing = usExtendConfigFlag[2];
	printf("ucBearing  %d\r\n", ucBearing);
	printf("usExtendBearing  %d\r\n", usExtendBearing);
	{
		bool bBearingFlag = false;
		bool bExtendBearingFlag = false;
		for (int i = 0; i < ACC_MAX_BEARING_COUNT; i++)
		{
			bBearingFlag = (bool)((ucBearing >> i) & 0x01);
			bExtendBearingFlag = (bool)((usExtendBearing >> i) & 0x01);
			printf("bBearingFlag %d, bExtendBearingFlag %d\r\n", bBearingFlag, bExtendBearingFlag);
			if (i == 0)
			{
				uiAddress = SENSOR_BEARING1_FEATURE_APKG_ADDR;
				aliasaddr = SENSOR_BEARING1_ALIAS_APKG_ADDR;
			}
			else
			{
				uiAddress = SENSOR_BEARING2_FEATURE_APKG_ADDR;
				aliasaddr = SENSOR_BEARING2_ALIAS_APKG_ADDR;
			}
			if (bBearingFlag == true || bExtendBearingFlag == true)
			{
				if (eWCPReadDataAddress(uiAddr, SENSOR_CHR_WRITE_HANDLER, uiAddress, 100, pFeatureData->cFeature) == WCP_OK)
				{
					eWCPReadDataAddress(uiAddr, SENSOR_CHR_WRITE_HANDLER, aliasaddr, 1, &num);
					if (num == 0)
						num = i + 1;
					Acc_GetBearingFeatureData(bBearingFlag, bExtendBearingFlag, i, pFeatureData->cFeature, &g_xWITData.xSensFea, num);
				}
			}
		}
	}
	// 获取齿轮特征值
	ucGear = ucConfigFlag[1];
	usExtendGear = usExtendConfigFlag[3];
	printf("gear  %d.....\r\n", ucGear);
	printf("usExtendGear  %d.....\r\n", usExtendGear);
	{
		bool bGearFlag = false;
		bool bExtendGearFlag = false;
		for (int i = 0; i < ACC_MAX_GEAR_COUNT; i++)
		{
			bGearFlag = (bool)((ucGear >> i) & 0x01);
			bExtendGearFlag = (bool)((usExtendGear >> i) & 0x01);
			if (i == 0)
			{
				uiAddress = SENSOR_GEAR1_FEATURE_APKG_ADDR;
				aliasaddr = SENSOR_GEAR1_ALIAS_APKG_ADDR;
			}
			else if (i == 1)
			{
				uiAddress = SENSOR_GEAR2_FEATURE_APKG_ADDR;
				aliasaddr = SENSOR_GEAR2_ALIAS_APKG_ADDR;
			}
			else
			{
				uiAddress = SENSOR_GEAR3_FEATURE_APKG_ADDR;
				aliasaddr = SENSOR_GEAR2_ALIAS_APKG_ADDR;
			}
			if (bGearFlag == true || bExtendGearFlag == true)
			{
				if (eWCPReadDataAddress(uiAddr, SENSOR_CHR_WRITE_HANDLER, uiAddress, 92, pFeatureData->cFeature) == WCP_OK)
				{
					eWCPReadDataAddress(uiAddr, SENSOR_CHR_WRITE_HANDLER, aliasaddr, 1, &num);
					if (num == 0)
						num = i + 1;
					printf("gear num %d\r\n", num);
					Acc_GetGearFeatureData(bGearFlag, bExtendGearFlag, i, pFeatureData->cFeature, &g_xWITData.xSensFea, num);
				}
			}
		}
	}
	// 获取转速特征值
	ucSpeedEnable = ucConfigFlag[2];
	printf("ucSpeedEnable %d.....\r\n", ucSpeedEnable);
	{
		if (ucSpeedEnable == 0)
		{
			Acc_ClearSenSpeedFeature(&g_xWITData.xSensFea);
		}
	}
	// 获取输入转速特征值
	usInputFlag = usExtendConfigFlag[0];
	printf("input speed %d.....\r\n", usInputFlag);
	{
		if (usInputFlag == 0)
			Acc_ClearSenInputSpeedFeature(&g_xWITData.xSensFea);
	}

	// 获取输出转速特征值
	usOutputFlag = usExtendConfigFlag[1];
	printf("output speed %d.....\r\n", usOutputFlag);
	{
		if (usOutputFlag == 0)
			Acc_ClearSenOutputSpeedFeature(&g_xWITData.xSensFea);
	}
	// 获取风机特征值
	ucFanEnable = ucConfigFlag[3];
	printf("ucFanEnable %d\r\n", ucFanEnable);
	{
		if (ucFanEnable == 1)
		{
			if (eWCPReadDataAddress(uiAddr, SENSOR_CHR_WRITE_HANDLER, SENSOR_FAN_FEATURE_AKPG_ADDR, SENSOR_FAN_FEATURE_APKG_NUM, pFeatureData->cFeature) == WCP_OK)
			{
				Acc_GetFanFeatureData(pFeatureData->cFeature, &g_xWITData.xSensFea);
			}
		}
	}
	// 获取皮带特征值
	// if(eWCPReadDataAddress(uiAddr, SENSOR_CHR_WRITE_HANDLER, SENSOR_BELT_APKG_CONFIG_FLAG_ADDR, 1, &ucBeltEnable) == WCP_OK)
	ucBeltEnable = ucConfigFlag[4];
	printf("ucBeltEnable %d\r\n", ucBeltEnable);
	{
		if (ucBeltEnable == 1)
		{
			if (eWCPReadDataAddress(uiAddr, SENSOR_CHR_WRITE_HANDLER, SENSOR_BELT_FEATURE_APKG_ADDR, SENSOR_BELT_FEATURE_APKG_NUM, pFeatureData->cFeature) == WCP_OK)
			{
				Acc_GetBeltFeatureData(pFeatureData->cFeature, &g_xWITData.xSensFea);
			}
		}
	}
	// 获取螺杆特征值
	ucScrewEnable = ucConfigFlag[5];
	printf("ucScrewEnable %d\r\n", ucScrewEnable);
	{
		if (ucScrewEnable == 1)
		{
			if (eWCPReadDataAddress(uiAddr, SENSOR_CHR_WRITE_HANDLER, SENSOR_SCREW_FEATURE_AKPG_ADDR, SENSOR_SCREW_FEATURE_APKG_NUM, pFeatureData->cFeature) == WCP_OK)
			{
				Acc_GetScrewFeatureData(pFeatureData->cFeature, &g_xWITData.xSensFea);
			}
		}
	}
	// 获取传动轴特征值
	ucShaftEnable = ucConfigFlag[6];
	printf("ucShaftEnable %d\r\n", ucShaftEnable);
	{
		if (ucShaftEnable == 1)
		{
			if (eWCPReadDataAddress(uiAddr, SENSOR_CHR_WRITE_HANDLER, SENSOR_SHAFT_FEATURE_AKPG_ADDR, SENSOR_SHAFT_FEATURE_APKG_NUM, pFeatureData->cFeature) == WCP_OK)
			{
				Acc_GetShaftFeatureData(pFeatureData->cFeature, &g_xWITData.xSensFea);
			}
		}
	}
	// 获取阴螺杆特征值
	ucFemaleEnable = ucConfigFlag[7];
	printf("ucFemaleEnable %d\r\n", ucFemaleEnable);
	{
		if (ucFemaleEnable == 1)
		{
			if (eWCPReadDataAddress(uiAddr, SENSOR_CHR_WRITE_HANDLER, SENSOR_FEMALE_SCREW_FEATURE_AKPG_ADDR, SENSOR_FMMALE_FEATURE_APKG_NUM, pFeatureData->cFeature) == WCP_OK)
			{
				Acc_GetFeMaleFeatureData(pFeatureData->cFeature, &g_xWITData.xSensFea);
			}
		}
	}
	// 获取阳螺杆特征值
	ucMaleEnable = ucConfigFlag[8];
	printf("ucMaleEnable %d\r\n", ucMaleEnable);
	{
		if (ucMaleEnable == 1)
		{
			if (eWCPReadDataAddress(uiAddr, SENSOR_CHR_WRITE_HANDLER, SENSOR_MALE_SCREW_FEATURE_AKPG_ADDR, SENSOR_FMMALE_FEATURE_APKG_NUM, pFeatureData->cFeature) == WCP_OK)
			{
				Acc_GetMaleFeatureData(pFeatureData->cFeature, &g_xWITData.xSensFea);
			}
		}
	}
	// // 固有频率特征值
	ucPwrFreqEnable = ucConfigFlag[9];
	{
		if(ucPwrFreqEnable == 1)
		{
			if(eWCPReadDataAddress(uiAddr, SENSOR_CHR_WRITE_HANDLER, SENSOR_POWER_FREQUENCY_AKPG_ADDR , 24, pFeatureData->cFeature) == WCP_OK)
			{
				Acc_GetPowerFeatureData(pFeatureData->cFeature, &g_xWITData.xSensFea);
			}
		}
	}
}

bool WIT_GetSensorDataAndUpload(unsigned int uiAddr, sGroupTable *pxGroupTable, bool *bWaveStatus, bool *bFeaStatus)
{
	unsigned long ulsampleNum = 0;
	unsigned long feature_push = 0;
	unsigned long wave_push = 0;
	unsigned char groupId;

	if (pxGroupTable == NULL)
		return false;
	
	feature_push = pxGroupTable->ulFeaPushEnable;
	wave_push = pxGroupTable->ulWavePushEnable;
	groupId = pxGroupTable->ulGroup - 1;
	// 判断原始波形是否采集
	g_ulUploadTheWaveform = wave_push;
	if (feature_push == 1 && g_bIsLowPower == false)
	{
	 	clear_feature_use();
	 	WIT_FeatureOperation(uiAddr);
	 	WIT2FSM_SendSensorFeatureToServer(groupId, uiAddr + 1);
	 	*bFeaStatus = true;
	}
    unsigned char flag = 0;
	
	if(g_bIsLowPower == true)
	{
		unsigned char buf[240] = {0};
		unsigned char *pbuf = NULL;
		if(eWCPReadDataAddress(uiAddr, SENSOR_CHR_WRITE_HANDLER, SENSOR_DEVICE_SN_APKG_ADDR, 102, buf) == WCP_OK)
		{
			Acc_GetDeviceInfoData(buf, g_cSensorSerialNumber[uiAddr], g_cSensorType[uiAddr], g_cSensorChildVersion[uiAddr], g_cSensorLogoType[uiAddr], g_cSensorVersion[uiAddr]);
		}
		else
		{
			return false;
		}		
		pbuf = &buf[SENSOR_DEVICE_INFO_APKG_NUM];
		g_xWITData.xSensWave.xWave.ulWaveCount[uiAddr] = *(unsigned short *)(&pbuf[0]);
		pbuf += 2;
		printf("g_cSensorType[%d]:%s\n",uiAddr, g_cSensorType[uiAddr]);
		if(strstr((char *)g_cSensorType[uiAddr], "BV3C") != NULL)
		{
			printf("judgement 3C\n");
			flag = 1;
		}
		else if (strstr((char *)g_cSensorType[uiAddr], "BV3A") != NULL)
		{
			printf("judgement 3A\n");
			flag = 2;
		}
		Acc_GetSensorData(pbuf, flag, &g_usAccSamplingRate[uiAddr], NULL);
	}
	if (wave_push == 1 || g_bIsLowPower == true)
	{
		dev_wave *pDev = NULL;
		pDev = get_dev_wave(uiAddr);
		ulsampleNum = g_xWITData.xSensWave.xWave.ulWaveCount[uiAddr];
		if(ulsampleNum > 8182)
		{
			g_xWITData.xSensWave.xWave.ulWaveCount[uiAddr] = 8192;
			ulsampleNum = 8192;
		}
		ESP_LOGD("test", "ulsampleNum %ld", ulsampleNum);
		if( flag == 1)
        {
            if(eWCPReadDataAddress(uiAddr, SENSOR_CHR_WRITE_HANDLER, 0x00002710, ulsampleNum * 2, pDev->cWave) != WCP_OK)
            {
                return false;
            }	
            if(eWCPReadDataAddress(uiAddr, SENSOR_CHR_WRITE_HANDLER, 0x00006710, ulsampleNum * 2, pDev->cWave + (ulsampleNum * 2 * 1)) != WCP_OK)
            {
                return false;
            }
            if(eWCPReadDataAddress(uiAddr, SENSOR_CHR_WRITE_HANDLER, 0x0000A710, ulsampleNum * 2, pDev->cWave + (ulsampleNum * 2 * 2)) != WCP_OK)
            {
                return false;
            }
        }
        else if( flag == 2)
        {
            if(eWCPReadDataAddress(uiAddr, SENSOR_CHR_WRITE_HANDLER, 0x00002710, ulsampleNum * 4, pDev->cWave) != WCP_OK)
            {
                return false;
            }	
            if(eWCPReadDataAddress(uiAddr, SENSOR_CHR_WRITE_HANDLER, 0x00006710, ulsampleNum * 4, pDev->cWave + (ulsampleNum * 4 * 1)) != WCP_OK)
            {
                return false;
            }
            if(eWCPReadDataAddress(uiAddr, SENSOR_CHR_WRITE_HANDLER, 0x0000A710, ulsampleNum * 4, pDev->cWave + (ulsampleNum * 4 * 2)) != WCP_OK)
            {
                return false;
            }
        }
		
		if (WIT_IsNetBusy() == false && g_bIsLowPower == false)
			WIT_SendWaveToServer(groupId, uiAddr + 1, NULL);
		gSenInfo.xDevInfo[uiAddr].bDeliveries = true;
		*bWaveStatus = true;
	}
	return true;
}

void if_clear_multi_gateway_data(void)
{
	for (int i = 0; i < WIT_SENSOR_GROUP_MAX_NUM * WIT_SENSOR_MAX_CONNECT_NUM; i++)
	{
		g_xWITConfig.xSenTable[i].ulGroup = 0;
		g_xWITConfig.xSenTable[i].ulUseEnable = 0;
		g_xWITConfig.xSenTable[i].ulConfigEnable = 0;
	}

	for (int i = 0; i < WIT_SENSOR_GROUP_MAX_NUM; i++)
	{
		g_xWITConfig.xGroupTable[i].ulGroup = i + 1;
		g_xWITConfig.xGroupTable[i].ulGUseEnable = 0;
		g_xWITConfig.xGroupTable[i].ulPeriod = 3000;
		g_xWITConfig.xGroupTable[i].ulFeaPeriod = 3000;
		g_xWITConfig.xGroupTable[i].ulConnectNum = 0;
		g_xWITConfig.xGroupTable[i].ulConnectCnt = 0;
		g_xWITConfig.xGroupTable[i].ulOperation = 0;
		memset(g_xWITConfig.xGroupTable[i].groupNo, 0, sizeof(g_xWITConfig.xGroupTable[i].groupNo));
	}
}
static sGroupTable *getGroupFromGroupNo(char *monitorNo)
{
	for (int i = 0; i < WIT_SENSOR_GROUP_MAX_NUM; i++)
	{
		if (strcmp(monitorNo, g_xWITConfig.xGroupTable[i].groupNo) == 0)
			return &g_xWITConfig.xGroupTable[i];
	}
	return NULL;
}

#endif

void WIT_SetMotorWork(unsigned char ucIndex, bool bStatus)
{
	g_bIsMotorWork[ucIndex] = bStatus;
}

bool WIT_GetMotorWork(unsigned char ucIndex)
{
	return g_bIsMotorWork[ucIndex];
}

void clear_sensor_updata_flag(void)
{
	for (int i = 0; i < WIT_SENSOR_GROUP_MAX_NUM * WIT_SENSOR_MAX_CONNECT_NUM; i++)
	{
		g_xWITConfig.xSenTable[i].ulUpdate = 0;
		WIT_SetUpdateConfigureAreaFlag(true);
	}
}

void clear_feature_use(void)
{
	int i, j;
	char cAxis[4];
	Sensor_Feature *pxSenFea = &g_xWITData.xSensFea;
	Feature *pxFea = NULL;
	ExtendFeature *pxExtFea = NULL;

	for (i = 0; i < 3; i++)
	{
		if (i == 0)
		{
			sprintf(cAxis, "x");
			pxFea = &pxSenFea->x;
			pxExtFea = &pxSenFea->xExt;
		}
		else if (i == 1)
		{
			sprintf(cAxis, "y");
			pxFea = &pxSenFea->y;
			pxExtFea = &pxSenFea->yExt;
		}
		else if (i == 2)
		{
			sprintf(cAxis, "z");
			pxFea = &pxSenFea->z;
			pxExtFea = &pxSenFea->zExt;
		}

#if CONFIG_RS485_RTU > 0
		pxFea->xFreq1x.bIsUsed = false;
		pxFea->xFreq2x.bIsUsed = false;
		pxFea->xFreq3x.bIsUsed = false;
#endif
#if CONFIG_BLUETOOTH > 0
		pxFea->xFreq05x.bIsUsed = false;
		pxFea->xFreq10x.bIsUsed = false;
		pxFea->xFreq15x.bIsUsed = false;
		pxFea->xFreq20x.bIsUsed = false;
		pxFea->xFreq25x.bIsUsed = false;
		pxFea->xFreq30x.bIsUsed = false;
		pxFea->xFreq35x.bIsUsed = false;
		pxFea->xFreq40x.bIsUsed = false;
		pxFea->xFreq45x.bIsUsed = false;
		pxFea->xFreq50x.bIsUsed = false;
		pxFea->xFreq55x.bIsUsed = false;
		pxFea->xFreq60x.bIsUsed = false;
		pxFea->xFreq65x.bIsUsed = false;
		pxFea->xFreq70x.bIsUsed = false;
		pxFea->xFreq75x.bIsUsed = false;
		pxFea->xFreq80x.bIsUsed = false;
		pxFea->xFreq85x.bIsUsed = false;
		pxFea->xFreq95x.bIsUsed = false;
		pxFea->xFreq100x.bIsUsed = false;
#endif
		// 扩展特征
		pxExtFea->xBaseExtFea.xInputEnvEnergy.bIsUsed = false;
		pxExtFea->xBaseExtFea.xOutputEnvEnergy.bIsUsed = false;
		// 轴承
		for (j = 0; j < ACC_MAX_BEARING_COUNT; j++)
		{
			pxFea->xBearingFea[j].xFreqBO.bIsUsed = false;
			pxFea->xBearingFea[j].xFreqBI.bIsUsed = false;
			pxFea->xBearingFea[j].xFreqBB.bIsUsed = false;
			pxFea->xBearingFea[j].xFreqBF.bIsUsed = false;
			pxExtFea->xBearingExtFea[j].xEnvEnergy.bIsUsed = false;
			pxExtFea->xBearingExtFea[j].xEnv2Energy.bIsUsed = false;
			pxExtFea->xBearingExtFea[j].xEnv3Energy.bIsUsed = false;
		}
		// 齿轮
		for (j = 0; j < ACC_MAX_GEAR_COUNT; j++)
		{
			pxFea->xGearFea[j].xFreq1xGMF.bIsUsed = false;
			pxFea->xGearFea[j].xFreq2xGMF.bIsUsed = false;
			pxFea->xGearFea[j].xFreq3xGMF.bIsUsed = false;
			// 齿轮扩展特征
			pxExtFea->xGearExtFea[j].x1xSideEnergy.bIsUsed = false;
			pxExtFea->xGearExtFea[j].x2xSideEnergy.bIsUsed = false;
			pxExtFea->xGearExtFea[j].x3xSideEnergy.bIsUsed = false;
			pxExtFea->xGearExtFea[j].xEnvEnergy.bIsUsed = false;
		}
		// 风机
		pxFea->xFan.xSPM1x.bIsUsed = false;
		pxFea->xFan.xSPM2x.bIsUsed = false;
		pxFea->xFan.xSPM3x.bIsUsed = false;
		// 皮带
		pxFea->xBelt.xBelt1x.bIsUsed = false;
		pxFea->xBelt.xSelfVirb.bIsUsed = false;
		// Screw
		pxFea->xScrew.xScrew1x.bIsUsed = false;
		pxFea->xScrew.xScrew2x.bIsUsed = false;
		pxFea->xScrew.xScrew3x.bIsUsed = false;
		// Male Screw
		pxFea->xMaleScrew.xMaleScrew1x.bIsUsed = false;
		pxFea->xMaleScrew.xMaleScrew2x.bIsUsed = false;
		pxFea->xMaleScrew.xMaleScrew3x.bIsUsed = false;
		// Female Screw
		pxFea->xFemaleScrew.xFemaleScrew1x.bIsUsed = false;
		pxFea->xFemaleScrew.xFemaleScrew2x.bIsUsed = false;
		pxFea->xFemaleScrew.xFemaleScrew3x.bIsUsed = false;
		// Shaft
		pxFea->xShaft.xShaft1x.bIsUsed = false;
		pxFea->xShaft.xShaft2x.bIsUsed = false;
		pxFea->xShaft.xShaft3x.bIsUsed = false;
		// Sleeve
		pxFea->xSleeve.xShaftA05xFn.bIsUsed = false;
		pxFea->xSleeve.xShaftA1xFn.bIsUsed = false;
		pxFea->xSleeve.xShaftA2xFn.bIsUsed = false;
		pxFea->xSleeve.xShaftA3xFn.bIsUsed = false;
		pxFea->xSleeve.xShaftA4xFn.bIsUsed = false;
		pxFea->xSleeve.xShaftA5xFn.bIsUsed = false;
		pxFea->xSleeve.xShaftA6xFn.bIsUsed = false;
		pxFea->xSleeve.xShaftA7xFn.bIsUsed = false;
		pxFea->xSleeve.xShaftA8xFn.bIsUsed = false;
		pxFea->xSleeve.xShaftA9xFn.bIsUsed = false;
		pxFea->xSleeve.xShaftA10xFn.bIsUsed = false;
		pxFea->xSleeve.xmaxFreqA05xFn.bIsUsed = false;

		// PowerFrequency
		pxFea->xPowerFrequency.xNaAmFreq.bIsUsed = false;
		pxFea->xPowerFrequency.xEnergy.bIsUsed = false;
	}
}

unsigned long *get_ulPort(void)
{
	return &g_xWITConfig.ulPort;
}

char *sent_mqtt_ip(void)
{
	return (char *)&g_xWITConfig.cTargetIP;
}

#if CONFIG_W5100S_SPI_ETHERNET > 0
unsigned long get_status_dhcp(void)
{
	return g_xWITConfig.g_xConfigIp.dhcp;
}

wiz_NetInfo *get_gw_eth_info(void)
{
	return &g_xWITConfig.g_xConfigIp;
}

#endif

void iot_Sensor_Init(void)
{
    for (int i = 0; i < IOTDEV_MAX_NUM; i++)
    {
        // g_xWITConfig.my_IOTDev[i].addr = 0;
        // g_xWITConfig.my_IOTDev[i].id = 0;
        // g_xWITConfig.my_IOTDev[i].sn = 0;
        // g_xWITConfig.my_IOTDev[i].bIsUsed = false;

        g_xWITConfig.my_IOTDev[i].bIsTimeout = true;
        g_xWITConfig.my_IOTDev[i].bIsPlugIn = false;
        g_xWITConfig.my_IOTDev[i].rope_value = 0;
        g_xWITConfig.my_IOTDev[i].beingWaiting = false;
        g_xWITConfig.my_IOTDev[i].status = 0;
    }
}

void iot_Sensor_Print(void)
{
    for (int i = 0; i < IOTDEV_MAX_NUM; i++)
    {
        printf("my_IOTDev[%d].addr = %d\n", i, g_xWITConfig.my_IOTDev[i].addr);
        printf("my_IOTDev[%d].id = %d\n", i, g_xWITConfig.my_IOTDev[i].id);
        printf("my_IOTDev[%d].sn = %d\n", i, g_xWITConfig.my_IOTDev[i].sn);
        printf("my_IOTDev[%d].bIsTimeout = %d\n", i, g_xWITConfig.my_IOTDev[i].bIsTimeout);
        printf("my_IOTDev[%d].bIsUsed = %d\n", i, g_xWITConfig.my_IOTDev[i].bIsUsed);
        printf("my_IOTDev[%d].bIsPlugIn = %d\n", i, g_xWITConfig.my_IOTDev[i].bIsPlugIn);
        printf("my_IOTDev[%d].rope_value = %d\n", i, g_xWITConfig.my_IOTDev[i].rope_value);
        printf("my_IOTDev[%d].beingWaiting = %d\n", i, g_xWITConfig.my_IOTDev[i].beingWaiting);
        printf("my_IOTDev[%d].status = %d\n", i, g_xWITConfig.my_IOTDev[i].status);
    }
}


// 修改：增加二值信号量，回复响应状态
void vSensorDataFreshTimerCallback()
{
    xSemaphoreDataResp =  xSemaphoreCreateBinary();
    for (int i = 0; i < IOTDEV_MAX_NUM; i++)
    {
        if (g_xWITConfig.my_IOTDev[i].bIsUsed && g_xWITConfig.my_IOTDev[i].bIsPlugIn)
        {
            Get_SensorIOT_Data(g_xWITConfig.my_IOTDev[i].id, g_xWITConfig.my_IOTDev[i].sn);
            if (xSemaphoreDataResp != NULL && xSemaphoreTake(xSemaphoreDataResp, pdMS_TO_TICKS(500)) == pdTRUE) 
            {
                // printf("data resp success\n");
            }
            else
            {
                printf("data resp fail, maybe timeout or be below the threshold\n");
            }
        }
        vTaskDelay(pdMS_TO_TICKS(10));
    }
}


bool IOTConfigFunction()
{
    bool status = true;
    // 创建二值信号量
    xSemaphoreWaitResp = xSemaphoreCreateBinary();
    // 遍历每个传感器
    for (int i = 0; i < IOTDEV_MAX_NUM; i++)
    {
        if (g_xWITConfig.my_IOTDev[i].bIsUsed && g_xWITConfig.my_IOTDev[i].bIsPlugIn) //配置阈值条件：传感器是否使用，传感器是否插上
        {
            g_xWITConfig.my_IOTDev[i].beingWaiting = true;
            Set_Threshold(g_xWITConfig.my_IOTDev[i].id, g_xWITConfig.my_IOTDev[i].sn, g_xWITConfig.my_IOTThreshold);
            if (xSemaphoreWaitResp != NULL && xSemaphoreTake(xSemaphoreWaitResp, pdMS_TO_TICKS(500)) == pdTRUE) {
                //printf("config success\n");
                g_xWITConfig.my_IOTDev[i].beingWaiting = false;
            }
            else{
                status = false;
                printf("config threshold fail\n");
                g_xWITConfig.my_IOTDev[i].beingWaiting = false;
            }
            //定时器等待配置之后得到传感器响应
        }
        vTaskDelay(pdMS_TO_TICKS(10));
    }

    // 删除二值信号量
    vSemaphoreDelete(xSemaphoreWaitResp);
    xSemaphoreWaitResp = NULL; 

    return status;
}


// 定时器回调函数， 用于检查心跳包是否超时
void vSensorTimerCallback()
{
    unsigned long last_ts = 0;
    unsigned long now_ts = 0;
    unsigned long diff_ts = 0;
    // 遍历每个传感器
    for (int i = 0; i < IOTDEV_MAX_NUM; i++)
    {
        if (g_xWITConfig.my_IOTDev[i].bIsUsed && g_xWITConfig.my_IOTDev[i].bIsPlugIn)
        {
            last_ts = g_ts[i];
            now_ts = xTaskGetTickCount();
            diff_ts = now_ts - last_ts;
            if(diff_ts < 0)
            {
                printf("ticks[now:%ld][old:%ld]\n", now_ts, last_ts);
                return;
            }
            // printf("my_IOTDev[%d] diff_ts[%ld]\n", i, diff_ts);
            if (diff_ts > pdMS_TO_TICKS(70000 * 1)) // 70s
            {
                // printf("Sensor[%d] is disconnected\n", i);
                g_xWITConfig.my_IOTDev[i].bIsTimeout = true; // next send error to mqtt broker
                if(g_xWITConfig.my_IOTDev[i].bIsPlugIn) 
                {
                    g_bErrorUpdateFlag = true;
                    g_xWITConfig.my_IOTDev[i].bIsPlugIn = false;
                }
            }
            else{
                // printf("Sensor[%d] is online\n", i);
            }
        }
        vTaskDelay(pdMS_TO_TICKS(10));
    }
}

void Message_Deal_Function(void)
{
    sIOT01B_Format g_data_frame;
    Tcp_Message tcp_message_deal = {0};
    bool ret = 0;
    unsigned long _ts = 0;

    xSensorTimer = xTimerCreate("SensorTimer", pdMS_TO_TICKS(1000), pdTRUE, NULL, vSensorTimerCallback);
    xSensorDataFreshTimer = xTimerCreate("SensorDataFreshtimer", pdMS_TO_TICKS(60000*3), pdTRUE, NULL, vSensorDataFreshTimerCallback);

    xTimerStart(xSensorTimer, 0);
    xTimerStart(xSensorDataFreshTimer, 0);

    while(1)
    {
        if (pdTRUE  == xQueueReceive(xMessageQueueHandle, &tcp_message_deal, portMAX_DELAY))
        {
            // printf("rec-len[%d]\n", tcp_message_deal.rec_len);
            // message_hex(tcp_message_deal.message, tcp_message_deal.rec_len);
            
            ret = data_frame_parsing(tcp_message_deal.message, tcp_message_deal.rec_len, &g_data_frame);
            if(ret == true)
            {
                g_slaver_status = g_data_frame.ucCmd;
                printf("CMD = [0X%02X]\n", g_slaver_status);
            }
            else
            {
                printf("data_frame_parsing fail\n");
                continue;
            }

            switch (g_slaver_status)
            {

            case SLAVE_POWER_ON_CONTACK:
            {
                _ts = xTaskGetTickCount();
                for(int i = 0;  i< IOTDEV_MAX_NUM; i++)
                {
                    if(g_xWITConfig.my_IOTDev[i].bIsUsed && (g_data_frame.ulDevID == g_xWITConfig.my_IOTDev[i].id) && (g_data_frame.ulDevSN == g_xWITConfig.my_IOTDev[i].sn))
                    {
                        ACK_0AH(g_data_frame.ulDevID, g_data_frame.ulDevSN);
                        Start_Detect(g_data_frame.ulDevID, g_data_frame.ulDevSN);
                        //printf("IOT[%d] POWER ON\n", i);
                        g_ts[i] = _ts; // update time
                        g_xWITConfig.my_IOTDev[i].bIsPlugIn = true; // 第一次检测到设备上电，意味着设备已经接入网关
                        g_xWITConfig.my_IOTDev[i].bIsTimeout = false;
                    }
                }
                break;
            }

            case SLAVE_ACK:
                _ts = xTaskGetTickCount();
                for(int i = 0;  i< IOTDEV_MAX_NUM; i++)
                {
                    if(g_xWITConfig.my_IOTDev[i].bIsUsed && g_data_frame.ulDevID == g_xWITConfig.my_IOTDev[i].id && g_data_frame.ulDevSN == g_xWITConfig.my_IOTDev[i].sn)
                    {
                        g_ts[i] = _ts; // update time
                        printf("SLAVE_ACK: %d\n", *g_data_frame.data);
                        if(g_xWITConfig.my_IOTDev[i].beingWaiting && *g_data_frame.data == 1)
                        {
                            if(xSemaphoreWaitResp != NULL)
                            {
                                xSemaphoreGive(xSemaphoreWaitResp);
                                Stop_Detect(g_data_frame.ulDevID, g_data_frame.ulDevSN);
                                vTaskDelay(10);
                                ACK_0AH(g_data_frame.ulDevID, g_data_frame.ulDevSN);
                                Start_Detect(g_data_frame.ulDevID, g_data_frame.ulDevSN);
                                // printf("Give Response\n");
                            }
                        }
                    }

                }
                break;
            
            case SLAVE_HEARTBEAT:
            {
                _ts = xTaskGetTickCount();
                for(int i = 0;  i< IOTDEV_MAX_NUM; i++)
                {
                    if(g_xWITConfig.my_IOTDev[i].bIsUsed)
                    {
                        g_ts[i] = _ts; // update time
                        printf("HeartBeat... g_ts[%d] = %ld\n", i, g_ts[i]);
                        if(g_xWITConfig.my_IOTDev[i].bIsTimeout == true || g_xWITConfig.my_IOTDev[i].bIsPlugIn == false)
                        {
                            g_xWITConfig.my_IOTDev[i].bIsPlugIn = true; // 已经配置的传感器中，且已经超时的，这里意味着传感器断开之后，又重新连接网关
                            g_xWITConfig.my_IOTDev[i].bIsTimeout = false;
                            printf("dev[%d] online again now..... \n", i);
                        }
                    }

                }
                break;
            }

            case SLAVE_LINE_DETECT_DATA:
            {
                _ts = xTaskGetTickCount();
                // save line 
                unsigned short data = 0;
                IOT01B_Get_Collect_Data(&g_data_frame, &data);
                //printf("Get Data: %f%%\n", data / 10.0);
                for(int i = 0; i < IOTDEV_MAX_NUM; i++)
                {
                    //printf("g_xWITConfig.my_IOTDev[%d].bIsUsed = %d\n", i, g_xWITConfig.my_IOTDev[i].bIsUsed);
                    if(g_xWITConfig.my_IOTDev[i].bIsUsed && g_data_frame.ulDevID == g_xWITConfig.my_IOTDev[i].id && g_data_frame.ulDevSN == g_xWITConfig.my_IOTDev[i].sn)
                    {
                        // printf("g_xWITConfig.my_IOTDev[%d].bIsUsed = %d\n", i, g_xWITConfig.my_IOTDev[i].bIsUsed);
                        g_ts[i] = _ts; // update time
                        g_xWITConfig.my_IOTDev[i].rope_value = data;
                        // printf("g_xWITConfig.my_IOTDev[%d].rope_value = %d\n", i, g_xWITConfig.my_IOTDev[i].rope_value);
                        if(xSemaphoreDataResp != NULL)
                        {
                            xSemaphoreGive(xSemaphoreDataResp);
                        }
                        g_DataUpdateFlag = true; // MQTT Upload Data
                        Stop_Detect(g_data_frame.ulDevID, g_data_frame.ulDevSN);    
                    }
                }
                break;
            }

            default:
                break;
            }

            //Set_Threshold(g_data_frame.ulDevID, g_data_frame.ulDevSN, 280); 设置门阈值
        }
        vTaskDelay(10);
    }
}


unsigned long getIotConnectNum(void)
{
    unsigned long ulExistNum = 0;
    for(int i = 0 ; i < IOTDEV_MAX_NUM; i++ ) 
    {
        if(g_xWITConfig.my_IOTDev[i].bIsPlugIn == true) 
        {
            ulExistNum++;        
        }
    }
    return ulExistNum;
}


